c
c
      subroutine ffttest

c
c     TESTS FFT PERFORMANCE ON DIFFERENT SYSTEMS
c     
      implicit none
c
c     maximum sizes
c

      integer 
     $     nmax,lotmax,
     $     naux1,naux2
      parameter(nmax=256+1,
     $          lotmax=256,
     $          naux1=20000,
     $          naux2=20000)


      double precision gimmetime
      external gimmetime

      integer
     $     i,nloop,j,iodd, ipot, ipot2,n,lot,ierr,k


      double precision
     $     t0,t1,dn,nops, errsum,dscale,fft_opcount
      external fft_opcount
      
c
c     the work arrays etc
c

      complex(kind=8)
     $     a(nmax*lotmax),
     $     b(nmax*lotmax),
     $     aux1(naux1), dcerr,
     $     aux2(naux2)

      FFT_HEADER

      m4_define([NODEBUG])
c
c     -------------------------------------------------------------
c

c
c     Must initialize to nonzero in order to measure true performance.
c     Some library routines, e.g. the ESSL, optimize if it is =0!
c      


      write(6,10)

      do iodd =0,1              ! whether the lda is n or n+1
         if(iodd.eq.0) write(6,40)
         if(iodd.eq.1) write(6,50)
         do ipot=4,8            ! loop through different sizes n
            n  = 2**ipot
            dn = dble(n)
            dscale = 1.d0/dn
            write(6,30) n
            do ipot2=4,max(6,ipot)
               m4_ifdef([DEBUG],[lot=1],[lot = 2**ipot2])
c
c              initialize FFT
c
               nops = 2.d0*dble(lot)*fft_opcount(n)
               nloop = max(int(10.d7/nops),1)
               m4_ifdef([DEBUG],[nloop=1])

               do i=1, nmax*lotmax
                  a(i)=cmplx(sin(dble(i)),sin(dble(i))**2)
                  b(i)=a(i)
               end do

               FFT_PERM_INIT(n,aux1)
                
               FFT_AUX_SPACE(aux1,naux1,aux2,naux2,fs,dxml_ffts)



               t1=0
               do i=1,nloop
                  t0=gimmetime()

                  FFT_LOCAL_INIT(n,a,(n+iodd),lot,-1)
                  FFT_MULTIPLE_BACKWARD(n,a,(n+iodd),lot,k,a,1)
                  FFT_LOCAL_INIT(n,a,(n+iodd),lot,1)
                  FFT_MULTIPLE_FORWARD(n,a,(n+iodd),lot,k,a,1)

                  t1 = t1+gimmetime()-t0
                  a(1:lot*(n+iodd))=b(1:lot*(n+iodd))     ! renormalize

               end do
               FFT_LOCAL_FREE
               FFT_LOCAL_FREE

               
               write(6,20) lot,dble(nloop)*nops/1d6/t1
               m4_ifdef([CHECK_FOR_ERROR],[
               errsum = 0
               do i=1, nmax*lotmax
                  dcerr = a(i)-cmplx(sin(dble(i)),sin(dble(i))**2)
                  errsum = errsum + dble (dcerr*conjg(dcerr))
               end do
               write(6,22) errsum
               ])
               call myflush(6)

            end do
         end do
      end do

      return

 10   format(' ============== FFT PERFORMANCE ==============')
 20   format(' number of sequences:',i4,' MFLOPS:',f12.6)
 22   format('                            ERROR: ',g20.10)
 30   format(' ------- fft lenght: ',i4,' ----------------')
 40   format(' <<<<<<<< LDA = LENGTH OF FFT >>>>>>>>>>>>>>>')
 50   format(' <<<<<<<< LDA = LENGTH OF FFT +1 >>>>>>>>>>>>')
      end

