c
      subroutine adjustfft(nfft, symms)

      include 'use.h'
      implicit none             ! implicit? Just say no
      include 'interface.h'

c
c     1996 Bernd Pfrommer
c     
c     
      
c
c     INPUT:
c     -----

      type (symmetry) symms     ! the symmetry operations of the crystal
c
c     INPUT/OUTPUT:
c     ------------
      integer 
     $     nfft(3)              ! fft grid dimensions
c
c     DESCRIPTION
c     -----------
c
c     Increases the fft grid in nfft(3) to fit a suitable value, e.g.
c     a power of two, three ...
c
c     Also checks to make sure the resulting grid is compatible with
c     the nonprimitive translations, e.g. if there is a translation
c     of 1/3, the grid must be a multiple of 3.
c
c
c     ------------------- local arrays ---------------------------

      integer
     $     i,j, k,ns,
     $     numfast
      double precision 
     $     tau,pi2

      


      logical ifit

c
m4_ifdef([SGI_PC],[
      parameter(numfast=74) 
      integer
     $     fastsize(numfast)

      data fastsize 
     $     /1,  2,  4, 8, 16, 18,
     $     20, 24, 25, 32, 36, 40, 50, 
     $     54, 64, 72, 75, 80,
     $     81, 90, 96,100,108,
     $    120,125,128,144,150,
     $    160,162,180,192,200,
     $    216,225,240,243,250, 256,
     $     270, 288, 300, 320, 324, 360, 375,
     $     384, 400, 405, 432, 450, 480, 486,
     $     500, 512, 540, 576, 600, 625, 640,
     $     648, 675, 720, 729, 750, 768, 800,
     $     810, 864, 900, 960, 972, 1000, 1024/])

m4_ifdef([RS6K],[
      parameter(numfast=68)

      integer
     $     fastsize(numfast)
      data fastsize 
     $     /2,4,6,8,10,12,14,16,18,
     $     20,22,24,28,30,32,36,40,42,
     $     44,48,56,60,64,66,70,72,80,
     $     84,88,90,96,110,112,120,126,128,
     $     132,140,144,154,160,168,176,180,192,
     $     198,210,220,224,240,252,256,264,280,
     $     288,308,320,330,336,352,360,384,396,
     $     420,440,448,462,480/])

m4_ifdef([ALPHA],[
      parameter(numfast=52)
      integer
     $     fastsize(numfast)
      data fastsize 
     $      /1,   2,   3,   4,   5 ,  6,   8,   9,  10, 
     $      12,  15,  16,  18,  20,  24,  25,  27,  30, 
     $      32,  36,  40,  45,  48,  50,  54,  60,  64, 
     $      72,  75,  80,  81,  90,  96, 100, 108, 120,
     $     125, 128, 135, 144, 150, 160, 162, 180, 192, 
     $     200, 216, 225, 240, 243, 250, 256/])

m4_ifdef([CVX],[
      parameter(numfast=85)
      integer
     $     fastsize(numfast)
      data fastsize 
     $     /1, 2, 3, 4, 5, 6, 8, 9, 10, 
     $     16, 18, 20, 24, 25, 27, 30, 
     $     32, 36, 40, 45, 48, 50, 54, 60, 64, 
     $     72, 75, 80, 81, 90, 96, 100, 108, 120,  
     $     125, 128, 135, 144, 150,160, 162, 180, 192, 
     $     200, 216, 225, 240, 243, 250, 256, 270, 288, 
     $     300, 320, 324, 360, 375, 384, 400, 405, 432,
     $     450, 480, 486, 500, 512, 540, 576, 600, 625, 
     $     640, 648, 675, 720, 729, 750, 768, 800, 810, 
     $     864, 900, 960, 972, 1000, 1024 / ])
m4_ifdef([CRAY],[
      parameter(numfast=86)
      integer
     $     fastsize(numfast)
      data fastsize 
     $      /1,   2,   3,   4,   5 ,  6,   8,   9,  10, 
     $      12,  15,  16,  18,  20,  24,  25,  27,  30, 
     $      32,  36,  40,  45,  48,  50,  54,  60,  64, 
     $      72,  75,  80,  81,  90,  96, 100, 108, 120,
     $     128, 135, 144, 150, 160, 162, 180, 192, 
     $     200, 216, 225, 240, 243, 250, 256, 
     $     270, 288, 300, 320, 324, 360, 375,
     $     384, 400, 405, 432, 450, 480, 486,
     $     500, 512, 540, 576, 600, 625, 640,
     $     648, 675, 720, 729, 750, 768, 800,
     $     810, 864, 900, 960, 972, 1000, 1024/])
m4_ifdef([SUN],[
      parameter(numfast=86)
      integer
     $     fastsize(numfast)
      data fastsize 
     $      /1,   2,   3,   4,   5 ,  6,   8,   9,  10, 
     $      12,  15,  16,  18,  20,  24,  25,  27,  30, 
     $      32,  36,  40,  45,  48,  50,  54,  60,  64, 
     $      72,  75,  80,  81,  90,  96, 100, 108, 120,
     $     128, 135, 144, 150, 160, 162, 180, 192, 
     $     200, 216, 225, 240, 243, 250, 256, 
     $     270, 288, 300, 320, 324, 360, 375,
     $     384, 400, 405, 432, 450, 480, 486,
     $     500, 512, 540, 576, 600, 625, 640,
     $     648, 675, 720, 729, 750, 768, 800,
     $     810, 864, 900, 960, 972, 1000, 1024/])

      pi2 = 8.d0*atan(1.d0)


      do i=1,3 
         do j=nfft(i), fastsize(numfast)
c
c           check if j is compatible with the nonprimitive translations
c
            ifit = .true.
            do ns = 1,symms%ntrans
               tau = symms%tnp(i,ns)/pi2
               if((tau*dble(j)-dble(nint(tau*dble(j))))
     $              .gt.1d-6) ifit = .false.
            end do
c
c           check if this is also an allowed fft grid size
c
            if(ifit) then
               do k=1,numfast
                  if(fastsize(k).eq.j) goto 10
               end do
            endif
         end do
         write(9,110) 
         stop

 10      continue
         nfft(i)=j
      end do

      return

 110  format(' *** ERROR IN ADJUSTFFT: ',
     $     'FFT GRIDSIZE EXCEEDS IN DIRECTION',i3,
     $     /'     THIS CAN BE DUE TO STRANGE NONPRIMITIVE'
     $     /'     TRANSLATIONS IN THE SYMMETRY OPERATIONS',
     $     /'     OR JUST BECAUSE THE SYSTEM IS TOO BIG.',
     $     /'     ADD LARGER FFT SIZES IN ADJUSTFFT OR',
     $     /'     IMPROVE THE COORDINATES/LATTICE VECTORS.')


      end


