c
c
      module all_to_all_module
c
c     first some global variables. 
c
      integer ata_nproc         ! number of processors

m4_ifdef([SHMEM],[
      complex(kind=8) ata_target_dc_buf(1)
      pointer(ata_target_dc_buf_p,ata_target_dc_buf)
      complex(kind=8) ata_source_dc_buf(1)
      pointer(ata_source_dc_buf_p, ata_source_dc_buf)
      integer ata_pSync(1)
      pointer(ata_pSync_p, ata_pSync)
      complex(kind=8) ata_pWrk(1)
      pointer(ata_pWrk_p, ata_pWrk)
      integer ata_pWrk_len      ! length of the pWrk array
      integer ata_dc_buf_len    ! length of the complex(kind=8) work arrays
])
      contains

      subroutine allocate_shmem_work(nproc,len)
      implicit none
      integer nproc,ierr,len
m4_ifdef([SHMEM],[
      include 'mpp/shmem.fh'
      ata_nproc = nproc
c
c     allocate the sync array once and forever
c
      call shpalloc(ata_pSync_p,SHMEM_REDUCE_SYNC_SIZE,ierr,-1)
      ata_pSync(1:SHMEM_REDUCE_SYNC_SIZE)=
     $     SHMEM_SYNC_VALUE
c
c     allocate the pWrk array
c
      call shpalloc(ata_pWrk_p,max((len/2+1)*2,
     $     SHMEM_REDUCE_MIN_WRKDATA_SIZE),ierr,-1)
      ata_pWrk_len = len
      
c
c     allocate the complex(kind=8) buffers
c
      call shpalloc(ata_target_dc_buf_p,2*len,ierr,-1)
      call shpalloc(ata_source_dc_buf_p,2*len,ierr,-1)
      ata_dc_buf_len = len
])
      end subroutine
c
c
c      
      subroutine deallocate_shmem_work()
      implicit none
      integer nproc,ierr
m4_ifdef([SHMEM],[
      
      include 'mpp/shmem.fh'
      call shpdeallc(ata_pSync_p,ierr,-1)
      call shpdeallc(ata_pWrk_p,ierr,-1)
      ata_pWrk_len = 0
c
c     deallocate the complex(kind=8) work arrays
c
      call shpdeallc(ata_target_dc_buf_p,ierr,-1)
      call shpdeallc(ata_source_dc_buf_p,ierr,-1)
      ata_dc_buf_len = 0
])
      end subroutine

c
c     allocs and initializes the global reduction work arrays
c     required  for shmem
c
c     
c
      subroutine fast_all_sum_all_alloc_dc(len)

      implicit none
      integer len,              ! number of elements to be reduced
     $     ierr
m4_ifdef([SHMEM],[
      include 'mpp/shmem.fh'
      if(len.gt.ata_dc_buf_len) then
         call shpdeallc(ata_target_dc_buf_p,ierr,-1)
         call shpdeallc(ata_source_dc_buf_p,ierr,-1)

         call shpalloc(ata_target_dc_buf_p,2*len,ierr,-1)
         call shpalloc(ata_source_dc_buf_p,2*len,ierr,-1)
         ata_dc_buf_len = len
      endif
      if(len.gt.ata_pWrk_len) then
         call shpdeallc(ata_pWrk_p,ierr,-1)
         call shpalloc(ata_pWrk_p,max((len/2+1)*2,
     $        SHMEM_REDUCE_MIN_WRKDATA_SIZE),ierr,-1)
         ata_pWrk_len = len
      endif
])
      end subroutine



m4_ifdef([CRAY],[m4_define([MPI_INTEGER4],[MPI_INTEGER])])
c
c     ================ summation subroutines ==============================
c

c
c     this subroutine uses the fast shmem routines,
c     but needs a call to the allocation routine before
c     it is called the first time
c

      subroutine fast_all_sum_all_complex(data,len)
      implicit none
      integer ierr
      integer len
      complex(kind=8) data(*)

m4_ifdef([SHMEM],[
      ata_source_dc_buf(1:len)=data(1:len)
      call barrier()
      call shmem_comp8_sum_to_all(ata_target_dc_buf, 
     $     ata_source_dc_buf, len, 0,  0, ata_nproc,ata_pWrk,ata_pSync)
      data(1:len)=ata_target_dc_buf(1:len)
],[
c
c     just the regular thing for MPI
c
m4_ifdef([MPI],[      
      include 'mpif.h'
      complex(kind=8), allocatable :: mpibuf(:)
      if(len.le.0) return
      allocate(mpibuf(len))
      call mpi_allreduce(data(1),mpibuf(1),len,
     $     MPI_DOUBLE_COMPLEX,MPI_SUM, MPI_COMM_WORLD,ierr)
      data(1:len)=mpibuf(1:len)
      deallocate(mpibuf)
])

])
      end subroutine



      subroutine all_sum_all_complex(data,len)
      integer ierr
      integer len
      complex(kind=8) data(*)
      complex(kind=8), allocatable :: mpibuf(:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(len))
      call mpi_allreduce(data(1),mpibuf(1),len,
     $     MPI_DOUBLE_COMPLEX,MPI_SUM, MPI_COMM_WORLD,ierr)
      data(1:len)=mpibuf(1:len)
      deallocate(mpibuf)
])
      end subroutine
      
      subroutine all_sum_all_complex1(data)
      integer ierr
      complex(kind=8) data
      complex(kind=8) mpibuf
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_allreduce(data,mpibuf,1,
     $     MPI_DOUBLE_COMPLEX,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=mpibuf
])
      end subroutine

      subroutine all_sum_all_complex2(data,len)
      integer ierr
      integer len
      complex(kind=8) data(:,:)
      complex(kind=8), allocatable :: mpibuf(:,:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(size(data,1),size(data,2)))
      call mpi_allreduce(data(1,1),mpibuf(1,1),len,
     $     MPI_DOUBLE_COMPLEX,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=mpibuf

      deallocate(mpibuf)
])
      end subroutine

      subroutine all_sum_all_complex3(data,len)
      integer ierr
      integer len
      complex(kind=8) data(:,:,:)
      complex(kind=8), allocatable :: mpibuf(:,:,:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(size(data,1),size(data,2),size(data,3)))
      call mpi_allreduce(data(1,1,1),mpibuf(1,1,1),len,
     $     MPI_DOUBLE_COMPLEX,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=mpibuf
      deallocate(mpibuf)
])
      end subroutine

      subroutine all_sum_all_complex4(data,len)
      integer ierr
      integer len
      complex(kind=8) data(:,:,:,:)
      complex(kind=8), allocatable :: mpibuf(:,:,:,:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(size(data,1),size(data,2),size(data,3),
     $     size(data,4)))
      call mpi_allreduce(data(1,1,1,1),mpibuf(1,1,1,1),len,
     $     MPI_DOUBLE_COMPLEX,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=mpibuf
      deallocate(mpibuf)
])
      end subroutine

      subroutine all_sum_all_complex_sub(comm, data,len)
      integer ierr
      integer len,comm
      complex(kind=8) data(*)
      complex(kind=8), allocatable :: mpibuf(:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(len))
      call mpi_allreduce(data(1),mpibuf(1),len,
     $     MPI_DOUBLE_COMPLEX,MPI_SUM, comm,ierr)
      data(1:len)=mpibuf(1:len)
      deallocate(mpibuf)
])
      
      end subroutine

      subroutine all_sum_all_complex1_sub(comm, data)
      integer ierr
      integer comm
      complex(kind=8) data
      complex(kind=8) mpibuf
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_allreduce(data,mpibuf,1,
     $     MPI_DOUBLE_COMPLEX,MPI_SUM, comm,ierr)
      data=mpibuf
])
      
      end subroutine


      subroutine all_sum_all_double(data,len)
      integer ierr
      integer len
      double precision data(len)
      double precision, allocatable :: mpibuf(:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(len))
      call mpi_allreduce(data(1),mpibuf(1),len,
     $     MPI_REAL8,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=mpibuf
      deallocate(mpibuf)
])
      end subroutine
 
      subroutine all_sum_all_double1(data)
      integer ierr
      double precision data, databuf
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_allreduce(data,databuf,1,
     $     MPI_REAL8,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=databuf
])
      
      end subroutine


      subroutine all_sum_all_double2(data,len)
      integer ierr
      integer len
      double precision :: data(:,:)
      double precision, allocatable :: mpibuf(:,:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(size(data,1),size(data,2)))
      call mpi_allreduce(data(1,1),mpibuf(1,1),len,
     $     MPI_REAL8,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=mpibuf
      deallocate(mpibuf)
])
      end subroutine


      subroutine all_sum_all_double3(data,len)
      integer ierr
      integer len
      double precision :: data(:,:,:)
      double precision, allocatable :: mpibuf(:,:,:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(size(data,1),size(data,2),size(data,3)))
      call mpi_allreduce(data(1,1,1),mpibuf(1,1,1),len,
     $     MPI_REAL8,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=mpibuf
      deallocate(mpibuf)
])
      
      end subroutine






      subroutine all_sum_all_integer(data,len)
      integer ierr
      integer len
      integer data(*)
      integer, allocatable :: mpibuf(:)
m4_ifdef([MPI],[      
      include 'mpif.h'
      if(len.le.0) return
      allocate(mpibuf(len))
      call mpi_allreduce(data,mpibuf,len,
     $     MPI_INTEGER4,MPI_SUM, MPI_COMM_WORLD,ierr)
      data(1:len)=mpibuf(1:len)

      deallocate(mpibuf)
])
      
      end subroutine

      subroutine all_sum_all_integer1(data)
      integer ierr
      integer data, databuf
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_allreduce(data,databuf,1,
     $     MPI_INTEGER4,MPI_SUM, MPI_COMM_WORLD,ierr)
      data=databuf
])
      
      end subroutine
c
c     ================= maximum subroutines ===========================
c
      subroutine all_max_all_double1(data)
      integer ierr
      double precision data
      double precision mpibuf
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_allreduce(data,mpibuf,1,
     $     MPI_REAL8,MPI_MAX, MPI_COMM_WORLD,ierr)
      data=mpibuf

])
      end subroutine

      subroutine all_min_all_double1(data)
      integer ierr
      double precision data
      double precision mpibuf
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_allreduce(data,mpibuf,1,
     $     MPI_REAL8,MPI_MIN, MPI_COMM_WORLD,ierr)
      data=mpibuf

])
      end subroutine

      subroutine all_max_all_int1(data)
      integer ierr
      integer data
      integer mpibuf
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_allreduce(data,mpibuf,1,
     $     MPI_INTEGER4,MPI_MAX, MPI_COMM_WORLD,ierr)
      data=mpibuf

])
      end subroutine
c
c     ================= broadcast subroutines ===========================
c
      subroutine my_broadcast_int1(data, root)
      integer ierr
      integer data
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data,1, MPI_INTEGER4,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine

      subroutine my_broadcast_int(data, count, root)
      integer ierr
      integer count
      integer data
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data,count, MPI_INTEGER4,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine

      subroutine my_broadcast_dcmplx(data, count, root)
      integer ierr
      integer count
      complex(kind=8) data(count)
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data(1),count, MPI_DOUBLE_COMPLEX,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine

      subroutine my_broadcast_dcmplx2(data, count, dim, root)
      integer ierr
      integer count
      integer dim
      complex(kind=8) data(count,dim)
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data(1,1),count*dim, MPI_DOUBLE_COMPLEX,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine

      subroutine my_broadcast_dcmplx3(data, count,root)
      integer ierr
      integer count
      complex(kind=8) data
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data,count, MPI_DOUBLE_COMPLEX,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine


      subroutine my_broadcast_double(data, count, root)
      integer ierr
      integer count
      double precision data(count)
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data(1),count, MPI_REAL8,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine

      subroutine my_broadcast_double1(data, root)
      integer ierr
      double precision data
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data,1, MPI_REAL8,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine

      subroutine my_broadcast_double3(data, count, root)
      integer ierr
      integer count
      double precision data
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data,count, MPI_REAL8,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine

      subroutine my_broadcast_char(data, count, root)
      integer ierr
      integer count
      character *(count) data
      integer root
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_bcast(data,count, MPI_CHARACTER,
     $     root, MPI_COMM_WORLD,ierr)

])
      end subroutine

c
c     ================= dot product subroutines =======================
c
      complex(kind=8) function parallel_zdotc(n,x,incx,y,incy) 
      include 'flibcalls.ph'
      integer n, incx, incy
      complex(kind=8) 
     $     x(*), y(*)
      
      parallel_zdotc=mzdotc(n,x(1),incx,y(1),incy)
      call all_sum_all_complex1(parallel_zdotc)
      return
      end function

      complex(kind=8) function parallel_zdotc2(n,x,incx,y,incy) 
      include 'flibcalls.ph'
      integer n, incx, incy
      complex(kind=8) 
     $     x, y
      
      parallel_zdotc2=mzdotc(n,x,incx,y,incy)
      call all_sum_all_complex1(parallel_zdotc2)
      return
      end function

c
c     ================ parallel barrier routine =======================

      subroutine parallel_barrier()
      integer info
m4_ifdef([MPI],[      
      include 'mpif.h'
      call mpi_barrier(MPI_COMM_WORLD,info)
])
      end subroutine
c
c
c     ================ subgroup communication routines =======================

c
c     creates communicators and corresponding groups for a list of ranks
c
      subroutine all_split_all(key,rank, subcomm)
      integer rank, key
      integer subcomm ! output
m4_ifdef([MPI],[      
      include 'mpif.h'
      integer info
      call mpi_comm_split(MPI_COMM_WORLD, key, rank, subcomm,info)
])
      end subroutine



      end module


