	SUBROUTINE angular_wavefn(wavefn,nkpt,kx,ky,kz,
     >                    bess,ngrid,result,tau,istart,
     >                        gmax,rc,step,bdot,cart)
m4_ifdef([MPI],[
      include 'mpif.h' ])
	  
	  integer nkpt,ngrid,istart,ii,jj,kk,ll,index
	  complex alpha(4,7),phase,wavefn(nkpt)
	  double precision bess(ngrid,4),tau(3),result(4,7)
	  integer kx(nkpt),ky(nkpt),kz(nkpt)
	  double precision k1,k2,k3,kc(3),norm
	  double precision vec(3),ylm(4,7),ff(4),gval
	  double precision gmax,rc,step,bdot(3,3),angle
	  double precision cart(3,3),sum
	  double precision cellvol ,j0,j1,j2,j3,rval
	  double precision ekin(nkpt)
c
c
c
c
c DESCRIPTION: computes the projected wavefunctions onto
c              angular momentum eigenstates with a square-well
c              radial part with 
c
c             nkpt = number of g-vectors on this processor
c             wavefn = complex wavefunction            
c             rc = cutoff radius of square-well function
c             alpha = value of scalar product 
c             tau = relative coordinate of atom in question
c                     (in lattice coordinates) 
c             bdot = metric used to compute scalar products of
c                     vectors in reciprocal space
c             cart = transformation matrix from lattice coordinates
c                     to cartesian coordinates
c             istart = if istart =1 generate bessel function integral
c
c
c
c
c
c
c COMPUTE RELEVANT PARAMETERS    
        cellvol = abs((2.0d0*3.141592645)**3*
     >          1.0d0/(cart(1,1)*cart(2,2)*cart(3,3)+
     >              cart(1,2)*cart(2,3)*cart(3,1)+
     >              cart(1,3)*cart(2,1)*cart(3,2)-
     >              cart(3,1)*cart(2,2)*cart(1,3)-
     >              cart(3,2)*cart(2,3)*cart(1,1)-
     >              cart(3,3)*cart(2,1)*cart(1,2))  )
        step = gmax*rc/(1.0d0*(ngrid)) 
        if (istart .eq. 1) then
c GENERATE BESSEL FUNCTION INTEGRAL
          bess(:,:) = 0.0d0
          do ii=1,ngrid
            do jj=1,ii
              rval = jj*step 
c              
c BESSEL FUNCTIONS TIMES rval**2*step
c
              j0 = sin(rval)*rval*step
              j1 = j0/rval - rval*cos(rval)*step
              j2 = 3.0d0/rval*j1 - j0
              j3 = 5.0d0/rval*j2 - j1
              bess(ii,1) = bess(ii,1) + j0
              bess(ii,2) = bess(ii,2) + j1
              bess(ii,3) = bess(ii,3) + j2
              bess(ii,4) = bess(ii,4) + j3
            enddo
          enddo
       endif  ! end of startup if then
c
c GENERATE KINETIC ENERGY
c
       ekin(:) = 0.0d0
       do ii=1,nkpt
         vec(1) = kx(ii)
         vec(2) = ky(ii)
         vec(3) = kz(ii)
         do jj=1,3
           do kk=1,3
             ekin(ii) = ekin(ii)+vec(jj)*vec(kk)*bdot(jj,kk)
           enddo
         enddo
       enddo
c
c COMPUTE alpha
c
      alpha(:,:) = 0.0d0
      do ii=1,nkpt   ! sum over gvectors
         gval = sqrt(ekin(ii))
         index = floor(gval*rc/step) 
         k1 = kx(ii)
         k2 = ky(ii)
         k3 = kz(ii)
         angle = k1*tau(1)+k2*tau(2)+k3*tau(3)
         phase = exp(2.0d0*3.141592645*cmplx(0.0d0,angle))

         if (index .gt. ngrid) index = ngrid

         if (index .eq. 0) then
            alpha(1,1) = alpha(1,1) + phase*wavefn(ii)*rc**3/3.0d0
         else

           kc(1) = cart(1,1)*k1+cart(1,2)*k2+cart(1,3)*k3
           kc(2) = cart(2,1)*k1+cart(2,2)*k2+cart(2,3)*k3
           kc(3) = cart(3,1)*k1+cart(3,2)*k2+cart(3,3)*k3
           k1 = kc(1)
           k2 = kc(2)
           k3 = kc(3)
           do ll=1,4
             ff(ll) = bess(index,ll)
           enddo
           ylm(1,1) = 1.0d0

           ylm(2,1) = 1.0d0/sqrt(3.0d0)*k1/gval
           ylm(2,2) = 1.0d0/sqrt(3.0d0)*k2/gval
           ylm(2,3) = 1.0d0/sqrt(3.0d0)*k3/gval
           
           ylm(3,1) = 1.0d0/sqrt(3.0d0)*k1*k2/gval**2
           ylm(3,2) = 1.0d0/sqrt(3.0d0)*k2*k3/gval**2
           ylm(3,3) =  1.0d0/sqrt(3.0d0)*k1*k3/gval**2
           ylm(3,4) = sqrt(3.0d0)/2.0d0*(k1**2-k2**2)/gval**2
           ylm(3,5) = 1.5d0*k3**2/gval**2 - 0.5d0

           ylm(4,1) = 2.5d0*k1**3/gval**3 - 1.5d0*k1/gval
           ylm(4,2) = 2.5d0*k2**3/gval**3  - 1.5d0*k2/gval
           ylm(4,3) = 2.5d0*k3**3/gval**3  - 1.5d0*k3/gval 
           ylm(4,4) = sqrt(3.75d0)*k1*(k2**2-k3**2)/gval**3 
           ylm(4,5) = sqrt(3.75d0)*k3*(k1**2-k2**2)/gval**3
           ylm(4,6) = sqrt(3.75d0)*k2*(k3**2-k1**2)/gval**3
           ylm(4,7) = sqrt(15.0d0)*k1*k2*k3/gval**3

           do kk=1,4
            do ll=1,2*kk-1
              alpha(kk,ll) = alpha(kk,ll) + 
     >               phase*wavefn(ii)*ff(kk)*ylm(kk,ll)/gval**3  
            enddo
          enddo
           
         endif ! end of if  statement for g=0 component
       enddo ! end of sum over g-vectors
m4_ifdef([MPI],[ 
       call mpi_allreduce(alpha,alpha,28,mpi_complex,mpi_sum,
     >                mpi_comm_world,ierr)])
       norm = 0.0d0
       do kk=1,4
         do ll=1,2*kk-1
           norm = norm + 
     >           alpha(kk,ll)*conjg(alpha(kk,ll))
         enddo
       enddo 

       result(:,:) = alpha(:,:)*conjg(alpha(:,:))/norm
       return
       end
