c
c
c
      subroutine apply_ham_wavefn(imonitor,hpsi,ham,psi,nwfn,work)

      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!

      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'

 
c
c     INPUT:
c     -----
      type (hamiltonian)
     $     ham                  ! hamiltonian at a given k-point

      complex(kind=8) 
     $     psi(*) ! input wave functions

      integer
     $     imonitor,            ! monitor flags
     $     nwfn                 ! number of wave functions

c
c     OUTPUT:
c     ------

      complex(kind=8) hpsi(*)

c
c     applies the hamiltonian ham to the first nwfn wave functions contained 
c     in psi, and returns the answer in the first nwfn wave functions in hpsi
c

c
c     WORK:
c     -----

      complex(kind=8) work(*) ! work(nanl*nwfn)

c
c     ------------- local variables --------------------------------
c
      integer i,len,nanl,j, dnx, nrodst1, nrodst2
      double precision emod,ekinj,ekinmod(3),t0,t1,t2,n,opcount,t3
      complex(kind=8) dcone, dczero
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0))

      double precision myderf, gimmetime
      external myderf, gimmetime

c
c     apply local potential in realspace
c
      len=ham%gspace%length     ! length of gspace

      if(iand(imonitor,1).eq.1) 
     $     t0=gimmetime()
      do i=0,nwfn-1
         call fft_convolute(ham%gspace, ham%fftstruc, hpsi(i*len+1),
     $        ham%vloc(1), psi(i*len+1)) 
      end do
      if(iand(imonitor,1).eq.1)  then
         t2=gimmetime()-t0
         dnx=mod(ham%gspace%fftsize(4)-ham%gspace%fftsize(5)
     $        +ham%gspace%fftsize(1),ham%gspace%fftsize(1))+1
         nrodst1= (dnx*ham%gspace%fftsize(3))/ham%gspace%nproc 
         if( mod(dnx*ham%gspace%fftsize(3),ham%gspace%nproc).gt
     $        .ham%gspace%myproc) nrodst1=nrodst1+1
         nrodst2= (ham%gspace%fftsize(2)*ham%gspace%fftsize(3))
     $        /ham%gspace%nproc  
         if(mod(ham%gspace%fftsize(2)*ham%gspace%fftsize(3)
     $        ,ham%gspace%nproc).gt.ham%gspace%myproc)   nrodst2
     $        = nrodst2+1
         
c
c        assume base 2 cmplx-cmplx: flops = 5n log n-6n+6
c
         n=dble(ham%gspace%fftsize(3))
         opcount = dble(ham%gspace%lorder)*(5*n*log(n)/log(2.)-6*n+6)
         n=dble(ham%gspace%fftsize(2))
         opcount = opcount + dble(nrodst1)*(5*n*log(n)/log(2.)-6*n+6)
         n=dble(ham%gspace%fftsize(1))
         opcount = opcount + dble(nrodst2)*(5*n*log(n)/log(2.)-6*n+6)

         opcount = opcount*2 + ham%gspace%r_size*6 ! fft-1 and convolution
         opcount = opcount *nwfn ! for each wave function
         if(t2.gt.0) then
            write(9,'(a,i3,a12,f12.6,a,f12.6)') 
     $           'TIME FOR ',nwfn,' FFTS:',t2,', MFLOPS=',(opcount/t2
     $           /1d6)
            call myflush(9)
         endif
      endif


c     kinetic energy term: multiply complex vector with real matrix
  
      ekinmod=ham%ekinmod

      do i=0, nwfn-1
         do j=1, len
            ekinj= ham%gspace%ekin(j)
c
c           modified kinetic energy expression
c
            emod = ekinj-ham%shift + ekinmod(1)*
     $           (1.d0+myderf((ekinj-ekinmod(2))/ekinmod(3)))
            hpsi(j+i*len) = hpsi(j+i*len) + psi(j+i*len)*emod
         end do
      end do
c
c     apply the nonlocal part of the hamiltonian
c
      nanl=ham%vnloc%nvecs

      if(iand(imonitor,1).eq.1) then
        call parallel_barrier()
        t0=gimmetime()
      endif
      if(nanl.gt.0) then
         if(nwfn.eq.1) then     ! do matrix-vector multiply
            call mzgemv('C', len, nanl, dcone,ham%vnloc%data(1,1,1),
     $           len,psi(1),1,dczero,work(1),1)
         else
            call mzgemm('C', 'N', nanl, nwfn, len, dcone, 
     $           ham%vnloc%data(1,1,1), len, psi(1), len, dczero, 
     $           work(1),nanl)
         endif
      endif

c
c     scale with xnorm
c
      do i=1, nanl
         call mzdscal(nwfn, ham%xnorm(i), work(i), nanl);
      end do


      call fast_all_sum_all_alloc_dc(nanl*nwfn)
      call fast_all_sum_all_complex(work(1),nanl*nwfn)

c
c     linearly combine the projectors to get H_nloc*psi
c     
      if(nanl.gt.0) then
         if(nwfn.eq.1) then     ! do matrix-vector multiply
            call mzgemv('N', len, nanl, dcone,ham%vnloc%data(1,1,1)
     $           ,len,work(1),1,dcone,hpsi(1),1)
         else
            call mzgemm('N','N',len,  nwfn,nanl,dcone,
     $           ham%vnloc%data(1,1,1), len, work(1), nanl, dcone, 
     $           hpsi(1), len)
         endif
      endif
         
      if(iand(imonitor,1).eq.1)  then
         opcount = 2.d0*8.d0*dble(nanl)*dble(nwfn)*dble(len)
         t2=gimmetime()-t0
         if(t2.gt.0) 
     $        write(9,'(a,i3,a12,f12.6,a,f12.6)') 'TIME FOR ',nanl,
     $        ' PROJECTORS:',t2,', MFLOPS=',opcount/t2/1.d6
      endif

      return

      end



      
