c
c     @process extchk
      subroutine cg_blocksolve(imonitor, crys, p0,
     $     ham, maxit, delta,
     $     gapp, rk,
     $     n_ukq, n_ukqapp,  
     $     e_k, e_kapp, 
     $     e_kq, e_kqapp, 
     $     ukqapp,
     $     ukq, uk, ukqtil, iguess, phi)
c
c     1996 by Bernd Pfrommer and Francesco Mauri
c     

      use all_to_all_module
      include 'use.h'
      implicit none
      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'
c
c     INPUT:
c     -----

      type (hamiltonian) ham    ! the hamiltonian at k+q
      type (crystal) crys       ! need the bvec information for gradient

      type (parallel_gspace)
     $     gapp                 ! the parallel gspace for the approx, at k+q

      integer
     $     iguess,              ! if iguess=1, compute starting guess
     $     p0,                  ! direction in which the p0 operator acts
     $     imonitor,            ! monitor flag indicating additional printout
     $     n_ukqapp,            ! number of approximate u_k+q  and u_k
     $     n_ukq,               ! number of u_k+q and u_k (must be same)
     $     maxit                ! max number of iterations

      double precision
     $     rk(3),               ! the kpoint for u_k
     $     e_k(n_ukq), e_kapp(n_ukqapp),  ! energy eigenvalues of u_k
     $     e_kq(n_ukq),e_kqapp(n_ukqapp), ! energy eigenvalues of u_k+q
     $     delta                ! convergence criterion

      complex(kind=8)
     $     ukqapp(*), ! approximate wave functions u_k+q
     $     phi(ham%dim,n_ukq),  ! contains grad|u_k>. Will be destroyed!
     $     ukq(ham%dim,n_ukq), ! the full occupied wave functions u_k+q
     $     uk(ham%dim,n_ukq) ! the full occupied states at k
c
c     INPUT/OUTPUT:
c     -------------
      
      complex(kind=8)
     $     ukqtil(ham%dim, n_ukq) ! the desired wave function ukqtilde

c
c     DESCRIPTION:
c     -----------

c
c     solves the equation
c     
c     (H_k+q - e_k*I + shift* Q) |u~_k+q> = -(1-Q)grad|u_k>
c     
c     by means of a conjugate gradient algorithm
c
c     Note: The ham%gspace and the gapp are expected to have rk=k+q
c     Note: If iguess.ne.1, then the ukqtil should contain starting
c           guess on input
c      
c
c     --------------------- local variables ----------------------
c
      integer 
     $     i,j, k, l, iwork,
     $     nunconv,             ! number of unconverged states
     $     nconvp1,             ! number of converged states plus one
     $     nnconvp1,            ! new number of unconverged states plus one
     $     iter,                ! the iteration number
     $     len,                 ! length of full vectors
     $     noccup,              ! number of occupied states
     $     nempty,              ! number of empty states
     $     ilumo,               ! index of lumo
     $     ihomo                ! index of homo

      double precision
     $     dgamma,              ! gamma 
     $     rkq(3),              ! k+q
     $     t0, gimmetime,
     $     a,c,
     $     kq(3)                ! temporary storage for k+q
      double precision, allocatable ::
     $     rhoold(:),           ! previous iterations rho
     $     rho(:),              ! absolute of the gradient
     $     eprecond(:),
     $     shift(:)             ! energy shift of the Hamiltonian [Ryd]

      complex(kind=8) dcone, dczero, dcmone
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcmone=(-1.d0,0.d0))


      complex(kind=8), allocatable :: 
     $     ukapp(:),            ! packed version of uk
     $     g(:,:),              ! the gradient
     $     gp(:,:),             ! the gradient
     $     h(:,:),              ! step direction
     $     hold(:,:),           ! previous iterations step direction 
     $     t(:,:),              ! some work vector
     $     work(:),             ! work arrays
     $     mat(:),              ! intermediate matrix
     $     gradukapp(:),        ! gradient of approximate uk
     $     me_k(:)              ! minus e_k, in complex(kind=8) form


      complex(kind=8)
     $     ssum,                ! debugging sum
     $     tkinav,              ! work variable for preconditioning
     $     dcgamma,             ! conjugation gamma
     $     dclambda             ! step length lambda, in complex(kind=8) form



      double precision    average_ekin

      external gimmetime, average_ekin

c
c     ------------------------------------------------------
c

c      write(9,*) 'ekq:',e_kq
c      write(9,*) 'uk on input',e_k
c      do i=1, ham%gspace%length
c         write(9,*) i,uk(i)
c      end do


      t0=gimmetime()

      len = ham%gspace%length
      ilumo=n_ukq+1             ! index of lumo
      ihomo=n_ukq               ! index of homo
      noccup = n_ukq
      nempty = n_ukqapp-n_ukq

      rkq=ham%gspace%rk         ! k+q

c
c     allocate a work array that is big enough for all purposes
c
      iwork =max(gapp%fftsize(1)*gapp%fftsize(2)*gapp%fftsize(3),
     $     ham%vnloc%nvecs*noccup, ! for apply_ham_wfn
     $     noccup*noccup)       ! for apply_proj_wfn

      allocate(work(iwork))
      work=0
c
c     -------- compute the rhs of the equation to solve. this is -------
c
c               psi = -(1-Q) * phi

c     
      call apply_proj_wavefn(len,-1.d0,1,ukq(1,1), noccup,
     $     phi(1,1),noccup,phi(1,1),work(1)) ! compute (1-Q) grad(u_k)

      call mzdscal(len*noccup,-1.d0,phi(1,1),1)
c
c     compute starting guess
c
      if(iguess.eq.1) then
         call cg_startguess(ham, crys,gapp, rk, ukqapp, uk, ukqtil,
     $        e_kapp, e_kqapp, p0, noccup, imonitor)
      endif

c
c     now ukqtil contains the full trial vectors
c
      
c
c     ------------ here starts the conjugate gradient part --------
c      

c
c     figure out how much the Hamiltonian must be shifted to make it 
c     positive  definite.
c
      allocate(shift(noccup))

      shift(1:noccup) = (e_k(1:noccup)-e_kq(1)) + 1.d0

      allocate(g(len,noccup))          ! used as scratch array here
      allocate(gp(len,noccup))
      allocate(h(len,noccup))
      allocate(hold(len,noccup))
      allocate(t(len,noccup))
      allocate(eprecond(noccup))
      allocate(rho(noccup)) ; allocate(rhoold(noccup))
      allocate(me_k(noccup))
      me_k(1:noccup)=cmplx(-e_k(1:noccup),0.d0,kind=8)

      do i=1, noccup
         eprecond(i) = 1.35*
     $        average_ekin(ukq(1,i),tkinav,ham%gspace%ekin(1),1,len)
      end do

      nconvp1 = 1
      nunconv = noccup - nconvp1 +1

      do iter=1, maxit
c
c        compute the gradient. can reuse information from previous step
c     
c

         if(iter.eq.1) then 
c           g=H*ukqtil
            call apply_ham_wavefn(imonitor,g(1,nconvp1),
     $           ham,ukqtil(1,nconvp1),nunconv,work(1))
c           g=g + shift *Q   * uktil
            call apply_proj_wavefn(len,shift(nconvp1),nunconv,
     $           ukq(1,1), noccup,
     $           ukqtil(1,nconvp1),nunconv,g(1,nconvp1),work(1)) 
c           g=g -e_k * uktil
            do i=nconvp1, noccup
               call mzaxpy(len,me_k(i),ukqtil(1,i),1,
     $              g(1,i),1)
            end do
            call mzaxpy(len*nunconv,dcmone,phi(1,nconvp1),1,
     $           g(1,nconvp1),1) !    g = g - b
         endif

c
c        compute residual
c
         gp(1:len,nconvp1:noccup) = g(1:len,nconvp1:noccup)
         nnconvp1 = nconvp1
         do i=nconvp1, noccup
            call precondition(gp(1,i),eprecond(i),
     $           ham%ekinmod(1),ham%gspace%ekin(1), 1,len)
            rho(i)=dble(parallel_zdotc(len,gp(1,i),1,g(1,i),1))
            
            if((rho(i).lt.delta).and.(i.eq.nnconvp1)) then
               nnconvp1=nnconvp1+1
            endif
         end do

c         if((iand(imonitor,8).eq.8).and.(nconvp1.lt.nnconvp1)) 
c     $        write(9,920) nconvp1,nnconvp1-1,iter


         nconvp1=nnconvp1
         nunconv = noccup - nconvp1 +1

         if(nconvp1.eq.noccup+1) goto 100

c
c        compute the step direction h. Conjugate it to previous step
c
         if(iter.eq.1) then
            h(1:len, nconvp1:noccup)=-gp(1:len, nconvp1:noccup)
         else
            do i=nconvp1,noccup
               dgamma = rho(i)/rhoold(i)
               h(1:len,i)=-gp(1:len,i) + dgamma*hold(1:len,i)
            end do
         endif
         
         rhoold(nconvp1:noccup)=rho(nconvp1:noccup)

         hold(1:len,nconvp1:noccup) = h(1:len,nconvp1:noccup)
c
c        compute t = A*h
c
         call apply_ham_wavefn(imonitor,t(1,nconvp1),ham,
     $        h(1,nconvp1),nunconv,work(1)) ! t = H*h
         call apply_proj_wavefn(len,shift(nconvp1),nunconv,ukq(1,1),
     $        noccup,h(1,nconvp1),nunconv,t(1,nconvp1),work(1)) ! t=t+shift*Q*h
c
c        compute the coefficients a and c for the line minimization
c        compute step length lambda

         do i=nconvp1, noccup
            call mzaxpy(len,me_k(i),h(1,i),1,t(1,i),1)
     $                          !t=t-e_k*h

            a=dble(parallel_zdotc(len,h(1,i),1,g(1,i),1))
            c=dble(parallel_zdotc(len,h(1,i),1,t(1,i),1))
            dclambda=cmplx(-a/c,0.d0,kind=8)
c
c           move to new position   
c
c           ukqtil = ukqtil+lambda*h
            call mzaxpy(len,dclambda,h(1,i),1,ukqtil(1,i),1) 

c           update to get the gradient
            call mzaxpy(len,dclambda,t(1,i),1, g(1,i),1) ! g = g + lambda*t
         end do


c         write(9,900) iter,rho(nconvp1:noccup)
c         call myflush(9)

      end do

      write(9,*) '*** WARNING: cg_blocksolve not converged in',
     $     maxit,' iterations'
      write(9,*) 'residual errors:', rho

 100  continue

      deallocate(me_k)
      deallocate(rho)   ;   deallocate(rhoold)
      deallocate(shift) ;   deallocate(eprecond)
      deallocate(g)     ;   deallocate(gp)
      deallocate(h)
      deallocate(hold)
      deallocate(t)
      deallocate(work)
      if(iand(imonitor,8).eq.8) then
         write(9,930) iter, gimmetime()-t0
         call myflush(9)
      endif


 900  format('iter:',i3,' rho=', 1000g12.6)
 910  format('cg_solve converged at iteration:',i3,
     $     ' with residual error rho=', g12.6)
 920  format(' STATES ',i3,'-',i3,' TOOK ',i3,' ITERATIONS ')
 930  format(' CGSOLVE TOOK ',i3,' ITERATIONS AND ', 
     $     f12.3, ' SECONDS')

      end

c
c     =====================================================================
c
c     @process extchk

      subroutine apply_proj_wavefn(len, mu, nff, proj,nproj, 
     $     wfn, nwfn, pwfn, work)
      
c
c     1996 Bernd Pfrommer
c
      use all_to_all_module
      implicit none
      include 'all_to_all.h'
      include 'flibcalls.ph'
c
c     INPUT:
c     -----
      integer
     $     len,                 ! length of wave functions and projectors
     $     nproj,               ! number of projectors
     $     nff,                 ! number of fore factors
     $     nwfn                 ! number of wave functions

      double precision mu(nff)  ! forefactors

      complex(kind=8) 
     $     proj(len, nproj), wfn(len, nwfn)
c
c     OUTPUT:
c     ------

      complex(kind=8) pwfn(len,nwfn) ! projector applied to the wavefunctions

c
c     WORK:
c     ----
      
      complex(kind=8) work(nproj*nwfn)
c
c     DESCRIPTION:
c     -----------

c
c     applies mu*projector to a wave function wfn and ADDS ! the result 
c     to pwfn:
c
c     pwfn = pwfn + mu*proj*wfn
c
c     The work array should be set to reasonable numbers,
c     because NaN*zero = NaN still!
c

      
c
c     ---------------------------------------------------------------------
c
      complex(kind=8) dcone, dczero
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0))

      integer i,j

      
      call mzgemm('C','N',nproj,nwfn,len, dcone,
     $     proj(1,1),len, wfn(1,1),len,dczero, work(1), nproj)

      call all_sum_all(work,nproj*nwfn)

      if(nff.eq.nwfn) then
         do i=1,nwfn
            do j=1, nproj
               work((i-1)*nproj+j)=work((i-1)*nproj+j)*mu(i)
            end do
         end do
      elseif(nff.eq.1) then
         work=work*mu(1)
      else
         write(9,*) 'apply_proj_wavefn: nff wrong:',nff
         stop
      endif

      call mzgemm('N','N',len,nwfn,nproj, dcone,
     $     proj(1,1),len, work(1),nproj,dcone, pwfn(1,1), len)


      return

      end 
      


