c
      subroutine cg_startguess(ham, crys, gapp, rk,ukqapp, uk, ukqtil,
     $     e_kapp,  e_kqapp, p0, noccup, ioutput)

      use all_to_all_module
      include 'use.h'
      implicit none
      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'
c
c     INPUT:
c     -----
c
      integer 
     $     ioutput,             ! output flag
     $     p0,                  ! direction in which the p0 operator acts
     $     noccup               ! number of occupied states

      type(hamiltonian) ham 
      
      type(crystal) crys 

      type(parallel_gspace) gapp ! small sub-gspace

      complex(kind=8) 
     $     ukqapp(*),           ! approximate wave functions u_k+q
     $     uk(ham%dim,noccup)   ! all occupied states at k

      double precision 
     $     rk(3),               ! the kpoint at which we are 
     $     e_kapp(gapp%length), ! energy eigenvalues of u_k
     $     e_kqapp(gapp%length) ! energy eigenvalues of u_k+q


c
c     OUTPUT:
c     ------
c
      complex(kind=8)
     $     ukqtil(ham%dim, noccup) ! the desired wave function ukqtilde

c
c     DESCRIPTION:
c     -----------
c
c     generate the starting guess for the conjugate gradient linear systems
c     solver in the magnetic susceptibility code.
c
c
c     1997 Bernd Pfrommer
c

c
c     
c     ------------------ local variables ------------------------------
c
      integer 
     $     i, j, info, 
     $     wfnlist(1),          ! dummy for put_gset
     $     nempty,              ! number of unoccupied states in subspace
     $     n,                   ! dimension of subspace
     $     nbc,igc,ibc,ic,      ! column index variables
     $     nbr,igr,ibr,ir,ngr,  ! row index variables
     $     ihomo, ilumo,        ! homo and lumo pointers
     $     nb,                  ! scalapack block size
     $     iam,                 ! blacs processor index
     $     nprow, npcol,        ! blacs number of proc rows and cols
     $     mypcol, myprow,      ! blacs processor column and row 
     $     nprocblacs,          ! blacs number of procs. must be nproc
     $     nproc,               ! number of processors
     $     myproc,              ! global rank of myproc
     $     ngmax, nfft(3), gr(3), subcomm, subcommc

      integer, allocatable ::  glistr(:)

      double precision kq(3), gimmetime, t0, rnum
      external gimmetime
      complex(kind=8), allocatable :: 
     $     mat(:),              ! some work array
     $     ukapp(:),            ! packed version of uk
     $     gradukapp(:)         ! gradient of approximate uk

      complex(kind=8) dcone, dczero, dcmone, ssum
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcmone=(-1.d0,0.d0))

c
c     ---------------------------------------------------------------------
c
      t0=gimmetime()

      n      = gapp%length      ! dimension of subspace
      nempty = gapp%length-noccup
      nproc = ham%gspace%nproc ; myproc = ham%gspace%myproc
      ilumo = noccup+1          ! index of lumo
      ihomo = noccup            ! index of homo

      m4_ifdef([SCALAPACK],[
c
c     since we are using scalapack, it is assumed that the grid
c     initialization etc has already been done when computing the
c     starting vectors. Here, the blacs calls are just to get info
c     about the variables.

      call layout_scalapack(n, nb, nproc, nprow, npcol)
      call blacs_pinfo(iam, nprocblacs) ! how many procs does BLACS see
      call blacs_gridinfo(blacs_context, nprow, npcol, myprow, mypcol) 
      if(iam.ne.myproc) then
         write(9,*) ' cg_startguess: blacs ranks different from mpi!'
         stop
      endif
c
c     now create an MPI subgroup to be used for the reduction later on
c
      call all_split_all(myprow, iam, subcomm)
      call all_split_all(mypcol, iam, subcommc)

      ],[
      nb=1; nprow=1; npcol=1; mypcol=0; myprow=0;
      ])


      write(9,400) n,nprow, npcol, nb
c
c     figure out number of blocks per processor in the row
c
      nbr = n/(nb *nprow) ; if(mod(n,(nb*nprow)).gt.myprow*nb) nbr=nbr+1
      nbc = n/(nb *npcol) ; if(mod(n,(nb*npcol)).gt.mypcol*nb) nbc=nbc+1
      ngmax     = nb*nbr        ! upper bound for g list
c
c     set up the list of row gvectors on this processor
c
      nfft=ham%gspace%fftsize(1:3)

      allocate(glistr(ngmax))
      ngr =0
      do ibr =0, nbr-1          ! loop over row blocks
         do ir=(ibr*nprow+myprow)*nb+1, 
     $        min((ibr*nprow+myprow)*nb+nb,n) ! loop over row
            ngr = ngr + 1
            gr(1:3) = gapp%gvec(1:3,ir)
            glistr(ngr) =  mod(gr(3)+nfft(3), nfft(3))
     $           + mod(gr(2)+nfft(2), nfft(2))*nfft(3)
     $           + mod(gr(1)+nfft(1), nfft(1))*nfft(3)*nfft(2)
         end do
      end do


      allocate(gradukapp(gapp%length))
      allocate(ukapp(gapp%length))
      allocate(mat(nempty)) ; mat=0

      do i=1, noccup
c
c        compute the gradient of the approximate wave function ukapp
c
         call regspace(ham%gspace,uk(1,i),gapp, ukapp(1)) ! pack large wavefn

         call derivative_gspace(p0,ukapp(1),gapp, crys,gradukapp(1))
c
c        dot into approximate u_k+q:   ukqapp
c
         m4_ifdef([SCALAPACK],[
         igc=0 ; mat=0 
         do ibc=0, nbc-1        ! loop over column blocks
            do ic=(ibc*npcol+mypcol)*nb+1, 
     $           min((ibc*npcol+mypcol)*nb+nb,n) 
               igc = igc + 1
               if(ic.gt.noccup) then
                  igr =0
                  do ibr =0, nbr-1 ! loop over row blocks
                     do ir=(ibr*nprow+myprow)*nb+1, 
     $                    min((ibr*nprow+myprow)*nb+nb,n) ! loop over row
                        igr = igr + 1
                        mat(ic-noccup)=mat(ic-noccup)+
     $                    conjg(ukqapp(igr+(igc-1)*ngr))*gradukapp(ir)
                     end do
                  end do
               endif
            end do
         end do
         call all_sum_all(mat,nempty)
         ],[
         call mzgemm('C','N',nempty,1,gapp%length, dcone,
     $         ukqapp(1+(ilumo-1)*gapp%length),gapp%length,
     $         gradukapp(1), gapp%length, dczero, mat(1), nempty)
         ])
c
c        prepare matrix mat with     1/(eps_ki - eps_k+q,j) factor
c     
         do j=1,nempty
            if(abs(e_kapp(i)-e_kqapp(ihomo+j)).lt.1d-6) then
               write(9,127) 
 127           format(' *** WARNING: SUBMATRIX INDICATES',
     $              ' METALLIC SYSTEM IN CGSTARTGUESS!')
               call random_number(rnum)
               mat(j)= mat(j)*rnum
            else
               mat(j)= mat(j)/(e_kapp(i)-e_kqapp(ihomo+j))
            endif
         end do

         
c
c        perform sum over empty states:
c
c        delta |phi_i> = sum_j     |phi_j> <phi_j| grad | phi_i>/(e_i-e_j)
c                                           `---------------------------'
c                                              = mat(j) for fixed i
c
         m4_ifdef([SCALAPACK],[
         gradukapp=0

         igc=0
         do ibc=0, nbc-1        ! loop over column blocks
            do ic=(ibc*npcol+mypcol)*nb+1, 
     $           min((ibc*npcol+mypcol)*nb+nb,n) 
               igc = igc + 1
               if(ic.gt.noccup) then
                  do igr=1,ngr
                     gradukapp(igr) = gradukapp(igr) +
     $                    ukqapp(igr+(igc-1)*ngr)*mat(ic-noccup)
                  end do
               endif
            end do
         end do
         call all_sum_all_sub(subcomm, gradukapp,ngr)

         ],[
         call mzgemm('N','N',gapp%length, 1, nempty, dcone,
     $         ukqapp(1+(ilumo-1)*gapp%length), gapp%length, mat(1), 
     $         nempty, dczero,   gradukapp(1), gapp%length)
         ])
c
c        gradukapp now contains the approximate u_kq~, 
c
         wfnlist(1)= 1
         kq(:)=ham%gspace%rk(:) ! save k+q
         ham%gspace%rk(:)=rk(:) ! set the kpoint to be k
         ukqtil(1:ham%gspace%length,i)=0

c         ssum =0
c         do j=1,ngr
c            ssum = ssum+gradukapp(j)*conjg(gradukapp(j))
c         end do
c         call all_sum_all_sub(subcommc, ssum)
c         write(9,*) i, ssum

         call compute_ekin_gspace(ham%gspace,crys) ! recompute kinetic energy
         call put_gset(ngr, glistr,1, wfnlist, gradukapp(1), 
     $        gapp%gmax, ham%gspace,    ukqtil(1,i), 1)
         ham%gspace%rk(:)=kq(:) ! undo changes from before
         call compute_ekin_gspace(ham%gspace,crys) ! now k+q gives energy again
      end do

      deallocate(glistr)
      deallocate(ukapp) ;  deallocate(gradukapp) ; deallocate(mat)

      if(iand(ioutput,8).eq.8) then
         write(9,930) gimmetime()-t0
         call myflush(9)
      endif

      m4_ifdef([SCALAPACK],[
      call mpi_comm_free(subcomm, info)
      call mpi_comm_free(subcommc, info)
      ])
      return


 400  format (' CG START GUESS SIZE ',i5,
     $     ' PROCESSOR GRID ',i3,' x ',i3,' BLOCKSIZE ',i3)
 930  format (' TIME FOR CG START GUESS:', f12.3, ' SECONDS')


      end
