c
      subroutine cgemin(pw_params,  maxit, iconv, bands,
     $     delta, kp, ham, wavefn, gs, crys, energs, 
     $     denc, rden, den,  vion, vbareion)
c     
c     1996 Bernd Pfrommer
c
c

      use all_to_all_module
      include 'use.h'
      implicit none
      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'
c
c     INPUT:
c     -----

      integer
      
     $     maxit                ! max number of iterations

      type (kpoint) kp
      type (hamiltonian) ham(kp%nrk) ! the hamiltonians at different kpoints
      type (pw_parameter) pw_params ! various parameters
      type (parallel_gspace) gs ! the gspace for the charge density
      type (crystal) crys


      complex(kind=8) 
     $     denc(gs%length),     ! the core charge density
     $     vbareion(gs%length)  ! the bare ionic potential in gspace


      double precision
     $     delta                ! accuracy of diagonalization

c
c     INPUT/OUTPUT:
c     ------------
      type (complex_gspace_array) 
     $     wavefn               ! wave functions for all kpoints, bands
      type (band) bands         ! eigenvalues and occupation numbers are touched
      type (energy) energs      ! various energy terms

      complex(kind=8)
     $     vion(gs%length)      ! vhxc on input, screened ionic pot. on output

c
c     OUTPUT:
c     -------

      logical 
     $     iconv                ! true if converged, false else


c
c     WORK ARRAYS:
c     -----------
      complex(kind=8) 
     $     rden(gs%r_size), den(gs%length)

c
c     DESCRIPTION:
c     -----------
c
c     Starting from given trial wavefunctions, this subroutine
c     minimizes the energy functional with a conjugate gradient
c     method. All bands and kpoints are moved simultaneously. 
c
c
c     --------------------- local variables ----------------------
c
      complex(kind=8) 
     $     dcone, dczero,dcminustwo, dcmhalf, dctrhalf
      
      double precision 
     $     t0,t1,t2,            ! timing variables
     $     oldrho,a,b,c,d,demax, de, lambda,eprecond, 
     $     delrho,a0,b0,c0,d0,
     $     rho,                 ! current error
     $     nops,                ! number of operations
     $     egap                 ! gap between highest computed and next higher 
     $                          ! eigenvalue

      double precision, pointer :: eval(:,:,:)

      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcminustwo=(-2.d0,0.d0), dcmhalf=(-0.5d0,0.d0), 
     $     dctrhalf=(1.5d0,0.d0))

      integer
     $     maxrescg,            ! num of resets to switch to steepest descend
     $     myproc,
     $     nreset,              ! counter for the number of resets done
     $     ierrmall,
     $     nrk,                 ! number of kpoints
     $     mnew,
     $     neig,                ! number of eigenvalues to be computed
     $     diaginfo,            ! return value of diagonalization routines
     $     m,                   ! current number of vectors
     $     m_squared,           ! square of m
     $     i,j,                 ! dummies
     $     irk,                 ! kpoint counter
     $     imax,                ! largest gap index
     $     n,                   ! itertion number
     $     ldawfn,              ! leading dimension of wave function arrays
     $     len(kp%nrk)          ! gspace length


      parameter(maxrescg=10)

      logical ishift, ireset, iconjug, isteepest_desc
      character*11 wfnname


      complex(kind=8), pointer :: x(:,:,:) ! alias for wave functions
      complex(kind=8)
     $     dclambda, dclambdasq, dcgamma, dcweight
c
c     work arrays
c
      complex(kind=8), allocatable :: ! of size m x m
     $     eps(:,:), s(:,:), xsi(:), eta(:), nu(:), mu(:), epsdum(:)

      complex(kind=8), allocatable :: ! of size nanl x m 
     $     ews(:,:)
      complex(kind=8), allocatable :: ! of size len x m
     $     g(:,:), gold(:,:), gp(:,:), h(:,:), y(:,:)
      integer, allocatable :: iord(:) ! size of m
      double precision, allocatable :: rwork(:)

c
c     local routines
      double precision 
     $     trace,  contract, average_ekin
      external trace, contract

c     external functions
      external findroot
      external average_ekin
      external gimmetime
      double precision gimmetime ! timing routine

c
c     --------------------------------------------------------------------
c

      t0=gimmetime()

      nrk  = kp%nrk             ! number of kpoints
      neig = bands%min
      eval => bands%energy      ! alias the energies to a more convenient name
      x    => wavefn%data       ! alias for the wave functions

      if (iand(pw_params%optimize,32).ne.32)      write(9,140) 

      if (iand(pw_params%optimize,128).eq.128) then
         isteepest_desc = .true.
      else 
         isteepest_desc = .false.
      endif


      nreset=0
      m=neig
      m_squared=m*m

      do irk=1,nrk
         len(irk) = ham(irk)%gspace%length
      end do
      ldawfn = size(x,1)
      myproc=ham(1)%gspace%myproc
c
c     allocate all the work spaces
c
      allocate(eps(m*m,nrk));  eps=0;
      allocate(s(m*m,nrk)); s=0; allocate(xsi(m*m)); xsi=0;
      allocate(eta(m*m)); eta=0;  
      allocate(nu(m*m)); nu=0;
      allocate(mu(m*m)); mu=0;
      allocate(epsdum(m*m)); epsdum=0
      allocate(rwork(max(1,3*m-2))); rwork=0
      if(ham(1)%vnloc%nvecs.gt.0) then
         allocate(ews(ham(1)%vnloc%nvecs,m))
         ews=0
      endif
      allocate(g(ldawfn,nrk),stat=ierrmall);  
      if(ierrmall.ne.0) then 
         write(9,930) 'g',ldawfn,nrk, ierrmall
         stop
      endif
      g=0;
      allocate(gold(ldawfn,nrk),stat=ierrmall); 
      if(ierrmall.ne.0) then 
         write(9,930) 'gold',ldawfn,nrk,ierrmall
         stop
      endif
      gold=0;
      allocate(h(ldawfn,nrk),stat=ierrmall); 
      if(ierrmall.ne.0) then 
         write(9,930) 'h',ldawfn,nrk,ierrmall
         stop
      endif
      h=0;
      allocate(y(ldawfn,nrk),stat=ierrmall); 
      if(ierrmall.ne.0) then 
         write(9,930) 'y',ldawfn,nrk,ierrmall
         stop
      endif
      y=0;
      allocate(gp(ldawfn,nrk),stat=ierrmall); 
      if(ierrmall.ne.0) then 
         write(9,930) 'gp',ldawfn,nrk,ierrmall
         stop
      endif
      gp=0;
      allocate(iord(m)); iord=0;


      ireset=.false.            ! generally no resetting
      iconjug = .false.         ! don't make conjugate in first step
      ishift=.true.             ! want to shift if error has decreased enough
      rho=1.0d4                 ! give some initial error
      nops=0                    ! reset operation counter 
      t1=0                      ! reset time counter

      write(9,100) 
      do irk=1,nrk
         write(9,105) irk, eval(1:neig+1,irk,1)
      end do

      egap = minval(eval(neig+1,:,1)-eval(neig,:,1))

      if(egap.lt.0) then        ! no good gap as input, guess
         write(9,*) ' *** MINIMIZATION: INPUT EIGENVALUES BAD!'
         stop
      endif
      
c
c     shift hamiltonian just enough to make the hessian matrix 
c     negative definite   around the minimum
c
      ham(1:nrk)%shift = 
     $     maxval(eval(neig,:,1))+egap/4+ pw_params%shiftsafety

c
c     --------------------- start iterative loop --------------------
c
      do n=1, maxit
 911     continue               ! go here for resetting
         do irk=1, nrk
            call apply_ham_wavefn(pw_params%output,y(1,irk), ! y = H * x
     $           ham(irk),x(1,irk,1),m,ews(1,1)) 
            if(iand(pw_params%output,1).eq.1) t2=gimmetime()
            call cheap_overlap_matrix(s(1,irk),x(1,irk,1),m,len(irk)) ! s=x^T*x
            call overlap_matrix(eps(1,irk),x(1,irk,1),y(1,irk), ! eps=x^T*y
     $           m,len(irk)) 
c
c           calculate the gradient for each kpoint with proper weight
c
            dcweight=kp%w(irk)
            call mzgemm('N','N',len(irk),m,m, dcweight, x(1,irk,1)
     $           ,len(irk),eps(1,irk), m,dczero,g(1,irk),len(irk)) 
            call mzgemm('N','N',len(irk),m,m,dcweight, y(1,irk),
     $           len(irk),s(1,irk),m,dcone, g(1,irk),len(irk)) 
            dcweight = -2.d0*dcweight
            call mzaxpy(len(irk)*m, dcweight, y(1,irk),1,
     $           g(1,irk),1)
            if(iand(pw_params%output,1).eq.1) then
               t1=t1+gimmetime()-t2
               nops =nops+(12.d0+16.d0)*dble(m)*dble(ldawfn)
            endif
         end do

c         rho =0
c         do irk=1,nrk
c            rho = rho+dble(parallel_zdotc(len(irk)*m,g(1,irk),1,
c     $           h(1,irk),1))
c         end do
c         write(9,*) 'check:',rho

c
c        precondition and compute rho
c
         rho =0
         do irk=1, nrk  
            call mzcopy(len(irk)*m,g(1,irk),1,gp(1,irk),1) !prepare precond
            if(iand(pw_params%optimize,32).eq.32) then
               eprecond=1.35d0*average_ekin(x(1,irk,1),xsi(1),
     $              ham(irk)%gspace%ekin(1),m,len(irk))
               call precondition(gp(1,irk),eprecond,ham(irk
     $              )%ekinmod(1),ham(irk)%gspace%ekin(1), m,len(irk))
            endif
            rho = rho + dble(parallel_zdotc(len(irk)*m,g(1,irk),1,
     $           gp(1,irk),1))
         end do

         write(9,'(1x,a,i3,a,g12.6)') 'iteration',n,
     $        ' total energy estimated error [Ryd]=',rho
         call myflush(9)
         irk=1
c
c        make conjugate
c
         if((n.gt.1).and.(.not.ireset).and.(.not.isteepest_desc)) then
            delrho=0
            do irk=1, nrk
               delrho = delrho+dble(parallel_zdotc(len(irk)*m,
     $              gold(1,irk), 1,gp(1,irk),1))
            end do
            dcgamma =cmplx((rho-delrho)/oldrho,0.d0,kind=8)
            do irk=1, nrk
               call mzaxpy(len(irk)*m, dcgamma, h(1,irk), 1, 
     $              gp(1,irk), 1)
            end do
         else
            ireset=.false.
         endif

         do irk=1, nrk
            call mzcopy(len(irk)*m, gp(1,irk), 1, h(1,irk), 1)
c           save gradient into old gradient
            call mzcopy(len(irk)*m, g(1,irk), 1, gold(1,irk), 1) 
         end do

         oldrho = rho

         if(rho.lt.delta) then
            iconv=.true.
            goto 200
         endif

c
c        compute the matrices and coefficients for the line minimization
c
         a=0 ; b=0; c=0; d=0

         do irk=1, nrk
            if(iand(pw_params%output,1).eq.1) t2=gimmetime()
            call hermit_overlap_matrix(eta(1),h(1,irk),
     $           x(1,irk,1),m,len(irk))
            call cheap_overlap_matrix(nu(1),h(1,irk),m,len(irk))

            call hermit_overlap_matrix(mu(1),h(1,irk),y(1,irk),
     $           m,len(irk))
            if(iand(pw_params%output,1).eq.1) then
               t1=t1+gimmetime()-t2 ; nops = nops + 
     $              12.d0*dble(m*m)*dble(len(irk))
            endif
            call apply_ham_wavefn(pw_params%output,g(1,irk),
     $           ham(irk),h(1,irk),m,ews(1,1)) ! g=H * h
            if(iand(pw_params%output,1).eq.1) t2=gimmetime()
            call overlap_matrix(xsi(1),h(1,irk),g(1,irk),
     $           m,len(irk))
            if(iand(pw_params%output,1).eq.1) then
               t1=t1+gimmetime()-t2 
               nops = nops + 8*dble(m*m)*dble(len(irk))
            endif
c
c           calculate the coefficients for the minimization routine 
c

            a0 = kp%w(irk)*(2.0*trace(mu(1),m)-contract(eta(1),
     $           eps(1,irk),m)  - contract(mu(1),s(1,irk),m))
            b0 =  kp%w(irk)*(2.0*trace(xsi(1),m) 
     $           - contract(mu(1),eta(1),m) 
     $           - contract(eps(1,irk),nu(1),m) 
     $           - contract(xsi(1),s(1,irk),m))
            c0 = kp%w(irk)*(-contract(xsi(1),eta(1),m) 
     $           - contract(nu(1),mu(1),m))
            d0 = kp%w(irk)*(-contract(xsi(1),nu(1),m))

            a=a+a0; b=b+b0; c=c+c0; d=d+d0
         end do

c     
c        do the line minimization 
c
         call findroot(lambda,4*d,3*c,2*b,a)
         if(lambda.lt.0) then
            nreset = nreset+1
            if(nreset.gt.maxrescg) then
               write(9,700)
               isteepest_desc=.true.
            endif
            if(nreset+n.gt.maxit) goto 199 ! too many resets ... bye bye
            ham(1:nrk)%shift=ham(1:nrk)%shift + 0.5
            write(9,800) ham(1)%shift
            ireset=.true.
            goto 911
         endif


         dclambda=lambda
c
c        move wave functions (contained in x) to new position 
c                     x= x+lambda*h
c
         do irk=1,nrk
            call mzaxpy(len(irk)*m, dclambda, h(1,irk), 
     $           1, x(1,irk,1), 1)
         end do
c
c        compute new overlap matrix, and (3/2 -1/2*S) |x> 
c
         do irk=1, nrk
            if(iand(pw_params%output,1).eq.1) t2=gimmetime()
            call cheap_overlap_matrix(s(1,irk),x(1,irk,1),
     $           m,len(irk))    ! s = x^T*x
            call mzgemm('N','N',len(irk),m,m,dcmhalf,x(1,irk,1),
     $           len(irk),s(1,irk),m,dczero,gp(1,irk),len(irk))
            call mzaxpy(len(irk)*m, dctrhalf, x(1,irk,1), 1, 
     $           gp(1,irk), 1)
            
            if(iand(pw_params%output,1).eq.1) then
               t1=t1+gimmetime()-t2 
               nops = nops + 12.d0*dble(m*m)*dble(len(irk))
            endif
         end do
c
c        diagonalize to find eigenvalues
c
         if (iand(pw_params%optimize,64).eq.64) then
            do irk=1, nrk
c
c              now diagonalize the matrix
c
               eps(1:m*m,irk)=eps(1:m*m,irk)+lambda*mu(1:m*m) 
     $              + lambda*lambda*xsi(1:m*m) ! adjust eps for new x
               epsdum(1:m*m)=eps(1:m*m,irk)
               call matrix_polish(s(1,irk),epsdum(1), eta,nu,m)
                  
               if(myproc.eq.0) then
                  call mzheev('V','L',m,epsdum(1),m, eval(1,irk,1),
     $                 g(1,irk), m*len(irk),  rwork(1),diaginfo) 
                  if(diaginfo.ne.0) then
                     if(myproc.eq.0)
     $                    write(9,*)
     $                    'diagonalization failed with info='
     $                    ,diaginfo
                     stop
                  endif
               endif
               call my_broadcast(epsdum(1),m*m,0)
               call my_broadcast(eval(1,irk,1),m,0)
               
c
c           combine the eigenvectors into g
  
               if(iand(pw_params%output,1).eq.1) t2=gimmetime()
               call mzgemm('N','N',len(irk),m,m, dcone, gp(1,irk),
     $              len(irk), epsdum(1),m, dczero,g(1,irk), len(irk)) 
               if(iand(pw_params%output,1).eq.1) then
                  t1=t1+gimmetime()-t2
                  nops = nops + 8*dble(m*m)*dble(len(irk))
               endif
               eval(1:m,irk,1) = eval(1:m,irk,1)+ham(irk)%shift
            end do
            gp=g
         endif
         
c
c        now update the hamiltonian with the new, transformed wavefunctions
c
         call update_hamiltonian(kp, ham(1), ldawfn, len(1),
     $        m,gp(1,1),gs, bands, 
     $        crys, energs, pw_params, denc(1), rden(1), den(1), 
     $        vion(1), vbareion(1))

         if(mod(n,pw_params%checkpoint_wfn).eq.0) then
            write(9,130)
            call writegsdat(0, gs, den(1), 1,
     $           1, 1, 'CD',2)
            do irk=1, kp%nrk
               write(wfnname,120) irk,1
               call writegsdat(0,ham(irk)%gspace, gp(1,irk),m,
     $              m,1,wfnname,10)
            end do
            if(myproc.eq.0) then
               call mysystem('\rm -f BAND')
               call write_band('BAND',bands)
            endif
         endif

      end do                    ! ----------- end of big iteration loop ------

 199  iconv=.false.

 200  continue

      if(n.eq.1) then           ! converged in first step, need to update vion
         vion = vion+ vbareion
         if(gs%ig0.gt.0) vion(gs%ig0)=0
      endif


      do irk=1, nrk

c
c     Done with the iterative part. 
c
c     Now first orthogonalize the vectors approximately. This step reduces
c     linear errors in the eigenvalues, and is important.
  
         if(iand(pw_params%output,1).eq.1) t2=gimmetime()
         call mzgemm('N','N',len(irk),m,m,dcmhalf,x(1,irk,1),
     $        len(irk),s(1,irk),m,dczero, gold(1,irk),len(irk))
         if(iand(pw_params%output,1).eq.1) then
            t1=t1+gimmetime()-t2 
            nops = nops + 8*dble(m*m)*dble(len(irk))
         endif
      
         call mzaxpy(len(irk)*m, dctrhalf, x(1,irk,1), 1, 
     $        gold(1,irk), 1)
         call mzcopy(len(irk)*m, gold(1,irk), 1, x(1,irk,1), 1)

c        calculate transformed eps=1/4 (9eps-3St*eps-3eps*S+St*eps*S) 

         call matrix_polish(s(1,irk),eps(1,irk), eta,nu,m)


c     the vectors x we have now span the right subspace, but are not yet the 
c     eigenvectors. to get those, we diagonalize the epsilon matrix. 
c     the eigenvalues are the eigenvalues we are looking for. 
c     the eigenvectors tell us how to superimpose the vectors x to get 
c     the true eigenvalues 
c     There are parallel issues here: If the eps matrix is not
c     exactly identical, the eigenvectors will acquire a different
c     phase on different processors. Therefore, we diagonalize only
c     on proc 0, and broadcast the result.
c
c

         if(myproc.eq.0) then
            call mzheev('V','L',m,eps(1,irk),m, eval(1,irk,1),
     $           g(1,irk), m*len(irk),  rwork(1),diaginfo) 
            if(diaginfo.ne.0) then
               if(myproc.eq.0)
     $              write(9,*) 'diagonalization failed with info='
     $              ,diaginfo
               stop
            endif
         endif
         call my_broadcast(eps(1,irk),m*m,0)
         call my_broadcast(eval(1,irk,1),m,0)
         
c
c        combine the eigenvectors into g
  
         if(iand(pw_params%output,1).eq.1) t2=gimmetime()
         call mzgemm('N','N',len(irk),m,m, dcone, x(1,irk,1),
     $        len(irk), eps(1,irk),m, dczero,g(1,irk), len(irk)) 
         if(iand(pw_params%output,1).eq.1) then
            t1=t1+gimmetime()-t2
            nops = nops + 8*dble(m*m)*dble(len(irk))
         endif

c
c        save the eigenvalues
c
         eta(1:m) = cmplx(eval(1:m,irk,1)+ham(irk)%shift,0.d0)

c        in case the calculation did not converge, preform
c        the orthonormalization of the  subspace: 
c     
c        a) Compute S^-1/2 = AT * D^-1/2 * A   
c        b) Compute x_new= S^-1/2 * x
c     
c        The eigenvalue array is used as temporary storage
c
         if(.not.iconv) then
            write(9,'(a,i3,a,g12.6)') 
     $           ' minimization not conv. after ',n,
     $           ' iter., residual error = ',rho
            call cheap_overlap_matrix(s(1,irk),g(1,irk),
     $           m,len(irk));
c           diagonalize S, find eigenvectors A (returned in S) 

            call mzheev('V','L',m, s(1,irk),m,eval(1,irk,1),
     $           x(1,irk,1),m*len(irk),rwork(1), diaginfo);  

c        compute At * D^-1/2 * A  

            do i=1,m
               eval(i,irk,1)=1.0/sqrt(eval(i,irk,1))
               do j=1,m
                  mu(i+m*(j-1))=eval(i,irk,1)*conjg(s(j+m*(i-1),irk))
               end do
            end do

c        compute nu = A * D^-1/2* AT 
            call mzgemm('N','N', m,m,m, dcone, s(1,irk),m,
     $           mu(1),m,dczero, nu(1),m) 

c        compute g = nu * x  
            call mzcopy(len(irk)*m, g(1,irk), 1, 
     $           x(1,irk,1), 1); !   x <- g 
            if(iand(pw_params%output,1).eq.1) t2=gimmetime()
            call mzgemm('N','N',len(irk),m,m, dcone,x(1,irk,1),
     $           len(irk), nu(1),m, dczero,g(1,irk),len(irk)) 
            if(iand(pw_params%output,1).eq.1) then
               t1=t1+gimmetime()-t2
               nops = nops + 8*dble(m*m)*dble(len(irk))
            endif
         endif
c
c     copy return eigenvectors into x array
c
         call mzcopy(len(irk)*m, g(1,irk), 1, x(1,irk,1), 1) !   x <- g 
c
c     retrieve eigenvalues
c
         eval(1:m,irk,1)=dble(eta(1:m))
         eval(m+1,irk,1)=eval(m,irk,1)+egap ! and remember gap

      end do                    ! end of loop over kpoints

c
c     performance printout
c
      if(iand(pw_params%output,1).eq.1) 
     $     write(9,920) t1,nops/t1  /1d6
c     
c     timing printout
c
      if(iand(pw_params%output,8).eq.8) then 
         write(9,910) n,gimmetime()-t0
         call myflush(9)
      endif

c
c     deallocate the work spaces
c
      deallocate(eps);  deallocate(s); deallocate(xsi);
      deallocate(eta);  deallocate(nu); deallocate(mu);
      deallocate(rwork); deallocate(epsdum)
      if(ham(1)%vnloc%nvecs.gt.0) deallocate(ews)
      deallocate(g);  deallocate(gold); deallocate(h); 
      deallocate(gp);
      deallocate(y);
      deallocate(iord);

      return
 100  format(/' EIGENVALUES FROM SUBMATRIX DIAGONALIZATION:',
     $     /1000(5f12.6/)) 
 105  format(/' KPOINT:',i4,/1000(5f12.6/)) 

 120  format('WFN',i5.5,'.',i1)
 130  format(' <<<<<< checkpointing wave functions and charge >>>>>>')
 140  format(/' DIRECT ENERGY MINIMZATION WITHOUT PRECONDITIONING!',
     $     /' THIS CAN BE A REAL PERFORMANCE HIT.')

 700  format(' *** cgemin: switching over to steepest descend')
 800  format(' *** cgemin: resetting and shifting to: ', f12.6)
 910  format(/' MINIMIZATION TOOK ',i3,' ITERATIONS AND ', 
     $     f12.3, ' SECONDS')
 920  format(' TIME FOR MATRIX-MATRIX:',f12.6,', MFLOPS=',f12.6)
 930  format('*** DIAGONALIZE_HAMILTONIAN: ALLOCATE(',a,'(',i6,
     $        ',',i4,') FAILED. ERROR CODE=',i4)
      end

