C*
c     @process extchk
      subroutine charge(ffts, pw_params,
     $     cd_gs, gs, bands, kpoints, wavefn,
     $     den,energs)
c     
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'

c
c     INPUT:
c     -----

c     
      type (fftstruc) ffts
      type (parallel_gspace) 
     $     cd_gs,               ! the gspace for the charge density and pot
     $     gs(*)                ! gspaces where wavefn were computed

      type (pw_parameter) pw_params ! for the smearing parameter
      type (band) bands         ! for nrk, occupation numbers ...
      type (kpoint) kpoints     ! for the weights kpoints%w
      type (complex_gspace_array) wavefn
c
c     OUTPUT:
c     ------

      type (energy) energs      ! efermi is input. compute eband,ektot,esmear.

      complex(kind=8)
     $     den(cd_gs%length,bands%nspin) ! charge density 

c
c     computes the charge density and eband, ektot, from the eigenvectors
c     and eigenvalues

c     1996 Bernd Pfrommer, UCB


c     ---------------- local variables ----------------------------------
       
      double precision
     $     t0, gimmetime,
     $     dsmear,              ! gaussian smearing parameter [Ryd]
     $     ryd,                 ! the rydberg constant in eV
     $     small, dmin, dmax, cmin, cmax, zero, sqrpi, pi,spinf,delta
      external gimmetime
      complex(kind=8) 
     $     zzero,
     $     cdsum, cdsumtot , cdsumtotr

      parameter (small = 1.0d-12, zzero=(0.d0, 0.d0), zero=0.d0,
     $     pi=3.141592653589793d0,
     $     ryd   =  13.60580d0)
      integer
     $     is,                  ! spin counter
     $     ns,                  ! number of stars
     $     ntot,                ! total number of points in realspace block
     $     irk,                 ! kpoint counter
     $     neig, ndim, iel, 
     $     jmin, jmax,
     $     i,j,k,
     $     ierr, idum, ntottot

      complex(kind=8), allocatable::
     $     dhd(:),              ! realspace array 
     $     rden(:)              ! accu for rspace dens



c     -----------------------------------------------------------------------
c
c     transfer data from structures to local variables
c
      t0=gimmetime()

      ns     = cd_gs%nstar
      ntot   = ffts%r_size     ! total number of points in realspace block
      dsmear = pw_params%smearing/ryd
      sqrpi  = sqrt(pi)
c
c     get dimensions of realspace block
c
      ntottot = ntot
      call all_sum_all(ntottot)
c
c     allocate various charge densities
c

      allocate(rden(ntot))    
      allocate(dhd(ntot))    
      cdsumtot =zzero
      cdsumtotr =zzero


      
      if(bands%nspin.ge.2) then ; spinf = 0.5d0 ; else ;  spinf =1.d0
      endif

      energs%eband=0
      energs%esmear=0
      energs%ektot=0



      do is=1,bands%nspin        ! loop over spins
         rden=zzero

         do irk=1, bands%nrk    ! ------------   loop over k points ---------

            neig = bands%nband(irk,is)
            ndim = gs(irk)%length
c
c        adds to sum of occupied eigenvalues and kinetic energy
c
            do j=1,neig
               energs%eband = energs%eband + 
     $              bands%occup(j,irk,is)*bands%energy(j,irk,is)
               delta=(energs%efermi(is)-
     $              (bands%energy(j,irk,is)+
     $              energs%evalshift(is)))/dsmear
               energs%esmear = energs%esmear - dsmear/sqrpi*
     $              2.d0 * kpoints%w(irk) * 0.5d0 *
     $              dexp(-delta*delta)
               energs%ektot = energs%ektot + bands%occup(j,irk,is)*
     $              bands%ekn(j,irk,is)
            end do
c
c     -----------       find min and max band with nonzero occupation
            jmin = 0
            jmax = 0
            do i=1,neig
               if (dabs(bands%occup(i,irk,is)) .ge. small 
     $              .and. jmin .eq. 0) jmin = i
               if (dabs(bands%occup(i,irk,is)) .ge. small 
     $              .and. jmin .ne. 0) jmax = i
            end do

       
            if(jmin.ne.0) then
               do j=jmin,jmax   ! ------------- loop over occupied bands ----
c
c                 fourier transform to real space
c
                  call fourier_transform(-1, ffts,
     $                 gs(irk),wavefn%data((j-1)*ndim+1,irk,is),dhd)
c
c                 square and add
c
                  do i=1, ntot
                     rden(i) = rden(i) + dhd(i)*conjg(dhd(i))*
     $                 bands%occup(j,irk,is)*spinf
                  end do
               end do
            endif
         end do                 ! ------------ end of loop over k points ----

c
c        fourier transform back to momentum space
c
         cdsum =zzero
         do i=1, ntot
            cdsum = cdsum + rden(i)
         end do
      
         call all_sum_all(cdsum)
         cdsum = cdsum/ntottot

         call fourier_transform(1, ffts, cd_gs, den(1,is), rden)

c
c     symmetrize only if the star information are available
c     Mauri Francesco
c
         call symmetrize_scalar_global(den(1,is),cd_gs)
c

      end do                    ! end of loop over spins 

      energs%eband = energs%eband/dble(bands%nspin)
      energs%ektot = energs%ektot/dble(bands%nspin)
      energs%esmear = energs%esmear/dble(bands%nspin)

      deallocate(dhd)
      deallocate(rden)
      if(iand(pw_params%output,8).eq.8) 
     $     write(9,920) gimmetime()-t0

      return

 920  format(' TIME FOR CHARGE DENSITY CALCULATION:',f12.3)


      end 



