c-*-Fortran-*-
c     @process extchk
      subroutine crstl(ioutput, crys, syms, miscflag)

c
      use flibcalls_module
      include 'use.h'
      implicit none
      include 'interface.h'
c
c
c     INPUT:
c     -----

      integer 
     $     miscflag,            ! for inv symm rest
     $     ioutput              ! for timing printout
c
c     INPUT/OUTPUT:
c     ------------

      type(crystal) crys
      type(symmetry) syms

c     
c     DESCRIPTION:
c     -----------
c
c     Subroutine computes various crystal parameters and the symmetry
c     Based on Alberto's crystal subroutine
c     1996 Bernd Pfrommer
c
c     input:
c     -----
c
c     crys%natom,     crys%ntype,     crys%mxdatm,
c     crys%rat,       crys%avec,      crys%szet,
c     crys%nameat
c
c     syms%ntrans   (if <0, compute symmetry operations)
c
c     action:
c     ------
c     
c     computes 
c
c     crys%vcell,    crys%adot,     crys%bdot,
c     crys%bvec
c
c
c     syms%ntrans,   syms%mtrx,     syms%rmtrx,
c     syms%rsymmat,  syms%tnp
c
c
c     --------------- local variables ----------------------------------

      double precision 
     $     pi2, gimmetime, t0

      integer 
     $     i,j,k, natomi, nops, icar, itest, lmtrx(48,3,3),
     $     lrmtrx(48,3,3),   invers_no, ipr

      double precision 
     $     a1(3),a2(3),a3(3), am(3), ba(3), b1(3),b2(3),b3(3),
     $     ltnp(48,3)


      character group*10
      double precision, allocatable :: lcoorat(:)
C     ..
C     .. External Subroutines ..
      external inpmat, symchk, symgen, fatal, gimmetime
C     ..
C     .. Intrinsic Functions ..
      intrinsic atan, sqrt
C     ..
C     .. External Functions ..
      logical rest_inv
      external rest_inv
C     ..
c

c     ----------------------------------------------------------------

      group = 'unknown'
      ipr =1

      lmtrx=0;
      ltnp=0;

      t0 = gimmetime()
      allocate(lcoorat(3*crys%mxdatm*crys%ntype))

      pi2 = 8.d0*datan(1.d0)
      if(ipr .eq. 1) then
         write(9,9000)
 9000    format(1x,'Lattice information',/1x,19('-'))

c
         write(9,9020)
 9020    format(/1x,'atomic positions relative to lattice vectors:'/)
c
         do i = 1, crys%ntype
            natomi = crys%natom(i)
            do j = 1, natomi
               write(9,9030) crys%nameat(i), j,crys%rat(:,j,i),
     $              crys%szet(i) 
            end do
         end do
 9030    format(1x,a2,' #',i2,':',3f18.12,4x,f5.2)
         write(9,9021)
 9021    format(/1x,'atomic positions in cartesian coordinates:'/)
         do i = 1, crys%ntype
            natomi = crys%natom(i)
            do j = 1, natomi
               write(9,9030) crys%nameat(i),j,
     $              matmul(crys%avec,crys%rat(:,j,i)),crys%szet(i) 
            end do
         end do


      endif

c
c      Compute reciprocal lattice and reset scales.
c
c               T                       T        -1          T -1
c              B A  = 2pi (1), so b =  B  = 2pi(A  )  = 2pi(a )
c
c     and we can compute the determinant (celvol) and the inverse.
c     But note that the 2pi factor is NOT needed here yet. (It
c     will be put in below)
c
      crys%bvec=transpose(crys%avec)

      call invers(crys%bvec,crys%vcell)
      if (crys%vcell .lt. 0) then
         if(ipr .eq. 1) then
            write(9,'(/,1x,a,/)')
     $           'Note: The lattice vectors as given'
     $           //' do not form a direct triad'
         endif
         crys%vcell = -crys%vcell
      endif
c
c
      if(ipr .eq. 1) then
         write(9,9050) crys%vcell
 9050    format(/' Cell volume =',f10.4,' au.')
      endif
c
c     Compute the inner products ba, put in the 2pi factor and clean up.
c
      a1(:)= crys%avec(:,1)
      a2(:)= crys%avec(:,2)
      a3(:)= crys%avec(:,3)
      crys%bvec=pi2*crys%bvec
c
      am(1) = sqrt(a1(1)*a1(1)+a1(2)*a1(2)+a1(3)*a1(3))
      am(2) = sqrt(a2(1)*a2(1)+a2(2)*a2(2)+a2(3)*a2(3))
      am(3) = sqrt(a3(1)*a3(1)+a3(2)*a3(2)+a3(3)*a3(3))
c
      ba(:)=pi2/am(:)
      b1(:) = crys%bvec(:,1)
      b2(:) = crys%bvec(:,2)
      b3(:) = crys%bvec(:,3)
c
c      printout
c
      if(ipr .eq. 1) then
         write(9,9130) a1, a2, a3
 9130    format(/' lattice vectors (a.u.)',/3(/1x,3f18.12))
         write(9,9140) b1, b2, b3
 9140    format(/' reciprocal basis (a.u.)',/3(/1x,3f18.12))
      endif
c
c     Compute metrics crys%bdot(i,j) and crys%adot(i,j)
c
      crys%bdot=matmul(transpose(crys%bvec),crys%bvec)
      crys%adot=matmul(transpose(crys%avec),crys%avec)

c     Symmetry operations for the symmetry group of the crystal.
c
c
c      If nops is negative,  calculate the symmetry operations.
c      If icar=1 the matrices and primitive translations should
c      be given in cartesian coordinates and the translation vectors
c      should be scaled the same way as the basis vectors (atomic units
c      divided by scale). Otherwise the matrices must be given in components
c      of the lattice wave vectors and the translations in units of the
c      lattice basis vectors.
c
c      if ipr=1 the matrices are printed.
c
c      itest is ignored. The test is always performed.
c
c      group is some group identifier.
c

c
c     transform the shit to weird coordinates
c
      do k=1,crys%ntype
         do i=1, crys%natom(k)
            do j=1,3
               lcoorat((j-1)*crys%mxdatm*crys%ntype+(i-1)*crys%ntype
     $              +k)=crys%rat(j,i,k) 
            end do
         end do
      end do

      nops= syms%ntrans



 400  continue

      if (nops .le. 0) then
         nops = -1
         call symgen(ipr, crys%avec,lcoorat,crys%natom,crys%ntype
     $        ,invers_no,syms%ntrans, lmtrx, ltnp, crys%mxdatm,
     $        lrmtrx, syms%rsymmat)
c
c        Restore manifest inversion symmetry if possible and desired
c
         if(iand(miscflag,1).eq.0) then
            if (rest_inv(ipr, invers_no,crys%ntype,
     $           crys%natom,lcoorat,crys%nameat,ltnp, crys%mxdatm))
     $           then
               nops = -1
               go to 400
            endif
         endif
         do k=1,48 ;  do i=1,3
            syms%mtrx(i,:,k)=lmtrx(k,i,:)
            syms%rmtrx(i,:,k)=lrmtrx(k,i,:)
         end do ;  end do
         syms%tnp=pi2*transpose(ltnp)
      else
         call inpmat(crys,syms)
      end if

      if(syms%ntrans.le.0) then
         write(9,*) ' *** SYMMETRY  COULD NOT FIND IDENTITY.'
         write(9,*) ' ***  ERROR    CHECK INPUT FILE COORDINATES'
         write(9,*) ' ***           ARE TWO ATOMS AT SAME COORD?'
         stop
         
      endif




c     ------------- rescale coordinates and clean up --------------------
c

      do k=1,crys%ntype
         do i=1, crys%natom(k)
            do j=1,3
               crys%rat(j,i,k) = pi2*lcoorat((j-1)*crys%mxdatm
     $              *crys%ntype+(i-1)*crys%ntype+k) 
            end do
         end do
      end do
c
      deallocate(lcoorat)


c
c     --------- debugging printout ----------------------
c

c      do i=1, syms%ntrans
c         write(9,9010) i, syms%mtrx(:,:,i), syms%tnp(:,i)
c         write(9,9010) i, syms%rmtrx(:,:,i)
c         write(9,9022) i, syms%rsymmat(:,:,i)
c      end do

      if(iand(ioutput,8).eq.8) then 
         write(9,910) gimmetime()-t0
         call myflush(9)
      endif
      



      return
 910  format(' TIME FOR CRYSTAL SYMMETRY DETERMINATION: ',f12.3
     $     ,' SECONDS')

c

 9010 format(i5,3(3x,3i3),4x,3f8.5)
 9022 format(i5,3(3x,3f10.6))
      end


