c
      subroutine derivative_gspace(dir, data,gs,crys,grd)

c
c     computes derivative wrt spatial direction: dir
c     we compute data(g)*(k+G), so set the k-point of the gspace
c     to zero if you just want the gradient!
c
c     Strictly speaking, it does NOT compute the gradient, but 
c     -i * gradient.
c
c
c     (1996) Bernd Pfrommer
c
 
      include 'use.h'
      implicit none             ! never
      include 'interface.h'


c
c     INPUT:
c     ------

      integer dir               ! the spatial direction, 1,2,3

      type (crystal) crys       ! for the bvec array
      type (parallel_gspace) gs ! the gspace 
      complex(kind=8) data(gs%length) ! the data to be gradiented

c
c     OUTPUT:
c     ------
c
      complex(kind=8) grd(gs%length) ! the derivative of this data


c
c     ----------   local variables  --------------------------
c

c
c     ----- variables for the gspace loop ---------
c
      integer
     $     igv(4),fftn(4), ffth(4),igv3, irod,iord,igs

      double precision gv(3)    ! if required

c     ----------------------------------------------

      if((dir.lt.1).or.(dir.gt.3)) then
         write(9,*) ' derivative_gspace: illegal direction:',dir
         stop
      endif


      fftn(1:3)=gs%fftsize(1:3)
      fftn(4)=gs%fftsize(3)
      ffth(:)=fftn(:)/2
      igs = 0 
      do iord=1, gs%lorder    ! loop through x/y gspace 
         irod=gs%order(1,iord)
         igv(1)= irod/gs%fftsize(2)
         igv(2)= mod(irod,gs%fftsize(2))
         igv(3)= gs%order(2,iord)
         igv(4)= gs%order(3,iord)
         igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
         gv(1:2)=dble(igv(1:2))+gs%rk(1:2)
         do igv3=igv(3),igv(4)     ! loop over z axis
            gv(3)=dble(igv3)+gs%rk(3)
            igs = igs + 1

            grd(igs)=data(igs)*(crys%bvec(dir,1)*gv(1)+
     $           crys%bvec(dir,2)*gv(2)+crys%bvec(dir,3)*gv(3))
         end do
      end do
      return
      end

