c     
c     find the lowest eigenvectors and eigenvalues for a given 
c     Hamiltonian with a conjugate gradient minimization scheme
c
      subroutine diagonalize_hamiltonian(pw_params,
     $     maxit, iconv,  neigmin, neig,
     $     delta, eval,  ham, x)

      use all_to_all_module
      include 'use.h'
      implicit none
      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'
c
c     INPUT:
c     -----

      type (hamiltonian) ham    ! the hamiltonian to diagonalize
      type (pw_parameter) pw_params ! various parameters

      integer
     $     maxit,               ! max number of iterations
     $     neigmin              ! minimum number of eigenvalues to be computed

      double precision
     $     delta                ! accuracy of diagonalization
c
c     INPUT/OUTPUT:
c     ------------

      integer   neig            ! max/real number of eigenvalues computed
      complex(kind=8) x(*)       ! start/final wave functions on input/output
      double precision eval(neig+1) ! start/final eigenvalues, eval(neig+1)
c
c     OUTPUT:
c     -------

      logical 
     $     iconv                ! true if converged, false else


c
c
c     --------------------- local variables ----------------------
c
      complex(kind=8) 
     $     dcone, dczero,dcminustwo, dcmhalf, dctrhalf
      
      double precision 
     $     t0,t1,t2,            ! timing variables
     $     oldrho,a,b,c,d,demax, de, lambda,eprecond, delrho,
     $     rho,                 ! current error
     $     nops,                ! number of operations
     $     egap                 ! gap between highest computed and next higher 
     $                          ! eigenvalue

      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcminustwo=(-2.d0,0.d0), dcmhalf=(-0.5d0,0.d0), 
     $     dctrhalf=(1.5d0,0.d0))


      integer
     $     myproc,
     $     ierrmall,
     $     mnew,
     $     diaginfo,            ! return value of diagonalization routines
     $     m,                   ! current number of vectors
     $     m_squared,           ! square of m
     $     i,j,                 ! dummies
     $     imax,                ! largest gap index
     $     n,                   ! iteration number
     $     nreset,              ! number of resets performed
     $     len                  ! gspace length

      complex(kind=8)
     $     dclambda, dclambdasq, dcgamma
c
c     work arrays
c
      complex(kind=8), allocatable :: ! of size m x m
     $     eps(:), s(:), xsi(:), eta(:), nu(:), mu(:)
      complex(kind=8), allocatable :: ! of size nanl x m 
     $     ews(:,:)
      complex(kind=8), allocatable :: ! of size len x m
     $     g(:,:), gold(:,:), gp(:,:), h(:,:), y(:,:)
      integer, allocatable :: iord(:) ! size of m
      double precision, allocatable :: rwork(:)


      logical ishift, ireset, iconjug

c
c          local routines
      double precision 
     $     trace,  contract, average_ekin
      external trace, contract

      double precision gimmetime ! timing routine


c     external functions
      external findroot
      external average_ekin
      external gimmetime

      t0=gimmetime()

      m=neig                    ! start with maximum allowed
      m_squared=m*m
      len=ham%gspace%length
      myproc=ham%gspace%myproc
c
c     allocate all the work spaces
c
      allocate(eps(m*m));  eps=0;
      allocate(s(m*m)); s=0; allocate(xsi(m*m)); xsi=0;
      allocate(eta(m*m)); eta=0;  
      allocate(nu(m*m)); nu=0;
      allocate(mu(m*m)); mu=0;
      allocate(rwork(max(1,3*m-2))); rwork=0;
      if(ham%vnloc%nvecs.gt.0) then
         allocate(ews(ham%vnloc%nvecs,m))
         ews=0
      endif
      allocate(g(len,m),stat=ierrmall);  
      if(ierrmall.ne.0) then 
         write(9,930) 'g',len,m,ierrmall
         stop
      endif
      g=0;
      allocate(gold(len,m),stat=ierrmall); 
      if(ierrmall.ne.0) then 
         write(9,930) 'gold',len,m,ierrmall
         stop
      endif
      gold=0;
      allocate(h(len,m),stat=ierrmall); 
      if(ierrmall.ne.0) then 
         write(9,930) 'h',len,m,ierrmall
         stop
      endif
      h=0;
      allocate(y(len,m),stat=ierrmall); 
      if(ierrmall.ne.0) then 
         write(9,930) 'y',len,m,ierrmall
         stop
      endif
      y=0;
      allocate(gp(len,m),stat=ierrmall); 
      if(ierrmall.ne.0) then 
         write(9,930) 'gp',len,m,ierrmall
         stop
      endif
      gp=0;
      allocate(iord(m)); iord=0;


      ireset=.true.             ! first step is like a reset
      iconjug = .false.         ! don't make conjugate in first step
      ishift=.true.             ! want to shift if error has decreased enough
      rho=1.0d4                 ! give some initial error
      nops=0
      t1=0

c      write(9,*) 'eigenvalues from submatrix:', eval(1:neig+1)
c      write(9,*) 'starting vectors:'
c      call printmatrix(x,len,m)
c      call overlap_matrix(s,x,x,m,len) !     s = x^T*x
c      write(9,*) 's matrix:'
c      call printmatrix(s,m,m)

c      call apply_ham_wavefn(pw_params%output,y(1,1),ham,x(1),
c     $     m,ews(1,1))          ! y = H * x
c      call overlap_matrix(eps,x,y,m,len) ! eps = x^T*y

c      write(9,*) 'eps matrix:'
c      call printmatrix(eps,m,m)

c      m = 4
c      write(9,*) 'number of states computed:', m
c      egap=0.283134674988261281-(-0.588457289928866700)
c      ham%shift = -0.588457289928866700 + 
c     $     egap/4 + 0.2

      egap = eval(neig+1)-eval(neig)

      if(egap.lt.0) then        ! no good gap as input, guess
         write(9,*) ' *** DIAGONALIZATION: INPUT EIGENVALUES BAD!'
         ham%shift=0.d0
         call apply_ham_wavefn(pw_params%output,y(1,1),ham,x(1),
     $        m,ews(1,1))       ! y = H * x
         call overlap_matrix(eps(1),x(1),y(1,1),m,len) ! eps = x^T*y
         egap= -4*eps(neigmin+(neigmin-1)*m)
         write(9,*) ' *** GUESSED GAP:', egap
      endif
c
c     shift hamiltonian just enough to make it negative definite
c
      ham%shift = eval(neig)+egap/4 + pw_params%shiftsafety
      nreset =0 

      do n=1, maxit             ! ----------- start of big iteration loop ----
 100     continue               ! restart

         if(ireset) then
            call apply_ham_wavefn(pw_params%output,y(1,1),ham,
     $           x(1),m,ews(1,1)) ! y=H * x
            if(iand(pw_params%output,1).eq.1) t2=gimmetime()
            call cheap_overlap_matrix(s(1),x(1),m,len) ! s = x^T*x
            call overlap_matrix(eps(1),x(1),y(1,1),m,len) ! eps = x^T*y
            if(iand(pw_params%output,1).eq.1) then
               t1=t1+gimmetime()-t2 
               nops = nops + 12*dble(m*m)*dble(len)
            endif
            ireset=.false.      ! no resetting any more
         else
            call mzaxpy(len*m,dclambda, g(1,1), 1, y(1,1), 1);
            dclambdasq=dclambda*dclambda
            call matrix_complete(eta(1),m)
            call matrix_complete(nu(1),m)
            call mzaxpy(m*m,dclambda, eta(1),1,s(1),1)
            call mzaxpy(m*m,dclambdasq, nu(1),1,s(1),1)
            call matrix_complete(xsi(1),m)
            call matrix_complete(mu(1),m)
            call mzaxpy(m*m, dclambda,  mu(1),1,eps(1),1)
            call mzaxpy(m*m, dclambdasq, xsi(1),1,eps(1),1)
         endif

c
c        adjust shift, find gap. In principle, this is a hack,
c        because the energy surface changes, and we should
c        restart the algorithm. In practice, we find that it 
c        hardly has any impact on the performance.
c

         if(ishift.and.(rho.lt.0.1)) then ! adjust shift of Hamiltonian
            xsi=eps
            call matrix_polish(s,xsi,mu,nu,m)
            mu=xsi
            call  mzheev('V','L',m, mu(1),m, eval(1),
     $           nu(1),m_squared,rwork(1),  diaginfo)
            if(diaginfo.ne.0) then
               write(9,*) 'diagonalization failed:',diaginfo
               call printmatrix(mu,m,m)
               stop
            endif
            do i=1,m ; iord(i)=i; end do
            call sort(m,eval(1),iord(1)) ! sort with numerical recipes
            if(neigmin.gt.m) then
               write(9,*) 'diagonalize_hamiltonian: neigmin > m.!'
               write(9,*) neigmin, m
               stop
            elseif(neigmin.lt.m) then
c
c               find largest gap, and throw other vectors away
c
               demax=0.d0
               imax=neigmin+1
               do i=neigmin+1,m
                  de=(eval(i)-eval(i-1))/(dble(i)**2.5)
                  if(demax.lt.de) then
                     demax=de; imax=i-1
                  endif
               end do
               mnew=imax        
c
c              superimpose x to form eigenvectors, and rearrange 
               if(iand(pw_params%output,1).eq.1) t2=gimmetime()
               call mzgemm('N','N',len,m,m,dcone,x(1),len,
     $              mu(1),m,dczero,g(1,1),len)
               if(iand(pw_params%output,1).eq.1) then
                  t1=t1+gimmetime()-t2
                  nops = nops + 8*dble(m*m)*dble(len)
               endif
               call wavefn_rearrange(len,m,g(1,1),x(1),
     $              iord(1),mnew)
               egap = eval(mnew+1)-eval(mnew)
               ham%shift = ham%shift+eval(mnew) +egap/4 + 
     $              pw_params%shiftsafety
               iconjug = .false. ! don't reuse old conjugate vector
               ireset  = .true. ! recompute H psi from scratch
               m = mnew
               ishift = .false.
               goto 100 
            else                ! neigmin.eq.m, no rearrangement
               ham%shift = ham%shift+eval(m)+egap/4+
     $              pw_params%shiftsafety
               ireset = .true.  ! recompute H psi from scratch
               ishift = .false.
            endif
         endif

c
c        calculate the gradient
c
         if(iand(pw_params%output,1).eq.1) t2=gimmetime()
         call mzgemm('N','N',len,m,m, dcone, x(1),len, eps(1), m,
     $        dczero,g(1,1),len) 
         call mzgemm('N','N',len,m,m,dcone, y(1,1),len, s(1),  m,
     $        dcone, g(1,1),len) 
         if(iand(pw_params%output,1).eq.1) then
            t1=t1+gimmetime()-t2
            nops = nops + 16*dble(m*m)*dble(len)
         endif

         call mzaxpy(len*m, dcminustwo, y(1,1), 1, g(1,1), 1)
c
c        make conjugate
c
         call mzcopy(len*m, g(1,1), 1, gp(1,1), 1) !prepare for preconditioning
c
c        change the preconditioning energy during the iteration.
c        this is not good style, since it effectively changes the
c        matrix under consideration, and we lose the property of
c        conjugation. we tried it out, and it doesn't hurt performance.
c        the factor of 1.35 was found by numerical experiment to give
c        best performance. Smaller factors are dangerous, as they kill
c        good components, and that hurts a lot.
c
         eprecond=average_ekin(x(1),xsi(1),
     $        ham%gspace%ekin(1),m,len)*1.35

         call precondition(gp(1,1),eprecond,ham%ekinmod(1),
     $        ham%gspace%ekin(1), m,len)

         rho = dble(parallel_zdotc(len*m,g(1,1),1,gp(1,1),1))

c         if(.not.ireset) then
         if((n.gt.1).and. (.not.ireset)) then
c           step for Polak-Ribiere. Fletcher-Reeves would not require gold,
c           but is performing terribly in many cases. 

            delrho = dble(parallel_zdotc(len*m,gold(1,1),1,gp(1,1),1))
            dcgamma =cmplx((rho-delrho)/oldrho,0.d0,kind=8)
c     dcgamma = cmplx(rho/oldrho) ! -------- do this for fletcher-Reeves
            call mzaxpy(len*m, dcgamma, h(1,1), 1, gp(1,1), 1)
         endif

         call mzcopy(len*m, gp(1,1), 1, h(1,1), 1)
c        save gradient into old gradient
         call mzcopy(len*m, g(1,1), 1, gold(1,1), 1) 

         oldrho = rho

         if(rho.lt.delta) then
            iconv=.true.
            goto 200
         endif
c
c        compute some of the matrices
c
         if(iand(pw_params%output,1).eq.1) t2=gimmetime()
         call hermit_overlap_matrix(eta(1),h(1,1),x(1),m,len)
         call cheap_overlap_matrix(nu(1),h(1,1),m,len)
         call hermit_overlap_matrix(mu(1),h(1,1),y(1,1),m,len)
         if(iand(pw_params%output,1).eq.1) then
            t1=t1+gimmetime()-t2
            nops = nops + 12*dble(m*m)*dble(len)
         endif
         call apply_ham_wavefn(pw_params%output, g(1,1),ham,
     $        h(1,1),m,ews(1,1)) ! g=H * h
         if(iand(pw_params%output,1).eq.1) t2=gimmetime()
         call overlap_matrix(xsi(1),h(1,1),g(1,1),m,len)
         if(iand(pw_params%output,1).eq.1) then
            t1=t1+gimmetime()-t2
            nops = nops + 8*dble(m*m)*dble(len)
         endif

c
c        calculate the coefficients for the minimization routine 
c
         a = 2.0*trace(mu(1),m)-contract(eta(1),eps(1),m) 
     $        - contract(mu(1),S(1),m)
         b = 2.0*trace(xsi(1),m) - contract(mu(1),eta(1),m) 
     $        - contract(eps(1),nu(1),m) 
     $        - contract(xsi(1),S(1),m)
         c = -contract(xsi(1),eta(1),m) - contract(nu(1),mu(1),m)
         d = -contract(xsi(1),nu(1),m)
c
c        do the line minimization 
c
         call findroot(lambda,4*d,3*c,2*b,a)
         if(lambda<0) then
            ham%shift=ham%shift + 0.5
            nreset = nreset+1
            write(9,*) '*** diagonalize_hamiltonian: resetting!'
            ireset=.true.
            if(nreset+n.gt.maxit) goto 199 ! too many resets ... bye bye
            goto 100
         endif

         dclambda=lambda
c
c        move to new position
c
         call mzaxpy(len*m, dclambda, h(1,1), 1, x(1), 1)

c         write(9,*) 'iter',n,' m=',m,' rho=',rho,' shift', ham%shift
c         call myflush(9)
      end do                    ! ----------- end of big iteration loop ------

 199  continue
      iconv=.false.

 200  continue

c
c     Done with the iterative part. 
c
c     Now first orthogonalize the vectors approximately. This step reduces
c     linear errors in the eigenvalues, and is important.
  
      if(iand(pw_params%output,1).eq.1) t2=gimmetime()
      call mzgemm('N','N',len,m,m,dcmhalf,x(1),len,s(1),m,dczero,
     $     gold(1,1),len)
      if(iand(pw_params%output,1).eq.1) then
         t1=t1+gimmetime()-t2
         nops = nops + 8*dble(m*m)*dble(len)
      endif
      
      call mzaxpy(len*m, dctrhalf, x(1), 1, gold(1,1), 1)
      call mzcopy(len*m, gold(1,1), 1, x(1), 1)

c     calculate transformed new eps=1/4 (9eps-3St*eps-3eps*S+St*eps*S) 

      call matrix_polish(s,eps, eta,nu,m)

c     the vectors x we have now span the right subspace, but are not yet the 
c     eigenvectors. to get those, we diagonalize the epsilon matrix. 
c     the eigenvalues are the eigenvalues we are looking for. 
c     the eigenvectors tell us how to superimpose the vectors x to get 
c     the true eigenvalues.
c     There are parallel issues here: If the eps matrix is not
c     exactly identical, the eigenvectors will acquire a different
c     phase on different processors. Therefore, we diagonalize only
c     on proc 0, and broadcast the result.
c
c

      if(myproc.eq.0) then
         call mzheev('V','L',m,eps(1),m, eval(1),g(1,1),m*len,
     $        rwork(1),diaginfo) 
         if(diaginfo.ne.0) then
            if(myproc.eq.0)
     $           write(9,*) 'diagonalization failed with info='
     $           ,diaginfo
            stop
         endif
      endif
      call my_broadcast(eps,m*m,0)
      call my_broadcast(eval,m,0)

c
c     combine the eigenvectors into g

      if(iand(pw_params%output,1).eq.1) t2=gimmetime()
      call mzgemm('N','N',len,m,m, dcone, x(1),len, eps(1),m, dczero,
     $     g(1,1), len) 
      if(iand(pw_params%output,1).eq.1) then
         t1=t1+gimmetime()-t2
         nops = nops + 8*dble(m*m)*dble(len)
      endif

c
c     save the eigenvalues
c
      do i=1,m
         eta(i)=cmplx(eval(i)+ham%shift,0.d0,kind=8)
      end do

      neig = m                  ! return number of eigenvectors calculated 

c     in case the calculation did not converge, preform
c     the orthonormalization of the  subspace: 
c     
c     a) Compute S^-1/2 = AT * D^-1/2 * A   
c     b) Compute x_new= S^-1/2 * x
c     
c     The eigenvalue array is used as temporary storage
c

      if(.not.iconv) then
         write(9,'(a,i3,a,g12.6)') 
     $        ' diagonalization not conv. after ',n,
     $        ' iter., residual error = ',rho
         call cheap_overlap_matrix(s,g,m,len);
c        diagonalize S, find eigenvectors A (returned in S) 

         call mzheev('V','L',m, s(1), m, eval(1),x(1),m*len,
     $        rwork(1), diaginfo);  

c        compute At * D^-1/2 * A  

         do i=1,m
            eval(i)=1.0/sqrt(eval(i))
            do j=1,m
               mu(i+m*(j-1))=eval(i)*conjg(s(j+m*(i-1)))
            end do
         end do

c        compute nu = A * D^-1/2* AT 
         call mzgemm('N','N', m,m,m, dcone, s(1),m,mu(1),m,dczero,
     $        nu(1),m) 

c        compute g = nu * x  
         call mzcopy(len*m, g(1,1), 1, x(1), 1); !   x <- g 
         if(iand(pw_params%output,1).eq.1) t2=gimmetime()
         call mzgemm('N','N',len,m,m, dcone,x(1),len, nu(1),m,dczero,
     $        g(1,1),len) 
         if(iand(pw_params%output,1).eq.1) then
            t1=t1+gimmetime()-t2
            nops = nops + 8*dble(m*m)*dble(len)
         endif


      endif
c
c     copy return eigenvectors into x array
c
      call mzcopy(len*m, g(1,1), 1, x(1), 1); !   x <- g 
c
c     retrieve eigenvalues
c
      eval(1:m)=dble(eta(1:m))
      eval(m+1)=eval(m)+egap    ! and remember gap
c
c     performance printout
c
      if(iand(pw_params%output,1).eq.1) write(9,920) t1,nops/t1/1d6
c
c     timing printout
c
      if(iand(pw_params%output,8).eq.8) then 
         write(9,910) n,gimmetime()-t0
         call myflush(9)
      endif

c
c     deallocate the work spaces
c
      deallocate(eps);  deallocate(s); deallocate(xsi);
      deallocate(eta);  deallocate(nu); deallocate(mu);
      deallocate(rwork); 
      if(ham%vnloc%nvecs.gt.0) deallocate(ews)
      deallocate(g);  deallocate(gold); deallocate(h); 
      deallocate(gp);
      deallocate(y);
      deallocate(iord);

      return
 910  format(' DIAGONALIZATION TOOK ',i3,' ITERATIONS AND ', 
     $     f12.3, ' SECONDS')
 920  format(' TIME FOR MATRIX-MATRIX:',f12.6,', MFLOPS=',f12.6)
 930  format('*** DIAGONALIZE_HAMILTONIAN: ALLOCATE(',a,'(',i6,
     $        ',',i4,') FAILED. ERROR CODE=',i4)
      end

