c-*-Fortran-*- 
c     @process extchk
      subroutine direct_emin(pot_gspace, k_gspace,
     $     energs, pw_params, crys, syms, ffts,
     $     pspot, kpoints, bands, wavefn, lrwfn,
     $     vion, vhxc, vout, den, denc, rden)


      include 'use.h'
      implicit none         
      include 'interface.h'
c
c     INPUT:
c     -----
      
      type (symmetry) syms       ! symmetry operations
      type (crystal) crys       ! crystal structure
      type (fftstruc) ffts      ! information for the fast fourier transform
      type (pw_parameter) pw_params ! plane wave parameters
      type (pseudo_potential) pspot ! all info about the pseudo potential
      type (kpoint) kpoints     ! BZ integration kpoints

      type (parallel_gspace) 
     $     pot_gspace, k_gspace(*)

      complex(kind=8) 
     $     denc(pot_gspace%length) ! core charge density
      logical lrwfn             ! true if wavefn is to be reused, false if not

c
c     INPUT/OUTPUT:
c     -------------

      type (energy) energs      ! energies and other information
      type (band) bands         ! eigenvalues, occupation numbers etc
      complex(kind=8) 
     $     vion(pot_gspace%length,crys%nspin), !screened ionic potential
     $     vhxc(pot_gspace%length, crys%nspin) ! V_Hartree + V_xc

c
c     OUTPUT:
c     ------

      complex(kind=8) 
     $     den(pot_gspace%length,crys%nspin), ! valence charge density
     $     vout(pot_gspace%length, crys%nspin) ! V_Hartree + V_xc

      type (complex_gspace_array) 
     $     wavefn               ! wave functions for all kpoints, bands

c
c     WORK:
c     ----

      complex(kind=8)
     $     rden(pot_gspace%r_size)

c
c     DESCRIPTION:
c     -----------
c
c     Does a direct minimization of the energy functional. The system
c     must be an insulator for this to work. Also, the memory costs
c     are very high when multiple k-points are used.
c
c
c     --------------- local variables -------------------------
c
      type (parallel_gspace)
     $     sub_gspace           ! for the starting guess

      complex(kind=8), pointer ::
     $     vloc(:)            ! local potential in realspace
      type (hamiltonian)
     $     ham(kpoints%nrk)     ! hamiltonian at all kpoints
      type (dense_hamiltonian)
     $     densham              ! subspace hamiltonian
      type (energy)
     $     oldenergs            ! as a dummy for etotal

      complex(kind=8), allocatable ::
     $     vbareion(:) ! bare ionic potential

      double precision 
     $     dsmear, sqrpi, ryd, pi, delta,
     $     ddum                 ! dummy for etotal

      parameter (pi=3.141592653589793d0, ryd   =  13.60580d0)

      integer 
     $     i,j,n,
     $     nsp, nvecs, 
     $     ierr,                ! error flag
     $     is,                  ! spin counter
     $     irk                  ! kpoint counter

      logical iconv             ! indicates if minimization is converged

      character*11 wfnname

c
c     --------------------------------------------------------------
c
      write(9,100);      call myflush(9)

      dsmear = pw_params%smearing/ryd
      sqrpi  = sqrt(pi)

      allocate(vloc(ffts%r_size)) ! local potential in realspace
c
c     Set up the Hamiltonians at all k-points. Only one spin
c     component is set up, and later the local potential component of
c     the Hamiltonians is set to either spin up or spin down local
c     potential. This is to save memory. After all, the Kleinman
c     -Bylander projectors are the same for spin up and spin down
c
      do irk=1, kpoints%nrk
         call create_hamiltonian(ham(irk),k_gspace(irk),pspot,
     $        vloc,ffts)
         ham(irk)%ekinmod=pw_params%ekinmod
         call setup_nonlocal_potential(ham(irk),pspot,crys)
      end do

c     
c     ------ Now read or generate the starting guesses ----------
c                  also set up the local potential 
c     
      do is=1, crys%nspin
         call setup_local_potential(0,ffts,pot_gspace,
     $        vion(1,is),vloc(1))
         do irk=1, kpoints%nrk
            ierr=1
            if(.not.lrwfn) then
               if(iand(pw_params%input,1).eq.1) then ! read from file
                  call read_band('BAND',bands)
                  if(bands%nrk.le.0) then ! error
                     write(9,155)
                     bands%nrk =kpoints%nrk ; goto 20
                  endif
                  write(wfnname,120) irk,is
                  wavefn%data(:,irk,is)=0
                  nsp = 1 ; nvecs = bands%nband(irk,is)
                  call readgsdat(1, ierr, k_gspace(irk), 
     $                 wavefn%data(1,irk,is), bands%nband(irk,is),
     $                 nvecs, nsp, wfnname,10)
                  if(ierr.ne.0) then
                     write(9,150) wfnname ; goto 20
                  endif
                  write(9,140) nvecs, wfnname
                  ierr=0
 20               continue
               endif
               if(ierr.eq.1) then ! ------- have to do submatrix diag ------
                  sub_gspace%gmax=sqrt(pw_params%emaxsub)
                  sub_gspace%rk=ham(irk)%gspace%rk !  shift from big brother
                  sub_gspace%nproc=1; ! scatter across all processors
                  sub_gspace%name='sub'
                  sub_gspace%myproc=0;
                  sub_gspace%fftsize=(/0,0,0,0,0/)
                  sub_gspace%istar=.false. ;sub_gspace%igvec=.true.
                  sub_gspace%iekin=.true.;sub_gspace%iinverse=.false.
                  sub_gspace%istruc=.false.
                  sub_gspace%ipackinfo=.false.
                  call generate_gspace(pw_params%output,sub_gspace, 
     $                 crys, syms)
                  call create_dense_hamiltonian(densham,sub_gspace)
                  call start_vectors(0,pw_params,ham(irk),
     $                 densham,bands%energy(1,irk,is), bands%max,
     $                 wavefn%data(1,irk,is),pot_gspace, sub_gspace,
     $                 vion(1,is),crys)
                  call destroy(densham)
                  call destroy(sub_gspace)
               endif
            endif
            
            if(pw_params%bandgap.gt.0) then ! put in bandgap by hand
               bands%energy(bands%min+1,irk,is)=
     $              bands%energy(bands%min,irk,is)+pw_params%bandgap
            endif
            bands%nband(irk,is)=bands%min ! it is an insulator
         end do
      end do
c
c     ------- Do the direct minimization ----------------------------
c
      if(lrwfn) write(9,160)
      pw_params%isymmetrize = .false.
      if((kpoints%nrk.gt.1).or.(kpoints%shift(1).ne.0) ! need to symmetrize?
     $     .or.(kpoints%shift(2).ne.0)
     $     .or.(kpoints%shift(3).ne.0)) then
         if(syms%ntrans.gt.1) then
            pw_params%isymmetrize = .true.
         endif
      endif
      if(crys%nspin.gt.1) then
         write(9,*) '*** ENERGY MINIMIZATION WORKS ONLY FOR NSPIN=1'
         stop
      endif

      allocate(vbareion(pot_gspace%length))
      vbareion(:)=vion(:,1)-vhxc(:,1) ! bare ionic potential

      if(pot_gspace%ig0.gt.0) 
     $    vbareion(pot_gspace%ig0)=0
       

      call cgemin(pw_params,pw_params%maxitdiag, iconv, bands,
     $     pw_params%epsdiag, kpoints,
     $     ham(1), wavefn, pot_gspace, 
     $     crys, energs, denc(1), rden(1), 
     $     den(1,1),  vhxc(1,1), vbareion(1))

c
c     ------- compute and print total energy ----------------------
c
c
c     compute kinetic and band energy and set band occupation numbers
c
      energs%ektot=0
      energs%eband=0
      energs%esmear=0

      call flevel(1, 2,pw_params,energs, crys,bands,kpoints)
      
      do is=1, crys%nspin
         do irk=1, kpoints%nrk
            call kinet(ham(irk),wavefn%data(1,irk,is), 
     $           bands%nband(irk,is), bands%ekn(1,irk,is))
            do n=1, bands%nband(irk,is)
               delta=(energs%efermi(is)-
     $           (bands%energy(n,irk,is)+energs%evalshift(is)))/dsmear
               energs%ektot = energs%ektot+ bands%occup(n,irk,is)
     $              *bands%ekn(n,irk,is)
               energs%eband = energs%eband+ bands%occup(n,irk,is)
     $              *bands%energy(n,irk,is)
               energs%esmear = energs%esmear - dsmear/sqrpi*
     $              kpoints%w(irk) * dexp(-delta*delta)
            end do
         end do
      end do
      

      write(9,*) 'vxc0=', energs%vxc0(1)
      write(9,110)
      do irk=1, kpoints%nrk
         write(9,115)irk,bands%energy(1:bands%min,irk,1)
      end do
c
c     prepare potentials for output
c

      oldenergs = energs
      do is=1, crys%nspin
         vion(:,is) = vhxc(:,is) 
         vhxc(:,is) = vhxc(:,is)-vbareion(:)
         vout(:,is) = vhxc(:,is)
         if(pot_gspace%ig0.gt.0) 
     $        vout(pot_gspace%ig0,is)=energs%vxc0(is)
      end do


      call etotal(crys,1, iconv, ddum, energs, oldenergs,
     $     pw_params, pot_gspace, vion(1,1), 
     $     vhxc(1,1), vhxc(1,1), den(1,1))

c
c     ------- Write charge density to disk -------------------------
c
      call writegsdat(0, pot_gspace, den(1,1), 1,
     $     1, crys%nspin, 'CD',2)
c
c     ------- Write eigenvectors to disk if requested ---------------
c
      if(iand(pw_params%output,512).eq.512) then
         write(9,130)
         do is=1, crys%nspin
            do irk=1, kpoints%nrk
               write(wfnname,120) irk,is
               call writegsdat(0,k_gspace(irk), wavefn%data(1,irk,is)
     $              ,bands%nband(irk,is),bands%nband(irk,is),1
     $              ,wfnname,10)
            end do
         end do
         if(pot_gspace%myproc.eq.0) then
            call mysystem('\rm -f BAND')
            call write_band('BAND',bands)
         endif
      endif

c
c     ----------- deallocate all memory -----------------------
c
      do irk=1, kpoints%nrk
         call destroy_hamiltonian(ham(irk))
      end do

      deallocate(vloc)
      deallocate(vbareion)



      return
C
 100  format(/,17('-'),' DIRECT MINIMIZATION OF ENERGY FUNCTIONAL ',
     $     17('-'))

 110  format(/' FINAL EIGENVALUES (WITHOUT VXC0):')
 115  format(/' KPOINT:',i4,/1000(5f12.6/)) 


 120  format('WFN',i5.5,'.',i1)
 130  format(/' <<<<<<<< WRITING WAVEFUNCTIONS TO DISK >>>>>>>>>>')
 140  format(' <<<<<<<< READ ',i4,' BANDS FROM FILE ',a,'>>>>>>>>')
 150  format(' *** COULD NOT FIND WAVE FUNCTION ',a,'ON DISK!')
 155  format(' *** COULD NOT FIND BAND FILE ON DISK!')
 160  format(' <<<<<<<< REUSING WAVE FUNCTIONS >>>>>>>>')
      end
