c     
      subroutine dos(myproc, nproc,
     $     p,crys, bands, energs, kp, pw_params)

      use all_to_all_module
      include 'use.h'
      implicit none 
      include 'interface.h'
      include 'all_to_all.h'
c
c     1996 Bernd Pfrommer
c
c     Based on subroutines by Alberto Garcia, Paul Delaney, and
c     Michel Cote.
c
c     INPUT:
c     -----

      type(crystal) crys        ! the crystal structure
      type(band) bands
      type(energy) energs
      type(kpoint) kp
      type (pw_parameter) pw_params


      integer myproc, nproc
c
c     The weights for different angular momenta, different bands,
c     kpoints, and spins. A fancy interpolation between the function
c     values at different kpoints is done inside the subroutine
c     spec_tet.
c
      double precision
     $     p(7,crys%mxdatm,crys%ntype,bands%max,bands%nrk,crys%nspin)

c
c     DESCRIPTION:
c     -----------
c
c     computes DOS and angular momentum resolved DOS using a 
c     tetrahedron method. The data must be given on a Monkhorst-Pack grid
c
c     -------------------local variables ------------------------------
c
      integer 
     $     n, i, j, k, l, nx, ny, nz, is, ie, it, ilm, ia, b,
     $     i1, j1, k1, ibnd, nmesh, kidx(4), itet, ntetra, na,
     $     nmytet, itetstart, itetend, neqtet, idum

      double precision 
     $     e(0:3), f(0:3),  tpi, t0, ecorn(4), fcorn(4),
     $     vol, ef, mineigen, maxeigen, delta, ryd, fav,t, ebound(2)

      double precision, allocatable ::
     $     et(:,:,:),ados(:,:,:,:,:), edos(:,:), dummy(:)

      integer, allocatable::
     $     itetdef(:,:)

      parameter(ryd=13.60580d0)

      double precision tetrados, gimmetime
      external gimmetime

c     --------------------------------------------------------------
c

      t0=gimmetime()

      write(9,110)
      nmesh=1000
      allocate(edos(nmesh,2))
      edos = 0
      allocate(dummy(nmesh))
      dummy = 0

      tpi = 8.d0*atan(1.d0)

      nx=kp%grid(1)
      ny=kp%grid(2)
      nz=kp%grid(3)
      
      if(.not.associated(kp%kmap)) then
         write(9,*) 'DOS: kpoint map is not set up!'
         stop
      endif

c
      vol = (tpi)**3/crys%vcell/(nx*ny*nz)/6.d0
c
c     rewrite the eigenvalues in eV according to fermi level
c
      allocate(et(bands%max,bands%nrk,bands%nspin))
      allocate(ados(nmesh,7,crys%mxdatm,crys%ntype,crys%nspin))
      ados=0

      do is=1,bands%nspin
         do i=1,bands%nrk
            do ibnd=1,bands%min
               et(ibnd,i,is)=(bands%energy(ibnd,i,is)+
     $              energs%evalshift(is)-
     $              energs%efermi(is))*ryd
            enddo
         enddo
      enddo
c
c     determine lowest and highest eigenvalues
c
      mineigen=et(1,1,1)
      maxeigen=et(bands%min,1,1)
      do is=1,bands%nspin
         do i=1, bands%nrk
            if(et(1,i,is).lt.mineigen) mineigen=et(1,i,is)
            if(et(bands%min,i,is).gt.maxeigen) 
     $           maxeigen=et(bands%min,i,is)
         enddo
      enddo
      delta=dabs(maxeigen-mineigen)/dble(nmesh)

c
c     -------- COMPUTE (ANGULAR MOMENTUM RESOLVED) DOS ----------
c
      ef = mineigen - 10.0*delta ! set ef to start value
      ebound(1)=mineigen-10.0*delta
      ebound(2)=maxeigen+10.0*delta
      delta=abs(ebound(2)-ebound(1))/dble(nmesh)


      ntetra = 6*nx*ny*nz       ! number of tetrahedra

      allocate(itetdef(4,ntetra))
      call setup_tetrahedra(kp%grid, kp%kmap(1), itetdef, crys%bvec)
c
c     now figure out which tetrahedra are on this processor
c
      b = ntetra/nproc       ! minimum number of tetrahedra
      na = mod(ntetra,nproc)  ! number of procs with one more tet.
      nmytet = b
      if(na.gt.myproc) nmytet=nmytet+1

      itetstart= b*myproc + min(myproc,na)+1
      itetend = itetstart + nmytet-1
c
c     run the loop over all tetrahedra
c

      neqtet=0
      idum=0
      do itet =itetstart, itetend ! loop through all tetrahedra
         kidx = itetdef(:,itet)
         do is=1, crys%nspin
            do ibnd = 1, bands%min
               ecorn(:)=et(ibnd,kidx(:),is) ! energy at corners
               fcorn(:)=1.d0
               call spec_tet(vol,ecorn, fcorn,nmesh,ebound, 
     $              edos(1,is),dummy(1),neqtet)
               if(iand(pw_params%output,1024).eq.1024) then
               do it=1, crys%ntype
                  do ia=1, crys%natom(it)
                     do ilm=1,7
                        ecorn(:)=et(ibnd,kidx(:),is) ! energy at corners
                        fcorn(:)=p(ilm,ia,it,ibnd,kidx(:),is)
                        call spec_tet(vol,ecorn, fcorn,nmesh,ebound, 
     $                       dummy(1), ados(1,ilm,ia,it,is),idum)
                     end do
                  end do
               end do
               endif
            end do
         end do
      end do
      neqtet = neqtet/(crys%nspin*bands%min)
      call all_sum_all(neqtet)
      write(9,210) ntetra
      write(9,200) neqtet


c
c     now sum across all processors
c
      do is=1, crys%nspin
         dummy(:)=edos(:,is)
         call all_sum_all(dummy,nmesh)
         edos(:,is) = dummy(:)
      end do
      if(iand(pw_params%output,1024).eq.1024) then
         do is=1, crys%nspin
            do it=1, crys%ntype
               do ia=1, crys%natom(it)
                  do ilm=1,7
                     dummy(:)=ados(:,ilm,ia,it,is)
                     call all_sum_all(dummy,nmesh)
                     ados(:,ilm,ia,it,is)=dummy(:)
                  end do
               end do
            end do
         end do
      endif

      
      if(myproc.ne.0) then
         deallocate(itetdef)
         deallocate(ados)
         deallocate(edos)
         deallocate(et)
         return
      endif



c
c     ------------ print out ---------------------------------------
c
      open(11,file='DOS',status='unknown', form='formatted')

      write(11,'(a15,2f12.6)') '# FERMI LEVELS:',
     $     energs%efermi(1),energs%efermi(2)
      write(11,'(a17)') '# SPIN DENSITIES:'
      
      do is=1,bands%nspin
         do ie=1,nmesh
            write(11,*) ebound(1)+ie*delta, 
     $           edos(ie,is)*crys%vcell/(tpi)**3/bands%nspin
         enddo
         write(11,*) '&'
      end do

      if(bands%nspin.gt.1) then
         write(11,'(a100)') '# sum of spin densities:'
         do ie=1,nmesh
            write(11,*) ebound(1)+ie*delta, 
     $           edos(ie,1)*crys%vcell/(tpi)**3/bands%nspin+
     $           edos(ie,2)*crys%vcell/(tpi)**3/bands%nspin 
         end do
         write(11,*) '&'
      endif
c
c     ------------------------------------------------------------------
c
c
c     print angular resolved DOS if it is requested
c
      if(iand(pw_params%output,1024).eq.1024) then
         do is=1,bands%nspin
            do it=1, crys%ntype
               do ia=1, crys%natom(it)
                  do ilm=1,7
                     if(ilm.eq.1) then
                        write(11,124) crys%nameat(it),ia,is
                     elseif(ilm.lt.4) then
                        write(11,123) crys%nameat(it),ia,ilm-2,is
                     elseif(ilm.eq.7) then
                        write(11,123) crys%nameat(it),ia,2,is
                     elseif(ilm.lt.7) then
                        write(11,125) crys%nameat(it),ia,ilm-5,is
                     endif
                     do ie=1,nmesh
                        write(11,*) ebound(1)+ie*delta, 
     $                       ados(ie,ilm,ia,it,is)*
     $                       crys%vcell/(tpi)**3.d0/dble(bands%nspin)
                     enddo
                     write(11,*) '&'
                  end do
               end do
            end do
         end do
      endif

c
      close(11)

      write(9,987)

      if(iand(pw_params%output,8).eq.8) then
         write(9,988) gimmetime()-t0
      endif
      
      deallocate(itetdef)
      deallocate(ados)
      deallocate(edos)
      deallocate(et)

      return
 110  format(/' GENERATION OF DOS',/1x,17('-'))
 123  format('#',a2,1x,i4,' with angular momentum:',i2,', spin:',i2)
 124  format('#',a2,1x,i4,' local density of states',', spin:',i2) 
 125  format('#',a2,1x,i4,' p-state with m-quantum number:',
     $     i2,', spin:',i2)
 200  format(' NUMBER OF TETRAHEDRA WITH EQUAL CORNERS:',i10)
 210  format(' TOTAL NUMBER OF TETRAHEDRA:             ',i10)

 987  format(/' WROTE DENSITY OF STATES TO FILE "DOS"'/)
 988  format(/' TIME FOR DENSITY OF STATES: ',f14.4)

      end

c
c     =================================================================
c
      subroutine setup_tetrahedra(grid,k_index, itetra, bvec)
c
c     1996 Bernd Pfrommer
c
c     originated from a code by Paul Delaney
c
      implicit none
c
c     INPUT:
c     -----
c
      integer 
     $     grid(3),             ! the Monkhorst pack k-grid size kx,ky,kz
     $     k_index(0:grid(1)-1,0:grid(2)-1,0:grid(3)-1) ! map to irred kpts

      double precision 
     $     bvec(3,3)            ! the reciprocal lattice vectors, as columns
     $                          ! in the matrix

c
c     OUTPUT:
c     ------
c
      integer 
     $     itetra(4,6*grid(1)*grid(2)*grid(3)) ! index into irred. k-pt array
c
c     
c     DESCRIPTION:
c     -----------
c
c     1) Finds optimal 6-tetrahedral disection of the monkhorst-pack
c     grid, such that the edgelengths of the tetrahedra are as
c     regular as possible
c
c     2) With this, generates the index array itetra which maps each
c     corner of the 6*nx*ny*nz tetrahedra to an irreducible kpoint
c
c
c
c     -------------------------- local variables ------------------
c

      integer
     $     ntetra, i,j,k,it,iv,idx,idy,idz, ivert,
     $     tset(3,4,6), ivert2, ivert1, goodi, goodj, goodk,
     $     initialtetraoffset(3,4,6)

      double precision 
     $     thisedgemin, thisedgemax, cart(3,4), edge, 
     $     edgemin, edgemax
c
c
c
      data initialtetraoffset/
     $     0,0,0, 0,1,0, 1,1,0, 1,1,1,
     $     0,0,0, 1,0,0, 1,1,0, 1,1,1,
     $     0,0,0, 1,0,0, 1,0,1, 1,1,1,
     $     0,0,0, 0,1,0, 0,1,1, 1,1,1,
     $     0,0,0, 0,0,1, 0,1,1, 1,1,1,
     $     0,0,0, 0,0,1, 1,0,1, 1,1,1 /

      edgemin=0.d0
      edgemax=1.0d10

c     First set up the matrix of components of the reciprocal lattice
c     vectors along the Cartesian basis. bcomp(i,j) holds the dot
c     product of dual basis vector b_i with Cartesian basis vector
c     e_j.

c     Now go through all the four long diagonals, specified by the
c     corner (i,j,k)
      k=0
      do i=0,1
         do j=0,1

            do ntetra=1,6
               do ivert=1,4
                  call mirrorijk(initialtetraoffset(1,ivert,ntetra),
     $                 tset(1,ivert,ntetra),i,j,k)
               end do
            end do

            thisedgemin=1.0d10
            thisedgemax=0.d0

            do ntetra=1,6
               do ivert=1,4
                  cart(:,ivert)=
     $                 matmul(bvec,dble(tset(:,ivert,ntetra)))
               end do
               
               do ivert1 = 1,3
                  do ivert2 = ivert1+1,4
                     edge=(cart(1,ivert1)-cart(1,ivert2))**2
     $                  + (cart(2,ivert1)-cart(2,ivert2))**2
     $                  + (cart(3,ivert1)-cart(3,ivert2))**2
                     edge = sqrt(edge)
                     thisedgemax=max(thisedgemax,edge)
                     thisedgemin=min(thisedgemin,edge)
                  end do
               end do

            end do

            if (thisedgemax .lt. edgemax) then
               goodi=i
               goodj=j
               goodk=k
               edgemax=thisedgemax
               edgemin=thisedgemin
            end if
            
         end do
      end do

      do ntetra=1,6
         do ivert=1,4
            call mirrorijk(initialtetraoffset(1,ivert,ntetra),
     $           tset(1,ivert,ntetra),goodi,goodj,goodk)
         end do
      end do

c      write(0,*)
c      write(0,*) 'Choosing long-diagonal based at i,j,k=',
c     $     goodi,goodj,goodk
c      write(0,*) 'The longest edge length is ',edgemax,' a.u.'
c      write(0,*) 'The shortest is ',edgemin,' a.u.'


c     -------------------------------------------------------------------
c     ----------- now use the optimial tetrahedron to set up the --------
c     -------------------------- index array ----------------------------
c

c      write(9,*) 'TETRAHEDRON GENERATION:'

      ntetra=0

      do i=0,grid(1)-1
         do j=0,grid(2)-1
            do k=0,grid(3)-1
               do it=1,6
c                  write(9,*) 'tetrahedron:',it
                  do iv=1,4
                     idx = mod(i+ tset(1,iv,it),grid(1))
                     idy = mod(j+ tset(2,iv,it),grid(2))
                     idz = mod(k+ tset(3,iv,it),grid(3))
                     itetra(iv,ntetra+it)= abs(k_index(idx, idy,idz))
c                     write(9,*) idx,idy,idz,itetra(iv,ntetra+it)
                  end do
               end do
               ntetra = ntetra+6
            end do
         end do
      end do



      end 

c
c     =================================================================
c

      subroutine mirrorijk(in,out,i,j,k)
c     Do mirrors in i around i=.5 if i=1; similarly for j,k
c
c     1995/96 by Paul Delaney
c
      integer i,j,k      
      integer in(3),out(3)

      out(1)=in(1)
      out(2)=in(2)
      out(3)=in(3)

      if (i .eq. 1) out(1) = 1-out(1)
      if (j .eq. 1) out(2) = 1-out(2)
      if (k .eq. 1) out(3) = 1-out(3)

      return
      end
