c
      subroutine dxplot(imode, ioutput,
     $     data,  gs, ffts,  crys,  outfilename)
c
c
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'


c
c     prepare a realspace  plot of a complex data array data.
c     writes a .dx file which can be imported by the IBM data explorer,
c     a visualization software package. If ioutput has the khoros flag
c     set, it produces a file suitable for the khoros graphics
c     package instead of a dx file.
c
c     1996, 1997 Bernd Pfrommer

c
c     INPUT
c     -----

      type (parallel_gspace) gs ! gspace on which the data is defined
      type (fftstruc) ffts
      type (crystal) crys

      integer  
     $     ioutput,             ! flag to determine format (dx/khoros)
     $     imode                ! mode flag
     $                          ! imode = 1 print real part of data
     $                          !       = 2 print imaginary part of data
     $                          !       = 3 print absolute squared
     $                          !       = 0 print absolute of data

      character*7   outfilename ! name of output file. Suffix .dx will be added

      complex(kind=8)
     $     data(gs%length)      ! data to be visualized



c     
c     ------------- local variables
c

      integer iproc, i,j,k, ic

      logical ikhoros

      double precision delta(3,3),
     $     sc1, sc2, sc3, ri(3), x(3)

      complex(kind=8), allocatable ::
     $     chd(:)     ! for realspace data
c
c     -------------------------------------------------------------
c
      ikhoros = iand(ioutput,32768).eq.32768 ! if khoros or dx

      call dx_latticevec(crys,gs%myproc)

      call parallel_barrier()

      allocate(chd(ffts%r_size))

      

c
c     inverse fourier transform the data to real space
c
      call fourier_transform(-1,ffts,gs,  data, chd)

c
c     --------- open files and write the header
c
      if(gs%myproc.eq.0) then
         if(ikhoros) then
            open(unit=18,file= 'vis/' // outfilename // '.avs',
     $           status='unknown',form='formatted')
            write(18,105) gs%fftsize(1),gs%fftsize(2),gs%fftsize(3) 
         else                   ! dx header 
            open(unit=18,file= 'vis/' // outfilename // '.dx',
     $           status='unknown',form='formatted')
            write(18,5)         ! write the header for the array
            write(18,10) gs%fftsize(1)*gs%fftsize(2)*gs%fftsize(3)
         endif
         close(18)
      endif
      
      call parallel_barrier()
c
c     --------- now write the data
c
c     Due to the specific data layout, the a1 axis runs the fastest,
c     then the a2 axis, and finally the a3 axis runs slowest.
c



      sc1 = dble(1.d0/gs%fftsize(1))
      sc2 = dble(1.d0/gs%fftsize(2))
      sc3 = dble(1.d0/gs%fftsize(3))
      ic=0

      do iproc =gs%nproc-1, 0,-1
         
         if(iproc.eq.gs%myproc) then
            if(ikhoros) then
               open(unit=18,file= 'vis/' // outfilename // '.avs',
     $              position='append',
     $              status='unknown',form='formatted')
               do i=1, ffts%r_size
                  ri(1) = sc1*mod(ic,gs%fftsize(1))
                  ri(2) = sc2*mod(ic/(gs%fftsize(1)),gs%fftsize(2))
                  ri(3) = sc3*(ic/(gs%fftsize(1)*gs%fftsize(2)))
                  x=matmul(crys%avec,ri)
                  ic=ic+1
                  if(imode.eq.1) then
                     write(18,180) x,dble(chd(ffts%r_size-i+1))
                  elseif(imode.eq.2) then
                     write(18,180) x,aimag(chd(ffts%r_size-i+1))
                  elseif(imode.eq.3) then
                     write(18,180) x,abs(chd(ffts%r_size-i+1))*
     $                               abs(chd(ffts%r_size-i+1))
                  else
                     write(18,180) x,abs(chd(ffts%r_size-i+1))
                  endif
                  
               end do
            else
               open(unit=18,file= 'vis/' // outfilename // '.dx',
     $              position='append',
     $              status='unknown',form='formatted')
               if(imode.eq.1) then
                  write(18,80) (dble(chd(ffts%r_size:1:-1)))
               elseif(imode.eq.2) then
                  write(18,80) (aimag(chd(ffts%r_size:1:-1)))
               elseif(imode.eq.3) then
                  write(18,80) (abs(chd(ffts%r_size:1:-1))*
     $                          abs(chd(ffts%r_size:1:-1)))
               else
                  write(18,80) (abs(chd(ffts%r_size:1:-1)))
               endif
            endif
            call myflush(18)
            close(18)
         endif
         call my_broadcast(ic,iproc)
      end do

c     now the dx trailer

      if(.not.ikhoros) then
      if(gs%myproc.eq.0) then
         open(unit=18,file= 'vis/' // outfilename // '.dx', 
     $        position='append',
     $        status='unknown',form='formatted')

c        grid connections object

         write(18,20) gs%fftsize(3),gs%fftsize(2), gs%fftsize(1)
         
c        calculate deltas

         delta(1:3,1)=crys%avec(1:3,3)/gs%fftsize(3)
         delta(1:3,2)=crys%avec(1:3,2)/gs%fftsize(2)
         delta(1:3,3)=crys%avec(1:3,1)/gs%fftsize(1)

c     write header for gridpositions
         write(18,30) gs%fftsize(3), gs%fftsize(2), gs%fftsize(1)
c     compute and write origin
         write(18,40) 0.d0,0.d0,0.d0
c     write deltas
         write(18,50) ((delta(i,j),i=1,3), j=1,3)
c     write header for field object
         write(18,60) 
                  
         close(18)
      endif                     ! i am proc 0
      endif                     ! khoros

      if(ikhoros) then
         write(9,199) outfilename
      else
         write(9,99) outfilename
      endif

      deallocate(chd)

      return


c     ----------------- dx output -----------------------------

 5    format(/'# this is the object defining the data values')
 10   format('object 1 class array type float rank 0 items ',i8, 
     $     ' data follows')
 20   format(/'# this is the object defining the grid connections',
     $     /'object 2 class gridconnections counts ',3i4) 

 30   format(//'# this is the object defining the grid ',
     $     /'object 3 class gridpositions counts ',3i4 )

 40   format('origin', 3f20.10)
 50   format('delta ', 3f20.10)

 60   format(/'# this is the collective object, one for each grid',
     $     /'object 4 class field',
     $     /'component "positions"   value 3',
     $     /'component "connections" value 2',
     $     /'component "data"        value 1')


 80   format(5g15.7)
 99   format(//' GRAPHICAL OUTPUT:',
     $     /' ----------------',
     $     //' INPUT DATA FOR IBM DATA EXPLORER WRITTEN TO FILE',
     $     /'    - ',a7,'.dx')

c     ----------------- khoros output -----------------------------

 105  format('3',/'3',3(/i6),/'1')
 180  format(3f10.6,5x, g15.7)

 199  format(//' GRAPHICAL OUTPUT:',
     $     /' ----------------',
     $     //' INPUT DATA FOR AVS/KHOROS WRITTEN TO FILE',
     $     /'    - ',a7,'.avs')
      end




