c
      subroutine electric_field(pw_params,crys,den,vion,vin,gs,
     $     nlineplot, line_plot, bra,ket)

c     
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'
c
c     computes and plots electric field
c
c
c     INPUT:
c     -----

      integer nlineplot         ! number of line plots to do

      double precision
     $     bra(3),              ! for tensor data left vector
     $     ket(3),              ! for tensor data right vector
     $     line_plot(7,*)       ! line plot ((start,end,bins), lines)

      type (crystal) crys       ! for the lattice vectors
      type (parallel_gspace) 
     $     gs                   ! potential gspace
      type (pw_parameter) pw_params ! plane-wave parameters

      complex(kind=8)
     $     den(gs%length, crys%nspin), ! charge density
     $     vion(gs%length,crys%nspin), ! screened ionic potential
     $     vin(gs%length,crys%nspin) ! vxc+vhartree
c
c
c     1997 Bernd Pfrommer, UCB
c
c     computes the electric field due to the ionic and hartree
c     potential. The electric field is
c
c     E = grad V,           
c
c     where V is V_hartree + V_ion. There is no minus sign, because
c     the electronic potential is the negative of the electric
c     potential. 
c
c     The spin polarized version is programmed, but NOT DEBUGGED!
c
c
c     ---------------- local variables ----------------------------------
       
      integer i,iat,nt, idir,is
      
      double precision twopi,r(3),t0, gimmetime, fac,g,gr,gvc(3)
      complex(kind=8) td, gvfac(3)
      
      complex(kind=8), allocatable :: v(:), egs(:,:),efield(:,:,:)

      external gimmetime
c
c     ----- variables for the gspace loop ---------
c
      integer
     $     igv(4),fftn(4), ffth(4),igv3, irod,iord,igs
      double precision gv(3)    ! if required

c
c     ----------------------------------------------------------
c
      twopi = 8.d0*datan(1.d0)
      fac = 4.d0*twopi/crys%vcell

      if(crys%nspin.gt.1) then
         write(9,*) 'SPIN POLARIZED VERSION OF ELECTRIC_FIELD'
         write(9,*) 'IS NOT DEBUGGED!'
      endif


      t0=gimmetime()
c
c     first compute the bare ionic potential 
c
      allocate(v(gs%length))
      v(1:gs%length)= vion(1:gs%length,1)-vin(1:gs%length,1)

c     
c     now add the hartree potential
c

      if(crys%nspin.gt.1) then
         do i=1, gs%length
            if(gs%ekin(i).gt.0) then
               td = fac*(den(i,1)+den(i,2))/gs%ekin(i)
               v(i) = v(i) + td
            endif
         end do
      else
         do i=1, gs%length
            if(gs%ekin(i).gt.0) then
               v(i) = v(i) + fac*den(i,1)/gs%ekin(i)
            endif
         end do
      endif

c
c     take the derivative in gspace
c
c     E = - gradient (V_hartree + V_ion)
c
      allocate(egs(gs%length,3))

      do i=1,3
         call derivative_gspace(i,v(1),gs,crys,egs(1,i))
      end do



c     to get the values at the atoms, run the fourier sum 
c     explicitly rather than FFT
c
      allocate(efield(3,crys%mxdatm,crys%ntype))
      efield=0

      fftn(1:3)=gs%fftsize(1:3)
      fftn(4)=gs%fftsize(3)
      ffth(:)=fftn(:)/2

      do idir=1,3
         do nt=1,crys%ntype
            do iat=1,crys%natom(nt)
               r=crys%rat(:,iat,nt)
               igs = 0 
               do iord=1, gs%lorder ! loop through x/y gspace 
                  irod=gs%order(1,iord)
                  igv(1)= irod/gs%fftsize(2)
                  igv(2)= mod(irod,gs%fftsize(2))
                  igv(3)= gs%order(2,iord)
                  igv(4)= gs%order(3,iord)
                  igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
                  gv(1:2)=dble(igv(1:2))+gs%rk(1:2)
                  do igv3=igv(3),igv(4) ! loop over z axis
                     gv(3)=dble(igv3)+gs%rk(3)
                     igs = igs + 1
                     efield(idir,iat,nt) = efield(idir,iat,nt)+
     $                    egs(igs,idir)*exp(cmplx(0.d0, 
     $                    (r(1)*gv(1)+r(2)*gv(2)+r(3)*gv(3)),kind=8))
                  end do
               end do
            end do
         end do
      end do

      call all_sum_all(efield,3*crys%mxdatm*crys%ntype)

      write(9,100) 
      do nt=1,crys%ntype
         do iat=1,crys%natom(nt)
            write(9,1254) crys%nameat(nt),iat, 0.d0,
     $          -aimag(efield(:,iat,nt))
         end do
      end do

 1254 format(1x,a2,i3,2x,f9.3,3x,3f12.6)
      
      call lineplot(2, crys,-1.d0, egs(1,1), 
     $     gs,  nlineplot, line_plot, bra,ket,3,
     $     'E_FIELD')

      call myflush(9)

c     do an intergration of the MT sphere
c
      efield=0

      fftn(1:3)=gs%fftsize(1:3)
      fftn(4)=gs%fftsize(3)
      ffth(:)=fftn(:)/2

      igs = 0 
      do iord=1, gs%lorder      ! loop through x/y gspace 
         irod=gs%order(1,iord)
         igv(1)= irod/gs%fftsize(2)
         igv(2)= mod(irod,gs%fftsize(2))
         igv(3)= gs%order(2,iord)
         igv(4)= gs%order(3,iord)
         igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
         gv(1:2)=dble(igv(1:2))+gs%rk(1:2)
         do igv3=igv(3),igv(4)  ! loop over z axis
            igs = igs + 1
            g = sqrt(gs%ekin(igs))
            if(g.gt.0.d0) then
               gv(3)=dble(igv3)+gs%rk(3)
               gvc = matmul(crys%bvec,gv)
               do nt=1, crys%ntype
                  gr = g*crys%mtsphere(nt)
                  gvfac = gvc*v(igs)*(sin(gr)-gr*cos(gr))/gr**3.d0
                  do iat=1,crys%natom(nt)
                     r=crys%rat(:,iat,nt)
                     efield(:,iat,nt) = efield(:,iat,nt)+
     $                    gvfac*exp(cmplx(0.d0, 
     $                    (r(1)*gv(1)+r(2)*gv(2)+r(3)*gv(3))))
                  end do
               end do
            endif
         end do
      end do

      efield = cmplx(0.d0,3.d0)*efield
      call all_sum_all(efield,3*crys%mxdatm*crys%ntype)

      write(9,110) 
      do nt=1,crys%ntype
         do iat=1,crys%natom(nt)
            write(9,1254) crys%nameat(nt),iat, 
     $           crys%mtsphere(nt),dble(efield(:,iat,nt))
         end do
      end do

      write(9,120) 

      deallocate(v)
      deallocate(egs)
      deallocate(efield)

      if(iand(pw_params%output,8).eq.8) 
     $     write(9,940) gimmetime()-t0


      return
 100  format(/' -------- ELECTRIC FIELD ----------'/,
     $     1x,' Atom     Radius',9x,'E x',9x,'E y',9x,'E z')
 110  format(/' ---- AVERAGED ELECTRIC FIELD -----'/,
     $     1x,' Atom     Radius',9x,'E x',9x,'E y',9x,'E z')
 120  format(/' Electric field must be multiplied by 8.5763*10^6',
     $     ' to convert to statvolt/cm')
 940  format(/' TIME FOR ELECTRIC FIELD:',f12.3)

      end 




