c
      subroutine etotal(crys,
     $     iter, iconv, dvmaxmax,
     $     enew,eold, pw_params,
     $     gs, vion, vin, vout, den)
c
c     checks for convergence, and finds the total energy.
c     adapted from sverre froyen plane wave program
c     written june 8 1987.jlm
c
c     parallel fortran 90 version 1996 by Bernd Pfrommer, UCB
c
      use all_to_all_module
      include 'use.h'
      implicit none 
      include 'interface.h'
      include 'all_to_all.h'


c     INPUT:
c     ------

      type (crystal) crys       ! crystal parameters: vcell, ztot

      type (parallel_gspace) 
     $     gs                   ! the gspace for potential and density

      type (pw_parameter)
     $     pw_params
      integer
     $     iter                 ! iteration number


      complex(kind=8)
     $     den(gs%length,crys%nspin), ! symmetrized charge density in gspace
     $     vion(gs%length,crys%nspin), ! ionic potential, but screened with vin
     $     vin(gs%length,crys%nspin), ! inp  vhxc (with which wavefn are calc)
     $     vout(gs%length,crys%nspin) ! output vhxc (calc from new wavefn)



C     OUTPUT:
c     -------

      logical
     $     iconv                ! if true then convergence was achieved

      double precision
     $     dvmaxmax             ! maximum potential deviation of all procs

c     INPUT/OUTPUT:
c     -------------

      type (energy)
     $     enew,
     $     eold                 ! store old energies


c     -----------------------------------------------------------------------
c                            local variables

      double precision
     $     pi, zero, t0, gimmetime,
     $     dvmax,
     $     spindiff,            ! differences of spinup/spin down
     $     eps,                 ! maximum loss of charge acceptable
     $     epscmplx,            ! maximum complex energy to be tolerated
     $     diff                 ! deviation of n(G=0) from ztot

      external gimmetime

      type (energy)
     $     de                   ! energy differences

      complex(kind=8)
     $     dv, e2, e4, cdtot

      parameter (pi=3.141592653589793d0, zero = 0.0d0,
     $     eps      = 1.0d-6,
     $     epscmplx = 1.d-7)
      

      integer
     $     nnotconv,            ! number of unconverged potential components
     $     i,j,k, idvmax, is
      
      logical isp

c     ------------------------------------------------------
c
      t0=gimmetime()
      isp = .false.
      if(crys%nspin.eq.2) isp =.true.
c
c     ------------------ check for convergence -----------------------
c
      nnotconv =0
      dvmax =0.d0
      idvmax=-1

      do is=1, crys%nspin
         do i=1, gs%length
            dv = vout(i,is) - vin(i,is)
            if(abs(dv).gt.dvmax) then
               if(gs%ekin(i).gt.0.d0) then
                  dvmax = abs(dv)
                  idvmax  = i
               endif
            endif
            if ((abs(dble(dv)) .gt. pw_params%epscv) .or.
     $           (abs(aimag(dv)).gt.pw_params%epscv)) then
               if(gs%ekin(i).gt.0.d0) then
                  nnotconv = nnotconv +1
               endif
            endif
         end do
      end do

      call all_sum_all(nnotconv)
      if(nnotconv.gt.0) then 
         iconv=.false.
      else 
         iconv=.true.
      endif

      write(9,210) nnotconv
      dvmaxmax=dvmax
      call all_max_all(dvmaxmax)
      write(9,220) dvmaxmax
      if(idvmax.gt.0)    write(9,240) gs%ekin(idvmax)
      call myflush(9)
c
c     -------------------- initialize energies -------------------------
c
      enew%hxc = zero
      enew%ion = zero
      enew%hartree = zero
c
c     start loop over gspace
c
      e2 = cmplx(0.d0,0.d0,kind=8)
      e4 = cmplx(0.d0,0.d0,kind=8)
      spindiff=0.d0
      diff =0.d0

      if(isp) then
         do i=1, gs%length
c
c        compute the hxc correction from the potential of the
c        valence electrons. this will be subtracted from the sum
c        of the eigenvalues.
c        then compute the contribution to the total energy from
c        the interaction with the ions and the electrostatic
c        (hartree) interaction between the electrons.
c
            if(gs%ekin(i).gt.0.d0) then
               e2 = e2 + vin(i,1)*conjg(den(i,1))+
     $              vin(i,2)*conjg(den(i,2))
               e4 = e4 + (vion(i,1)-vin(i,1))*conjg(den(i,1))
     $              + (vion(i,2)-vin(i,2))*conjg(den(i,2))
               cdtot = den(i,1) + den(i,2)
               enew%hartree = enew%hartree + 
     $              dble(cdtot*conjg(cdtot))/gs%ekin(i)
            else
               diff = dble(den(i,1)+den(i,2)) - crys%ztot
c               write(9,*) den(i,1),den(i,2)
               spindiff = dble(den(i,1)-den(i,2))
            endif
         end do
      else
         do i=1, gs%length
            if(gs%ekin(i).gt.0.d0) then
               e2 = e2 + vin(i,1)*conjg(den(i,1))
               e4 = e4 + (vion(i,1)-vin(i,1))*conjg(den(i,1))
               enew%hartree = enew%hartree+
     $              dble(den(i,1)*conjg(den(i,1)))/gs%ekin(i)
            else
               diff = dble(den(i,1)) - crys%ztot
            endif
         end do
      endif
      call all_sum_all(diff)
      call all_sum_all(enew%hartree)
      call all_sum_all(e2)
      call all_sum_all(e4)
      call all_sum_all(spindiff) ! propagate spindiff to all processors

      if(aimag(e2).gt.epscmplx) then
         write(9,*) 'etotal: complex energy in e2 encountered:',e2
         stop
      endif
      if(aimag(e4).gt.epscmplx) then
         write(9,*)  'etotal: complex energy in e4 encountered:',e4
         stop
      endif

      enew%hxc = dble(e2)
      enew%ion = dble(e4)
      enew%hartree = 4.d0*pi*enew%hartree/crys%vcell
c
c     find the nonlocal energy.
c
      enew%nloc = enew%eband-enew%hxc-enew%ektot-enew%ion

c
c     the total energy.
c
c     the magnetic contribution is subtracted
c     the smearing energy (-TS term) is added
c
      enew%total=enew%eband-enew%hxc+enew%hartree
     $     + enew%xc+enew%alpha+enew%ewald
     $     + enew%esmear - enew%magnetic
      enew%totnonv=enew%total-0.5d0*enew%esmear

c
c     find differences from last iteration.
c
      de=enew-eold
      eold=enew
      if(iter.eq.1) de=0.d0
c
c     printout
c
      write(9,102) iter, enew%eband, de%eband,
     $     enew%hxc, de%hxc, enew%ektot, de%ektot, enew%ion, de%ion,
     $     enew%nloc, de%nloc, enew%hartree, de%hartree, 
     $     enew%xc, de%xc, enew%alpha, de%alpha, 
     $     enew%ewald, de%ewald,
     $     enew%esmear, de%esmear,  -enew%magnetic, -de%magnetic,
     $     enew%totnonv, de%totnonv,
     $     enew%total, de%total

      if (abs(diff) .gt. eps) call warn(290,diff,i,'etotal')
c
      if (isp) then
         write(9,200) spindiff
         write(9,205) enew%afmagmom
      endif

      if(iand(pw_params%output,8).eq.8) 
     $     write(9,940) gimmetime()-t0

      call myflush(9)

      return



 102  format(///,11X,'ITERATION NUMBER',I3,' :',/,11X,21('-'),/
     $     /,11X,'BAND ENERGY          =',2X,F17.10,2X,F12.8,
     $     /,11X,'HXC  CORRECTION      =',2X,F17.10,2X,F12.8,
     $     /,11X,55('-'),
     $     /,11X,'KINETIC  ENERGY      =',2X,F17.10,2X,F12.8,
     $     /,11X,'IONIC    ENERGY      =',2X,F17.10,2X,F12.8,
     $     /,11X,'NONLOCAL ENERGY      =',2X,F17.10,2X,F12.8,
     $     /,11X,'HARTREE  ENERGY      =',2X,F17.10,2X,F12.8,
     $     /,11X,'EXCHANGE CORRELATION =',2X,F17.10,2X,F12.8,
     $     /,11X,55('-'),
     $     /,11X,'ALPHA TERM           =',2X,F17.10,2X,F12.8,
     $     /,11X,'EWALD TERM           =',2X,F17.10,2X,F12.8,
     $     /,11X,'SMEARING CORRECTION  =',2X,F17.10,2X,F12.8,
     $     /,11X,'MAGNETIC CORRECTION  =',2X,F17.10,2X,F12.8,
     $     /,11X,55('-'),
     $     /,11X,'NON-VARIATIONAL ETOT =',2X,F17.10,2X,F12.8,
     $     /,11X,'TOTAL ENERGY         =',2X,F17.10,2X,F12.8)


 103  format(/,' TOTAL ENERGY =',F17.10)

 200  format(/,11x,'TOTAL MAGNETIC MOMENT [MU_BOHR/UCELL]=',f12.8)
 205  format(11x,'TOTAL ANTIFERROMAGNETIC MOMENT [MB/U]=',f12.8)

 210  format(/' NUMBER OF UNCONVERGED POTENTIAL COMPONENTS:',i10)
 220  format(' LARGEST POTENTIAL DIFFERENCE  (ALL PROCS): ',g12.6)
 230  format(' LARGEST POTENTIAL DIFFERENCE  (THIS PROC): ',g12.6)
 240  format(' WITH KINETIC ENERGY:', f12.6)
 940  format(' TIME FOR ETOTAL:',f12.3)
      END
