C*
C*
      subroutine ewald(ioutput,
     $     crys, gs,  eewald, ew)
c
c
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'



c     INPUT:
c     -----

      type (crystal) crys
      type (parallel_gspace) gs 


      integer 
     $     ioutput              ! print flag

c     OUTPUT:
c     ------

      double precision
     $     eewald               ! ewald energy (Ryd)

      type (force) ew           ! structure containing the forces

c
c     DESCRIPTION:
c     -----------
c
c     computes the ewald contribution to the
c     total energy, forces and stresses.
c     adapted from Sverre Froyen plane wave program
c     written december 16 1987. JLM
c
c     parallel version:          1996 Bernd Pfrommer, UC Berkeley
c
c     better scaling of the gspace loop 1997 Bernd Pfrommer
c

c     ---------------- local variables ----------------------------------
       
      double precision
     $     tol, t0,
     $     eps,                 ! corresponds to Kittels eta
     $     seps, sepi,
     $     rmax,
     $     zz2,                 ! sum_{ij}    zv(i)*zv(j)
     $     adot(3,3),           ! metric in real space
     $     pi, twopi, zero, one, two, three, four, small,
     $     factor, rmod, arg,
     $     esumg,  esumr, esub, enorm,
     $     ssumg(6),            ! stress sum in gspace
     $     ssumr(6),            ! stress sum in real space
     $     ssum0(6),            ! stress sum for the case of a=b
     $     tau(3),              ! difference vector
     $     esum0, exp1, exp2, exp3,
     $     emax,                ! energy cutoff
     $     ssub(6), fsub(3), rp(3), gdt, gimmetime

      external gimmetime
c
c     variables for the gspace loop:
c
      integer igs,iord,irod,igv(4),igv3,ffth(4),fftn(4)
      double precision gv(3)



c     WORK ARRAYS:
c     -----------
      double precision, allocatable ::
     $     zz(:),               ! size zz(crys%ntype*crys%mxdatm)
     $     rc(:,:),             ! size rc(3,crys%ntype*crys%mxdatm)
     $     fsumr(:,:), fsumg(:,:), ! size fsum(3,crys%ntype*crys%mxdatm)
     $     expg(:)              ! size expg(gs%length)


      complex(kind=8) sfac
      complex(kind=8), allocatable :: cfsumg(:,:)
      double precision cutfac

      parameter (pi = 3.141592653589793d0, twopi = 2.0d0*pi)
      parameter (zero = 0.0d0, one = 1.0d0 , two = 2.0d0)
      parameter (three = 3.0d0, four = 4.0d0)
      parameter (small = 1.0d-12)

      integer
     $     imx, jmx, kmx,
     $     natot,               ! total number of atoms, no matter which kind
     $     i, j, k, n, im, l, m, ii, jj, kk,
     $     ir(3), im2, jm2, km2

      double precision myderfc

      external myderfc


c     -------------------------------------------------------------------

      t0= gimmetime()

c     alloc mem


      allocate(expg(gs%length))
      allocate(zz(crys%ntype*crys%mxdatm))
      allocate(rc(3,crys%ntype*crys%mxdatm))
      allocate(fsumr(3,crys%ntype*crys%mxdatm))
      allocate(fsumg(3,crys%ntype*crys%mxdatm))

      allocate(ew%force(3,crys%mxdatm,crys%ntype))

c     
c     compute metric in real space
c
      factor    = (crys%vcell/(twopi*twopi))*(crys%vcell/(twopi*twopi
     $     )) 
      adot(1,1) = factor*(crys%bdot(2,2)*crys%bdot(3,3) - crys%bdot(2
     $     ,3)*crys%bdot(2,3)) 
      adot(2,2) = factor*(crys%bdot(3,3)*crys%bdot(1,1) - crys%bdot(3
     $     ,1)*crys%bdot(3,1)) 
      adot(3,3) = factor*(crys%bdot(1,1)*crys%bdot(2,2) - crys%bdot(1
     $     ,2)*crys%bdot(1,2)) 
      adot(1,2) = factor*(crys%bdot(1,3)*crys%bdot(3,2) - crys%bdot(1
     $     ,2)*crys%bdot(3,3)) 
      adot(1,3) = factor*(crys%bdot(1,2)*crys%bdot(2,3) - crys%bdot(1
     $     ,3)*crys%bdot(2,2)) 
      adot(2,3) = factor*(crys%bdot(2,1)*crys%bdot(1,3) - crys%bdot(2
     $     ,3)*crys%bdot(1,1)) 
      adot(2,1) = adot(1,2)
      adot(3,1) = adot(1,3)
      adot(3,2) = adot(2,3)

c
c     store data for atoms into new arrays
c
      natot = 0
      do i=1,crys%ntype
         do j=1,crys%natom(i)
            natot = natot + 1
            zz(natot) = crys%zv(i)
            rc(1,natot) = crys%rat(1,j,i)/twopi
            rc(2,natot) = crys%rat(2,j,i)/twopi
            rc(3,natot) = crys%rat(3,j,i)/twopi
            rc(1,natot) = rc(1,natot) - dble(int(rc(1,natot)))
            if(rc(1,natot) .lt. 0) rc(1,natot) = rc(1,natot) + 1.d0
            rc(2,natot) = rc(2,natot) - dble(int(rc(2,natot)))
            if(rc(2,natot) .lt. 0) rc(2,natot) = rc(2,natot) + 1.d0
            rc(3,natot) = rc(3,natot) - dble(int(rc(3,natot)))
            if(rc(3,natot) .lt. 0) rc(3,natot) = rc(3,natot) + 1.d0
         end do
      end do
c
      zz2 = zero
      do i=1,natot
         do j=1,natot
            zz2 = zz2 + zz(i)*zz(j)
         end do
      end do
c
c     compute various parameters
c
      tol = -log(small) 
      eps = 2.d0*gs%gmax/(four*tol)

      seps = sqrt(eps)
      sepi = two*seps/sqrt(pi)
      rmax = sqrt((tol+log(dble(natot)))/eps) ! increase precision with number of atoms
c
c     Changed the following to have +2 instead of +1
c     to avoid problems with large gspace cutoffs. The max()
c     expression provides a safeguard against the case where the
c     atoms are way outside of the unit cell. Then, the rspace sum
c     must be extended appropriately.
c
      imx = int(rmax*sqrt(crys%bdot(1,1))/twopi) + 2
      jmx = int(rmax*sqrt(crys%bdot(2,2))/twopi) + 2
      kmx = int(rmax*sqrt(crys%bdot(3,3))/twopi) + 2
c
c      initialize sums : esum(g,r)   -   energy
c                        fsum(g,r)   -   force
c                        ssum(g,r)   -   stress
c
c      note that the sums are in units of basis vectors
c      (g) in units of bi's, and (r) in units of ai's
c
      esumg = zero
      esumr = zero
      do i=1,3
         ssumg(i+3) = zero
         ssumr(i+3) = zero
         ssumg(i)   = zero
         ssumr(i)   = zero
         do j=1,natot
            fsumg(i,j) = zero
            fsumr(i,j) = zero
         end do
      end do

c     ---------------------------------------------------------------------
c                          DO SUM IN GSPACE
c
c     (this is done in parallel)

c
c     compute forces 
c
      allocate(cfsumg(3,natot))
      cfsumg = 0
      esumg = 0 

      fftn(1:3)=gs%fftsize(1:3)
      fftn(4)=gs%fftsize(3)
      ffth(:)=fftn(:)/2

      igs = 0 
      do iord=1, gs%lorder    ! loop through x/y gspace 
         irod=gs%order(1,iord)
         igv(1)= irod/gs%fftsize(2)
         igv(2)= mod(irod,gs%fftsize(2))
         igv(3)= gs%order(2,iord)
         igv(4)= gs%order(3,iord)
         igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
         gv(1:2)=dble(igv(1:2))
         do igv3=igv(3),igv(4)     ! loop over z axis
            gv(3)=dble(igv3)
            igs = igs + 1
            if(gs%ekin(igs).gt.0.0) then 
               cutfac = exp(-gs%ekin(igs)/(four*eps))/gs%ekin(igs) 
c
c              first precompute the structure factor
c
               sfac = 0
               do i=1, natot
                  gdt = twopi*(gv(1)*rc(1,i)+gv(2)*rc(2,i)+ 
     $                 gv(3)*rc(3,i))
                  sfac = sfac + zz(i)*cmplx(cos(gdt),-sin(gdt),kind=8)
               end do
c
c              contribution to the energy ...
c
               esumg =esumg +dble(sfac*conjg(sfac))*cutfac
c
c              .... to the stress ...
c
               exp3 = - (1.d0/(2.d0*eps) + 2.d0/gs%ekin(igs))
               exp3 = exp3 * dble(sfac*conjg(sfac))*cutfac

               ssumg(1) = ssumg(1) + exp3 * gv(1)*gv(1)
               ssumg(2) = ssumg(2) + exp3 * gv(2)*gv(2)
               ssumg(3) = ssumg(3) + exp3 * gv(3)*gv(3)
               ssumg(4) = ssumg(4) + exp3 * gv(1)*gv(2)
               ssumg(5) = ssumg(5) + exp3 * gv(2)*gv(3)
               ssumg(6) = ssumg(6) + exp3 * gv(3)*gv(1)
               
c
c              and to the forces.
c
               sfac = sfac*cutfac
               do i=1,natot
                  gdt = twopi*(gv(1)*rc(1,i)+gv(2)*rc(2,i)+ 
     $                 gv(3)*rc(3,i))
                  cfsumg(:,i) = cfsumg(:,i)+
     $                 sfac*cmplx(cos(gdt),sin(gdt),kind=8)*gv(:)
               end do

            endif
         end do
      end do

c
c     put in the prefactors etc
c

      do j=1,natot
         cfsumg(1:3,j) = 8.d0*pi*zz(j)*cfsumg(1:3,j)/crys%vcell
      end do
      esumg = 4.d0*pi*esumg/crys%vcell
      ssumg(:) = four*pi*ssumg(:)/crys%vcell

c
c     sum across the processors
c
      call all_sum_all(cfsumg, 3*natot)
      call all_sum_all(esumg)
      call all_sum_all(ssumg,6)

      fsumg(1:3,1:natot) = aimag(cfsumg(1:3,1:natot))

      deallocate(cfsumg)

c     
c     ------------------------------------------------------------
c     
C                          START SUM IN R SPACE


C     (no parallization done. all processors compute the same thing)

      im2 = 2*imx+1
      jm2 = 2*jmx+1
      km2 = 2*kmx+1
      esum0 = zero
      ssum0 = 0

      do i=1,im2
         ir(1) = i-imx-1
         do j=1,jm2
            ir(2) = j-jmx-1
            do k=1,km2
               ir(3) = k-kmx-1
               rmod = zero
               do l=1,3
                  do m=1,3
                     rmod = rmod + dble(ir(l))*adot(l,m)*dble(ir(m))
                  end do
               end do
               if (rmod .ne. zero) then
                  rmod = sqrt(rmod)
                  arg = seps*rmod
                  if (arg .lt. 25.0) then
                     exp1 = myderfc(arg) / rmod
                     exp2 = (exp1 + sepi*exp(-arg*arg))/(rmod*rmod)
                     esum0 = esum0 + exp1
                     ssum0(1) = ssum0(1) + exp2 * dble(ir(1)*ir(1))
                     ssum0(2) = ssum0(2) + exp2 * dble(ir(2)*ir(2))
                     ssum0(3) = ssum0(3) + exp2 * dble(ir(3)*ir(3))
                     ssum0(4) = ssum0(4) + exp2 * dble(ir(1)*ir(2))
                     ssum0(5) = ssum0(5) + exp2 * dble(ir(2)*ir(3))
                     ssum0(6) = ssum0(6) + exp2 * dble(ir(3)*ir(1))
                  endif
               endif
            end do
         end do
      end do

      esum0 = esum0 - pi/(eps*crys%vcell) - sepi

c
c      start loop over atoms in cell
c
      do 50 i=1,natot
c
c        term with a=b
c
         esumr = esumr + zz(i)*zz(i)*esum0
         do j=1,6
            ssumr(j) = ssumr(j) + zz(i)*zz(i) * ssum0(j)
         end do
         im = i-1
         if (im .ne. 0) then
c
c           terms with a#b
c
            do j=1,im           ! loop over other atoms in unit cell
               esub = zero
               do k=1,3
                  fsub(k) = zero
                  ssub(k+3) = zero
                  ssub(k) = zero
               end do
               do ii=1,im2      ! ---------  loop over lattice points
                  ir(1) = ii-imx-1
                  do jj=1,jm2
                     ir(2) = jj-jmx-1
                     do kk=1,km2
                        ir(3) = kk-kmx-1
                        rp(1) = dble(ir(1)) + rc(1,i) - rc(1,j)
                        rp(2) = dble(ir(2)) + rc(2,i) - rc(2,j)
                        rp(3) = dble(ir(3)) + rc(3,i) - rc(3,j)
                        rmod = zero
                        do l=1,3
                           do m=1,3
                              rmod = rmod + rp(l)*adot(l,m)*rp(m)
                           end do
                        end do
                        rmod = sqrt(rmod)
                        arg = seps*rmod
                        if (arg .lt. 25.d0) then
                           exp1 = myderfc(arg) / rmod
                           exp2 = (exp1 + sepi*exp(-arg*arg))
     $                          /(rmod*rmod)
                           esub = esub + exp1
                           fsub(1) = fsub(1) + rp(1) * exp2
                           fsub(2) = fsub(2) + rp(2) * exp2
                           fsub(3) = fsub(3) + rp(3) * exp2
                           ssub(1) = ssub(1) + rp(1) * exp2 * rp(1)
                           ssub(2) = ssub(2) + rp(2) * exp2 * rp(2)
                           ssub(3) = ssub(3) + rp(3) * exp2 * rp(3)
                           ssub(4) = ssub(4) + rp(1) * exp2 * rp(2)
                           ssub(5) = ssub(5) + rp(2) * exp2 * rp(3)
                           ssub(6) = ssub(6) + rp(3) * exp2 * rp(1)
                        endif
                     end do
                  end do
               end do
               esub = esub - pi/(eps*crys%vcell)
               esumr = esumr + two*zz(i)*zz(j)*esub
               do k=1,6
                  ssumr(k) = ssumr(k) + two*zz(i)*zz(j)*ssub(k)
               end do

               do k=1,3
                  fsumr(k,i) = fsumr(k,i) + two*zz(i)*zz(j)*fsub(k)
                  fsumr(k,j) = fsumr(k,j) - two*zz(i)*zz(j)*fsub(k)
               end do
            end do              ! end of loop over other atoms in unit cell
         endif
 50   continue
c
c      end r sum
c
      eewald = esumg + esumr

c
c      force
c      note - returned force is in units of real space lattice vectors
c             printed force is in cartesian coordinates
C
      n = 0
      do i=1,crys%ntype
         do j=1,crys%natom(i)
            n = n + 1
            do k=1,3
               ew%force(k,j,i) = fsumr(k,n)
               do l=1,3
                  ew%force(k,j,i)=
     $                 ew%force(k,j,i)+crys%bdot(k,l)*fsumg(l,n)
     $                 /twopi 
               end do
            end do
         end do
      end do
c
c     stress
c      note - both returned and printed stress are
c             in cartesian coordinates
c
      do i=1,6
         j = i
         k = i
         if (i .gt. 3) j = i - 3
         if (i .gt. 3) k = j + 1
         if (k .gt. 3) k = 1
         ew%stress(i) = 
     $        crys%bvec(j,1)*ssumg(1)*crys%bvec(k,1) + crys%avec(j,1)
     $        *ssumr(1)*crys%avec(k,1)+ crys%bvec(j,2)*ssumg(2)
     $        *crys%bvec(k,2) + crys%avec(j,2)*ssumr(2)*crys%avec(k,2
     $        )+ crys%bvec(j,3)*ssumg(3)*crys%bvec(k,3) + crys%avec(j
     $        ,3)*ssumr(3)*crys%avec(k,3)+ crys%bvec(j,1)*ssumg(4)
     $        *crys%bvec(k,2) + crys%avec(j,1)*ssumr(4)*crys%avec(k,2
     $        )+ crys%bvec(j,2)*ssumg(5)*crys%bvec(k,3) + crys%avec(j
     $        ,2)*ssumr(5)*crys%avec(k,3)+ crys%bvec(j,3)*ssumg(6)
     $        *crys%bvec(k,1) + crys%avec(j,3)*ssumr(6)*crys%avec(k,1
     $        )+ crys%bvec(k,1)*ssumg(4)*crys%bvec(j,2) + crys%avec(k
     $        ,1)*ssumr(4)*crys%avec(j,2)+ crys%bvec(k,2)*ssumg(5)
     $        *crys%bvec(j,3) + crys%avec(k,2)*ssumr(5)*crys%avec(j,3
     $        )+ crys%bvec(k,3)*ssumg(6)*crys%bvec(j,1) + crys%avec(k
     $        ,3)*ssumr(6)*crys%avec(j,1)
         if (i .le. 3) then
            ew%stress(i) = ew%stress(i) + esumg - zz2*pi/(eps
     $           *crys%vcell) 
         endif
      end do
c
c      printout
c
      enorm = eewald*crys%vcell**(one/three)
      write(9,100) enorm

      write(9,101) eewald,enorm
      n = 0
      do i=1,crys%ntype
         do j=1,crys%natom(i)
            n = n + 1
            do k=1,3
               tau(k) = crys%avec(k,1)*ew%force(1,j,i)
     $              + crys%avec(k,2)*ew%force(2,j,i)
     $              + crys%avec(k,3)*ew%force(3,j,i)
            end do
            write(9,102) n,(crys%rat(k,j,i)/twopi,k=1,3),(tau(k),k
     $           =1,3) 
         end do
      end do
      write(9,103) (ew%stress(i),i=1,6)

      deallocate(expg)
      deallocate(zz)
      deallocate(rc)
      deallocate(fsumr)
      deallocate(fsumg)

      if(iand(ioutput,8).eq.8) 
     $     write(9,940) gimmetime()-t0

      call myflush(9)

      

      return



 100   FORMAT(/,' NORMALIZED EWALD ENERGY =',F16.10)
 101   FORMAT(/,' EWALD ANALYSIS',/,1X,14('*'),/,50X,'1/3',/,
     1 ' ENERGY :',12X,'ENERGY (RY)',16X,
     2 '*V    (RY*A.U.)',/,16X,F16.10,14X,F16.9,//
     3 ' FORCES(CARTESIAN) :',3X,'N',11X,'COORD',19X,'FORCE (RY/A.U.)',/
     4 19X,'A1',4X,'A2',4X,'A3',11X,'-X-',7X,'-Y-',7X,'-Z-')
 102   FORMAT(10X,I3,3X,3F6.3,5X,3F10.6)
 103   FORMAT(/,' STRESS(CARTESIAN) :',25X,'SIGMA * V (RY)',/,
     1 14X,'-XX-',6X,'-YY-',6X,'-ZZ-',6X,'-XY-',6X,'-YZ-',6X,'-ZX-',/,
     2 9X,6F10.6)
 940  format(' TIME FOR EWALD CALCULATION:',f12.3)

       end
