c
      subroutine excorr(ipr,icorr, ispn,
     $     rsize,  norm,   chd, vcell,exc, afmagm)

      use all_to_all_module

      implicit none             ! implicit? Just say no!
      include 'all_to_all.h'


c
c
c     -----------------------------------------------------------------------
c
c     INPUT:
c     ------
      integer 
     $     ipr,
     $     ispn,                ! number of spin components
     $     norm,                ! normalization: nfft(1)*nfft(2)*nfft(3)
     $     rsize                ! declared dimension of the array chd

      character*2 icorr         ! determines the type of correlation potential
c                                 that is used.
c                                 xa  x-alpha method. 
c                                     x-alpha must be set in the subroutine.
c                                 wi  wigners interpolation formula is used.
c                                 hl  hedin-lundqvist.
c                                 ca  ceperly-alder.

      double precision vcell    ! cell volume

c
c     INPUT/OUTPUT:

      complex(kind=8) 
     $     chd(rsize, ispn) ! @  charge density on input,
c                                      in electrons/unit cell, 
c                                  @  v_xc on output, in rydberg
c     OUTPUT:
c
      double precision 
     $     afmagm,            ! antiferromagnetic moment
     $     exc
c
c     DESCRIPTION:
c     -----------
c      
C
c     computes the exchange correlation potential
c     (in rydberg) given the charge density den (in electrons
c     per unit cell). 
c
c     the potential replaces the density in the array den.
c     adapted from sverre froyen plane wave program
c     written june 11 1987. jlm
c
c     exc        the total exchange correlation energy given by the
c                integral of the density times epsilon xc (rydberg).
c
c     parallel version, explicit typing, cmplx data type,  1995
c
c
c     --------------------- local variables -------------------------

       double precision 
     $      pi, zero, one, two, three, four, third, ftrd,
     $      dmax, dmin, cmax, abschd, a0, alp, ro, ro1, ro2, eps, 
     $      rs, a,b,c, x, aln, ec, alrs, xdum, sqrs, 
     $      cmin, fa, zeta, fzeta, dfdz, 
     $      ecu, ecp, ect, vc, vcp, vcu, vcd,
     $      g,  b1, b2, aca, bca, cca,  dca,
     $      gp, b1p,b2p,acap,bcap,ccap, dcap,
     $      ax, rs_kf, tf_kin(2), lda_exchange(2), gga_exchange(2),
     $      lda_correlation, gga_correlation

       logical isp              ! spin polarization


       integer 
     $      i,j,k, ierr, idum, is
       

       parameter (pi=3.141592653589793d0)
       parameter (zero = 0.0d0, one = 1.0d0)
       parameter (two = 2.0d0, three = 3.0d0, four = 4.0d0, 
     $      third =1.d0/3.d0, ftrd = 4.d0/3.d0)

       parameter (eps = 1.0d-9) ! maximum cmplx cd to tolerate

c     -------------------------------------------------------------------



      isp = .false.
      if(ispn.eq.2) isp =.true.

c
c
c     find max and min value for real space charge density, check for complex
c
      do is=1, ispn
         call check_array(chd(1,is), 
     $        rsize, dmax, dmin, 
     $        cmax, cmin, eps, ierr)
         
         if (ierr .ne. 0) then
            write(9,200) ierr, cmax
         endif
         if (dmin .lt. zero) then
            if(ipr.ge.1)
     $           call warn(272,dmin,idum,'excorr')
         endif
         
         if (ipr .ne. 0) then
            if(isp) then
               write(9,304) is, dmax,dmin
               write(9,106) cmax,cmin
            else
               write(9,104) dmax,dmin
               write(9,106) cmax,cmin
            endif
         endif
      end do

      exc = zero

      if(rsize .le. 0) goto 999
c
c     compute the antiferromagnetic moment. Core charge is assumed to be
c     not polarized, so it need not be subtracted.
c

      afmagm=0.d0

      if(isp) then              ! calculate antiferromagnetic moment
         do i=1,rsize
            afmagm=afmagm+abs(chd(i,1)-chd(i,2))
         end do
      endif
      
      call all_sum_all(afmagm)

      if(norm.gt.0) then
         afmagm=afmagm/dble(norm)          ! normalize
      else
         write(9,*) 'excorr: norm=0. Something is wrong.'
         stop
      endif

      a0 = (four/(three*three*pi))**(third)
      fa = 2*(6.d0/pi)**(1.d0/3.d0)
      ect =0

      if(
     $     (icorr.eq.'wi') .or.
     $     (icorr.eq.'xa') .or.
     $     (icorr.eq.'hl') ) then
         if(isp) then
            write(9,*) 'excorr: No spin polarization for icorr=',icorr
            write(9,*) '        Implement it if you need it!'
            stop
         else
            write(9,*) 'excorr: Correlation type icorr=',icorr
            write(9,*) '        is implemented, but not debugged!'
            stop
         endif
      endif


      if (icorr .eq. 'xa' .or. icorr .eq. 'xa') then
c        x-alpha correlation
         alp = one
         write(9,100) alp
         do i=1, rsize
            ro = 0.5d0*dble(chd(i,1)+
     $           conjg(chd(i,1)))/vcell
            chd(i,1) = cmplx(0.d0,0.d0,kind=8)
            if (ro .gt. zero) then
               rs = (three/(four*pi*ro))**(third)
               chd(i,1) = cmplx(-three*alp/(pi*a0*rs), zero,kind=8)
               exc = exc + three*ro*dble(chd(i,1))/four
            endif
         end do
      else if (icorr .eq. 'wi' .or. icorr .eq. 'wi') then
c        wigner correlation
         a = -0.875529
         b = 7.8
         do i=1,rsize
            ro = 0.5d0*dble(chd(i,1)+
     $           conjg(chd(i,1)))/vcell
            chd(i,1) = cmplx(0.d0,0.d0,kind=8)
            if (ro .gt. zero) then
               rs = (three/(four*pi*ro))**(third)
               chd(i,1) = cmplx(-two/(pi*a0*rs),0.d0,kind=8)
               exc = exc + three*ro*dble(chd(i,1))/four
               chd(i,1) = chd(i,1) +   cmplx(a*((4.0*rs)/3.0+b)
     $              /((rs+b)*(rs+b)),0.d0,kind=8)
               exc = exc + ro*a/(rs+b)
            endif
         end do

      else if (icorr .eq. 'hl' .or. icorr .eq. 'hl') then
c        hedin-lundqvist
         a = 21.
         c = 0.045
         do i=1,rsize
            ro = 0.5d0*dble(chd(i,1)+
     $           conjg(chd(i,1)))/vcell
            chd(i,1) = cmplx(0.d0,0.d0,kind=8)
            if (ro .gt. zero) then
               rs = (three/(four*pi*ro))**(third)
               chd(i,1) = cmplx(-two/(pi*a0*rs), 0.d0, kind=8)
               exc = exc + three*ro*dble(chd(i,1))/four
               x = rs/a
               aln = log(one+one/x)
               chd(i,1) = chd(i,1) -cmplx(c*aln, 0.d0, kind=8)
               exc = exc - 
     $              ro*c*((one+x*x*x)*aln+x/two-x*x-third)
            endif
         end do
      else if (icorr .eq. 'ca') then
c        ceperly alder.
         g = -0.2846d0          ! unpolarized constants
         b1 = 1.0529d0
         b2 = 0.3334d0
         aca = 0.0311d0 * 2
         bca =-0.048d0 * 2
         cca= 0.0020d0 * 2
         dca=-0.0116d0 * 2

         gp = -0.0843d0 * 2     ! polarized constants
         b1p = 1.3981d0
         b2p = 0.2611d0
         acap = 0.01555d0 * 2
         bcap = -0.0269d0 * 2
         ccap = 0.0007d0 * 2
         dcap = -0.0048d0 * 2
         

         if(.not.isp) then
            do i=1,rsize
               ro = dble(chd(i,1))/vcell
               if (ro .gt. zero) then
                  rs = (three/(four*pi*ro))**(third)
c     exchange potential
                  chd(i,1) = cmplx( -two/(pi*a0*rs), 0.d0, kind=8)
                  exc = exc + three*ro*dble(chd(i,1))/four
                  if (rs .ge. one) then
                     sqrs = sqrt(rs)
                     ec = g/(one+b1*sqrs+b2*rs)
                     chd(i,1) = chd(i,1) + 
     $                    cmplx(ec*ec*(1.0+7.0*b1*sqrs/6.0+
     $                    4.0*b2*rs/3.0)/g, 0.d0, kind=8)
                     ect = ect + ro*ec
                     exc = exc + ro*ec
                  else
                     alrs = log(rs)
                     ec=0.0622*alrs-0.096+(0.0040*alrs-0.0232)*rs
                     chd(i,1) = chd(i,1) + 
     $                    cmplx(ec-(0.0622+(0.0040*alrs-
     $                    0.0192)*rs)/3.0, 0.d0, kind=8)
                     ect = ect + ro*ec
                     exc = exc + ro*ec
                  endif
               else
                  chd(i,1) = cmplx(0.d0,0.d0,kind=8)
               endif
            end do
         else                   ! do spin polarized version
            do i=1, rsize
               ro1  = dble(chd(i,1))/vcell
               ro2  = dble(chd(i,2))/vcell
               ro   = ro1 +ro2
               
               if((ro.gt.0).and.(ro1.ge.0).and.(ro2.ge.0)) then

                  zeta = (ro1-ro2)/ro ! spin polarization
                  fzeta = ( (1+zeta)**(ftrd) + 
     $                 (1.d0-zeta)**(ftrd)-2.d0)/
     $                 ( 2**(ftrd) - 2.d0 )
                  dfdz = (ftrd) * (
     $                 (1+zeta)**(third)-(1-zeta)**(third)
     $                 )/( 2**(ftrd) - 2.d0 )
                  
c                 exchange potentials
                  chd(i,1)=cmplx(-fa*ro1**third,0.d0,kind=8)
                  chd(i,2)=cmplx(-fa*ro2**third,0.d0,kind=8)
                  rs = (three/(four*pi*ro))**(third)

c                 exchange energy
                  exc = exc + three/four*
     $                 (chd(i,1)*ro1 + chd(i,2)*ro2)

c                  correlation potential and energy. 

                  if (rs .ge. one) then
                     sqrs = sqrt(rs)
                     ecu= g/(1.d0+b1*sqrs+b2*rs) ! unpolarized
                     vc = ecu*ecu*(1.d0+7.d0*b1*sqrs/6.d0+
     $                    4.d0*b2*rs/3.d0)/g

                     ecp = gp/(1.d0+b1p*sqrs+b2p*rs) ! polarized
                     vcp = ecp*ecp*(
     $                    1.d0+7.d0*b1p*sqrs/6.d0+
     $                    4.d0*b2p*rs/3.d0)/gp
                  else
                     alrs = log(rs)
                     ecu= aca*alrs + bca + (cca*alrs+dca)*rs
                     vc = ecu- (aca+(cca*alrs+cca+dca)*rs)/3.d0
                     ecp = acap*alrs + bcap + 
     $                    (ccap*alrs+dcap)*rs
                     vcp = ecp - (acap+(ccap*alrs+
     $                    ccap+dcap)*rs)/3.d0
                  endif

c                 now take spin into acoount for correlation
                        
                  vcu = vc +fzeta*(vcp-vc) + 
     $                 (ecp-ecu)*(1-zeta)*dfdz
                  vcd = vc +fzeta*(vcp-vc) + 
     $                 (ecp-ecu)*(-1-zeta)*dfdz
                  ec = ecu + fzeta*(ecp - ecu)

                  chd(i,1) = chd(i,1) + vcu
                  chd(i,2) = chd(i,2) + vcd
                  exc = exc + ro*ec
                  ect = ect + ro*ec ! total correlation energy
               else
                  chd(i,1) = cmplx(0.d0,0.d0,kind=8)
                  chd(i,2) = cmplx(0.d0,0.d0,kind=8)
               endif
            end do
         endif
      else
         call warn(280,xdum,idum,'excorr')
      endif

 999  continue

      exc  = exc  * vcell / dble(norm)
      ect  = ect  * vcell / dble(norm)

      call all_sum_all(exc)


      return

 100  FORMAT(/,' X-ALPHA CORRELATION, X-ALPHA =',F5.3,/)
 104  FORMAT(/,'  MAX AND MIN OF CHARGE DENSITY ',2F10.4)
 106  FORMAT('  MAX AND MIN COMPLEX COMPONENT   ',2g10.2)
 200  FORMAT(' *** WARNING: EXCORR FINDS ',i6,
     $     ' POINTS WITH COMPLEX CHARGE, MAX=',g14.4)
 304  FORMAT(/,'  MAX AND MIN OF CHARGE DENSITY SPIN (',i1,') '
     $     ,2F10.4)

      END
