c
c
      subroutine expand(gsub, gfull, input, output,  unfolded)
c     
c     1996 Bernd Pfrommer while at  UC Berkeley
c     

      include 'use.h'
      implicit none             !
      include 'interface.h'

c     INPUT:
c     -----

      type(parallel_gspace)
     $     gsub,                ! sub gspace
     $     gfull                ! full gspace

      complex(kind=8)
     $     input(gsub%length)   ! input data, folded 



c     OUTPUT:
c     ------
      complex(kind=8)
     $     output(gfull%length) ! the output data, folded



c     WORK:
c     -----

      complex(kind=8)
     $     unfolded(gsub%fftsize(3), gsub%fftsize(2), gsub%fftsize(1))
      
c
c     DESCRIPTION:
c     -----------
c
c     Takes data on a small, replicated gspace, and sets it on a 
c     large, possibly distributed gspace.
c
c     To avoid any searches, this is done as follows:
c
c      1) Set the data from the small gspace on a fourier grid
c      2) Pick up the data from the fourier grid into the large gspace
c
c
c     As a consequence, the expansion scales only linearly with the
c     size of the gspace.
c
c

c     ====================================================================
c
c     local variables:


      integer 
     $     ii, istart,iend, iid, nz, i, ix, iy, 
     $     fftmargin(3,2), ift(3), ! pointer into the fourier grid
     $     ifg(3)               
c
c
c
c     ----- variables for the gspace loop ---------
c
      integer
     $     igv(4),fftn(4), ffth(4),igv3, irod,iord,igs


      if(gsub%nproc.gt.1) then
         write(9,*)'expand: works only for local subspaces'
         stop
      endif



      unfolded=0        ! clear unfolded array

c
c     first put data from small space to fourier grid
c
      nz=gsub%fftsize(3)

      ii=1
      do iord=1,gsub%lorder
         irod=gsub%order(1,iord)
         ix=irod/gsub%fftsize(2)+1
         iy=mod(irod,gsub%fftsize(2))+1
         istart = gsub%order(2,iord)
         iend   = gsub%order(3,iord)
         if(istart.le.iend) then ! there is only one contiguous block to copy:
            iid    = ii + mod(iend-istart+nz,nz)
            unfolded(istart+1:iend+1,iy,ix)=input(ii:iid)
         else                   ! there are two blocks to copy
            iid    = ii + nz-istart -1
            unfolded(istart+1:nz,iy,ix)=input(ii:iid)
            ii     = iid+1
            iid    = ii + iend
            unfolded(1:iend+1,iy,ix) = input(ii:iid)
         endif
c         write(9,*) iord, unfolded(:,iy,ix)

         ii = iid+1             ! new start  =    old end +1

      end do

c
c     now pick up the stuff for the large gspace if
c
c            -(fftsub-1)/2 < index < (fftsub+1)/2

      fftn(1:3)=gfull%fftsize(1:3)
      fftn(4)=gfull%fftsize(3)
      ffth(:)=fftn(:)/2

      fftmargin(:,1)= -(gsub%fftsize(1:3)-1)/2
      fftmargin(:,2)=  (gsub%fftsize(1:3)-1)/2

c      write(9,*)'sub fftsize:',gsub%fftsize(1:3)
c      write(9,*)'fftmargin:',fftmargin
      igs = 0 
      do iord=1, gfull%lorder    ! loop through x/y gspace 
         irod=gfull%order(1,iord)
         igv(1)= irod/gfull%fftsize(2)
         igv(2)= mod(irod,gfull%fftsize(2))
         igv(3)= gfull%order(2,iord)
         igv(4)= gfull%order(3,iord)
         igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
         do igv3=igv(3),igv(4)     ! loop over z axis
            igs = igs + 1
            if(                 ! within range of sub gspace
     $           ((fftmargin(1,1).le.igv(1)).and.(igv(1).le
     $           .fftmargin(1,2))).and.
     $           ((fftmargin(2,1).le.igv(2)).and.(igv(2).le
     $           .fftmargin(2,2))).and.
     $           ((fftmargin(3,1).le.igv3).and.(igv3.le
     $           .fftmargin(3,2)))
     $           ) then
               ift(1:2)=mod(igv(1:2)+gsub%fftsize(1:2),
     $              gsub%fftsize(1:2))
               ift(3)=mod(igv3+gsub%fftsize(3),gsub%fftsize(3))

               output(igs)=unfolded(ift(3)+1,ift(2)+1,ift(1)+1)
c               write(9,*) igs,ift(:), igv(1:2),igv3,output(igs)
            else
               output(igs)=(0.d0,0.d0)
            endif
            
         end do
      end do

      return
      
      end
