C
C========================================================================
C
      double precision function scp(v1,v2)
      
cp    scalar product without metric
      double precision v1(3),v2(3)
      scp=v1(1)*v2(1)+v1(2)*v2(2)+v1(3)*v2(3)
      return
      end
C
C========================================================================
C
      subroutine findbound(qmax,b,maxi,maxj,maxk)
cp    
cp    adapted from subroutine ban 3.2.93, Bernd Pfrommer 
cp
cp    ! bi,bj,bk should form a right-handed system !
cp
cp    subroutine findbound finds boundaries of the space built up by
cp    the vectors bi,bj,bk if a sphere of
cp    radius qmax should be placed inside that space. In maxi, maxj, maxk
cp    integers values are returned representing the coordinates along bi,bj,bk
cp    that are necessary to involve the qmax-sphere entirely.
cp
c     -------------------- documentation of variables --------------------

cp    qmax      radius of involved sphere
cp    b(i,k)    ith component of recip. lattice vector k.
cp    maxi,maxj,maxk integer variables used to return required minimum
cp              boundaries.
cp    volp      volume produkt bi*(bj x bk)
cp    proi,proj,prok  see documentation in source
cp    bj_times_bk, bk_times_bj, bi_times_bk   absolut value of cross product
cp
      implicit none
      double precision qmax     ! radius of involved sphere 
      integer maxi,maxj,maxk    ! return values: boundaries
      external scp              ! standard scalar product
      double precision scp
      double precision b(3,3)   ! reciprocal lattice vectors
      
cp    ------- local variables -------------------

      double precision bi(3),bj(3),bk(3) ! rec lattice vectors (or any vector)

      integer i
      double precision volp     ! volume product 
      double precision proi,proj,prok ! see below
      double precision bj_times_bk, bk_times_bi, bi_times_bj 
c      abs. value of cross prod.
      

      do i=1,3
         bi(i) = b(i,1)
         bj(i) = b(i,2)
         bk(i) = b(i,3)
      end do

cp    volume product bi*(bj x bk)

      volp = bi(1)*(bj(2)*bk(3)-bj(3)*bk(2)) +
     $       bi(2)*(bj(3)*bk(1)-bj(1)*bk(3)) +
     $       bi(3)*(bj(1)*bk(2)-bj(2)*bk(1))

cp    project bi on normal vector to the plane (bk,bj) to determine 
cp    the minimum component a vector has to have to be surely outside
cp    the sphere regardless of the component along bk and bj.

cp    proi=bi*(bj x bk)/|bj x bk|

      bj_times_bk=sqrt(scp(bj,bj)*scp(bk,bk)-scp(bj,bk)**2)
      if(bj_times_bk.le.1d-6) then
         write(9,*) 'error in findbound: bj,bk parallel'
         stop
      else
         proi= abs(volp/bj_times_bk)
         maxi= int(qmax/proi)
      endif

cp    project bj on normal vector to the plane (bk,bi) to determine 
cp    the minimum component a vector has to have to be surely outside
cp    the sphere regardless of the component along bk and bi.

cp    proj=bj*(bk x bi)/|bk x bi|

      bk_times_bi=sqrt(scp(bk,bk)*scp(bi,bi)-scp(bk,bi)**2)
      if(bk_times_bi.le.1e-6) then
         write(9,*) 'error in findbound: bk,bi parallel'
         stop
      else
         proj= abs(volp/bk_times_bi)
         maxj= int(qmax/proj)
      endif

cp    project bk on normal vector to the plane (bi,bj) to determine 
cp    the minimum component a vector has to have to be surely outside
cp    the sphere regardless of the component along bi and bj.

cp    prok=bk*(bi x bj)/|bi x bj|

      bi_times_bj=sqrt(scp(bi,bi)*scp(bj,bj)-scp(bi,bj)**2)
      if(bi_times_bj.le.1e-6) then
         write(9,*) 'error in findbound: bi,bj parallel'
         stop
      else
         prok= abs(volp/bi_times_bj)
         maxk= int(qmax/prok)   
      endif

      end


