c/*
c**                 determines the solution of 
c**
c**                 a*x^3 + b*x^2 + c*x +d = 0
c**
c**       which has   3*a*x^2 + 2*b*x + c >0
c**
c**       and is the minimum of 
c**        (1/4) a*x^4 + (1/3) b*x^3 +(1/2) c*x^2 + d*x
c**
c*/


      double precision function myasinh(x)
      implicit none
      double precision x
      myasinh=log(x+sqrt(x*x+1.d0))
      return
      end

      double precision function myacosh(x)
      implicit none
      double precision x
      myacosh=log(x+sqrt(x*x-1.d0))
      return
      end


      subroutine findroot(ret,ainp,binp,cinp,dinp)

      implicit none

      double precision 
     $     ret,                 ! the return value
     $     ainp,                ! a
     $     binp,                ! b
     $     cinp,                ! c
     $     dinp

c
c     ------------- local variables ---------------------------
c
      double precision 
     $     find_root_eps,  myacosh, myasinh,
     $     pi3rd, asq, acubed, bsq, bcubed, p,q,r, phi3rd,a,b,c,d,
     $     x, x1,x2,x3, x1p, x2p, x3p, x1f, x2f, x3f;
      integer imode
      
      external myasinh, myacosh;
      parameter(pi3rd = 3.141592653589793/3.d0, 
     $     find_root_eps=1d-40, imode=0)
c
c     --------------------------------------------------------
c

c      write(9,*) 'a=',ainp
c      write(9,*) 'b=',binp
c      write(9,*) 'c=',cinp
c      write(9,*) 'd=',dinp

      a=ainp; b=binp; c=cinp; d=dinp;
      asq = a*a; acubed = asq*a; bsq = b*b; bcubed = bsq*b;


      if(abs(a).le.FIND_ROOT_EPS) then ! it is a polynomial of second order 
    
c     this situation should rarely occur, and could be due to a 
c     failure of the conjugate gradient method. In case anything
c     suspicious is detected, findroot returns a negative value to
c     initiate  a reset of the conjugate gradient algortihm
    

         p = c*c-4.d0*b*d;
         if(p.lt.0) then
            write(9,*)   
     $           'findroot: 2nd order polynomial has no real roots.'
            write(9,*) '      resetting algorithm'
            ret=-0.001
            return
         endif
         if(abs(b).lt.FIND_ROOT_EPS) then
            write(9,*) 'findroot: encountered linear polynomial.'
            write(9,*) '          resetting algorithm.'
            ret=-0.0011
            return
         endif

c        only the positive root is of interest, since the other one 
c        corresponds to a maximum 

         x=(-c+sqrt(p))/(2.d0*b)

         if(x.gt.5.0)  then
            write(9,*) 
     $      'findroot: 2nd order polynomial shows runaway  solution.'
            write(9,*) '          resetting algorithm.'
            ret=-0.0012
            return
         endif
         ret=x
         return
      endif


c     /* calculate p and q */
c      q = (2.0*bcubed-9.0*a*b*c + 27.0*asq*d)  / (54.0 * acubed);
c     

      q= (b/a)*(b/a)*(b/a)/27.d0 - (b/a)*(c/a)/6.d0 + d/(2.d0*a)

c     /*  p = (3.0*a*c-bsq)/(9.0 *asq); */
  
      p = c/(a*3.d0) - (b/a)*(b/a)/9.d0
  
c     /*  printf("p=%.5e, q=%.5e\n",p,q); */

c     /* Find real roots depending on p, q */


      if(abs(p).lt.FIND_ROOT_EPS) then ! easy real solution   ?? 
         if(q.lt.0) then
            x=  (2.d0*abs(q))**(1.d0/3.d0) - b/(3.d0*a)
         else
            x= -(2.d0*abs(q))**(1.d0/3.d0) - b/(3.d0*a)
         endif
         if((3*x*x + 2*b*x + c .gt.0).and.(x.ge.0)) then
            ret=x; return
         else 
            ret=-0.002; return
c            write(9,100);   stop
         endif
      else                      ! no, then have a closer   look 
         if(q.ge.0.0) then
            r = sqrt(abs(p))
         else
            r =-sqrt(abs(p))
         endif
         if(p.gt.0) then        ! one real solution 
            x = -2.0 *r*sinh(myasinh(q/(r*r*r))/3.d0)    
     $           -  b/(3.d0*a)
c            write(9,*) 3.d0*a*x*x + 2.d0*b*x + c
c            write(9,*) x
            if((3.d0*a*x*x + 2.d0*b*x + c .gt.0).and.(x.ge.0)) then
               ret=x;    return
            else 
               ret=-0.003; return
            endif
         else                   ! look at the discriminant 
            if((q*q + p*p*p).le.0) then ! three real solutions
               phi3rd = acos(q/(r*r*r))/3.0;
               x1 = -2.0 *r*cos(phi3rd)       -  b/(3.0*a);
               x2 =  2.0 *r*cos(pi3rd-phi3rd) -  b/(3.0*a);
               x3 =  2.0 *r*cos(pi3rd+phi3rd) -  b/(3.0*a);

               x1p = 3.d0*a*x1*x1 + 2.d0*b*x1 + c;
               x2p = 3.d0*a*x2*x2 + 2.d0*b*x2 + c;
               x3p = 3.d0*a*x3*x3 + 2.d0*b*x3 + c;

               x1f = 0.25d0*a*(x1*x1*x1*x1)+ (1.d0/3.d0)*b*(x1*x1*x1) + 
     $              0.5d0*c*x1*x1 + d*x1
               x2f = 0.25*a*(x2*x2*x2*x2)+ (1.d0/3.d0)*b*(x2*x2*x2) + 
     $              0.5d0*c*x2*x2 + d*x2
               x3f = 0.25d0*a*(x3*x3*x3) + (1.d0/3.d0)*b*(x3*x3*x3) +
     $              0.5d0*c*x3*x3 + d*x3

c              sort them according to size 

               x=-1.0d20;

               if(IMODE==1) then ! take the one that lowers the energy the most 
                  if(x3.gt.0 .and. x3p.gt.0) then
                     if(x2.gt.0 .and. x2p.gt.0) then
                        if(x1.gt.0 .and. x1p.gt.0) then
                           if(x1f.le.x2f .and. x1f.le.x3f) x=x1;
                           if(x2f.le.x1f .and. x2f.le.x3f) x=x2;
                           if(x3f.le.x1f .and. x3f.le.x2f) x=x3;
                        else    ! only x2, x3 left 
                           if(x2f.le.x3f) then
                              x=x2;
                           else       
                              x=x3;
                           endif
                        endif
                     else       ! only x1, x3 left 
                        if(x1.gt.0 .and. x1p.gt.0) then
                           if(x1f.le.x3f) then
                              x=x1
                           else
                              x=x3;
                           endif
                        else 
                           x=x3;
                        endif
                     endif
                  else          ! only x1, x2 left
                     if(x2.gt.0 .and. x2p.gt.0) then
                        if(x1.gt.0 .and. x1p.gt.0) then
                           if(x1f.le.x2f) then
                              x=x1
                           else 
                              x=x2
                           endif
                        else 
                           x=x2;
                        endif
                     else
                        if(x1.gt.0 .and. x1p.gt.0)  x=x1;
                     endif
                  endif
               else             ! take smallest root which represents  minimum 
                  if(x3.gt.0 .and. x3p.gt.0) then
                     if(x2.gt.0 .and. x2p.gt.0) then
                        if(x1.gt.0 .and. x1p.gt.0) then
                           if(x1.le.x2 .and. x1.le.x3) x=x1;
                           if(x2.le.x1 .and. x2.le.x3) x=x2;
                           if(x3.le.x1 .and. x3.le.x2) x=x3;
                        else !  only x2, x3 left 
                           if(x2.le.x3) then
                              x=x2;
                           else      
                              x=x3;
                           endif
                        endif
                     else       ! only x1, x3 left 
                        if(x1.gt.0 .and. x1p.gt.0) then
                           if(x1.le.x3) then
                              x=x1;
                           else      
                              x=x3;
                           endif
                        else 
                           x=x3;
                        endif
                     endif
                  else          ! only x1, x2 left 
                     if(x2.gt.0 .and. x2p.gt.0) then
                        if(x1.gt.0 .and. x1p.gt.0) then
                           if(x1.le.x2) then
                              x=x1;
                           else 
                              x=x2;
                           endif
                        else 
                           x=x2;
                        endif
                     else
                        if(x1.gt.0 .and. x1p.gt.0) x=x1;
                     endif
                  endif
               endif
               if(x.gt.0) then
                  ret=x; return;
               else
                  ret=-0.004; return
c                  write(9,100); stop
               endif
            else                !  one real solution  
               x = -2.d0 *r*cosh(myacosh(q/(r*r*r))/3.d0)    
     $              -  b/(3.d0*a)
               if((3.d0*a*x*x + 2.d0*b*x + c .gt.0).and.(x.ge.0)) then
                  ret=x;          return;
               else 
                  ret=-0.005; return
c                  write(9,100); stop
               endif
            endif
         endif                  ! look at the discriminant
      endif                     ! then have a closer look



      ret=-1.0d20
      return

 100  format('*** MATRIX DIAGONALIZATION BOMBOUT!',
     $     /'-->  CHECK INPUT CHARGE DENSITY,'
     $     /'-->  MAYBE REDUCE MIXING,'
     $     /'-->  INCREASE DIAGSAFETY')


      end





