c
c     @process extchk
      subroutine force_stress(crys, syms, bands, kpoints,
     $   pspot, energs, pw_params, pot_gspace, k_gspace,
     $   vion, vin, vout, den, denc, dvql, ddc, vql, dnc,
     $   wavefn, ewaldf, ssum, fsum, ffts)

c
c     calls the right subroutines to compute force and stress 
c

c     1996 by Bernd Pfrommer, while at UCB
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
c     INPUT:
c     -----

      type (fftstruc) ffts      ! information for the fast fourier transform
      type (symmetry) syms
      type (crystal) crys
      type (band) bands     
      type (kpoint) kpoints
      type (pseudo_potential) pspot
      type (energy) energs
      type (force) ewaldf
      type (pw_parameter) pw_params
      type (parallel_gspace) 
     $     pot_gspace,          ! gspace for the potential
     $     k_gspace(*)          ! gspaces for the different kpoints

      complex(kind=8)
     $     vion(pot_gspace%length,crys%nspin), ! the screened ionic potential
     $     vin(pot_gspace%length,crys%nspin), ! electronic V_hxc input potential
     $     vout(pot_gspace%length,crys%nspin), ! electronic V_hxc output potential
     $     den(pot_gspace%length,crys%nspin), ! valence charge density
     $     denc(pot_gspace%length), ! core charge density
     $     dvql(pot_gspace%length), ! derivative of the local pseudopotential
     $     ddc(pot_gspace%length) ! derivative of core charge density

      double precision
     $     vql(pot_gspace%length,crys%ntype), ! local pspot of different atoms
     $     dnc(pot_gspace%length,crys%ntype) ! core charge density of atoms

      type (complex_gspace_array)
     $     wavefn


c     INPUT/OUTPUT:
c     ------------

      double precision
     $     ssum(6),             ! sigma * V  [Ryd]
     $     fsum(3,crys%mxdatm,crys%ntype) ! dE/dq



c     ------------------ local variables ------------------------

      double precision
     $     aadot(6)             ! realspace metric

c
c     --------------------------------------------------------------------
c
      fsum=0
      ssum=0

      call forstressloc(1,crys%nspin, !  local part of force and stress 
     $     fsum, ssum, aadot,
     $     energs%xc, energs%alpha,
     $     pot_gspace%length, pot_gspace%gvec,pot_gspace%ekin,
     $     vion, vin, vout, 
     $     den, denc, dvql, ddc,
     $     vql, dnc,
     $     crys%ntype,crys%natom,crys%rat,crys%bdot,crys%vcell, 
     $     crys%mxdatm)

      call forstressnloc(1,crys,
     $     bands, k_gspace, pw_params%ekinmod, pspot, kpoints%rk,
     $     aadot, wavefn, fsum, ssum)

      if(pw_params%icorr.eq.'pw') then
        call stresspw91(ffts,crys,pot_gspace,den,denc,ssum)
      endif 

      if(pw_params%icorr.eq.'pb') then
        call stresspbe(ffts,crys,pot_gspace,den,denc,ssum)
      endif

      call forstresssym(1,1,fsum,ssum,pw_params%icorr,
     $     ewaldf%force,ewaldf%stress,
     $     crys%ntype,crys%natom,crys%rat,crys%avec,crys%bvec,
     $     crys%bdot,crys%vcell,
     $     syms%ntrans,syms%mtrx, syms%tnp, syms%rsymmat,
     $     crys%mxdatm)

      return

      end subroutine
