c
      subroutine forstressloc(ipr, ispn,
     $     fsum, ssum, aadot,
     $     exc, ealpha,
     $     ng, gsvec, ek,
     $     vion, vin, vout, 
     $     den, denc, dvql, ddc,
     $     vql, dnc,
     $     ntype, natom, rat, bdot, vcell, mxdatm)

c
c     computes the local potential contributions to the Hellman-Feynman
c     Forces and the stress. Adapted from a code by J.L. Martins 1988
c
c     parallel version 1995 Bernd Pfrommer
c     
c
      implicit none             ! never do implicit !

c
c     INPUT:
c     -----
      
      integer
     $     ipr,                 ! print flag
     $     ispn,                ! number of spin components: 1 or 2
     $     ng,                  ! number of gvectors
     $     ntype,               ! number of atomic types
     $     gsvec(3,*),          ! gvectors
     $     natom(*),            ! number of atoms of each type
     $     mxdatm               ! dimensions for arrays


      double precision
     $     vcell,               ! unit cell volume
     $     bdot(3,3),           ! reciprocal lattice metric
     $     vql(ng,ntype),        ! local pseudopotential of different atoms
     $     dnc(ng,ntype),        ! core charge density of atoms
     $     rat(3,mxdatm,*),     ! atomic coordinates in lattice *2*pi
     $     exc,                 ! the exchange correlation energy
     $     ealpha,              ! the divergent term to the energy contribution
     $     ek(ng)               ! kinetic energy of gvectors

      complex(kind=8)
     $     vion(ng,ispn),       ! the screened ionic potential
     $     vin(ng,ispn),        ! electronic V_hxc input potential
     $     vout(ng,ispn),       ! electronic V_hxc output potential
     $     den(ng,ispn),        ! valence charge density
     $     denc(ng),            ! core charge density
     $     dvql(ng),            ! derivative of the local pseudopotential
     $     ddc(ng)              ! derivative of core charge density

c     OUTPUT:
c     ------

      double precision
     $     fsum(3,mxdatm,*),    ! unsymmetrized forces[lattice coordinates] 
     $                          ! from local part on each atom
     $     aadot(6),            ! metric in realspace:
     $                          ! adot(11,22,33,12,23,31)/(2*pi)**2
     $     ssum(6)              ! unsymmetrized, metric stress tensor [Ryd]

c     ---------------- local variables -----------------------------------

      integer i,j,k,l,is
      double precision pi, zero, two, four, twopi, fi, fac,
     $     vld, dvld, vxd, dvxd, exp1, exp2, vhd, denr, deni
      complex(kind=8) vdg, vd, vxc, densum, dentot
      logical isp

      parameter ( pi = 3.141592653589793d0, twopi = 2.0d0*pi)
      parameter ( zero = 0.0d0, two = 2.d0, four = 4.0d0)
      
      isp = .false.
      if(ispn.eq.2) isp=.true.

c
c     initialize fsum
c
      do i=1,ntype
         do j=1,natom(i)
            do  k=1,3
               fsum(k,j,i) = zero
            end do
         end do
      end do
      
      fac =  four*twopi/vcell

c     -----------------------------------------------------------------
c              first the local contribution to the forces
c

      do i=1,ntype              ! loop over atomic types
         do j=1,ng              ! loop over gvectors, exclude G=0
            if(ek(j).gt.0.d0) then
c
c           vql * cc(den) + cc(vxc) * denc
c           vxc = vout - vhartree
c
c    spinpolarized:  (vxc_up-vhartree + vxc_down-vhartree) * dnc/2
c
c
               
               if(isp) then
                  dentot = den(j,1) + den(j,2)
                  vdg  = vql(j,i) * conjg(dentot)
     $                 + conjg((vout(j,1)+vout(j,2))/2.d0
     $                 - fac*dentot/ek(j))* dnc(j,i)
               else
                  vdg = vql(j,i) * conjg(den(j,1))
     $                 + conjg(vout(j,1)-
     $                 fac*den(j,1)/ek(j))* dnc(j,i)
               endif

               do l=1,natom(i)  ! loop over atoms of same type
                  
                  fi =   dble(gsvec(1,j)) * rat(1,l,i)
     $                 + dble(gsvec(2,j)) * rat(2,l,i)
     $                 + dble(gsvec(3,j)) * rat(3,l,i)
                  exp1 = sin(fi)*dble(vdg) - cos(fi)*aimag(vdg)
c     
c                 add to forces
c     
                  fsum(1,l,i) = fsum(1,l,i) + dble(gsvec(1,j)) * exp1
                  fsum(2,l,i) = fsum(2,l,i) + dble(gsvec(2,j)) * exp1
                  fsum(3,l,i) = fsum(3,l,i) + dble(gsvec(3,j)) * exp1
               end do
            endif
         end do
      end do

c     ---------------------------------------------------------------------
c                            local part of the stress

c
c     initialize ssum array
c
      do i=1,6
         ssum(i) = zero
      end do
c
c     compute metric in real space
c     aadot(1-6) = adot(11, 22, 33, 12, 23, 31) / (2*pi)**2
c
      fac = vcell/(twopi*twopi*twopi)
      fac = fac * fac
      aadot(1) = fac*(bdot(2,2)*bdot(3,3) - bdot(2,3)*bdot(2,3))
      aadot(2) = fac*(bdot(3,3)*bdot(1,1) - bdot(3,1)*bdot(3,1))
      aadot(3) = fac*(bdot(1,1)*bdot(2,2) - bdot(1,2)*bdot(1,2))
      aadot(4) = fac*(bdot(1,3)*bdot(3,2) - bdot(1,2)*bdot(3,3))
      aadot(5) = fac*(bdot(2,1)*bdot(1,3) - bdot(2,3)*bdot(1,1))
      aadot(6) = fac*(bdot(1,2)*bdot(2,3) - bdot(1,3)*bdot(2,2))
      
c     -------------------------------------------------------------
c                 local part of stress contribution
c

C

      fac =  four*twopi/vcell

      do i=1,ng                 ! loop over gvectors
         if(ek(i).gt.0) then
c
c           vlocal * den
c            
            if(isp) then
               dentot = den(i,1) + den(i,2)
            else
               dentot = den(i,1)
            endif

            denr = dble(dentot)
            deni = aimag(dentot)
            
            vd   = vion(i,1)-vin(i,1) ! ionic potential, same for both spins
            vld = dble(vd)*denr + aimag(vd)*deni
c
c           derivative of v local * den
c
            dvld = dble(dvql(i)) * denr
     $           + aimag(dvql(i)) * deni
c
c           vhartree * den / 2
c
            vhd = fac * (denr*denr +deni*deni)/(two*ek(i))
c
c           vxc * den and vxc * d(den)/d(g**2)  (for g .ne. 0)
c           (vxc (g=0) and exc done below)
c
c
c           in the spin polarized case, take  denc/2.0, and ddc/2.0,
c           but sum over both spins
c

            vxd  = 0.d0
            dvxd = 0.d0

            do is=1,ispn
               vxc = vout(i,is) - fac *dentot/ek(i)
               densum = den(i,is) + denc(i)/dble(ispn)
               vxd  = vxd + 
     $              dble(vxc)*dble(densum) + aimag(vxc)*aimag(densum)
               dvxd = dvxd + (dble(vxc)*dble(ddc(i)) 
     $              + aimag(vxc)*aimag(ddc(i)))/dble(ispn)
            end do

c
c           diagonal (exp1) and nondiag (exp2) contributions
c
            exp1 = (vld + vhd + vxd)
            exp2 = two * (dvld - vhd/ek(i) + dvxd)
c
c           add diagonal terms to stress tensor
c
            do j=1,6
               ssum(j) = ssum(j) + exp1 * aadot(j)
            end do
c
c           and add nondiagonal terms
c
            ssum(1) = ssum(1) + exp2 * dble(gsvec(1,i)*gsvec(1,i))
            ssum(2) = ssum(2) + exp2 * dble(gsvec(2,i)*gsvec(2,i))
            ssum(3) = ssum(3) + exp2 * dble(gsvec(3,i)*gsvec(3,i))
            ssum(4) = ssum(4) + exp2 * dble(gsvec(1,i)*gsvec(2,i))
            ssum(5) = ssum(5) + exp2 * dble(gsvec(2,i)*gsvec(3,i))
            ssum(6) = ssum(6) + exp2 * dble(gsvec(3,i)*gsvec(1,i))
         else                   ! ha! here we have G=0
c
c           vxc (g=0), exc, and ealpha contribution
c
            if(isp) then
               exp1 = dble(vout(i,1))*dble(den(i,1)+denc(i)/2.d0)
     $              + dble(vout(i,2))*dble(den(i,2)+denc(i)/2.d0)
     $              - exc + ealpha
            else
               exp1 = dble(vout(i,1))*dble(den(i,1)+denc(i)) 
     $              - exc + ealpha
            endif

            do j=1,6
               ssum(j) = ssum(j) + exp1 * aadot(j)
            end do
         endif 
      end do                    ! loop over gvectors 


      return

      end
c
c
c
c
c    _________________________________________________________________
c
      subroutine stresspw91(ffts, crys, gs, den, denc, ssum)

      use all_to_all_module
      include 'use.h'
      implicit none
      include 'interface.h'
      include 'all_to_all.h'

c
c     INPUT:
c     ------

      type (fftstruc) ffts      ! for the fourier transform
      type (crystal) crys       ! for the bvec array to take the gradient
      type (parallel_gspace) gs ! the gspace for the gradient computation


c
c     INPUT/OUTPUT:
c     ------------
c
c     the charge density in fourier space on input, the exchange
c     -correlation potential (Rydbergs) in fourier space on output
c
      complex(kind=8) 
     $     den(gs%length,crys%nspin),denc(gs%length) 

c     --------------------- local variables -------------------------

      double precision 
     $ ssum(6),ggss(6),exp3,exp4,
     $ bb1,bb2,bb3,bb4,bb5,alfa,beta,eta,cc0,cc1,c1,c2,c3,c4,c5,c6,c7,
     $ a,alfa1,bt1,bt2,bt3,bt4,a_p,alfa1_p,bt1_p,bt2_p,bt3_p,bt4_p,
     $ a_alf,alfa1_alf,bt1_alf,bt2_alf,bt3_alf,bt4_alf,alfr,alfden,
     $ alflog,dalfdrs,zeta,zeta3,zeta4,fzeta,fzetap,fzetazeta4,
     $ invfzz,gam,invgam,phi,phi2,phi3,invphi,invphi3,phip,
     $ pi,pi2,delt,x13,x23,x43,x76,x16m,ax,pider1,pider2,pider3,
     $ abder1,abder2,abder3,vcell,invcell,spscale,invspscale,
     $ roe,grx,gry,grz,roeth,rs,gkf,sd,s,s2,s3,s4,
     $ ysr,ys,ysl,sysl,fxexp,fxnum,fxden,rs12,rs32,rs2,rs3,invrs12,
     $ td,t,t2,t3,t4,aexp,abig,abig2,h0num,h0den,h0arg,h0,
     $ ccrnum,ccrden,ccr,h1,dys,bys,dfs,dfs2,ecr_dif,
     $ dh0dt2,dh1dt2,dh1ds2,normfac
C
C     _________________________________________________________________
      integer 
     $ k,ir,nspin,rsize,is,isign,norm,i,j

      double precision, dimension(1:3) :: aroe,agr,ecr
      double precision, dimension(1:2) :: ecden,eclog,
     $                                    dfxd,dfxdg,dexcd
      double precision, dimension(3,3) :: avec
      double precision, allocatable ::
     $     rhoe(:,:),         ! charge density in rspace
     $     gradr(:,:,:)       ! gradient of charge density in rspace

      complex(kind=8), allocatable :: 
     $     gswork(:),         ! work array in gspace
     $     gswork1(:),        ! work array in gspace
     $     rswork(:)          ! work array in rspace
c
      parameter(pi=3.14159265358979323846264338327950d0)
      parameter(ax=-0.738558766382022405884230032680836d0)
      parameter(gam=0.5198420997897463295344212145565d0)
      parameter(invgam=1.d0/gam,invfzz=9.d0*gam/8.d0)
      parameter(eta=1.d-12,delt=1.d-12)
      parameter(bb1=0.19645,bb2=0.27430,bb3=-0.15084,bb4=0.004,
     1 bb5=7.7956,alfa=0.09,beta=0.06672632268006,cc0=15.75592034948314,
     2 cc1=0.00352057142857,c1=0.001667,c2=0.002568,c3=0.023266,
     3 c4=7.389e-6,c5=8.723,c6=0.472,c7=7.389e-2,a=0.0621814,
     4 alfa1=0.2137,bt1=7.5957,bt2=3.5876,bt3=1.6382,bt4=0.49294,
     5 a_p=0.0310907,alfa1_p=0.20548,bt1_p=14.1189,bt2_p=6.1977,
     6 bt3_p=3.3662,bt4_p=0.62517,a_alf=0.033774,alfa1_alf=0.11125,
     7 bt1_alf=10.357,bt2_alf=3.6231,bt3_alf=0.88026,bt4_alf=0.49671) 
c
      logical isp               ! spin polarization
c
c     -------------------------------------------------------------------
c
      x13=1.d0/3.0d0
      x16m=-1.d0/6.0d0
      x23=2.0d0/3.0d0
      x43=4.0d0/3.0d0
      x76=7.0d0/6.0d0
c     _________________________________________________________________
c     derived parameters from pi
c
      pi2=pi*pi
      pider1=(0.75d0/pi)**x13
      pider2=(3.0d0*pi2)**x13
      pider3=(3.0d0*pi2/16.0d0)**x13
c     _________________________________________________________________
c     derived parameters from alfa and beta 
c
      abder1=beta*beta/(2.0d0*alfa)
      abder2=1.d0/abder1
      abder3=2.0d0*alfa/beta
c
      norm  = gs%fftsize(1)*gs%fftsize(2)*gs%fftsize(3)
      normfac=crys%vcell/norm
      vcell=crys%vcell
      invcell=1.d0/crys%vcell
      avec(:,:)= crys%avec(:,:)
c
      nspin=crys%nspin
      spscale=dble(nspin)
      invspscale=1.d0/spscale
      isp = .false.
      if(nspin.eq.2) isp =.true.
c
      rsize=gs%r_size
      phi=1.d0
      invphi=1.d0
      phi2=1.d0
      phi3=1.d0
      invphi3=1.d0
c
      allocate(rswork(rsize))
      allocate(gswork(gs%length))
      allocate(gswork1(gs%length))
c
      allocate(rhoe(rsize,nspin))
      allocate(gradr(rsize,3,2*nspin-1))
c
      do i=1,6
         ggss(i) = 0.d0
      end do
      do is=1,nspin
         gswork(:) = (den(:,is)+denc(:)/dble(nspin))*invcell
         call fourier_transform(-1,ffts,gs,gswork(1),rswork(1))
         rhoe(:,is) = real(rswork(:))
         do k=1,3
            call derivative_gspace(k,gswork(1),gs,crys,gswork1(1))
            gswork1(:) = gswork1(:) * (0.d0,1.d0)
            call fourier_transform(-1,ffts,gs,gswork1(1),rswork(1))
            gradr(:,k,is) = real(rswork(:))
         enddo
      enddo
      if(isp) gradr(:,:,3)=gradr(:,:,1)+gradr(:,:,2)
c
c     _________________________________________________________________
c     main loop
c
      do 100 ir=1,rsize
         do is=1,nspin
            roe=rhoe(ir,is)*spscale
            if((.not.isp).and.(roe.le.0.d0)) then
               gradr(ir,:,1)=0.d0
               goto 100
            endif
            if(isp.and.(roe.lt.0.d0)) then
               gradr(ir,:,:)=0.d0
               goto 100
            endif
            aroe(is)=abs(roe)
            grx=gradr(ir,1,is)*spscale
            gry=gradr(ir,2,is)*spscale
            grz=gradr(ir,3,is)*spscale
            agr(is)=sqrt(grx*grx+gry*gry+grz*grz)
            roeth=aroe(is)**x13 
            if(isp.and.(roe.eq.0.d0)) then
               exp3=0.d0
            else
               rs= pider1/roeth
               gkf=pider2*roeth
               sd=1.d0/(2.0d0*gkf*aroe(is))
               s=agr(is)*sd
               s2=s*s
               s3=s*s2
               s4=s2*s2
               ysr=sqrt(1.d0+bb5*bb5*s2)
               ys=bb5*s+ysr
               ysl=log(ys)*bb1
               sysl=s*ysl
               fxexp=exp(-100.0d0*s2)
               fxnum=1.d0+sysl+(bb2+bb3*fxexp)*s2
               fxden=1.d0/(1.d0+sysl+bb4*s4)
               dys=bb5*(1.d0+bb5*s/ysr)/ys
               dfs=-fxnum*(ysl+bb1*s*dys+4.0d0*bb4*s3)*fxden*fxden
     $              +(ysl+bb1*s*dys+2.0d0*s*(bb2+bb3*fxexp)
     $              -200.0d0*s3*bb3*fxexp)*fxden
               isign=sign(1.d0,s-delt)
               bys=0.5d0*(1+isign)/(s+(1-isign)*delt)
               dfs2=0.5d0*dfs*bys
               exp3=spscale*ax*dfs2*roeth*normfac
     $              /(gkf*gkf*aroe(is)*4.d0*pi2)
               do i=1,3
                  do j=1,3
                     ggss(1)=ggss(1)+exp3*avec(i,1)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,1)
                     ggss(2)=ggss(2)+exp3*avec(i,2)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,2)
                     ggss(3)=ggss(3)+exp3*avec(i,3)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,3)
                     ggss(4)=ggss(4)+exp3*avec(i,1)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,2)
                     ggss(5)=ggss(5)+exp3*avec(i,2)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,3)
                     ggss(6)=ggss(6)+exp3*avec(i,3)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,1)
                  enddo
               enddo
            endif
         enddo
c     _________________________________________________________________
c     back to total charge density  
c
         if(isp) then
            roe=rhoe(ir,1)+rhoe(ir,2)
            if(roe.eq.0.d0) then
               rhoe(ir,:)=0.d0
               gradr(ir,:,:)=0.d0
               goto 100
            endif
            aroe(3)=abs(roe)
            grx=gradr(ir,1,3)
            gry=gradr(ir,2,3)
            grz=gradr(ir,3,3)
            agr(3)=sqrt(grx*grx+gry*gry+grz*grz)
            roeth=aroe(3)**x13
            rs= pider1/roeth
            gkf=pider2*roeth
            sd=1.d0/(2.0d0*gkf*aroe(3))
            s=agr(3)*sd
            s2=s*s
          endif
c     _________________________________________________________________
c     correlation ecr=ec(rho,zeta) 
c
         rs12=sqrt(rs)
         invrs12=1.d0/rs12
         rs32=rs12*rs
         rs2=rs*rs
         rs3=rs*rs2
         ecden(1)=a*(bt1*rs12+bt2*rs+bt3*rs32+bt4*rs2) 
         eclog(1)=log(1.d0+(1.d0/ecden(1)))
         ecr(1)=-a*(1.d0+alfa1*rs)*eclog(1)
         if(isp) then
            ecden(2)=a_p*(bt1_p*rs12+bt2_p*rs+bt3_p*rs32+bt4_p*rs2) 
            eclog(2)=log(1.d0+(1.d0/ecden(2)))
            ecr(2)=-a_p*(1.d0+alfa1_p*rs)*eclog(2)
c
            alfden=a_alf*(bt1_alf*rs12+bt2_alf*rs
     $             +bt3_alf*rs32+bt4_alf*rs2) 
            alflog=log(1.d0+(1.d0/alfden))
            alfr=a_alf*(1.d0+alfa1_alf*rs)*alflog
c
            zeta=(rhoe(ir,1)-rhoe(ir,2))/roe
            if(abs(zeta).gt.1.d0) zeta=dble(sign(1.d0,zeta))
            zeta3=zeta*zeta*zeta
            zeta4=zeta3*zeta
c           invgam=1.0/(2.d0**(3.d0/4.d0)-2.d0)
            fzeta=((1.d0+zeta)**x43+(1.d0-zeta)**x43-2.d0)*invgam
            fzetap=x43*((1.d0+zeta)**x13-(1.d0-zeta)**x13)*invgam
            fzetazeta4=fzeta*zeta4
            phi=((1.d0+zeta)**x23+(1.d0-zeta)**x23)*0.5d0
            phip=(((1.d0+zeta)**2+eta)**x16m 
     $           -((1.d0-zeta)**2+eta)**x16m)*x13
            invphi=1.d0/phi
            phi2=phi*phi
            phi3=phi2*phi
            invphi3=1.d0/phi3
c
            ecr_dif=ecr(2)-ecr(1)
            ecr(3)=ecr(1)+alfr*fzeta*invfzz*(1.d0-zeta4)
     $          +ecr_dif*fzetazeta4
         endif
c     _________________________________________________________________
c     correlation h0(t,ecr,zeta)
c
         td=pider3*sd*invrs12*invphi
         t=agr(2*nspin-1)*td
         t2=t*t
         t3=t*t2
         t4=t2*t2
         aexp=exp(-abder2*ecr(2*nspin-1)*invphi3)-1.d0
         abig=abder3/aexp
         abig2=abig*abig
         h0num=t2+abig*t4
         h0den=1.d0/(1.d0+abig*t2+abig2*t4)
         h0arg=1.d0+abder3*h0num*h0den
         h0=abder1*log(h0arg)*phi3
c     _________________________________________________________________
c     correlation h1(t,s,aroe,zeta)
c
         ccrnum=c2+c3*rs+c4*rs2
         ccrden=1.d0/(1.d0+c5*rs+c6*rs2+c7*rs3)
         ccr=c1+ccrnum*ccrden
         fxexp=exp(-100.0d0*s2*phi2)
         h1=phi3*cc0*(ccr-cc1)*t2*fxexp
         dh0dt2=phi3*abder1/h0arg*abder3*h0den
     $        *(1.0d0+2.0d0*abig*t2-h0num*h0den*(1.0d0*abig+2.0d0
     $        *abig2*t2))
         dh1dt2=phi3*cc0*(ccr-cc1)*fxexp
         dh1ds2=-100.0d0*phi2*cc0*(ccr-cc1)*phi3*t2*fxexp
         is=2*nspin-1
         exp4=(dh0dt2+dh1dt2)/(16.d0*phi2*gkf*aroe(is)*pi)*normfac
     $        +dh1ds2*normfac/(gkf*gkf*aroe(is)*4.d0*pi2)
         do i=1,3
            do j=1,3
               ggss(1)=ggss(1)+exp4*avec(i,1)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,1)
               ggss(2)=ggss(2)+exp4*avec(i,2)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,2)
               ggss(3)=ggss(3)+exp4*avec(i,3)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,3)
               ggss(4)=ggss(4)+exp4*avec(i,1)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,2)
               ggss(5)=ggss(5)+exp4*avec(i,2)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,3)
               ggss(6)=ggss(6)+exp4*avec(i,3)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,1)
            enddo
         enddo
 100  continue
      call all_sum_all(ggss(1))
      call all_sum_all(ggss(2))
      call all_sum_all(ggss(3))
      call all_sum_all(ggss(4))
      call all_sum_all(ggss(5))
      call all_sum_all(ggss(6))
      ssum(:)=ssum(:)+ggss(:)
C     _________________________________________________________________
      deallocate(rswork)
      deallocate(gswork)
      deallocate(gswork1)
c
      deallocate(rhoe)
      deallocate(gradr)
C     _________________________________________________________________
      return
      end
C     _________________________________________________________________
c
c
c
c
c
      subroutine stresspbe(ffts,crys,gs,den,denc,ssum)

      use all_to_all_module
      include 'use.h'
      implicit none
      include 'interface.h'
      include 'all_to_all.h'

c
c     INPUT:
c     ------

      type (fftstruc) ffts      ! for the fourier transform
      type (crystal) crys       ! for the bvec array to take the gradient
      type (parallel_gspace) gs ! the gspace for the gradient computation



c
c     INPUT/OUTPUT:
c     ------------
c
c     stress and the charge density in fourier space on input,
c     gga contribution of stress is added to ssum.
c
      complex(kind=8) 
     $     den(gs%length,crys%nspin),denc(gs%length) 

c     --------------------- local variables -------------------------

      double precision 
     $ ssum(6),ggss(6),bet,gamma,gam,invgam,um,uk,uk2,eta,delt,
     $ a,alfa1,bt1,bt2,bt3,bt4,a_p,alfa1_p,bt1_p,bt2_p,bt3_p,
     $ bt4_p,a_alf,alfa1_alf,bt1_alf,bt2_alf,bt3_alf,bt4_alf,
     $ alfr,alfden,alflog,dalfdrs,ecr_dif,
     $ zeta,zeta3,zeta4,fzeta,fzetap,fzetazeta4,invfzz,
     $ phi,phi2,phi3,invphi,invphi3,
     $ x13,x23,x43,x76,x16m,pi,pi2,ax,pider1,pider2,pider3,
     $ abder1,abder2,abder3,vcell,invcell,spscale,invspscale,
     $ roe,grx,gry,grz,roeth,rs,gkf,sd,s,s2,fxden,fxden2,fx,
     $ rs12,rs32,rs2,rs3,dfs,td,t,t2,t3,t4,aexp,abig,abig2,
     $ hnum,hden,harg,h,dhdt2,normfac,exp3,exp4
C
C     _________________________________________________________________
      integer 
     $ k,ir,nspin,rsize,is,isign,norm,i,j

      double precision, dimension(1:3) :: aroe,agr,ecr
      double precision, dimension(1:2) :: ecden,eclog
      double precision, dimension(3,3) :: avec
      double precision, allocatable ::
     $     rhoe(:,:),         ! charge density in rspace
     $     gradr(:,:,:)       ! gradient of charge density in rspace

      complex(kind=8), allocatable :: 
     $     gswork(:),         ! work array in gspace
     $     gswork1(:),        ! work array in gspace
     $     rswork(:)          ! work array in rspace
c
      parameter(pi=3.14159265358979323846264338327950d0)
      parameter(ax=-0.738558766382022405884230032680836d0)
      parameter(um=0.2195149727645171d0,uk=0.804d0)
      parameter(gam=0.5198420997897463295344212145565d0)
      parameter(invgam=1.d0/gam,invfzz=9.d0*gam/8.d0)
      parameter(gamma=0.03109069086965489503494086371273d0)
      parameter(bet=0.06672455060314922d0)
      parameter(eta=1.d-12,delt=1.d-12)    
      parameter(a=0.0621814,alfa1=0.2137,bt1=7.5957,bt2=3.5876,
     1 bt3=1.6382,bt4=0.49294,a_p=0.0310907,alfa1_p=0.20548,
     2 bt1_p=14.1189,bt2_p=6.1977,bt3_p=3.3662,bt4_p=0.62517,
     3 a_alf=0.033774,alfa1_alf=0.11125,bt1_alf=10.357,
     4 bt2_alf=3.6231,bt3_alf=0.88026,bt4_alf=0.49671) 
c
      logical isp               ! spin polarization
c
c     -------------------------------------------------------------------
c
      x13=1.d0/3.0d0
      x16m=-1.d0/6.0d0
      x23=2.0d0/3.0d0
      x43=4.0d0/3.0d0
      x76=7.0d0/6.0d0
c     _________________________________________________________________
c     derived parameters from pi
c
      pi2=pi*pi
      pider1=(0.75d0/pi)**x13
      pider2=(3.0d0*pi2)**x13
      pider3=(3.0d0*pi2/16.0d0)**x13
c     _________________________________________________________________
c     derived parameters from beta, gamma, and kappa
c
      abder1=gamma
      abder2=1.d0/abder1
      abder3=bet*abder2
      uk2=uk*uk
c
      norm  = gs%fftsize(1)*gs%fftsize(2)*gs%fftsize(3)
      normfac=crys%vcell/norm
      vcell=crys%vcell
      invcell=1.d0/crys%vcell
      avec(:,:)= crys%avec(:,:)
c
      nspin=crys%nspin
      spscale=dble(nspin)
      invspscale=1.d0/spscale
      isp = .false.
      if(nspin.eq.2) isp =.true.
c
      rsize=gs%r_size
      phi=1.d0
      invphi=1.d0
      phi2=1.d0
      phi3=1.d0
      invphi3=1.d0
c
      allocate(rswork(rsize))
      allocate(gswork(gs%length))
      allocate(gswork1(gs%length))
c
      allocate(rhoe(rsize,nspin))
      allocate(gradr(rsize,3,2*nspin-1))
      do i=1,6
         ggss(i) = 0.d0
      end do
c
      do is=1,nspin
         gswork(:) = (den(:,is)+denc(:)/dble(nspin))*invcell
         call fourier_transform(-1,ffts,gs,gswork(1),rswork(1))
         rhoe(:,is) = real(rswork(:))
         do k=1,3
            call derivative_gspace(k,gswork(1),gs,crys,gswork1(1))
            gswork1(:) = gswork1(:) * (0.d0,1.d0)
            call fourier_transform(-1,ffts,gs,gswork1(1),rswork(1))
            gradr(:,k,is) = real(rswork(:))
         enddo
      enddo
      if(isp) gradr(:,:,3)=gradr(:,:,1)+gradr(:,:,2)
c
c     _________________________________________________________________
c     main loop
c
      do 100 ir=1,rsize
         do is=1,nspin
            roe=rhoe(ir,is)*spscale        ! spin-scaled for exchange
            if((.not.isp).and.(roe.le.0.d0)) then
               gradr(ir,:,1)=0.d0
               goto 100
            endif
            if(isp.and.(roe.lt.0.d0)) then
               gradr(ir,:,:)=0.d0
               goto 100
            endif
            aroe(is)=abs(roe)
            grx=gradr(ir,1,is)*spscale
            gry=gradr(ir,2,is)*spscale
            grz=gradr(ir,3,is)*spscale
            agr(is)=sqrt(grx*grx+gry*gry+grz*grz)
            roeth=aroe(is)**x13 
            if(isp.and.(roe.eq.0.d0)) then
               exp3=0.d0
            else
               rs= pider1/roeth
               gkf=pider2*roeth
               sd=1.d0/(2.0d0*gkf*aroe(is))
               s=agr(is)*sd
               s2=s*s
               fxden=1.d0/(uk+um*s2)
               fxden2=fxden*fxden
               fx=1.d0+uk-uk2*fxden
c au to ry     exp3=spscale*ax*um*uk2*fxden2*roeth
c                   /(2.d0*gkf*gkf*aroe(is))*normfac*2.d0/(4.d0*pi2)
               exp3=spscale*ax*um*uk2*fxden2*roeth*normfac
     $              /(gkf*gkf*aroe(is)*4.d0*pi2)
               do i=1,3
                  do j=1,3
                     ggss(1)=ggss(1)+exp3*avec(i,1)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,1)
                     ggss(2)=ggss(2)+exp3*avec(i,2)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,2)
                     ggss(3)=ggss(3)+exp3*avec(i,3)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,3)
                     ggss(4)=ggss(4)+exp3*avec(i,1)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,2)
                     ggss(5)=ggss(5)+exp3*avec(i,2)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,3)
                     ggss(6)=ggss(6)+exp3*avec(i,3)*gradr(ir,i,is)
     $                       *gradr(ir,j,is)*avec(j,1)
                  enddo
               enddo
            endif
         enddo
c     _________________________________________________________________
c     back to total charge density  
c        Be careful! Total charge density is not spin-scaled!
c
         if(isp) then
            roe=rhoe(ir,1)+rhoe(ir,2)
            if(roe.eq.0.d0) goto 100
            aroe(3)=abs(roe)
            grx=gradr(ir,1,3)
            gry=gradr(ir,2,3)
            grz=gradr(ir,3,3)
            agr(3)=sqrt(grx*grx+gry*gry+grz*grz)
            roeth=aroe(3)**x13
            rs= pider1/roeth
            gkf=pider2*roeth
            sd=1.d0/(2.0d0*gkf*aroe(3))
            s=agr(3)*sd
            s2=s*s
          endif
c     _________________________________________________________________
c     correlation ecr=ec(rho,zeta) 
c
         rs12=sqrt(rs)
         rs32=rs12*rs
         rs2=rs*rs
         rs3=rs*rs2
         ecden(1)=a*(bt1*rs12+bt2*rs+bt3*rs32+bt4*rs2) 
         eclog(1)=log(1.d0+(1.d0/ecden(1)))
         ecr(1)=-a*(1.d0+alfa1*rs)*eclog(1)
         if(isp) then
            ecden(2)=a_p*(bt1_p*rs12+bt2_p*rs+bt3_p*rs32+bt4_p*rs2) 
            eclog(2)=log(1.d0+(1.d0/ecden(2)))
            ecr(2)=-a_p*(1.d0+alfa1_p*rs)*eclog(2)
            alfden=a_alf*(bt1_alf*rs12+bt2_alf*rs
     $             +bt3_alf*rs32+bt4_alf*rs2) 
            alflog=log(1.d0+(1.d0/alfden))
            alfr=a_alf*(1.d0+alfa1_alf*rs)*alflog
            zeta=(rhoe(ir,1)-rhoe(ir,2))/roe
            if(abs(zeta).gt.1.d0) zeta=dble(sign(1.d0,zeta))
            zeta3=zeta*zeta*zeta
            zeta4=zeta3*zeta
            fzeta=((1.d0+zeta)**x43+(1.d0-zeta)**x43-2.d0)*invgam
            fzetap=x43*((1.d0+zeta)**x13-(1.d0-zeta)**x13)*invgam
            fzetazeta4=fzeta*zeta4
            phi=((1.d0+zeta)**x23+(1.d0-zeta)**x23)*0.5d0
            invphi=1.d0/phi
            phi2=phi*phi
            phi3=phi*phi*phi
            invphi3=1.d0/phi3
            ecr_dif=ecr(2)-ecr(1)
            ecr(3)=ecr(1)+alfr*fzeta*invfzz*(1.d0-zeta4)
     $          +ecr_dif*fzetazeta4
         endif
c     _________________________________________________________________
c     correlation h(t,ecr,zeta)
c
         td=pider3*sd/rs12*invphi
         t=agr(2*nspin-1)*td
         t2=t*t
         t3=t*t2
         t4=t2*t2
         aexp=exp(-abder2*ecr(2*nspin-1)*invphi3)-1.d0
         abig=abder3/aexp
         abig2=abig*abig
         hnum=t2+abig*t4
         hden=1.d0/(1.d0+abig*t2+abig2*t4)
         harg=1.d0+abder3*hnum*hden
         h=abder1*log(harg)*phi3
         dhdt2=phi3*abder1/harg*abder3*hden
     $        *(1.0d0+2.0d0*abig*t2-hnum*hden*(1.0d0*abig+2.0d0
     $        *abig2*t2))
         is=2*nspin-1
c        exp4=dhdt2/(2.d0*phi2*4.0d0*gkf/pi*aroe(is))
c             2.d0*normfac/(4.d0*pi2)
         exp4=dhdt2/(16.d0*phi2*gkf*aroe(is)*pi)*normfac
         do i=1,3
            do j=1,3
               ggss(1)=ggss(1)+exp4*avec(i,1)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,1)
               ggss(2)=ggss(2)+exp4*avec(i,2)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,2)
               ggss(3)=ggss(3)+exp4*avec(i,3)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,3)
               ggss(4)=ggss(4)+exp4*avec(i,1)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,2)
               ggss(5)=ggss(5)+exp4*avec(i,2)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,3)
               ggss(6)=ggss(6)+exp4*avec(i,3)*gradr(ir,i,is)
     $                 *gradr(ir,j,is)*avec(j,1)
            enddo
         enddo
 100  continue
      call all_sum_all(ggss(1))
      call all_sum_all(ggss(2))
      call all_sum_all(ggss(3))
      call all_sum_all(ggss(4))
      call all_sum_all(ggss(5))
      call all_sum_all(ggss(6))
      ssum(:)=ssum(:)+ggss(:)
C     _________________________________________________________________
      deallocate(rswork)
      deallocate(gswork)
      deallocate(gswork1)
c
      deallocate(rhoe)
      deallocate(gradr)
C     _________________________________________________________________
      return
      end

