c
c     @process extchk
      subroutine forstressnloc(ipr, crys,
     $     bands, kgs, ekinmod, pspot,  rk,  
     $     aadot,  eigenvec,  fsum, ssum)

c
c     computes the kinetic energy and nonlocal potential contributions 
c     to the Hellman-Feynman forces and the stress. 
c     Adapted from a code by J.L. Martins 1988
c
c     parallel version 1995/1996 Bernd Pfrommer
c     
c

      use all_to_all_module
      include 'use.h'
      implicit none             ! never do implicit !
      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'
      

c
c     INPUT:
c     -----

      type (parallel_gspace) 
     $     kgs(*)               ! gspaces for the different k-points
      type (pseudo_potential)  pspot
      type (band)       bands
      type (crystal)    crys

      type(complex_gspace_array)
     $     eigenvec ! the groundstate wavefunctions

      
      integer
     $     ipr                  ! print flag


      double precision
     $     rk(3,bands%nrk),     ! k-points in units of reciprocal lattice vecs
     $     ekinmod(3),          ! parameters for the modified kinetic energy
     $     aadot(6)             ! metric in realspace:
     $                          ! adot(11,22,33,12,23,31)/(2*pi)**2



c     INPUT/OUTPUT:  (the nonlocal+kinetic part is added)
c     -------------

      double precision
     $     fsum(3,crys%mxdatm,crys%ntype), ! unsymm forces [latt. coordinates] 
     $                          ! from local part on each atom
     $     ssum(6)              ! unsymmetrized, metric stress tensor [Ryd]

c     ---------------- local variables -----------------------------------

c     dynamic arrays:
c     --------------

      complex(kind=8), allocatable ::
     $     anlga(:,:,:), dhd(:,:,:)
      double precision, allocatable ::
     $     xnlkb(:)

c     other simple variables:
c     -----------------------

      integer i,j,k,l, n, kk, kl, lm, ind, lo, ist, is2,is3,
     $     irk, neig, ndim, jmin, jmax, mxdpdim,
     $     nanl, nanl10, nanlga, nanlga10,ni, lmmin, lmmax, 
     $     bytes_per_double_complex,bytes_per_double,is 
      double precision 
     $     pi, zero, one, two, three, four, half, twopi, delta,
     $     rkpt(3), qki(3), qk(3), qi, qinv, dqi(6), qcar(3),
     $     dqcar(6,3), flm(9), dflm(6,9), forefac,
     $     zsumr, eps, fi, xni, vq, dvq, xi, enl





      logical isp

      complex(kind=8)
     $     st, zzero, zone

      parameter ( pi = 3.141592653589793d0, twopi = 2.0d0*pi)
      parameter ( zero = 0.0d0, one =1.d0, two = 2.d0, 
     $     three = 3.d0, four = 4.0d0, half = 0.5d0,
     $     zzero = (0.d0, 0.d0), zone = (1.d0, 0.d0))
     
      parameter ( eps = 1.0d-12)
      parameter ( bytes_per_double_complex=16,
     $            bytes_per_double=8 )


      isp = .false.
      if(crys%nspin.eq.2) isp =.true.


      forefac = 2.d0/(sqrt(pi)*ekinmod(3))*ekinmod(1)


c     ------- first allocate dynamic arrays ------------

      mxdpdim=0
      do i=1,bands%nrk
         if(kgs(i)%length.gt.mxdpdim) mxdpdim=kgs(i)%length
      end do

c
c     count the max number of nonlocal components nanlga for array dimensioning
c
      nanlga=0

      do k=1, crys%ntype
         ind = 0
         do l=1,5
            if(pspot%nkb(l,k) .ne. 0) then
               lmmin = pspot%lo(l,k)*pspot%lo(l,k) + 1
               lmmax = (pspot%lo(l,k)+1)*(pspot%lo(l,k)+1)
               do lm=lmmin,lmmax
                  ind = ind + 1
               end do
            endif
         end do
         nanlga=max(nanlga,ind);
      end do

      nanlga10=10*nanlga;


      allocate(anlga(10,nanlga,mxdpdim));      anlga=0
      allocate(dhd(10,nanlga,bands%max));      dhd=0
      if(pspot%nanl.gt.0) allocate(xnlkb(pspot%nanl))
      
      do is=1,crys%nspin
      do irk=1, bands%nrk             ! loop over all k-points

         neig = bands%nband(irk,is)
         ndim = kgs(irk)%length
         rkpt(1) = rk(1,irk)
         rkpt(2) = rk(2,irk)
         rkpt(3) = rk(3,irk)

c        ----------- kinetic energy contribution to the stress ------------

c        find min and max occupied band
         jmin = 0
         jmax = 0

         do i=1,neig
            if (bands%occup(i,irk,is) .ne. zero .and. jmin .eq. 0) 
     $           jmin = i
            if (bands%occup(i,irk,is) .ne. zero .and. jmin .ne. 0) 
     $           jmax = i
         end do
c
c      start loop over g(i)
c
         do i=1, ndim
c
c           compute |g+k|
c
            qki(1) = rkpt(1) + dble(kgs(irk)%gvec(1,i))
            qki(2) = rkpt(2) + dble(kgs(irk)%gvec(2,i))
            qki(3) = rkpt(3) + dble(kgs(irk)%gvec(3,i))
c      
c           do eigenvector multiplication and band sum
c
            zsumr = zero
            do k=jmin,jmax
               zsumr = zsumr + bands%occup(k,irk,is)/dble(crys%nspin)*
     $              (dble(eigenvec%data((k-1)*ndim+i,irk,is))* 
     $               dble(eigenvec%data((k-1)*ndim+i,irk,is))
     $              +aimag(eigenvec%data((k-1)*ndim+i,irk,is))*
     $               aimag(eigenvec%data((k-1)*ndim+i,irk,is)))
            end do
c
c           add the kinetic energy functional modifications
c

            delta=(kgs(irk)%ekin(i)-ekinmod(2))/ekinmod(3)
            zsumr = zsumr*(1.d0 +  forefac * exp(-delta*delta))

            zsumr = two * zsumr

            
            ssum(1) = ssum(1) + zsumr * qki(1) * qki(1)
            ssum(2) = ssum(2) + zsumr * qki(2) * qki(2)
            ssum(3) = ssum(3) + zsumr * qki(3) * qki(3)
            ssum(4) = ssum(4) + zsumr * qki(1) * qki(2)
            ssum(5) = ssum(5) + zsumr * qki(2) * qki(3)
            ssum(6) = ssum(6) + zsumr * qki(3) * qki(1)

         end do


      end do                    ! end of loop over k-points
      end do                    ! end of loop over spins





c     ----------- nonlocal potential contribution to forces and stress --

c
c     Consolidate across processors by summing. THIS CAN ONLY BE DONE ONCE!
c
      call all_sum_all(fsum,3*crys%mxdatm*crys%ntype)
      call all_sum_all(ssum,6)

      do irk=1, bands%nrk             ! loop over all k-points

         ndim = kgs(irk)%length
         rkpt(1) = rk(1,irk)
         rkpt(2) = rk(2,irk)
         rkpt(3) = rk(3,irk)

         do 10 k=1,crys%ntype
c
            nanl = 0
            do l=1,5
               if(pspot%nkb(l,k) .ne. 0) then
                  lmmin = pspot%lo(l,k)*pspot%lo(l,k) + 1
                  lmmax = (pspot%lo(l,k)+1)*(pspot%lo(l,k)+1)
                  do lm=lmmin,lmmax
                     nanl = nanl +1
                  end do
               endif
            end do

            nanl10 = 10*nanl
c     
            do 11 kk=1,crys%natom(k)
c
c                calculates the matrix elements
c
               do 20 i=1,ndim
                  qk(1) = rkpt(1) + dble(kgs(irk)%gvec(1,i))
                  qk(2) = rkpt(2) + dble(kgs(irk)%gvec(2,i))
                  qk(3) = rkpt(3) + dble(kgs(irk)%gvec(3,i))
                  qi = (qk(1)*crys%bdot(1,1) + qk(2)*crys%bdot(2,1) +
     +                 qk(3)*crys%bdot(3,1))*qk(1) +
     +                 (qk(1)*crys%bdot(1,2) + qk(2)*crys%bdot(2,2) +
     +                 qk(3)*crys%bdot(3,2))*qk(2) +
     +                 (qk(1)*crys%bdot(1,3) + qk(2)*crys%bdot(2,3) +
     +                 qk(3)*crys%bdot(3,3))*qk(3)
                  if(qi .gt. eps) then
                     qi = sqrt(qi)
                     qinv = one / qi
c
c                    derivatives of qi with respect to strain
c                    in lattice coordinates
c
                     dqi(1) = - qk(1)*qk(1)*qinv
                     dqi(2) = - qk(2)*qk(2)*qinv
                     dqi(3) = - qk(3)*qk(3)*qinv
                     dqi(4) = - qk(1)*qk(2)*qinv
                     dqi(5) = - qk(2)*qk(3)*qinv
                     dqi(6) = - qk(3)*qk(1)*qinv
c     
c                    qvector in cartesian coordinates and normalized
c
                     qcar(1) = crys%bvec(1,1)*qk(1)+
     $                    crys%bvec(1,2)*qk(2)+crys%bvec(1,3)*qk(3)
                     qcar(2) = crys%bvec(2,1)*qk(1)+
     $                    crys%bvec(2,2)*qk(2)+crys%bvec(2,3)*qk(3)
                     qcar(3) = crys%bvec(3,1)*qk(1)+
     $                    crys%bvec(3,2)*qk(2)+crys%bvec(3,3)*qk(3)
                     qcar(1) = qcar(1)*qinv
                     qcar(2) = qcar(2)*qinv
                     qcar(3) = qcar(3)*qinv
c
c                    derivatives of qcar with respect to strain
c                    in lattice coordinates
c
                     dqcar(1,1) = - crys%avec(1,1) * qk(1) / twopi
                     dqcar(2,1) = - crys%avec(1,2) * qk(2) / twopi
                     dqcar(3,1) = - crys%avec(1,3) * qk(3) / twopi
                     dqcar(4,1) = - crys%avec(1,1) * qk(2) / twopi
                     dqcar(5,1) = - crys%avec(1,2) * qk(3) / twopi
                     dqcar(6,1) = - crys%avec(1,3) * qk(1) / twopi
                     dqcar(1,2) = - crys%avec(2,1) * qk(1) / twopi
                     dqcar(2,2) = - crys%avec(2,2) * qk(2) / twopi
                     dqcar(3,2) = - crys%avec(2,3) * qk(3) / twopi
                     dqcar(4,2) = - crys%avec(2,1) * qk(2) / twopi
                     dqcar(5,2) = - crys%avec(2,2) * qk(3) / twopi
                     dqcar(6,2) = - crys%avec(2,3) * qk(1) / twopi
                     dqcar(1,3) = - crys%avec(3,1) * qk(1) / twopi
                     dqcar(2,3) = - crys%avec(3,2) * qk(2) / twopi
                     dqcar(3,3) = - crys%avec(3,3) * qk(3) / twopi
                     dqcar(4,3) = - crys%avec(3,1) * qk(2) / twopi
                     dqcar(5,3) = - crys%avec(3,2) * qk(3) / twopi
                     dqcar(6,3) = - crys%avec(3,3) * qk(1) / twopi
c
                     do kl=1,6
                      dqcar(kl,1) = (dqcar(kl,1)-dqi(kl)*qcar(1))*qinv
                      dqcar(kl,2) = (dqcar(kl,2)-dqi(kl)*qcar(2))*qinv
                      dqcar(kl,3) = (dqcar(kl,3)-dqi(kl)*qcar(3))*qinv
                     end do
c
c                    angular functions
c
                     flm(1) = one
                     flm(2) = qcar(1)
                     flm(3) = qcar(2)
                     flm(4) = qcar(3)
                     flm(5) = sqrt(three)*qcar(1)*qcar(2)
                     flm(6) = sqrt(three)*qcar(2)*qcar(3)
                     flm(7) = sqrt(three)*qcar(3)*qcar(1)
                     flm(8) = three*qcar(3)*qcar(3)/two - one/two
                     flm(9) = sqrt(three)*(qcar(1)*qcar(1)-
     $                    qcar(2)*qcar(2))/two
c
c                    strain derivatives of angular functions
c                    in lattice coordinates
c
                     do kl = 1,6
                        dflm(kl,1) = zero
                        dflm(kl,2) = dqcar(kl,1)
                        dflm(kl,3) = dqcar(kl,2)
                        dflm(kl,4) = dqcar(kl,3)
                        dflm(kl,5) = sqrt(three)*(dqcar(kl,1)*qcar(2)+
     $                       qcar(1)*dqcar(kl,2))
                        dflm(kl,6) = sqrt(three)*(dqcar(kl,2)*qcar(3)+
     $                       qcar(2)*dqcar(kl,3))
                        dflm(kl,7) = sqrt(three)*(dqcar(kl,3)*qcar(1)+
     $                       qcar(3)*dqcar(kl,1))
                        dflm(kl,8) = three * dqcar(kl,3)*qcar(3)
                        dflm(kl,9) = sqrt(three)*(dqcar(kl,1)*qcar(1)-
     $                       dqcar(kl,2)*qcar(2))
                     end do
                  else          ! treat the G=0 component
                     qi = zero
                     dqi(1) = zero               
                     dqi(2) = zero               
                     dqi(3) = zero               
                     dqi(4) = zero               
                     dqi(5) = zero               
                     dqi(6) = zero
                     do lm=1,9
                        flm(lm) = zero
                        dflm(1,lm) = zero
                        dflm(2,lm) = zero
                        dflm(3,lm) = zero
                        dflm(4,lm) = zero
                        dflm(5,lm) = zero
                        dflm(6,lm) = zero
                     end do
                     flm(1) = one
                  endif
c
c                 structure factor
c
                  fi = dble(kgs(irk)%gvec(1,i))*crys%rat(1,kk,k) +
     $                 dble(kgs(irk)%gvec(2,i))*crys%rat(2,kk,k) +
     $                 dble(kgs(irk)%gvec(3,i))*crys%rat(3,kk,k)
                  st = exp(cmplx(0.d0,fi,kind=8))
c
                  ind = 0
                  do 30 l=1,5
                     if(pspot%nkb(l,k) .ne. 0) then
                        xni = qi/pspot%delqnl(k) + two
                        ni  = xni + half
                        if(ni .le. 3) ni = 4
                        vq = zero
                        dvq = zero
                        if(ni .lt. pspot%nqnl(k)) then
                           xi  = xni - dble(ni)
                           vq = pspot%vkb(ni,l,k)*(one+xi)*(one-xi)
     $                          + half*(pspot%vkb(ni+1,l,k)*(one+xi)
     $                          -pspot%vkb(ni-1,l,k)*(one-xi)) * xi
                           dvq = -two * xi *pspot%vkb(ni,l,k) +
     $                          (xi + half) * pspot%vkb(ni+1,l,k) +
     $                          (xi - half) * pspot%vkb(ni-1,l,k)
                           dvq = dvq / pspot%delqnl(k)
                        endif
                        lmmin = pspot%lo(l,k)*pspot%lo(l,k) + 1
                        lmmax = (pspot%lo(l,k)+1)*(pspot%lo(l,k)+1)

c     
                        do lm=lmmin,lmmax
                           ind = ind + 1
                           anlga(1,ind,i) = st*flm(lm)*vq
                           xnlkb(ind) = dble(pspot%nkb(l,k))
c     
c                          forces
c
                           anlga(2,ind,i)=cmplx(zero,-one,kind=8)*st*
     $                         flm(lm)*vq*dble(kgs(irk)%gvec(1,i))
                           anlga(3,ind,i)=cmplx(zero,-one,kind=8)*st*
     $                         flm(lm)*vq*dble(kgs(irk)%gvec(2,i))
                           anlga(4,ind,i)=cmplx(zero,-one,kind=8)*st*
     $                         flm(lm)*vq*dble(kgs(irk)%gvec(3,i))
c     
c                          stresses
c     
                           do ist=1,6
                              anlga(4+ist,ind,i) = 
     $                             st*(dflm(ist,lm)*vq+
     $                             flm(lm)*dvq*dqi(ist))
                           end do
                        end do

                     endif
c      
 30               continue      ! loop over lo
c     
 20            continue         ! loop over gspace
c
c     calculates the matrix vector product
c
               do is=1, crys%nspin
                  neig = bands%nband(irk,is)
                  if(nanl10.gt.0)
     $                 call mzgemm('n','n',nanl10,neig,ndim, zone,
     $                 anlga(1,1,1),nanlga10, eigenvec%data(1,irk,is),
     $                 ndim, zzero,dhd(1,1,1),nanlga10)
         

c
c              cross-processor consolidation for dhd
c
                  call all_sum_all(dhd,nanlga10*bands%max)

                  do n=1,neig
                     do i=1,nanl
                        enl = bands%occup(n,irk,is)/dble(crys%nspin) 
     $                       * xnlkb(i) *
     $                       (dble(dhd(1,i,n))*dble(dhd(1,i,n))
     $                       +aimag(dhd(1,i,n))*aimag(dhd(1,i,n)))
                        
                        do ist=1,3
                           fsum(ist,kk,k) = fsum(ist,kk,k) + 
     $                      two * bands%occup(n,irk,is)/
     $                          dble(crys%nspin)*xnlkb(i)*
     $                      (dble(dhd(1+ist,i,n))*dble(dhd(1,i,n))
     $                      +aimag(dhd(1+ist,i,n))*aimag(dhd(1,i,n)))
                        end do

                        do ist=1,6
                           ssum(ist) = ssum(ist) + enl*aadot(ist)
     $                      -two*bands%occup(n,irk,is)/
     $                          dble(crys%nspin) *xnlkb(i)*
     $                      (dble(dhd(4+ist,i,n))*dble(dhd(1,i,n))
     $                      +aimag(dhd(4+ist,i,n))*aimag(dhd(1,i,n)))
                        end do
                     end do
                  end do
               end do           ! end of loop over spins
c
 11         continue            ! end of loop over atoms(type)
 10      continue               ! end of loop over atomic types


      end do                    ! end of loop over k-points


c     free the dynamic arrays

      deallocate(anlga)
      deallocate(dhd)
      if(pspot%nanl.gt.0) deallocate(xnlkb)

      return

      end

