c
c
c     @process extchk
      subroutine forstresssym(ipr,mode,fsum,ssum,icorr,
     $     force,stress,
     $     ntype,natom,rat,avec,bvec,bdot,vcell,
     $     ntrans,mtrx, tnp, rsymmat,
     $     mxdatm)
c
c     symmetrizes and computes the hellman-feynman forces and stress
c     adapted from sverre froyen plane wave program
c     written january 18 1988. jlmo

c     explicit typing and other cosmetics 1995 Bernd Pfrommer

      implicit none             ! implicit? Just say no!

C

c     INPUT:
c     -----

      integer 
     $     ipr,                 ! print flag
     $     natom(*),            ! number of atoms of each type
     $     mtrx(3,3,48),        ! symmetry operation: matrices
     $     mode,                ! if mode=1 then symmetrize stress/force
     $                          ! at the end
     $     ntype,               ! number of atomic types
     $     ntrans,              ! number of symmetry operations
     $     mxdatm               ! array dimension

      double precision
     $     fsum(3,mxdatm,ntype), ! unsymmetrized forces
     $     ssum(6),             ! unsymmetrized stress
     $     force(3,mxdatm,ntype), ! forces as from ewald subroutine (lattice)
     $     stress(6),           ! stress tensor as from ewald subroutine (ryd)
     $     bdot(3,3),           ! metric in reciprocal space
     $     bvec(3,3),           ! reciprocal lattice vectors
     $     avec(3,3),           ! realspace lattice vectors
     $     vcell, 
     $     tnp(3,48),           ! nonprimitive translations
     $     rsymmat(3,3,48),       ! the realspace point symmetry matrices.
     $                          ! they are needed for the stress-symmetrization
     $     rat(3,mxdatm,ntype)
      
      character*2 icorr         ! string for the exchange-correlation

c     OUTPUT:  fsum and ssum will return symmetrized force/stress
c     ------

c     ------------------------- local variables ---------------------------

      double precision
     $     fcart(3), select(6), ssym(6), cro(3),
     $     pi, twopi, zero, eps, fac, xdif,rotfor(3)

c     WORK ARRAYS:
c     -----------

      double precision
     $     fsym(3,mxdatm,ntype)


      integer i,j,k, natomi, natomj, kk,l,m,n, idif

      parameter ( pi = 3.141592653589793d0, twopi = 2.0d0*pi)
      parameter ( zero = 0.0d0)
      parameter ( eps = 1.0d-6)

c
c      symmetrize
c
      do i=1,ntype
         natomi = natom(i)
         do j=1,natomi
            do k=1,3
               fsym(k,j,i) = zero
            end do
         end do
      end do

      ssym=0
c
c     symmetrizes forces
c
      do i=1,ntrans
         do j=1,ntype
            natomj = natom(j)
            do k=1, natomj
c
c                     -1
c            find mtrx    * (rat - tnp)
c
               do l=1,3
                  cro(l) = zero
                  do m=1,3
                     cro(l) = cro(l)+dble(mtrx(m,l,i))*
     $                    (rat(m,k,j)-tnp(m,i))
                  end do
               end do
               do l=1,natomj
                  do m=1,3
                     xdif = abs(cro(m)-rat(m,l,j)) / twopi
                     idif = xdif + eps
                     if (abs(xdif-dble(idif)) .gt. eps) goto 16
                  end do
                  kk = l
                  goto 17
 16               continue
               end do
               write(9,*) 'subroutine forstresssym:'
               write(9,*) 'unable to find equivalent atom for k=',k
               stop
 17            continue
c
c              rotate force and add
c
               do l=1,3
                  do m=1,3
                     fsym(l,k,j) = fsym(l,k,j)+
     $                    dble(mtrx(l,m,i))*fsum(m,kk,j)
                  end do
               end do
            end do
         end do
      end do
c
c      symmetrize stress
c
      do i=1,6
         j = i
         k = i
         if (i .gt. 3) j = i - 3
         if (i .gt. 3) k = j + 1
         if (k .gt. 3) k = 1
         do l=1,ntrans
            ssym(i) = ssym(i)
     $           + ssum(1) * dble(mtrx(j,1,l) * mtrx(k,1,l))
     $           + ssum(2) * dble(mtrx(j,2,l) * mtrx(k,2,l))
     $           + ssum(3) * dble(mtrx(j,3,l) * mtrx(k,3,l))
     $           + ssum(4) * dble(mtrx(j,1,l) * mtrx(k,2,l))
     $           + ssum(5) * dble(mtrx(j,2,l) * mtrx(k,3,l))
     $           + ssum(6) * dble(mtrx(j,3,l) * mtrx(k,1,l))
     $           + ssum(4) * dble(mtrx(j,2,l) * mtrx(k,1,l))
     $           + ssum(5) * dble(mtrx(j,3,l) * mtrx(k,2,l))
     $           + ssum(6) * dble(mtrx(j,1,l) * mtrx(k,3,l))
         end do
         ssym(i) = ssym(i) / dble(ntrans)
      end do
c
c     transform to cartesian coordinates and printout
c
c     forces
c
c
      if (ipr .ne. 0) then
         write(9,100)
      endif
      n = 0
      do i=1,ntype
         natomi = natom(i)
         do j=1,natomi
            n = n + 1
c
c           compute force in cartesian coordinates: f_cart = a*f_q
c           where a has the lattice vectors as columns
c
            do k=1,3
               fcart(k) =  avec(k,1) * force(1,j,i)
     $              +  avec(k,2) * force(2,j,i)
     $              +  avec(k,3) * force(3,j,i)
     $              + (bvec(k,1) * fsym(1,j,i)
     $              +  bvec(k,2) * fsym(2,j,i)
     $              +  bvec(k,3) * fsym(3,j,i)) / dble(ntrans)
            end do

            if (ipr .ne. 0) write(9,101) n,
     $           (matmul(avec,rat(:,j,i))/twopi), (fcart(k),k=1,3)


c
c          compute the dE/dq, which is NOT the force in lattice coordinates,
c          but  dE/dq = aT* f_cart
c
c          f_sum = aT * f_cart
c
c

            do k=1,3
               fsum(k,j,i) =0
               do l=1,3
                  fsum(k,j,i) = fsum(k,j,i) + avec(l,k)*fcart(l)
               end do
            end do

         end do
      end do
c
c     stress:      select(j,k) = b_j * ssym * b_k
c
      fac = 14710.80 / vcell
      do i=1,6                  ! j,k are corresponding indices in full matrix
         j = i 
         k = i
         if (i .gt. 3) j = i - 3
         if (i .gt. 3) k = j + 1
         if (k .gt. 3) k = 1
         select(i) = bvec(j,1) * ssym(1) * bvec(k,1)
     $        + bvec(j,2) * ssym(2) * bvec(k,2)
     $        + bvec(j,3) * ssym(3) * bvec(k,3)
     $        + bvec(j,1) * ssym(4) * bvec(k,2)
     $        + bvec(j,2) * ssym(5) * bvec(k,3)
     $        + bvec(j,3) * ssym(6) * bvec(k,1)
     $        + bvec(k,1) * ssym(4) * bvec(j,2)
     $        + bvec(k,2) * ssym(5) * bvec(j,3)
     $        + bvec(k,3) * ssym(6) * bvec(j,1)
      end do

c     add electronic stress to ewald contribution 

      do i=1,6
         ssum(i) = stress(i) + select(i)
      end do


c      write(9,*) 'forces on atoms in dE/dq'

c      do i=1,ntype
c         natomi = natom(i)
c         do j=1,natomi
c            write(9,*)  fsum(1,j,i), fsum(2,j,i), fsum(3,j,i)
c         end do
c      end do






c
c
c     explicitly symmetrize stress and forces if desired.
c
c     this has to be done to avoid small, but significant errors:
c     the lattice vectors might not be perfectly symmetric. this 
c     in turn produces an error in the stress, which is used to 
c     move the lattice vectors. after about 10 iterations, the error
c     gets large enough to cause trouble.
c
c
c

      if(mode.ge.1) then 

c         write(9,*) 'mtrx:'
c         do l=1,ntrans
c            write(9,'(i3/,3(3i5/))') l,
c     $           mtrx(1,1,l),mtrx(2,1,l),mtrx(3,1,l),
c     $           mtrx(1,2,l),mtrx(2,2,l),mtrx(3,2,l),
c     $           mtrx(1,3,l),mtrx(2,3,l),mtrx(3,3,l)
c         end do


c
c        first the stress. This is the simple part. Ideally:
c                         -1   
c             sigma = rmat    *  sigma   * rmat
c

         do i=1,6
            j = i
            k = i
            if (i .gt. 3) j = i - 3
            if (i .gt. 3) k = j + 1
            if (k .gt. 3) k = 1

            ssym(i) = 0.d0
            do l=1,ntrans       !  loop through all symmetry operations
c
c                 do sigma = rmat * sigma *rmat
c
               ssym(i) = ssym(i)
     $              + ssum(1) * rsymmat(1,j,l) * rsymmat(1,k,l)
     $              + ssum(2) * rsymmat(2,j,l) * rsymmat(2,k,l)
     $              + ssum(3) * rsymmat(3,j,l) * rsymmat(3,k,l)
     $              + ssum(4) * rsymmat(1,j,l) * rsymmat(2,k,l)
     $              + ssum(5) * rsymmat(2,j,l) * rsymmat(3,k,l)
     $              + ssum(6) * rsymmat(3,j,l) * rsymmat(1,k,l)
     $              + ssum(4) * rsymmat(2,j,l) * rsymmat(1,k,l)
     $              + ssum(5) * rsymmat(3,j,l) * rsymmat(2,k,l)
     $              + ssum(6) * rsymmat(1,j,l) * rsymmat(3,k,l)
            end do
            ssym(i) = ssym(i) / dble(ntrans)
         end do

         do i=1,6
            ssum(i) = ssym(i)
         end do

c
c        now symmetrize the forces again.
c
c        Assume in realspace, a point-symmetry rotation matrix
c        looks like:
c
c          alpha = (cos(phi)  -sin(phi)    0)
c                  (sin(phi)   cos(phi)    0)
c                  (0          0           1)
c
c      *  the forces f_q in lattice coordinates transform like
c
c          f_q' = beta * f_q
c
c          where  beta := a^-1 * alpha * a
c
c      *  the quantity F_q:= dE/dq is the projection of the force
c         onto the lattice vectors, and transforms differently:
c
c                  T
c          F_q := a * a  *  f_q
c
c                          T
c          F_q' = (beta^-1)  * F_q
c
c          now:       beta     = rmtrx
c                     beta^-1  = mtrx
c
c

         do i=1,ntype
            natomi = natom(i)
            do j=1,natomi
               do k=1,3
                  fsym(k,j,i) = zero
               end do
            end do
         end do

         do i=1,ntrans
            do j=1,ntype
               natomj = natom(j)
               do k=1, natomj
c
c                           -1
c                  find mtrx    * (rat - tnp)
c
                  do l=1,3
                     cro(l) = zero
                     do m=1,3
                        cro(l) = cro(l)+dble(mtrx(m,l,i))*
     $                       (rat(m,k,j)-tnp(m,i))
                     end do
                  end do
                  do l=1,natomj
                     do m=1,3
                        xdif = abs(cro(m)-rat(m,l,j)) / twopi
                        idif = xdif + eps
                        if (abs(xdif-dble(idif)) .gt. eps) goto 116
                     end do
                     kk = l
                     goto 117
 116                 continue
                  end do
                  write(9,*) 'subroutine forstresssym:'
                  write(9,*) 'unable to find equivalent atom for k=',k
                  stop
 117              continue
c
c                 now we know that mtrx maps atom k to atom kk
c                 and mtrx maps atom kk to atom k.
c
c     
c                 rotate force and add
c     
c                  write(9,*) 'atom kk=',kk,'mapped to k=',k
c                  write(9,*) 'force on atom kk:',
c     $                 fsum(1,kk,j),fsum(2,kk,j),fsum(3,kk,j)

c
c                 apply transpose of mtrx to fsum
c

                  do l=1,3
                     rotfor(l)=0.0
                     do m=1,3
                        fsym(l,k,j) = fsym(l,k,j)+
     $                       dble(mtrx(l,m,i))*fsum(m,kk,j)
                        rotfor(l) =rotfor(l)+
     $                       dble(mtrx(l,m,i))*fsum(m,kk,j)
                     end do
                  end do
c                  write(9,*) 'rotated force:',
c     $                 rotfor(1),rotfor(2),rotfor(3)
c                  write(9,'(3i4,6f10.5)') i,j,k,
c     $                 fsum(1,k,j),fsum(2,k,j),fsum(3,k,j),
c     $                 fsym(1,k,j),fsym(2,k,j),fsym(3,k,j)
                  
               end do
            end do
c            write(9,*)
         end do

c
c        normalize coordinates and write them into the output array
c
         do i=1,ntype
            natomi = natom(i)
            do j=1,natomi
               do k=1,3
                  fsum(k,j,i) = fsym(k,j,i)/dble(ntrans)
               end do
            end do
         end do

      endif

c
      if (ipr .ne. 0) write(9,102) (ssum(i),i=1,6)
      if (ipr .ne. 0) write(9,103) (ssum(i)*fac,i=1,6)
c
      return

 100  FORMAT(//' FORCE/STRESS ANALYSIS',/,1X,21('*'),///,
     $     ' FORCES :',9X,'COORD',25X,
     $     'FORCE (Ry/a.u.)',/
     $     ' ',11X,'A1',4X,'A2',4X,'A3',11X,
     $     '-X-',12X,'-Y-',12X,'-Z-')
 101  FORMAT(I3,3X,3F8.3,2X,3F14.10)
 102  FORMAT(//,' STRESS :',
     $     25X,'SIGMA * V (Ry)',/,
     $     '(cartes)',2x,'-XX-',8X,'-YY-',8X,'-ZZ-',8X,
     $     '-XY-',8X,'-YZ-',8X,'-ZX-',/,
     $     5X,6F12.8)
 103  FORMAT(/,36X,'SIGMA (GPa)',/,
     $     9X,'-XX-',8X,'-YY-',8X,'-ZZ-',8X,
     $     '-XY-',8X,'-YZ-',8X,'-ZX-',/,
     $     5X,6F12.5)
 120  format(' *** WARNING: FORCE AND STRESS ARE NOT CORRECT IN GGA!')
      end

