c-*-Fortran-*-
c*
      subroutine fourier_transform(idir,  ffts, gs, gsdat, rspace)
c
c
m4_ifdef([SHMEM],[
c     switched on shmem for second transpose of inv fft m4_define([SHMEM2])])
m4_ifdef([SHMEM],[
c     switched on shmem for second transpose of fwd fft m4_define([SHMEM3])])

      include 'use.h' 
      implicit none             ! implicit? Just say no!
      include 'interface.h'
m4_ifdef([MPI],[
      include 'mpif.h' ])
m4_ifdef([TIMING],[      
      include 'timing.h'])

c     INPUT:
c     -----

      integer idir              ! if >=0  rspace ==> gspace
     $                          ! if <0   rspace <== gspace
      type (fftstruc) ffts      ! work arrays for the FFT
      type (parallel_gspace) gs ! the gspace for which the data is set up

c
c     INPUT/OUTPUT:
c     ------------
c
c     WARNING: UPON TRANSFORM REALSPACE => GSPACE, THE REALSPACE
c     ARRAY IS DESTROYED.
c
      complex(kind=8)
     $     rspace(gs%r_size)    ! the data in realspace rspace(ffts%r_size)
      complex(kind=8)
     $     gsdat(gs%length)     ! the data in gspace gsdat(gs%length)
c
c     --------------------------------------------------------------------
c     summer 1996 by Bernd Pfrommer while at UC Berkeley
c
c     
c     DESCRIPTION:
c     -----------
c
c     Takes an array gsdat in fourier space, and transforms it to
c     realspace, or the other way round. This is done in parallel
c     using the MPI message passing  library.
c
c
c     Returns in rspace a bunch of rods along the x direction which
c     belong to this processor. The data layout is as follows:
c
c
c           ------------ Nz
c           |          |
c           |          |        
c           |          |
c           |   2222222|
c           |2211111111|
c           |1000000000|
c           ------------
c           Ny
c
c
c     All the information about which processor receives what from
c     where is contained in the packinfo array, which is attached to
c     the gspace structure.
c
c
      FFT_HEADER

      double precision t0, ttot, gimmetime, normfac
      integer ix, iy,iz, icx, icy, icz

      integer i
      external gimmetime

      interface
        subroutine fourier_transform_12(idir,ffts,gs,gsdat,rspace)
          include 'use.h'
          integer idir
          type (fftstruc) ffts
          type (parallel_gspace) gs
          complex(kind=8), target :: rspace(gs%r_size)
          complex(kind=8) gsdat(gs%length)
        end subroutine
      end interface

      m4_ifdef([TIMING],[
      tunfold=0; txfft=0; tyfft=0; tzfft=0; t0rspace=0; ttrods=0
      t0t1buf=0; tlt1=0; tt1=0; tlt2=0; tt2=0 ])

      if(idir.ge.0) then        ! rspace ->gspace
c
c        do the first stage of the 3d FFT
c
         TIMEGET(t0)
         call fourier_transform_3(idir,  ffts, gs, rspace(1))
c
c        do the last two stages of the 3d FFT
c
         call fourier_transform_12(idir,  ffts, gs, gsdat, rspace(1))

         m4_ifdef([TIMING],[
         TIMEDIFF(ttot,t0)
         ttot=100/ttot
         write(9,123) txfft, tyfft, tzfft, tunfold, ttrods,
     $        t0t1buf,t0rspace,  tlt1, tlt2, tt1, tt2
         write(9,124) txfft*ttot, tyfft*ttot, tzfft*ttot, 
     $        tunfold*ttot, ttrods*ttot, t0t1buf*ttot,
     $        t0rspace*ttot,  tlt1*ttot, tlt2*ttot, 
     $        tt1*ttot, tt2*ttot    ])

      else                      ! gspace -> rspace
c
c        do the last two stages of the 3d FFT
c
         TIMEGET(t0)

         call fourier_transform_12(idir,  ffts, gs, gsdat, rspace(1))



c
c        do the first stage of the 3d FFT
c
         call fourier_transform_3(idir,  ffts, gs, rspace(1))


         m4_ifdef([TIMING],[
         TIMEDIFF(ttot,t0)
         ttot=100/ttot
         write(9,223) txfft, tyfft, tzfft, tunfold, ttrods,
     $        t0t1buf,t0rspace,  tlt1, tlt2, tt1, tt2
         write(9,224) txfft*ttot, tyfft*ttot, tzfft*ttot, 
     $        tunfold*ttot, ttrods*ttot, t0t1buf*ttot,
     $        t0rspace*ttot,  tlt1*ttot, tlt2*ttot, 
     $        tt1*ttot, tt2*ttot    
         ])
      endif

      return


 123  format('FFTI:',3f8.4,', UFOLD:',f8.4,', TRODS:',f8.4,
     $     ', ZBUF:', f8.4,', ZRSPC:', f8.4,', TLT1:', f8.4,
     $     ', TLT2:', f8.4,', TC1:', f8.4,', TC2:', f8.4)
 124  format('FFTI:',3f8.1,', UFOLD:',f8.1,', TRODS:',f8.1,
     $     ', ZBUF:', f8.1,', ZRSPC:', f8.1,', TLT1:', f8.1,
     $     ', TLT2:', f8.1,', TC1:', f8.1,', TC2:', f8.1)
 223  format('FFTF:',3f8.4,', UFOLD:',f8.4,', TRODS:',f8.4,
     $     ', ZBUF:', f8.4,', ZRSPC:', f8.4,', TLT1:', f8.4,
     $     ', TLT2:', f8.4,', TC1:', f8.4,', TC2:', f8.4)
 224  format('FFTF:',3f8.1,', UFOLD:',f8.1,', TRODS:',f8.1,
     $     ', ZBUF:', f8.1,', ZRSPC:', f8.1,', TLT1:', f8.1,
     $     ', TLT2:', f8.1,', TC1:', f8.1,', TC2:', f8.1)

      end

c
c     ================================================================
c
      subroutine fourier_transform_12(idir,  ffts, gs, gsdat, rspace)
c
c
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'
m4_ifdef([MPI],[
      include 'mpif.h' ])
m4_ifdef([TIMING],[      
      include 'timing.h'])

c     INPUT:
c     -----

      integer idir              ! if >=0  rspace ==> gspace
     $                          ! if <0   rspace <== gspace
      type (fftstruc) ffts      ! work arrays for the FFT
      type (parallel_gspace) gs ! the gspace for which the data is set up
c
c     INPUT/OUTPUT:
c     ------------
c
      complex(kind=8), target ::
     $     rspace(gs%r_size)    ! the data in realspace rspace(ffts%r_size)
      complex(kind=8)
     $     gsdat(gs%length)     ! the data in gspace gsdat(gs%length)
c
c     DESCRIPTION:
c     -----------
c
c     performs the first two stages of a forward or an inverse 3dFFT
c
c
c     ---------------------- local variables -------------------------

      integer 
     $     i,j,k, iord, nrods, b, b2, na, na2, rodrange(2),
     $     nx,ny,nz,nxp,ireqr,ireqs,ip, ierr,irod, ipack, iofy,
     $     psource, ptarget, 
     $     ldx, ldy, ldz,       ! leading dimensions in x, y, z
     $     nnx,nny,nnz,         ! sizes of the fourier grid
     $     nrodst1,             ! number of rods after first transposition
     $     nrodst2,             ! number of rods after second transposition
     $     dnx,                 ! number of nx slabs after t1
     $     ioffset,             ! rod offset for myproc on nz/ny grid
     $     srod, sy, slen, swidth, drod, dx, dlen, ix, ichunk, 
     $     poffset, dwidth,     !
     $     npacksend,           ! counter for num pack send
     $     dwx, dwy, dwz,dw2    ! pointers into the work array
m4_ifdef([MPI],[ 
      integer  istatus(MPI_STATUS_SIZE)])
      
      double precision 
     $     gimmetime, t0,t1,t2

      complex(kind=8) dummy

      FFT_HEADER
      
      external gimmetime

      m4_ifdef([SHMEM],[
      integer t1bufsize, ufbufsize, sendbufsize
      complex(kind=8) t1buf(1), ufb(1), sendbuf(1), dumbuf(1) 
      pointer(t1buf_p,t1buf)
      pointer(ufb_p,ufb)
      pointer(dumbuf_p,dumbuf)       
      pointer(sendbuf_p,sendbuf)
      t1bufsize=size(ffts%t1buf)*2; call all_max_all(t1bufsize)
      ufbufsize=(ffts%fftsize(3)+1)*gs%lorder*2
      call all_max_all(ufbufsize)
      sendbufsize=size(ffts%sendbuf)*2; call all_max_all(sendbufsize)


      call shpalloc(t1buf_p,t1bufsize,ierr,-1)
      call shpalloc(ufb_p,ufbufsize,ierr,-1)
      call shpalloc(sendbuf_p,sendbufsize,ierr,-1)
      ],[
      complex(kind=8), pointer :: t1buf(:), ufb(:), sendbuf(:)
      t1buf => ffts%t1buf
      ufb => rspace
      sendbuf => ffts%sendbuf
      ])


      nnx=gs%fftsize(1); nny=gs%fftsize(2); nnz= gs%fftsize(3);
      dnx=mod(gs%fftsize(4)-gs%fftsize(5)+nnx,nnx)+1

      nrodst1= (dnx*nnz)/gs%nproc 
      if( mod(dnx*nnz,gs%nproc).gt.gs%myproc) 
     $     nrodst1=nrodst1+1

      nrodst2= (nny*nnz)/gs%nproc 
      if(mod(nny*nnz,gs%nproc).gt.gs%myproc)
     $     nrodst2=nrodst2+1
c
c     variables for local T2 transpose. See setup_packinfo
c
      b  = (dnx*nnz)/gs%nproc ;        na = mod(dnx*nnz,gs%nproc)
      b2 = (nny*nnz)/gs%nproc ;       na2 = mod(nny*nnz,gs%nproc)
c
      rodrange(1)=b2*gs%myproc+min(gs%myproc,na2)
      rodrange(2)=rodrange(1)+b2
      if(na2.le.gs%myproc) rodrange(2)=rodrange(2)-1
c
      ioffset = b*gs%myproc + min(gs%myproc,na)

c
c     do some sanity checks
c
      if((ffts%fftsize(1).ne.gs%fftsize(1)).or.(ffts%fftsize(2).ne
     $     .gs%fftsize(2)).or.(ffts%fftsize(3).ne.gs%fftsize(3)))
     $     then 
         write(9,*) 'fftsize mismatch between gspace:',
     $        gs%fftsize(1:3), 'and ffts:', ffts%fftsize
         stop
      endif
c
c     figure out leading dimensions for the cray optimization
c
      ldx=nnx
      ldy=nny
      ldz=nnz
      m4_ifdef([VECTOR],[
      if(mod(ldx,2).eq.0) ldx=ldx+1 
      if(mod(ldy,2).eq.0) ldy=ldy+1
      if(mod(ldz,2).eq.0) ldz=ldz+1
      ])
      if(ldz*gs%lorder.gt.gs%r_size) then
         write(9,*) ' FFT: strange geometry used...'
         write(9,*) ' change code not to abuse rspace as buffer!'
         stop
      endif

c
c     compute pointers into the work array 
c
      dwx=1; dwy=ffts%naux(1)+1; dwz = ffts%naux(1)+ffts%naux(2)+1;
      dw2= ffts%naux(1)+ffts%naux(2)+ffts%naux(3)+1; 

      if(idir.lt.0) then ! from gspace to realspace
c
c     ================   FROM GSPACE TO REALSPACE ===================
c                        (inverse or backward FFT)
c
c        Set the data up for the first fourier transform.
c        All the rods on this processor are put next to each other
c        into the unfold buffer, and then fourier transformed with 
c        a block routine for optimum performance.
c 
         TIMEGET(t0)
         m4_ifdef([SGI_PC],[
         call fftunfoldc(gsdat(1),ufb(1),nnz,
     $        gs%order(1,1), gs%lorder)
         ],[
         call fftunfold(gsdat(1),gs,ufb(1),ldz)
         ])
         TIMEDIFF(tunfold,t0)

c
c     ---- inverse fourier transform in z direction locally ----
c
         TIMEGET(t0)
         m4_ifdef([VECTOR],[
         call ccfftm(0, nnz, gs%lorder, 1.d0, ufb(1), 
     $        ldz, ufb(1), ldz, ffts%workspace(dwz),
     $        ffts%workspace(dw2), 0)
         call ccfftm(-1, nnz, gs%lorder, 1.d0, ufb(1), 
     $        ldz, ufb(1), ldz, ffts%workspace(dwz),
     $        ffts%workspace(dw2), 0)
         ])

         FFT_AUX_SPACE(ffts%workspace(dwz),ffts%naux(3),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
         FFT_LOCAL_INIT(nnz,ufb(1),ldz,gs%lorder,-1)
         FFT_MULTIPLE_BACKWARD(nnz,ufb,ldz,gs%lorder,i,ufb,1)
         FFT_LOCAL_FREE
         TIMEDIFF(tzfft,t0)

         TIMEGET(t0)

         m4_ifdef([SGI_PC],[
         call fcleardcarray(t1buf(1),nrodst1*nny) ],[
         t1buf(1:nrodst1*ldy)=0])

         TIMEDIFF(t0t1buf,t0)
c
c        do local transpose without any message passing
c
         TIMEGET(t0)
         call frepack(gs%packsize(gs%myproc+1,2,1),
     $        t1buf(1), gs%packinfo(1,gs%myproc+1,2),
     $        ufb(1), gs%packinfo(1,gs%myproc+1,1))
         TIMEDIFF(tlt1,t0)
c
c        communication phase to transpose. New grid will be Nx/Nz
c
m4_ifdef([MPI],[
         TIMEGET(t2)
         m4_ifdef([SHMEM],[ call barrier() ! all data available?])
         do ip=1, gs%nproc-1
            ptarget=mod(ip+gs%myproc,gs%nproc) ! the target processor
            psource=mod(gs%myproc+gs%nproc-ip,gs%nproc) ! source processor
        
            m4_ifdef([SHMEM],[
            call shmem_ixget(ffts%recbuf(1), 
     $           ufb(1), 
     $           gs%packinfo(1,psource+1,1),
     $           gs%packsize(psource+1,1,1), 
     $           psource)

            do ipack = 1, gs%packsize(psource+1,2,1)
               t1buf(gs%packinfo(ipack,psource+1,2))
     $              = ffts%recbuf(ipack)
            end do
            ],[
c
c           post a non-blocking receive request to the source processor
c
            if(gs%packsize(psource+1,2,1).gt.0) 
     $           call mpi_irecv(ffts%recbuf(1),ffts%recbufsize,
     $           MPI_DOUBLE_COMPLEX, psource, psource
     $           ,MPI_COMM_WORLD,ireqr, ierr)
c
c           generate and send off the package for the target processor
c

            m4_ifdef([MPI_SCATTER],[
            if(gs%packsize(ptarget+1,1,1).gt.0) 
     $           call mpi_isend(ufb(1),1,gs%packtype(ptarget+1),
     $           ptarget, gs%myproc, MPI_COMM_WORLD, ireqs, ierr)
            ],[
            do ipack=1,gs%packsize(ptarget+1,1,1)
               sendbuf(ipack)= 
     $              ufb(gs%packinfo(ipack, ptarget+1,1))
            end do

            if(gs%packsize(ptarget+1,1,1).gt.0) 
     $           call mpi_isend(sendbuf(1),
     $           gs%packsize(ptarget+1,1,1), MPI_DOUBLE_COMPLEX, 
     $           ptarget, gs%myproc, MPI_COMM_WORLD,ireqs, ierr)
            ])
c     
c           unpack the package from the neighbor. 
c
            if(gs%packsize(psource+1,2,1).gt.0) 
     $           call mpi_wait(ireqr, istatus, ierr) ! wait til packet here

            do ipack = 1, gs%packsize(psource+1,2,1)
               t1buf(gs%packinfo(ipack,psource+1,2))
     $              = ffts%recbuf(ipack)
            end do
c
c              wait until send has completed
c
            if(gs%packsize(ptarget+1,1,1).gt.0) 
     $           call mpi_wait(ireqs, istatus, ierr)
            ])
         end do
         m4_ifdef([SHMEM],[ call barrier() ! wait until ubuf is avail])
         TIMEDIFF(tt1,t2)
         ])
         TIMEGET(t0)

c
c        inverse fourier transform in y direction locally 
c

         m4_ifdef([VECTOR],[ 
         call ccfftm(0, nny, nrodst1, 1.d0, t1buf(1), 
     $        ldy, t1buf(1), ldy, ffts%workspace(dwy),
     $        ffts%workspace(dw2), 0)
         call ccfftm(-1, nny, nrodst1, 1.d0, t1buf(1), 
     $        ldy, t1buf(1), ldy, ffts%workspace(dwy),
     $        ffts%workspace(dw2), 0) 
         ])
         
         FFT_AUX_SPACE(ffts%workspace(dwy),ffts%naux(2),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
         FFT_LOCAL_INIT(nny,t1buf(1),ldy,nrodst1,-1)
         FFT_MULTIPLE_BACKWARD(nny,t1buf,ldy,nrodst1,i,t1buf,1)
         FFT_LOCAL_FREE
         TIMEDIFF(tyfft,t0)
c
c     -------- second transposition phase. grid will be Ny/Nz  ------------
c                                                       Ny  running faster
c

c
c        clear the array if no fast convolution is wanted
c
         TIMEGET(t0)
         iofy=gs%fftsize(5)-gs%fftsize(4)
         
         m4_ifdef([FAST_CONVOLUTE],[],[
          m4_ifdef([VECTOR],[ffts%unfoldbuf(1:ldx*nrodst2)=0],[
          do irod=1, nrodst2
             i=(irod-1)*nnx+1
             rspace(i+gs%fftsize(4):i+gs%fftsize(5))=0 ! clear the array
          end do
          ])
         ])

         TIMEDIFF(t0rspace,t0)
         TIMEGET(t0)
c
c        do local transpose
c
         if(gs%packsize(gs%myproc+1,1,2).gt.0) then
m4_ifdef([CLT2],[
            call fftlt2(-1,gs%chunk(1,1,1,gs%myproc+1),
     $             m4_ifdef([VECTOR],[ffts%unfoldbuf],[rspace])(1),
     $             t1buf(1),nnx,nny,ldx,ldy)
            ],[
            ichunk=1
            do while(1.eq.1)
               srod    = gs%chunk(1,1,ichunk, gs%myproc+1)
               sy      = gs%chunk(2,1,ichunk, gs%myproc+1)+1
               slen    = gs%chunk(3,1,ichunk, gs%myproc+1)-1
               swidth  = abs(gs%chunk(4,1,ichunk,gs%myproc+1))
               drod    = gs%chunk(1,2,ichunk, gs%myproc+1)
               dx      = gs%chunk(2,2,ichunk, gs%myproc+1)
               dlen    = gs%chunk(3,2,ichunk, gs%myproc+1)-1
               do irod=srod, srod + swidth -1
                  ix = mod(dx + (irod-srod),nnx) + 1
                  m4_ifdef([VECTOR],[
                  ffts%unfoldbuf(drod*ldx+ix : (drod+dlen)*ldx+ix :
     $                 ldx)=t1buf(irod*ldy+sy:irod*ldy+sy
     $                 +slen)
                  ],[
                  rspace(drod*nnx+ix : (drod+dlen)*nnx+ix : nnx)
     $                 =t1buf(irod*nny+sy:irod*nny+sy  +slen)
                  ])
               end do
               if(gs%chunk(4,1,ichunk, gs%myproc+1).lt.0) exit
               ichunk=ichunk+1
            end do
            ])
         endif
         TIMEDIFF(tlt2,t0)
c
c        communicate with the other processors.
c

m4_ifdef([MPI],[
         TIMEGET(t0)
c
c        First pack all the packets to all processors
c        In case mpi is used, also send out immediately
c
         poffset=1 ; npacksend =0
         do ip=1, gs%nproc-1
            ptarget=mod(ip+gs%myproc,gs%nproc) ! the target processor
            if(gs%packsize(ptarget+1,1,2).gt.0) then
               ichunk=1
               ipack=poffset
               do while(1.eq.1)
                  srod    = gs%chunk(1,1,ichunk, ptarget+1)
                  sy      = gs%chunk(2,1,ichunk, ptarget+1)+1
                  slen    = gs%chunk(3,1,ichunk, ptarget+1)-1
                  swidth  = abs(gs%chunk(4,1,ichunk,ptarget+1))
                  do irod=srod, srod + swidth -1
                     sendbuf(poffset:poffset+slen)
     $                    =t1buf(irod*ldy+sy:irod*ldy+sy  +slen)
                     poffset=poffset+slen+1
                  end do
                  if(gs%chunk(4,1,ichunk, ptarget+1).lt.0) exit
                  ichunk=ichunk+1
               end do
               m4_ifdef([SHMEM2],[],[
               npacksend = npacksend +1
               call mpi_isend(sendbuf(ipack),
     $              gs%packsize(ptarget+1,1,2),MPI_DOUBLE_COMPLEX,
     $              ptarget, gs%myproc,MPI_COMM_WORLD, 
     $              ffts%mpireq(npacksend), ierr)
               ])
            endif
         end do                 ! loop over all packets to other processors
])m4_ifdef([MPI],[
         m4_ifdef([SHMEM2],[call barrier() ! everybody done packing? ])
c
c        now get the data from the other processor
c
c
c        The MPI code has room for improvement here. One should just
c        pull in the messages, read the tag, and dispatch accordingly.
c
c
         do ip=1, gs%nproc-1
            psource=mod(gs%myproc+gs%nproc-ip,gs%nproc) ! source processor
            if(gs%packsize(psource+1,2,2).gt.0) then
               m4_ifdef([SHMEM2],[
c              for shmem, get directly from source proc
               call shmem_get(ffts%recbuf(1), 
     $              sendbuf(gs%pickup(psource+1,1)),
     $              gs%packsize(psource+1,2,2)*2, 
     $              psource)
               ],[
c              for MPI, do a blocking receive on the source proc
               call mpi_recv(ffts%recbuf(1),ffts%recbufsize,
     $               MPI_DOUBLE_COMPLEX, psource, psource,
     $               MPI_COMM_WORLD,istatus, ierr)
               ])
c
c              unpack the received data
c
               ichunk=1
               poffset=1
               do while(1.eq.1)
                  drod    = gs%chunk(1,2,ichunk, psource+1)
                  dx      = gs%chunk(2,2,ichunk, psource+1)
                  dlen    = gs%chunk(3,2,ichunk, psource+1)-1
                  dwidth  = abs(gs%chunk(4,2,ichunk, psource+1))
                  do j=0, dwidth-1
                     ix = mod(dx + j,nnx) + 1
                     rspace(drod*ldx +ix : (drod+dlen)*ldx+ix : ldx)
     $                    =ffts%recbuf(poffset:poffset+dlen)
                     poffset=poffset+dlen+1
                  end do
                  if(gs%chunk(4,2,ichunk, psource+1).lt.0) exit
                  ichunk=ichunk+1
               end do
            endif
         end do                 ! loop over processors
         m4_ifdef([SHMEM2],[
         call barrier()         ! wait before deallocation
         ],[
         if(npacksend.gt.0)  call mpi_waitall(npacksend,
     $         ffts%mpireq(1),ffts%mpistat(1),ierr) 
         ])
         TIMEDIFF(tt2,t0)

])
      else

c
c     ================   FROM REALSPACE TO GSPACE ===================
c
c                        (forward FFT)
c
c
c     ---------------- second transposition phase -----------------------
c                 (really the first transpose for the forward FFT)

c
c        do local transpose without any message passing
c
         TIMEGET(t0)
         if(gs%packsize(gs%myproc+1,1,2).gt.0) then
m4_ifdef([CLT2],[
            call fftlt2(1,gs%chunk(1,1,1,gs%myproc+1),
     $             m4_ifdef([VECTOR],[ffts%unfoldbuf],[rspace])(1),
     $             ffts%t1buf(1),nnx,nny,ldx,ldy)
            ],[

            ichunk=1
            do while(1.eq.1)
               srod    = gs%chunk(1,1,ichunk, gs%myproc+1)
               sy      = gs%chunk(2,1,ichunk, gs%myproc+1)+1
               slen    = gs%chunk(3,1,ichunk, gs%myproc+1)-1
               swidth  = abs(gs%chunk(4,1,ichunk,gs%myproc+1))
               drod    = gs%chunk(1,2,ichunk, gs%myproc+1)
               dx      = gs%chunk(2,2,ichunk, gs%myproc+1)
               dlen    = gs%chunk(3,2,ichunk, gs%myproc+1)-1
               do irod=srod, srod + swidth -1
                  ix = mod(dx + (irod-srod),nnx) + 1
                  m4_ifdef([VECTOR],[
                  ffts%t1buf(irod*ldy+sy:irod*ldy+sy  +slen) =
     $                  ffts%unfoldbuf(drod*ldx+ix :drod*ldx+dlen*ldx
     $                  +ix : ldx)
                  ],[
                  ffts%t1buf(irod*nny+sy:irod*nny+sy  +slen) =
     $                  rspace(drod*nnx+ix : (drod+dlen)*nnx+ix : nnx)
                  ])
               end do
               if(gs%chunk(4,1,ichunk, gs%myproc+1).lt.0) exit
               ichunk=ichunk+1
            end do
            ])
         endif
         TIMEDIFF(tlt2,t0)
c
c        now communicate (t2 forward FFT)
c
         TIMEGET(t0)
m4_ifdef([MPI],[
c
c        First pack all the packets to all processors
c        In case mpi is used, also send out immediately
c
         poffset=1 ; npacksend =0
         do ip=1, gs%nproc-1
            ptarget= mod(ip+gs%myproc,gs%nproc) ! the target processor

            if(gs%packsize(ptarget+1,2,2).gt.0) then
               ichunk=1
               ipack=poffset
               do while(1.eq.1)
                  drod    = gs%chunk(1,2,ichunk, ptarget+1)
                  dx      = gs%chunk(2,2,ichunk, ptarget+1)
                  dlen    = gs%chunk(3,2,ichunk, ptarget+1)-1
                  dwidth  = abs(gs%chunk(4,2,ichunk, ptarget+1))
                  do j=0, dwidth-1
                     ix = mod(dx + j,nnx) + 1
                     sendbuf(poffset:poffset+dlen)=
     $                    rspace(drod*ldx+ix : (drod+dlen)*ldx+ix
     $                    :ldx)
                     poffset=poffset+dlen+1
                  end do
                  if(gs%chunk(4,2,ichunk, ptarget+1).lt.0) exit
                  ichunk=ichunk+1
               end do

               m4_ifdef([SHMEM3],[],[
               npacksend = npacksend +1
               call mpi_isend(sendbuf(ipack),gs%packsize(ptarget+1
     $              ,2,2),MPI_DOUBLE_COMPLEX,ptarget, gs%myproc
     $              ,MPI_COMM_WORLD,ffts%mpireq(npacksend), ierr)
               ])
            endif
         end do                 ! loop over all packets to other procs
c
c        now get the data from the other processor
c
c        The MPI code has room for improvement here. One should just
c        pull in the messages, read the tag, and dispatch accordingly.
c
c
         m4_ifdef([SHMEM3],[call barrier() ! everybody done packing?])
         do ip=1, gs%nproc-1
            psource= mod(gs%myproc+gs%nproc-ip,gs%nproc) ! the source proc
        
            if(gs%packsize(psource+1,1,2).gt.0) then
               m4_ifdef([SHMEM3],[
c              for shmem, get directly from source proc
               call shmem_get(ffts%recbuf(1), 
     $              sendbuf(gs%pickup(psource+1,2)),
     $              gs%packsize(psource+1,1,2)*2, 
     $              psource)
               ],[
c              for MPI, do a blocking receive on the source proc
               call mpi_recv(ffts%recbuf(1),ffts%recbufsize,
     $           MPI_DOUBLE_COMPLEX, psource, psource
     $           ,MPI_COMM_WORLD, istatus, ierr)
               ])
c
c              unpack the package from the neighbor. The packinfo array
c              gives the index for the incoming data.
c
               ichunk=1
               poffset=1
               do while(1.eq.1)
                  srod    = gs%chunk(1,1,ichunk, psource+1)
                  sy      = gs%chunk(2,1,ichunk, psource+1)+1
                  slen    = gs%chunk(3,1,ichunk, psource+1)-1
                  swidth  = abs(gs%chunk(4,1,ichunk,psource+1))
                  do irod=srod, srod + swidth -1
                     ffts%t1buf(irod*ldy+sy:irod*ldy+sy  +slen)=
     $                    ffts%recbuf(poffset:poffset+slen)
                     poffset=poffset+slen+1
                  end do
                  if(gs%chunk(4,1,ichunk, psource+1).lt.0) exit
                  ichunk=ichunk+1
               end do
            endif
         end do                 ! loop over processors

c        make sure the send buffer can be deallocated
         m4_ifdef([SHMEM3],[ call barrier()],[
         if(npacksend.gt.0)  call mpi_waitall(npacksend,
     $         ffts%mpireq(1),ffts%mpistat(1),ierr) 
         ])
         TIMEDIFF(tt2,t0)
])

c
c        fourier transform in y direction locally 
c
         TIMEGET(t0)
         m4_ifdef([VECTOR],[
         call ccfftm(0, nny, nrodst1, 1.d0, ffts%t1buf(1), 
     $        ldy, ffts%t1buf(1), ldy, ffts%workspace(dwy),
     $        ffts%workspace(dw2), 0)
         call ccfftm(1, nny, nrodst1, 1.d0, ffts%t1buf(1), 
     $        ldy, ffts%t1buf(1), ldy, ffts%workspace(dwy),
     $        ffts%workspace(dw2), 0) 
         ])

         TIMEGET(t0)

         FFT_AUX_SPACE(ffts%workspace(dwy),ffts%naux(2),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
         FFT_LOCAL_INIT(nny,ffts%t1buf(1),ldy,nrodst1,1)
         FFT_MULTIPLE_FORWARD(nny,ffts%t1buf,ldy,nrodst1,i,ffts%t1buf,1)
         FFT_LOCAL_FREE
         TIMEDIFF(tyfft,t0)
c
c     --------------------- first transpose --------------------
c
c
c        do local transpose without any message passing
c
         TIMEGET(t0)
         call frepack(gs%packsize(gs%myproc+1,2,1),
     $        ufb(1), gs%packinfo(1,gs%myproc+1,1),
     $        ffts%t1buf(1), gs%packinfo(1,gs%myproc+1,2))
         TIMEDIFF(tlt1,t0)

m4_ifdef([MPI],[
         TIMEGET(t0)
         m4_ifdef([SHMEM],[ call barrier()])
         do ip=1, gs%nproc-1

            ptarget=mod(ip+gs%myproc,gs%nproc) ! the target processor
            psource=mod(gs%myproc+gs%nproc-ip,gs%nproc) ! the source processor
            m4_ifdef([SHMEM],[
            do ipack = 1, gs%packsize(ptarget+1,2,1)
               sendbuf(ipack)=
     $              ffts%t1buf(gs%packinfo(ipack,ptarget+1,2))
            end do
            call barrier() 
            call shmem_ixput(ufb(1), sendbuf(1), 
     $        gs%packinfo(1,ptarget+1,1),
     $        gs%packsize(ptarget+1,1,1), ptarget)

            ],[
c
c           post a non-blocking receive request to the source processor
c
            if(gs%packsize(psource+1,1,1).gt.0)
     $           call mpi_irecv(ffts%recbuf(1),ffts%recbufsize,
     $           MPI_DOUBLE_COMPLEX, psource, psource,MPI_COMM_WORLD,
     $           ireqr, ierr)

c
c           generate and send off the package for the target processor
c
            if(gs%packsize(ptarget+1,2,1).gt.0) then
               do ipack = 1, gs%packsize(ptarget+1,2,1)
                  sendbuf(ipack)=
     $                 ffts%t1buf(gs%packinfo(ipack,ptarget+1,2))
               end do

               call mpi_isend(sendbuf(1),
     $              gs%packsize(ptarget+1,2,1), MPI_DOUBLE_COMPLEX, 
     $              ptarget, gs%myproc, MPI_COMM_WORLD,ireqs, ierr)
            endif
c
c           unpack the package from the neighbor. 
c
            if(gs%packsize(psource+1,1,1).gt.0) then
               call mpi_wait(ireqr, istatus, ierr) ! wait til packet here

               do ipack=1,gs%packsize(psource+1,1,1)
                  ufb(gs%packinfo(ipack, psource+1,1))=
     $                 ffts%recbuf(ipack)
               end do
            endif
c
c           wait until send has completed
c
            if(gs%packsize(ptarget+1,2,1).gt.0) 
     $           call mpi_wait(ireqs, istatus, ierr)
          ])

         end do
         m4_ifdef([SHMEM],[ call barrier()])
         TIMEDIFF(tt1,t0)
])
c
c     ---- fourier transform in z direction locally ----
c
         TIMEGET(t0)
         m4_ifdef([VECTOR],[ 
         call ccfftm(0, nnz, gs%lorder, 1.d0, ufb(1), 
     $        ldz, ufb(1), ldz, ffts%workspace(dwz),
     $        ffts%workspace(dw2), 0)

         call ccfftm(1, nnz, gs%lorder, 1.d0, ufb(1), 
     $        ldz, ufb(1), ldz, ffts%workspace(dwz),
     $        ffts%workspace(dw2), 0) 
         ])
         FFT_AUX_SPACE(ffts%workspace(dwz),ffts%naux(3),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
         FFT_LOCAL_INIT(nnz,ufb(1),ldz,gs%lorder,1)
         FFT_MULTIPLE_FORWARD(nnz,ufb,ldz,gs%lorder,i,ufb,1)
         FFT_LOCAL_FREE

         TIMEDIFF(tzfft,t0)

c        Set the data up for the first fourier transform.
c        All the rods on this processor are put next to each other
c        into the unfold buffer, and then fourier transformed with 
c        a block routine for optimum performance.
c 
         TIMEGET(t0)
         call fftfold(gsdat(1),gs,ufb(1),ldz)
         TIMEDIFF(tunfold,t0)
      endif

      m4_ifdef([SHMEM],[
      call shpdeallc(t1buf_p,ierr,-1)
      call shpdeallc(ufb_p,ierr,-1)
      call shpdeallc(sendbuf_p,ierr,-1)
      ])

      return


 100  format('TIME FOR ',a30,':',f12.6)


      end
c
c     ======================================================================
c

      subroutine fourier_transform_3(idir,  ffts, gs, rspace)
c
c
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
m4_ifdef([MPI],[
      include 'mpif.h' ])
m4_ifdef([TIMING],[      
      include 'timing.h'])

c     INPUT:
c     -----

      integer idir              ! if >=0  rspace ==> gspace
     $                          ! if <0   rspace <== gspace
      type (fftstruc) ffts      ! work arrays for the FFT
      type (parallel_gspace) gs ! the gspace for which the data is set up
c
c     INPUT/OUTPUT:
c     ------------
c
      complex(kind=8)
     $     rspace(gs%r_size)    ! the data in realspace rspace(ffts%r_size)
c                                 on the cray pvp, this is untouched, and the
c                                 output is in ffts%unfoldbuf

c
c     DESCRIPTION:
c     -----------
c
c     performs the first stage of a forward or the last stage of an
c     inverse 3dFFT.
c
c     ON THE VECTOR MACHINES, THE OUTPUT IS ACTUALLY RETURNED IN 
c     THE FFTS%UNFOLDBUF ARRAY. This is because on the vector
c     machines, the stride should not be a power of two, but that is
c     likely to be the case for a nice FFT grid.
c
c
c
      integer i,j, nnx, nny,nnz, nrodst2, dwx, dw2,ldx,ldy,ldz,
     $     nz,ny,k,ierr

      double precision 
     $     gimmetime, t0

      external gimmetime

      FFT_HEADER

      nnx=gs%fftsize(1); nny=gs%fftsize(2); nnz= gs%fftsize(3);
      nrodst2= (nny*nnz)/gs%nproc 
      if(mod(nny*nnz,gs%nproc).gt.gs%myproc)  nrodst2=nrodst2+1
      dwx=1; 
      dw2= ffts%naux(1)+ffts%naux(2)+ffts%naux(3)+1; 

      ldx=nnx;      ldy=nny;      ldz=nnz

      m4_ifdef([VECTOR],[
      if(mod(ldx,2).eq.0) ldx=ldx+1
      if(mod(ldy,2).eq.0) ldy=ldy+1
      if(mod(ldz,2).eq.0) ldz=ldz+1])

      if(idir.lt.0) then ! from gspace to realspace
c
c           inverse fourier transform in x direction locally
c
c     
         TIMEGET(t0)
         FFT_AUX_SPACE(ffts%workspace(dwx),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)

         m4_ifdef([FAST_CONVOLUTE],[
         FFT_LOCAL_INIT(nnx,rspace(1),ldx,1,-1)
         do i=0, nrodst2-1
            j=i*nnx+1
            if(gs%fftsize(5).gt.gs%fftsize(4)+1)
     $           rspace(j+gs%fftsize(4)+1:j+gs%fftsize(5)-1)=0 ! clear array
            FFT_BACKWARD(nnx,rspace(j),ldx)
         end do
         ],[
         FFT_LOCAL_INIT(nnx,rspace(1),ldx,nrodst2,-1)
         FFT_MULTIPLE_BACKWARD(nnx,rspace,ldx,nrodst2,i,ffts%unfoldbuf,1)
         ])

         FFT_LOCAL_FREE


         m4_ifdef([VECTOR],[ 
         call ccfftm(0, nnx, nrodst2, 1.d0,ffts%unfoldbuf(1), 
     $        ldx, rspace(1), nnx, ffts%workspace(dwx),
     $        ffts%workspace(dw2), 0)
         call ccfftm(-1, nnx, nrodst2, 1.d0,ffts%unfoldbuf(1), 
     $        ldx, rspace(1), nnx, ffts%workspace(dwx),
     $        ffts%workspace(dw2), 0) 
         ])
         
         TIMEDIFF(txfft,t0)

      else                      ! -------------- forward FFT ------------

c
c        fourier transform in x direction locally
c
         TIMEGET(t0)
         FFT_AUX_SPACE(ffts%workspace(dwx),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
         FFT_LOCAL_INIT(nnx,rspace(1),ldx,nrodst2,1)
         FFT_MULTIPLE_FORWARD(nnx,rspace,ldx,nrodst2,i,ffts%unfoldbuf,1)
         FFT_LOCAL_FREE

         m4_ifdef([VECTOR],[
         call ccfftm(0, nnx, nrodst2, 1.d0, rspace(1), 
     $        nnx, ffts%unfoldbuf(1), ldx, ffts%workspace(dwx),
     $        ffts%workspace(dw2), 0)
         call ccfftm(1, nnx, nrodst2, 1.d0, rspace(1), 
     $        nnx, ffts%unfoldbuf(1), ldx, ffts%workspace(dwx),
     $        ffts%workspace(dw2), 0) 
         ])

         TIMEDIFF(txfft,t0)

      endif

      return

      end

c
c     ======================================================================
c

      subroutine fast_convolute(ffts, gs, gsdat, vgsdat, rspace, vloc)
c
c
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
m4_ifdef([MPI],[
      include 'mpif.h' ])
m4_ifdef([TIMING],[      
      include 'timing.h'])

c     INPUT:
c     -----

      type (fftstruc) ffts      ! work arrays for the FFT
      type (parallel_gspace) gs ! the gspace for which the data is set up

      complex(kind=8) vloc(ffts%r_size) ! realspace potential
c
c     INPUT/OUTPUT:
c     ------------
c
      complex(kind=8)
     $     rspace(gs%r_size)    ! the data in realspace rspace(ffts%r_size)
      complex(kind=8)
     $     gsdat(gs%length)     ! the data in gspace gsdat(gs%length)

c
c     OUTPUT:
c     ------
c
      complex(kind=8)
     $     vgsdat(gs%length)    ! the data in gspace multiplied with vloc
c                                 vgsdat(gs%length)

c
c     DESCRIPTION:
c     -----------
c
c     performs the last stage of the inverse FFT, multiplies with
c     vloc, and does the first stage of the forward FFT
c
      integer 
     $     i,j,k,nnx,nny,nnz, nrodst2, dwx, dw2,ldx,ldy,ldz,nblock,n

      double precision 
     $     gimmetime, t0

      complex(kind=8), allocatable :: work(:)
      external gimmetime

      FFT_HEADER

      nnx=gs%fftsize(1); nny=gs%fftsize(2); nnz= gs%fftsize(3);
      nrodst2= (nny*nnz)/gs%nproc 
      if(mod(nny*nnz,gs%nproc).gt.gs%myproc)
     $     nrodst2=nrodst2+1
      dwx=1; 
      dw2= ffts%naux(1)+ffts%naux(2)+ffts%naux(3)+1; 
      nblock=1


      ldx=nnx;      ldy=nny;      ldz=nnz

      m4_ifdef([VECTOR],[
      if(mod(ldx,2).eq.0) ldx=ldx+1
      if(mod(ldy,2).eq.0) ldy=ldy+1
      if(mod(ldz,2).eq.0) ldz=ldz+1
      ])

      TIMEGET(t0)
      
      nblock = FFT_FCBLOCK

      m4_ifdef([RS6K],[allocate(work(ffts%naux(1)))
      FFT_AUX_SPACE(work(1),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
      FFT_LOCAL_INIT(nnx,rspace(1),ldx,nblock,1)
      ])

      FFT_AUX_SPACE(ffts%workspace(dwx),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
      FFT_LOCAL_INIT(nnx,rspace(1),ldx,nblock,-1)
c
c     now do the blocked loop
c
      do i=0, nrodst2-nblock,nblock
         j=i*ldx+1
         if(gs%fftsize(5).gt.gs%fftsize(4)+1) then
            do n=0,nblock-1     ! clear the array
               rspace(n*ldx+j+gs%fftsize(4)+1:n*ldx+j+gs%fftsize(5)-1)=0
            end do
         endif
         FFT_AUX_SPACE(ffts%workspace(dwx),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)

         m4_ifdef([HAS_MULT_FFT],[
         FFT_MULTIPLE_BACKWARD(nnx,rspace,ldx,nblock,j,rspace,1+i*nnx)  ],[
         FFT_BACKWARD(nnx,rspace(1+nnx*i),ldx)
         ])
c
c        mult with realspace potential
c
         FFT_CONVOL(nnx*nblock,rspace,vloc,j)

c        generic loop for arbitrary block size
c         do n=0,nblock-1    
c            rspace(j+n*ldx:j+n*ldx+nnx-1)=
c     $        rspace(j+n*ldx:j+n*ldx+nnx-1)*vloc(j+n*ldx:j+n*ldx+nnx-1)
c         end do

         m4_ifdef([RS6K],[
         FFT_AUX_SPACE(work(1),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
         ])

         m4_ifdef([HAS_MULT_FFT],[
         FFT_MULTIPLE_FORWARD(nnx,rspace,ldx,nblock,j,rspace,1+i*nnx)  ],[
         FFT_FORWARD(nnx,rspace(1+nnx*i),ldx)
         ])
      end do
      FFT_LOCAL_FREE

      m4_ifdef([RS6K],[
      FFT_AUX_SPACE(work(1),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
      FFT_LOCAL_INIT(nnx,rspace(1),ldx,1,1)
      ])

      FFT_AUX_SPACE(ffts%workspace(dwx),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
      FFT_LOCAL_INIT(nnx,rspace(1),ldx,1,-1)
c
c     now finish off what is left
c
      k=i
      do i=k, nrodst2-1
         j=i*nnx+1
         if(gs%fftsize(5).gt.gs%fftsize(4)+1)
     $        rspace(j+gs%fftsize(4)+1:j+gs%fftsize(5)-1)=0 ! clear array
         FFT_AUX_SPACE(ffts%workspace(dwx),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts)
         FFT_BACKWARD(nnx,rspace(1+nnx*i),ldx)

         FFT_CONVOL(nnx,rspace,vloc,j)

         m4_ifdef([RS6K],[FFT_AUX_SPACE(work(1),ffts%naux(1),ffts%workspace(dw2),ffts%naux(4),fs,dxml_ffts) ])
         FFT_FORWARD(nnx,rspace(1+nnx*i),ldx)

      end do

      FFT_LOCAL_FREE


      m4_ifdef([RS6K],[deallocate(work)])


      TIMEDIFF(txfft,t0)

      return

      end

c
c     ======================== SERVICE ROUTINES ==========================
c

      subroutine fftunfold(folded,gs,unfolded,ldz)
c
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'

c     INPUT:
c     -----

      type (parallel_gspace) gs ! the gspace for the data
      complex(kind=8)
     $     folded(gs%length)    ! the data in gspace
      integer ldz               ! leading dimension

c
c     OUTPUT:
c     ------

      complex(kind=8)
     $     unfolded(ldz, gs%lorder)

c
c     the unfold array must have the size Nz* number of rods held by
c     this proc, i.e. gs%lorder * Nz
c
c
c
c     DESCRIPTION:
c     -----------
c
c     Transfers the rods from the packed gspace arrangement onto
c     a linear grid of length gs%fftsize(3), such that a subsequent
c     1d FFT can be performed
c
c     ----------------- local variables -----------------------
c
      integer i,ii,nz,iord,istart,iend,iid,it


      unfolded=0                ! blank out target array

c
c     do not remove the it variable. It helps to circumvent a
c     compiler bug on the SGI PowerChallenge Fortran 90 v.7.0
c     
c

      nz=gs%fftsize(3)

      ii=1
      it=0
      do iord=1,gs%lorder
         istart = gs%order(2,iord)
         iend   = gs%order(3,iord)
         it=it+iend
         if(istart.le.iend) then ! there is only one contiguous block to copy:
            iid    = ii + mod(iend-istart+nz,nz)
            unfolded(istart+1:iend+1,iord)=folded(ii:iid)
         else                   ! there are two blocks to copy
            iid    = ii + nz-istart -1
            unfolded(istart+1:nz,iord)=folded(ii:iid)
            ii     = iid+1
            iid    = ii + iend
            unfolded(1:iend+1,iord) = folded(ii:iid)
         endif


         ii = iid+1             ! new start  =    old end +1

      end do


      if(it.lt.0) write(9,*) it

      end


c     -----------------------------------------------------------------------

      subroutine fftfold(folded,gs,unfolded,ldz)
c
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
c
c     1996 Bernd Pfrommer
c
c     INPUT:
c     -----

      type (parallel_gspace) gs ! the gspace for the data
      integer ldz
      complex(kind=8)
     $     unfolded(ldz, gs%lorder)
c
c     OUTPUT:
c     ------

      complex(kind=8)
     $     folded(gs%length)    ! the data in gspace

c     DESCRIPTION:
c     -----------
c
c     Collects the data from a 1d grid into the packed gspace
c     arrangement.
c
c     Scales the result by   1/nx*ny*nz, except for the alpha and convex
c
c     the unfold array must have the size Nz* number of rods held by
c     this proc, i.e. gs%lorder * Nz.
c
c     the fold array must have the length of the gspace
c
c
c     ----------------- local variables -----------------------
c
      integer i,ii,nz,iord,istart,iend,iid,it
c
c     do not remove the it variable. It helps to circumvent a
c     compiler bug on the SGI PowerChallenge Fortran 90 v.7.0
c
      double precision 
     $     sc

      sc=1.d0/dble(gs%fftsize(1)*gs%fftsize(2)*gs%fftsize(3))
      m4_ifdef([ALPHA],[sc=1.d0])
      m4_ifdef([CVX],[sc=1.d0])

      nz=gs%fftsize(3)

      ii=1
      it=0
      do iord=1,gs%lorder
         istart = gs%order(2,iord)
         iend   = gs%order(3,iord)
         it=it+iend
         if(istart.le.iend) then ! there is only one contiguous block to copy:
            iid    = ii + mod(iend-istart+nz,nz)
            folded(ii:iid)=unfolded(istart+1:iend+1,iord)*sc
         else                   ! there are two blocks to copy
            iid    = ii + nz-istart -1
            folded(ii:iid)=unfolded(istart+1:nz,iord)*sc
            ii     = iid+1
            iid    = ii + iend
            folded(ii:iid)=unfolded(1:iend+1,iord)*sc
         endif
c         write(9,*)  iord, unfolded(:,iord)
         ii = iid+1             ! new start  =    old end +1

      end do



      if(it.lt.0) write(9,*) it
      

      end

c
c     ============================================================

      subroutine fft_convolute(gs, ffts, vpsi,vloc, psi)
c
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'

      include 'timing.h'
c
c     1996 Bernd Pfrommer
c

c     INPUT:
c     -----

      type (parallel_gspace) gs ! the gspace for the data
      type (fftstruc) ffts      ! work arrays for the fft 

      complex(kind=8)
     $     vloc(gs%r_size),     ! the local potential in realspace
     $     psi(gs%length)       ! the original wave function
c
c     OUTPUT:
c     ------

      complex(kind=8)            ! vpsi = vloc * psi
     $     vpsi(gs%length)


c     DESCRIPTION:
c     -----------
c
c     Applies the local potential vloc to a wave function psi 
c     by 
c            1) inverse FFT to realspace
c            2) Multiply with vloc
c            3) FFT to gspace
c
c
c     ---------------------- local variables ----------------------
c
      integer i
      double precision gimmetime,t0,ttot, t12i,t12f,tfc
      external gimmetime

      interface
        subroutine fourier_transform_12(idir,ffts,gs,gsdat,rspace)
          include 'use.h'
          integer idir
          type (fftstruc) ffts
          type (parallel_gspace) gs
          complex(kind=8), target :: rspace(gs%r_size)
          complex(kind=8) gsdat(gs%length)
        end subroutine
      end interface

      FFT_HEADER

c
c     now convolute with local potential
c

      m4_ifdef([FAST_CONVOLUTE],[
c
c     this is the faster convolute technique
c
      TIMEGET(t0)
      call fourier_transform_12(-1, ffts, gs,psi(1),
     $     ffts%rspacebuf)
      m4_ifdef([TIMING],[
      TIMEDIFF(ttot,t0)
      t12i = ttot
      ttot=100/ttot
      write(9,123) tyfft, tzfft, tunfold, ttrods,
     $     t0t1buf,t0rspace,  tlt1, tlt2, tt1, tt2
      write(9,124) tyfft*ttot, tzfft*ttot, 
     $     tunfold*ttot, ttrods*ttot, t0t1buf*ttot,
     $     t0rspace*ttot,  tlt1*ttot, tlt2*ttot, 
     $     tt1*ttot, tt2*ttot    
      ])
      TIMEGET(t0)
      call fast_convolute(ffts, gs,psi(1),vpsi(1),
     $     ffts%rspacebuf,vloc(1))
      TIMEDIFF(tfc,t0)

      m4_ifdef([TIMING],[
      TIMEGET(t0)
      tunfold=0; txfft=0; tyfft=0; tzfft=0; t0rspace=0; ttrods=0
      t0t1buf=0; tlt1=0; tt1=0; tlt2=0; tt2=0
      ])
      call fourier_transform_12(1, ffts, gs,vpsi(1),ffts%rspacebuf)
      m4_ifdef([TIMING],[
      TIMEDIFF(ttot,t0)
      t12f = ttot
      ttot=100/ttot
      write(9,223) tyfft, tzfft, tunfold, ttrods,
     $     t0t1buf,t0rspace,  tlt1, tlt2, tt1, tt2
      write(9,224) tyfft*ttot, tzfft*ttot, 
     $     tunfold*ttot, ttrods*ttot, t0t1buf*ttot,
     $     t0rspace*ttot,  tlt1*ttot, tlt2*ttot, 
     $     tt1*ttot, tt2*ttot
      ttot=100/(t12i+t12f+tfc)
      write(9,110) 100/ttot,t12i*ttot, tfc*ttot,t12f*ttot])
      
      ],[
c
c     for some architectures, standard technique is better
c
      call fourier_transform(-1, ffts, gs,psi(1),ffts%rspacebuf(1))
      FFT_CONVOL(ffts%r_size,ffts%rspacebuf,vloc,1)
      call fourier_transform(1, ffts, gs,vpsi(1),ffts%rspacebuf(1))

      ])



 100  format('TIME FOR ',a30,f12.6)
 110  format('FFT:',f8.4,'   FFTI:',f8.1,'  CONV:',f8.1,'  FFTF:',f8.1)
 123  format('FFTI:',2f8.4,', UFOLD:',f8.4,', TRODS:',f8.4,
     $     ', ZBUF:', f8.4,', ZRSPC:', f8.4,', TLT1:', f8.4,
     $     ', TLT2:', f8.4,', TC1:', f8.4,', TC2:', f8.4)
 124  format('FFTI:',2f8.1,', UFOLD:',f8.1,', TRODS:',f8.1,
     $     ', ZBUF:', f8.1,', ZRSPC:', f8.1,', TLT1:', f8.1,
     $     ', TLT2:', f8.1,', TC1:', f8.1,', TC2:', f8.1)
 223  format('FFTF:',2f8.4,', UFOLD:',f8.4,', TRODS:',f8.4,
     $     ', ZBUF:', f8.4,', ZRSPC:', f8.4,', TLT1:', f8.4,
     $     ', TLT2:', f8.4,', TC1:', f8.4,', TC2:', f8.4)
 224  format('FFTF:',2f8.1,', UFOLD:',f8.1,', TRODS:',f8.1,
     $     ', ZBUF:', f8.1,', ZRSPC:', f8.1,', TLT1:', f8.1,
     $     ', TLT2:', f8.1,', TC1:', f8.1,', TC2:', f8.1)


      end

c



