C     
      subroutine generate_kpoints(ipr,
     $     kp,crys,symms,bands,iflag,qshift)

      include 'use.h'
      implicit none
      include 'interface.h'
      integer ipr               ! print flag
      integer iflag
      double precision qshift(3)
c
c     INPUT:
c     -----
c
      type (crystal) crys
      type (symmetry) symms
c
c     OUTPUT:
c     ------
c
      type (kpoint) kp
      type (band) bands
c
c     computes k-points for integration
c     over the brillouin zone. periodic mesh.
c     adapted from sverre froyen plane wave program
c     written june 1 1987. jlm
c     modified july 31 1987. jlm
c
c     fortran 90 version 1996 Bernd Pfrommer

c     DESCRIPTION:
c     -----------

c     * computes kpoints in irreducible zone
c     * computes kmap, which maps reducible to irreducible kpoints
c

c     ----------------- local variables ------------------------------
c
      double precision, allocatable :: 
     $     xk(:,:), ekk(:)
      double precision fk2(3,10000)
      integer nfk2,indsubq(48),ntranq,nrk2,indrq2(10000)
      
      integer
     $     nx,ny,nz, ntrans, neg, km, kl,
     $     i,j,k,l,n,maxn,nrk,jp,irk,im, jm,ip,jcar,jabs

      double precision
     $     wsum,                ! sum of weights
     $     sx,sy,sz, ek1, ek, ek2, dx,dy,dz, dw, diff,
     $     rkt(3), rktran(3), rkmod(3,4), rkcar(3), rktran_inv(3)

      double precision
     $     zero, um, eps, reduce_01, x_dummy, qnorm
      
      integer 
     $     nsafety              ! number of extra states to start diag with
      logical lshift
      integer nband1,ifmax1
      parameter (zero = 0.0d0, um = 1.0d0)
      parameter (eps = 1.0d-6)
      parameter(nsafety=5)

c     -----------------------------------------------------------------

c     
c
      reduce_01(x_dummy) = mod(x_dummy+500d0, 1.d0)
      ntrans=symms%ntrans
      lshift = .TRUE.
      qnorm = sqrt(qshift(1)**2+qshift(2)**2+qshift(3)**2)
      if (qnorm .lt. 1e-6) lshift=.FALSE.
c     
c     switch off reduction 
c
      if(.not.(kp%reduced)) ntrans=1
      
      sx=kp%shift(1)
      sy=kp%shift(2)
      sz=kp%shift(3)
      
      nx=kp%grid(1)
      ny=kp%grid(2)
      nz=kp%grid(3)

      nrk=kp%nrk

c
      maxn = nx*ny*nz
         
      nullify(kp%kmap)
      if(kp%generated) then
      
c     ------- count the number of irreducible k-points -------------

c     allocate dynamic arrays
       
         allocate(xk(3,maxn))
         allocate(kp%kmap(maxn))


c
c     set up uniform array of k points
c     kmap is used to mark reducible k points and also to
c     map reducible to irreducible k points
c
         dx = um/dble(nx)
         dy = um/dble(ny)
         dz = um/dble(nz)
         n = 0
         do i=1,nx
            do j=1,ny
               do k=1,nz
                  n = n+1
                  xk(1,n) = (dble(i-1)+sx)*dx
                  xk(2,n) = (dble(j-1)+sy)*dy
                  xk(3,n) = (dble(k-1)+sz)*dz
                  kp%kmap(n) = n
               end do
            end do
         end do
c     
c     reduce to irreducible zone
c
         nrk = 0
         dw = um/dble(n)
         do i=1,n
            if (kp%kmap(i) .eq. i) then
c     
c     new irreducible point
c     mark with negative kmap
c
               nrk = nrk+1

               rkt(1) = xk(1,i)
               rkt(2) = xk(2,i)
               rkt(3) = xk(3,i)
               kp%kmap(i) = -nrk
               if (i .ne. n) then
c     
c     operate on irreducible rk with the symmetry operations
c
                  do j=1,ntrans
                     rktran(1) = dble(symms%mtrx(1,1,j))*rkt(1) +
     +                    dble(symms%mtrx(1,2,j))*rkt(2) +
     +                    dble(symms%mtrx(1,3,j))*rkt(3)
                     rktran(2) = dble(symms%mtrx(2,1,j))*rkt(1) +
     +                    dble(symms%mtrx(2,2,j))*rkt(2) +
     +                    dble(symms%mtrx(2,3,j))*rkt(3)
                     rktran(3) = dble(symms%mtrx(3,1,j))*rkt(1) +
     +                    dble(symms%mtrx(3,2,j))*rkt(2) +
     +                    dble(symms%mtrx(3,3,j))*rkt(3)
c     
c     translate to interval 0-1.
c
                        
                     do k=1,3
                        rktran(k) = reduce_01(rktran(k))
                        rktran_inv(k) = reduce_01(-rktran(k))
                     end do
c
c     remove (mark) k points related to irreducible rk by symmetry
c
                     ip = i+1
                     do k=ip,n
                        if (kp%kmap(k) .eq. k) then
c     
c     both the transformed rk ...
c
                           do l=1,3
                              if (abs(rktran(l)-xk(l,k)) 
     $                             .gt. eps) goto 930
                           end do
                           kp%kmap(k) = nrk
                           goto 940
c     
c     ... and its inverse (time reversal)
c
 930                       continue
            if(iand(iflag,1048576).eq.1048576) goto 940
                           do l=1,3
                              diff = abs(rktran_inv(l)-xk(l,k))
                              if(.not.(kp%reduced)
     $                             .or.(diff .gt. eps)) goto 940
                           end do
                           kp%kmap(k) = nrk
 940                       continue
                        endif
                     end do
                  end do
               endif
            endif
         end do
      
      endif
      
c     -------- read kpt --------------------------------------------------

      if(.not.kp%generated) then
c
c     read in number of eigenvalues to be computed, the lowest and
c     highest filled band, the integration weight and the reduced
c     k-vector.
c
         open(unit=7,
     $        file='KPOINTS',
     $        status='unknown', 
     $        form='formatted')
         read(7,*) nrk
         kp%nrk = nrk
         call create_kpoint(kp,nrk)
         call create_band(bands,nrk,bands%min+nsafety, 
     $        crys%nspin)       ! give 5 bands safety
         allocate(ekk(nrk))

         do i = 1, nrk
            read(7,*) kp%label(i), bands%nband(i,1), 
     $           k, bands%ifmax(i,1), kp%w(i), (kp%rk(j,i),j=1,3)
c 9000       format(f8.3,3i4,f7.2,3f16.8)
         end do
         bands%nband=bands%nband+nsafety
         bands%ifmax(:,bands%nspin)=bands%ifmax(:,1)
         bands%nband(:,bands%nspin)=bands%nband(:,1)
         write(9,100) nrk

         close(7)

         do i=1, nrk            ! calculate length of vectors
            ekk(i)=0.d0
            do j=1,3
               do k=1,3
                  ekk(i)=ekk(i)+kp%rk(j,i)*crys%bdot(j,k)*kp%rk(k,i)
               end do
            end do
         end do
c     
c     --------------------- generate kpts ---------------------------
      else                     

c
c        allocate space for kpoints and other work arrays
c

         call create_kpoint(kp,nrk)
         call create_band(bands,nrk,bands%min+nsafety, 
     $        crys%nspin)       ! give 5 bands safety
         allocate(ekk(nrk))

      

c
c     the points will be computed from parameters
c     nx, ny, and nz are the number of points in the three
c     directions dermined by the lattice wave vectors. sx, sy, and
c     sz shifts the grid of integration points from the origin.
c

         maxn = nx*ny*nz
      
c
c     set up uniform array of k points
c     kmap is used to mark reducible k points and also to
c     map reducible to irreducible k points
c
         dx = um/dble(nx)
         dy = um/dble(ny)
         dz = um/dble(nz)
         n = 0
         do i=1,nx
            do j=1,ny
               do k=1,nz
                  n = n+1
                  xk(1,n) = (dble(i-1)+sx)*dx
                  xk(2,n) = (dble(j-1)+sy)*dy
                  xk(3,n) = (dble(k-1)+sz)*dz
                  kp%kmap(n) = n
               end do
            end do
         end do
c
c      reduce to irreducible zone
c
         nrk = 0
         dw = um/dble(n)
         do 20 i=1,n
            if (kp%kmap(i) .eq. i) then
c
c     new irreducible point
c     mark with negative kmap
c
               nrk = nrk+1
               kp%rk(1,nrk) = xk(1,i)
               kp%rk(2,nrk) = xk(2,i)
               kp%rk(3,nrk) = xk(3,i)
               kp%kmap(i) = -nrk
               kp%w(nrk) = dw
               if (i .ne. n) then
c
c     operate on irreducible rk with the symmetry operations
c
                  do 21 j=1,ntrans
                     rktran(1) = dble(symms%mtrx(1,1,j))*kp%rk(1,nrk) +
     +                    dble(symms%mtrx(1,2,j))*kp%rk(2,nrk) +
     +                    dble(symms%mtrx(1,3,j))*kp%rk(3,nrk)
                     rktran(2) = dble(symms%mtrx(2,1,j))*kp%rk(1,nrk) +
     +                    dble(symms%mtrx(2,2,j))*kp%rk(2,nrk) +
     +                    dble(symms%mtrx(2,3,j))*kp%rk(3,nrk)
                     rktran(3) = dble(symms%mtrx(3,1,j))*kp%rk(1,nrk) +
     +                    dble(symms%mtrx(3,2,j))*kp%rk(2,nrk) +
     +                    dble(symms%mtrx(3,3,j))*kp%rk(3,nrk)
c
c     translate to interval 0-1.
c
                     
                     do k=1,3
                        rktran(k) = reduce_01(rktran(k))
                        rktran_inv(k) = reduce_01(-rktran(k))
                     end do
c
c     remove (mark) k points related to irreducible rk by symmetry
c
                     ip = i+1
                     do 24 k=ip,n
                        if (kp%kmap(k) .eq. k) then
c
c     both the transformed rk ...
c
                           do l=1,3
                              if (abs(rktran(l)-xk(l,k)) 
     $                             .gt. eps) goto 130
                           end do
                           kp%w(nrk) = kp%w(nrk)+dw
                           kp%kmap(k) = nrk
                           goto 140
c
c     ... and its inverse (time reversal)
c
 130                       continue
            if(iand(iflag,1048576).eq.1048576) goto 140
                           do l=1,3
                              diff = abs(rktran_inv(l)-xk(l,k))
                              if (.not.kp%reduced
     $                             .or.(diff .gt. eps)) goto 140
                           end do
                           kp%w(nrk) = kp%w(nrk)+dw
                           kp%kmap(k) = nrk
 140                       continue
                        endif
 24                  continue
 21               continue
               endif
            endif
 20      continue

c     
c     set up array indk
c     indk(1-6,i) gives the index for irreducible points that
c     are neighbors of point i
c
         n = 0
         do i=1,nx
            do j=1,ny
               do k=1,nz
                  n = n+1
                  if (kp%kmap(n) .le. 0) then
                     irk = -kp%kmap(n)
                     ip = i+1
                     jp = j+1
                     kl = k+1
                     if (i .eq. nx) ip=1
                     if (j .eq. ny) jp=1
                     if (k .eq. nz) kl=1
                     im = i-1
                     jm = j-1
                     km = k-1
                     if (i .eq. 1) im=nx
                     if (j .eq. 1) jm=ny
                     if (k .eq. 1) km=nz
                     ip = ((ip-1)*ny + j -1)*nz + k
                     kp%ind(1,irk) = iabs(kp%kmap(ip))
                     im = ((im-1)*ny + j -1)*nz + k
                     kp%ind(2,irk) = iabs(kp%kmap(im))
                     jp = ((i -1)*ny + jp-1)*nz + k
                     kp%ind(3,irk) = iabs(kp%kmap(jp))
                     jm = ((i -1)*ny + jm-1)*nz + k
                     kp%ind(4,irk) = iabs(kp%kmap(jm))
                     kl = ((i -1)*ny + j -1)*nz + kl
                     kp%ind(5,irk) = iabs(kp%kmap(kl))
                     km = ((i -1)*ny + j -1)*nz + km
                     kp%ind(6,irk) = iabs(kp%kmap(km))
                  endif
               end do
            end do
         end do
         wsum = zero
         
         do i=1,nrk
            bands%nband(i,1) = bands%max
            bands%nband(i,bands%nspin) = bands%max
            bands%ifmax(i,1) = bands%min
            bands%ifmax(i,bands%nspin) = bands%min
            wsum = wsum + kp%w(i)
         end do
c
c     find rk**2, etc por the printout
c
         write(9,101) nrk,nx,ny,nz,sx,sy,sz,bands%min
         write(9,102)
c
         do 50 i=1,nrk
            do j=1,4
               do k=1,3
                  rkmod(k,j) = kp%rk(k,i)
               end do
            end do
            rkmod(1,2) = rkmod(1,2)-um
            rkmod(2,3) = rkmod(2,3)-um
            rkmod(3,4) = rkmod(3,4)-um
            ek = 2000.0
            do j=1,4
               ek1 = zero
               ek2 = zero
               do k=1,3
                  do l=1,3
                     ek1=ek1+rkmod(k,j)*crys%bdot(k,l)*rkmod(l,j)
                     ek2=ek2+(rkmod(k,j)-um)*
     $                    crys%bdot(k,l)*(rkmod(l,j)-um)
                  end do
               end do
               if (ek1 .lt. ek) then
                  ek = ek1
                  jcar = j
               endif
               if (ek2 .lt. ek) then
                  ek = ek2
                  jcar = -j
               endif
            end do
c
c        rk in cartesian coordinates
c
            neg = 0
            do k=1,3
               jabs = iabs(jcar)
               rkcar(k) = crys%bvec(k,1)*rkmod(1,jabs) +
     +              crys%bvec(k,2)*rkmod(2,jabs) +
     +              crys%bvec(k,3)*rkmod(3,jabs)
               if (jcar .lt. 0) rkcar(k) = rkcar(k) -
     -              crys%bvec(1,k)-crys%bvec(2,k)-crys%bvec(3,k)
               if (rkcar(k) .lt. zero) neg = neg + 1
            end do
            if (neg .le. 1) neg = 1
            if (neg .gt. 1) neg = -1
c
c     printout
c
            write(9,103) i,kp%w(i),(kp%rk(j,i),j=1,3),
     $           (dble(neg)*rkcar(j),j=1,3),ek
 50      continue
c
         if (abs(wsum) .lt. eps) wsum=zero

c     ------ calculate |k|^2 for all kpoints
       
         do i=1, nrk
            ekk(i)=0.d0
            do j=1,3
               do k=1,3
                  ekk(i)=ekk(i)+kp%rk(j,i)*crys%bdot(j,k)*kp%rk(k,i)
               end do
            end do
         end do
         deallocate(xk)
      endif                     ! for generate kpoints

      deallocate(ekk)

      call myflush(9)

cxav
      if (lshift) then

      call fullbzx(nrk,kp%rk,nfk2,fk2,symms)
      call subgrpqx(qshift,ntranq,indsubq,symms)
      call irrbzqx(nfk2,fk2,ntranq,indsubq,nrk2,indrq2,symms)
      deallocate(kp%rk)
      deallocate(kp%w)
      deallocate(kp%label)
      deallocate(kp%ind)     
      allocate(kp%rk(3,nrk2))
      allocate(kp%w(nrk2))
      allocate(kp%label(nrk2))
      allocate(kp%ind(6,nrk2))
      kp%label = 0      

      nband1 = bands%nband(1,1)
c      ifmin1 = bands%ifmin(1,1)
c      ifmin1 = 1 
      ifmax1 = bands%ifmax(1,1)

      deallocate(bands%nband)
c      deallocate(bands%ifmin)
      deallocate(bands%ifmax)
      deallocate(bands%occup)
      deallocate(bands%energy)
      deallocate(bands%ekn)

c      bands%occup = 0.0
c      bands%energy = 0.0
c      bands%ekn = 0.0
c      kp%ind = 0
c      kp%label = 0

      allocate(bands%nband(nrk2,bands%nspin))
c      allocate(kp%ifmin(nrk2,crys%nspin))
      allocate(bands%ifmax(nrk2,bands%nspin))
      allocate(bands%ekn(bands%max,nrk2,bands%nspin))
      allocate(bands%occup(bands%max,nrk2,bands%nspin))
      allocate(bands%energy(bands%max,nrk2,bands%nspin))
       
      bands%occup = 0.0
      bands%energy = 0.0
      bands%ekn = 0.0
      kp%ind = 0
      kp%label = 0
      kp%shift = 0.0
      kp%grid = 0

cxav
       kp%nrk = nrk2
       bands%nrk = nrk2
       kp%generated = .false.
       write(9,104) nrk2
       do i=1,nrk2
        kp%rk(1,i) =  fk2(1,indrq2(i)) + qshift(1)
        kp%rk(2,i) =  fk2(2,indrq2(i)) + qshift(2)
        kp%rk(3,i) =  fk2(3,indrq2(i)) + qshift(3)
        kp%w(i) = 1.0d0/dble(nrk2)
        bands%nband(i,1) = nband1       
c        bands%ifmin(i,1) = 1
        bands%ifmax(i,1) = bands%min
c        bands%ifmax(i,1) = 4
       enddo
       bands%ifmax(:,bands%nspin)=bands%ifmax(:,1)
       bands%nband(:,bands%nspin)=bands%nband(:,1)       
       endif

c      call qualitykp(crys, symms, kp)
c      stop 1494
      
      RETURN

 100  FORMAT(//' K POINTS READ FROM FILE: ',i5)
 101  FORMAT(//' K POINT GENERATION:',
     $     /' ------------------',
     $     /1X,I5,' K POINTS GENERATED BY PROGRAM FROM',
     $     ' PARAMETERS :',/,4X,'N =',3I5,6X,'S =',3F6.2,6X,
     $     'NB =',I5,/)
 102  FORMAT(4X,'I',7X,'W',17X,'RK',22X,'RKCAR',12X,'EKIN',/)
 103  FORMAT(1X,I4,3X,F8.4,3X,3F8.4,3X,3F7.3,3X,F7.3)
 104  FORMAT(/,1X,I5,' K POINTS GENERATED FOR',
     $     ' THE GW SLIGHTLY SHIFTED GRID')
      
 120  format(' READ ',i5,' K-POINTS FORM FILE KPOINTS')
      END

c
c
c
c ADDITIONAL SERVICE ROUTINES, X. Blase, E. Chang
c
c-------------------------------------
       subroutine subgrpqx(qi,ntranq,indsub,symms)
c
       include 'use.h'
       implicit double precision(a-h,o-z)
c
c       include 'symmetry.h'
c       include 'use.h'
       type (symmetry) symms
c
c......comunication scalar and arrays
       integer ntranq,indsub(48)
       double precision qi(3)
c
c......local arrays
       double precision qk(3),g0(3)
c
c we look for operations such that  R(q) = q + g0q
c
       ntranq=0
       do 200 itran=1,symms%ntrans
c
        do 110 i=1,3
        qk(i)=0
         do 105 j=1,3
          qk(i)=qk(i) + symms%mtrx(i,j,itran)*qi(j)
 105     continue
 110    continue
c
        call bring_to_01(qk,g0)
c
        diff = dabs(qk(1)-qi(1)) +
     1         dabs(qk(2)-qi(2)) +
     2         dabs(qk(3)-qi(3))
c
        if (diff .gt. 0.1d-5) goto 200
c
c store index element of subroup
c
        ntranq=ntranq+1
        indsub(ntranq)=itran
c
 200   continue
c
c print out
c
c      write(9,9000) ntranq,(indsub(i),i=1,ntranq)
c9000  format(/,i3,' operations in small group of q:',48i4)
c
       return
       end
ccccccccccccccccccccccccccccccccccccccccccccc
       subroutine fullbzx(NRK,RK,NFK,FK,symms)
C
        include 'use.h'
       IMPLICIT double precision (A-H,O-Z)
       integer nfkp
       parameter(nfkp=10000)
c       include 'bands.i'
C
C      SUBROUTINE TO DETERMINE POINTS IN FBZ FROM
C      K-POINTS IN IRR BZ
C
       dimension RK(3,10000)
c
c       include 'symmetry.h'
c       include 'use.h'
       type (symmetry) symms
c
c
C
C  O   NFK,FK             K-POINTS IN FULL BZ
C                         ONE POINT IN THE SET FK
c
c.....local array
       dimension FK(3,nfkp)
       DIMENSION QK(3)
       integer mati(3,3)
       dimension xmat(3,3)
       double precision G0(3)
C
       PI = 4.0D0*DATAN(1.0D0)
C
C      LOOP OVER Q-POINTS

C
       NFK = 0
       DO 60 IRK=1,NRK
C
C      LOOP OVER TRANSFORMATIONS
C
       DO 50 IT=1,symms%ntrans
C
C      ROTATE RK
C
       DO 20 I=1,3
       QK(I) = 0.0D0
       DO 10 J=1,3
       QK(I) = QK(I) + symms%mtrx(I,J,IT)*RK(J,IRK)
 10    CONTINUE
 20    CONTINUE
C
C      TRANSLATE TO INTERVAL 0-1
C
       call bring_to_01(QK,G0)
C
C      COMPARE TO OTHER POINTS IN FULL ZONE
C
       IF (NFK .EQ. 0) GOTO 40
       IFLAG = 0
       DO 30 IFK=1,NFK
       IF (DABS(QK(1)-FK(1,IFK)) .LT. 1.0D-05
     1   .AND. DABS(QK(2)-FK(2,IFK)) .LT. 1.0D-05
     2   .AND. DABS(QK(3)-FK(3,IFK)) .LT. 1.0D-05) IFLAG = 1
 30    CONTINUE
       IF (IFLAG .EQ. 1) GOTO 50
C
C      STORE NEW KPOINT IN FBZ
C
 40    NFK = NFK + 1
c
       IF (NFK .GT. nfkp) then
         write(9,*) 'NFK,nfkp=',NFK,nfkp
         STOP 210
       endif
       FK(1,NFK) = QK(1)
       FK(2,NFK) = QK(2)
       FK(3,NFK) = QK(3)
 50    CONTINUE
 60    CONTINUE
C
C      WRITE(9,70) NFK
C70    FORMAT(5X,4HNFK=,I5)
       RETURN
       END
c---------------------------------------------------------
       subroutine irrbzqx(NFK,FK,NTRANQ,INDSUB,NRQ,INDRQ,symms)
C
       include 'use.h'
       IMPLICIT REAL * 8 (A-H,O-Z)
c

       integer nfkp
       parameter(nfkp=10000)
C
C      SUBROUTINE REDUCES FULL BZ (FK) TO IRREDUCIBLE PART (RQ)
C      USING OPERATIONS IN THE SUBGROUP OF KN
C
       DIMENSION FK(3,*),INDRQ(*)
       DIMENSION INDSUB(48),KGO(48,3)
c
c       include 'symmetry.h'
c       include 'use.h'
       type (symmetry) symms
c
c
C
C  I   NFK,FK             K-POINTS IN FULL BZ
C  O   NRQ,INDRQ          Q-POINTS IN IRR BZ WITH RESPECT TO SUBGROUP,
C                         INDEX OF RQ STORED IN INDRQ REFERING TO FK
c
C      UTILITY ARRAYS
C
       DIMENSION QK(3)
       double precision g0(3)
       double precision xmati(3,3)
C
C      INITIALIZE NUMBER OF POINTS IN IRR. BZ
C
       NRQ = 0
C
C
C      LOOP OVER K-POINTS IN FULL ZONE
C
       DO 70 IK=1,NFK
       IF (IK .EQ. 1) GOTO 60
C
C      LOOP OVER TRANSFORMATION MATRICES
C
       DO 50 IT=1,NTRANQ
c
       do k=1,3
       do l=1,3
       xmati(k,l) = dble( symms%mtrx(k,l,INDSUB(IT)) )
       enddo
       enddo
       call invers(xmati,det)
c
       DO 20 I=1,3
       QK(I) = 0.0D0
       DO 10 J=1,3
       QK(I) = QK(I) + xmati(I,J)*FK(J,IK)
 10    CONTINUE
 20    CONTINUE
c
       call bring_to_01(QK,g0)
c
C      COMPARE TO OTHER K-POINTS IN THE IRR. BZ WITH RESPECT TO QVEC
C
       DO 40 IRQ=1,NRQ
       IFLAG = 0
       DO 30 I=1,3
       IF (DABS(FK(I,INDRQ(IRQ))-QK(I)) .GT. 1.0D-06) IFLAG = 1
 30    CONTINUE
       IF (IFLAG .EQ. 0) GOTO 70
 40    CONTINUE
 50    CONTINUE
C
C      END LOOP OVER TRANSFORMATION MATRICES
C
C      STORE INDEX OF NEW K-POINT IN IRR. BZ WITH RESPECT TO Q
C
 60    NRQ = NRQ + 1
       INDRQ(NRQ) = IK
 70    CONTINUE
C
C      END LOOP OVER FULL BZ
C
c print out
c
c      write(9,9000) NRQ
c9000  format(/,' irr k-points according to subgroup of q ',i5)
c      do IRQ=1,NRQ
c      write(9,9010) INDRQ(IRQ), (FK(I,INDRQ(IRQ)),I=1,3)
c9010  format(i5,3x,3f8.5)
c      enddo
c
       RETURN
       END
c---------------------------------------------------------
       subroutine bring_to_01(x,g)
c
       implicit none
       double precision one_eps,x(3),y(3),g(3)
c
       integer i
c
       one_eps=1.0d0 - 1.0d-6
c
       do 10 i=1,3
       y(i)=mod(x(i)+500.00,1.0d0)
       if (y(i) .gt. one_eps) y(i)=0.0d0
       g(i) = x(i) - y(i)
       g(i) = - g(i)
       x(i) = y(i)
 10    continue
       return
       end

      subroutine invers(mat,determinant)
c
c     Compute the inverse of a matrix (in situ) and its determinant
c     using the SCILIB (from LINPACK) routines sgeco and sgedi
c
      double precision mat(3,3), determinant
c
      integer ipvt(3)
      double precision det(2), work(3), rcond
c
      call sgeco(mat,3,3,ipvt,rcond,work)
      if (rcond .lt. 1.d-7) then 
         write(9,'(1x,a)') 'Ill-conditioned matrix in invers!!'
         stop
      endif
c
      call sgedi(mat,3,3,ipvt,det,work,11)
c
      determinant = det(1)*10.d0**det(2)
c
      return
      end
