
/* generates a file KPOINTS with the points for the bandstructure */

#include "generate_kpts.h"
#include <math.h>

int generate_kpts(FILE *infile, int nband, char **labels)
{

  char buf[500];
  double kpoint[7],kp[3],pi2; 
  FILE *kptfile;
  int i,j,k,ierr,istop,nrk,n,nitems;
  keyword key1,key2,key3;
  
  pi2= 8.0*atan(1.0);

  kptfile = fopen("KPOINTS","w");

  setkeyword(&key1,2,"begin","bandstructure");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {printf("cannot find 'begin bandstructure' in input file\n"); exit(-1);}

  setkeyword(&key1,1,"kpoint","");
  setkeyword(&key2,1,"end","");
  setkeyword(&key3,1,"label","");

  istop = 1;
  nrk  =  0;
  nitems=0;

  while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {

    if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new kpoint line */
      if(getdouble(buf,&key1,kpoint,7)<0)  yell(buf,"getdouble");
      if((n=(int)kpoint[6])<=0) {
        yell(buf,"number of bins in error:");
      }
      nitems++;
      nrk+=n+1;
    }
    else{
      if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
      else{
	if(keytest(buf,&key3)==KEYWORD_FOUND) {}
	else{
	  printf("found funny keyword in band structure input:\n%s\n", buf);}
      }
    }
  }

  fprintf(kptfile,"%4d\n", nrk);
  *labels=malloc(nitems*100*sizeof(char));
  memset(*labels,' ',100*nitems*sizeof(char));

  setkeyword(&key1,2,"begin","bandstructure");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {printf("cannot find 'begin bandstructure' in input file\n"); exit(-1);}

  setkeyword(&key1,1,"kpoint","");
  setkeyword(&key2,1,"end","");
  setkeyword(&key3,1,"label","");

  istop = 1;
  nrk  =  0;
  nitems=0;

  while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {

    if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new kpoint line */
      if(getdouble(buf,&key1,kpoint,7)<0)  yell(buf,"getdouble");
      if((n=(int)kpoint[6])<=0) {
        yell(buf,"number of bins in error:");
      }
      for(i=0;i<=n;i++) {
        for(j=0;j<3;j++) {
          kp[j]=i*(kpoint[j+3]-kpoint[j])/n;
        }
        fprintf(kptfile,"%8.3f %3d %3d %3d %6.2f %15.8f %15.8f %15.8f\n",
                (double) (i==n?0:1),nband, 1,nband, 1.0, 
		kp[0]+kpoint[0],kp[1]+kpoint[1],kp[2]+kpoint[2]);
	nrk++;
      }
      nitems++;
    }
    else{
      if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
      else{
	if(keytest(buf,&key3)==KEYWORD_FOUND) {
	  if(getstr(buf,&key3,*labels+100*nitems,100)<0) yell(buf,"getstring");
	  *(*labels+100*nitems+strlen(*labels+100*nitems))=' ';
	}
	else{
	  printf("found funny keyword in band structure input:\n%s\n", buf);}
      }
    }
  }
  
  

  fclose(kptfile);
  return(nrk);

}
