c     @process extchk
c
      subroutine generate_potential_gspace(
     $     pot_gspace, crys, syms, pw_params, myproc, nproc)
c
c     
c     
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
c
c     INPUT:
c     -----

      type (pw_parameter) pw_params ! plane wave parameters
      type (crystal) crys       ! for the lattice vectors
      type (fftstruc) ffts
      type (symmetry) syms


      integer nproc, myproc     ! parallel info

c
c     OUTPUT:
c     ------
c

      type (parallel_gspace) 
     $     pot_gspace           ! potential gspace

c
c     DESCRIPTION:
c     -----------

c
c     generates a large potential gspace with all the features:
c     stars, gvectors, kinetic energy, structure factors
c

c
c     -------------------------------------------------------------------
c
      double precision emax     ! energy cutoff for the wave functions!

      emax=pw_params%emax

      
      pot_gspace%gmax=2.d0 * sqrt(emax) ! 4*emax
      pot_gspace%rk=(/0,0,0/);  ! no shift
      pot_gspace%nproc=nproc;   ! scatter across all processors
      pot_gspace%name='potential'
      pot_gspace%myproc=myproc; pot_gspace%fftsize=(/0,0,0,0,0/)
      pot_gspace%istar=.true. ; pot_gspace%igvec=.true.
      pot_gspace%iekin=.true. ; pot_gspace%iinverse=.false.
      pot_gspace%istruc=.true.
      pot_gspace%ipackinfo=.true.
c     
c     F. Mauri istar information if there are symmetries
c
      pot_gspace%istar=.not.(syms%ntrans.eq.1)


      call generate_gspace(pw_params%output,pot_gspace, crys, syms)

      write(9,10) pot_gspace%length,pot_gspace%fftsize(1:3)
      call myflush(9)

 10   format(' SIZE OF POTENTIAL GSPACE = ', i8,
     $      ' WITH FFT GRID ', 3i6)

      end
