c
c     THE PERDEW-WANG SUBROUTINES AS RECEIVED FROM THE AUTHORS
c     
c     variable declaration by Alberto.
c     removed common blocks 1/97 Bernd
c
c
      subroutine exch(d,s,u,v,ex,vx)
c
c     gga91 exchange for a spin-unpolarized electronic system
c     input d : density
c     input s:  abs(grad d)/(2*kf*d)
c     input u:  (grad d)*grad(abs(grad d))/(d**2 * (2*kf)**3)
c     input v: (laplacian d)/(d*(2*kf)**2)
c     output:  exchange energy per electron (ex) and potential (vx)
c
c
C     .. Scalar Arguments ..
      double precision d, ex, s, u, v, vx
C     ..
C     .. Local Scalars ..
      double precision a, a1, a2, a3, a4, ax, b1, f, fac, fs, fss, 
     &                 p0, p1, p10,
     &       p11, p2, p3, p4, p5, p6, p7, p8, p9, s2, s3, s4, thrd,
     &       thrd4
C     ..
C     .. Intrinsic Functions ..
      intrinsic exp, log, sqrt
C     ..
C     .. Data statements ..
      data a1, a2, a3, a4/0.19645D0, 0.27430D0, 0.15084D0, 100.0D0/
      data ax, a, b1/-0.7385588D0, 7.7956D0, 0.004D0/
      data thrd, thrd4/0.333333333333D0, 1.33333333333D0/
C     ..
      fac = ax*d**thrd
      s2 = s*s
      s3 = s2*s
      s4 = s2*s2
      p0 = 1.0D0/sqrt(1.0D0+a*a*s2)
      p1 = log(a*s+1.0D0/p0)
      p2 = exp(-a4*s2)
      p3 = 1.0D0/(1.0D0+a1*s*p1+b1*s4)
      p4 = 1.0D0 + a1*s*p1 + (a2-a3*p2)*s2
      f = p3*p4
      ex = fac*f
C     LOCAL EXCHANGE OPTION
c      EX = FAC
      
C     ENERGY DONE. NOW THE POTENTIAL:
      p5 = b1*s2 - (a2-a3*p2)
      p6 = a1*s*(p1+a*s*p0)
      p7 = 2.0D0*(a2-a3*p2) + 2.0D0*a3*a4*s2*p2 - 4.0D0*b1*s2*f
      fs = p3*(p3*p5*p6+p7)
      p8 = 2.0D0*s*(b1-a3*a4*p2)
      p9 = a1*p1 + a*a1*s*p0*(3.0D0-a*a*s2*p0*p0)
      p10 = 4.0D0*a3*a4*s*p2*(2.0D0-a4*s2) - 8.0D0*b1*s*f -
     &      4.0D0*b1*s3*fs
      p11 = -p3*p3*(a1*p1+a*a1*s*p0+4.0D0*b1*s3)
      fss = p3*p3*(p5*p9+p6*p8) + 2.0D0*p3*p5*p6*p11 + p3*p10 + p7*p11
      vx = fac*(thrd4*f-(u-thrd4*s3)*fss-v*fs)
C     LOCAL EXCHANGE OPTION:
c      VX = FAC*THRD4

      return
c
      end
c
c     =================================================================
c
      subroutine corlsd(rs,zet,ec,vcup,vcdn,ecrs,eczet,alfc)

C     UNIFORM-GAS CORRELATION OF PERDEW AND WANG 1991
C     INPUT: SEITZ RADIUS (RS), RELATIVE SPIN POLARIZATION (ZET)
C     OUTPUT: CORRELATION ENERGY PER ELECTRON (EC), UP- AND DOWN-SPIN
C     POTENTIALS (VCUP,VCDN), DERIVATIVES OF EC WRT RS (ECRS) & ZET
c     (ECZET)
C     OUTPUT: CORRELATION CONTRIBUTION (ALFC) TO THE SPIN STIFFNESS
c
C     .. Scalar Arguments ..
      double precision alfc, ec, ecrs, eczet, rs, vcdn, vcup, zet
C     ..
C     .. Local Scalars ..
      double precision alfm, alfrsm, comm, ep, eprs, eu, eurs, f, 
     &                 fz, fzz, gam, thrd, thrd4, z4
C     ..
C     .. External Subroutines ..
      external gcor
C     ..
C     .. Data statements ..
      data gam, fzz/0.5198421D0, 1.709921D0/
      data thrd, thrd4/0.333333333333D0, 1.333333333333D0/
C     ..
      f = ((1.0D0+zet)**thrd4+(1.0D0-zet)**thrd4-2.0D0)/gam
      call gcor(0.0310907D0,0.21370D0,7.5957D0,3.5876D0,1.6382D0,
     &          0.49294D0,1.00D0,rs,eu,eurs)
      call gcor(0.01554535D0,0.20548D0,14.1189D0,6.1977D0,3.3662D0,
     &          0.62517D0,1.00D0,rs,ep,eprs)
      call gcor(0.0168869D0,0.11125D0,10.357D0,3.6231D0,0.88026D0,
     &          0.49671D0,1.00D0,rs,alfm,alfrsm)
C     ALFM IS MINUS THE SPIN STIFFNESS ALFC
      alfc = -alfm
      z4 = zet**4
      ec = eu*(1.0D0-f*z4) + ep*f*z4 - alfm*f*(1.0D0-z4)/fzz
C     ENERGY DONE. NOW THE POTENTIAL:
      ecrs = eurs*(1.0D0-f*z4) + eprs*f*z4 - alfrsm*f*(1.0D0-z4)/fzz
      fz = thrd4*((1.0D0+zet)**thrd-(1.0D0-zet)**thrd)/gam
      eczet = 4.0D0*(zet**3)*f*(ep-eu+alfm/fzz) +
     &        fz*(z4*ep-z4*eu-(1.0D0-z4)*alfm/fzz)
      comm = ec - rs*ecrs/3.0D0 - zet*eczet
      vcup = comm + eczet
      vcdn = comm - eczet
c
      return
c
      end
c
c     ================================================================
c
      subroutine corgga(rs,zet,t,uu,vv,ww,h,dvcup,dvcdn,
     $     ec, ecrs, eczet, fk, g, sk)

C     GGA91 CORRELATION
C     INPUT RS: SEITZ RADIUS
C     INPUT ZET: RELATIVE SPIN POLARIZATION
C     INPUT T: ABS(GRAD D)/(D*2.*KS*G)
C     INPUT UU: (GRAD D)*GRAD(ABS(GRAD D))/(D**2 * (2*KS*G)**3)
C     INPUT VV: (LAPLACIAN D)/(D * (2*KS*G)**2)
C     INPUT WW:  (GRAD D)*(GRAD ZET)/(D * (2*KS*G)**2)
C     OUTPUT H: NONLOCAL PART OF CORRELATION ENERGY PER ELECTRON
C     OUTPUT DVCUP,DVCDN:  NONLOCAL PARTS OF CORRELATION POTENTIALS

C     .. Scalar Arguments ..
      double precision 
     $     dvcdn, dvcup, h, rs, t, uu, vv, ww, zet,
     $     ec, ecrs, eczet, fk, g, sk
C     ..
C     ------------- local variables ----------------------------
c
c      double precision  a4, alf, b, b2, bec, bet, bg, c1, c2, c3, c4, 
     &       c5, c6, cc,
     &       cc0, ccrs, coeff, comm, cx, delt, fac, fact0, fact1, fact2,
     &       fact3, fact4, fact5, g3, g4, gz, h0, h0b, h0bt, h0rs,
     &       h0rst, h0t, h0tt, h0z, h0zt, h1, h1rs, h1rst, h1t, h1tt,
     &       h1z, h1zt, hrs, hrst, ht, htt, hz, hzt, pon, pref, q4, q5,
     &       q6, q7, q8, q82, q9, r0, r1, r2, r3, r4, rs2, rs3, rsthrd,
     &       t2, t4, t6, thrd2, thrdm, xnu
C     ..
C     .. Intrinsic Functions ..
      intrinsic exp, log
c
c
C     .. Data statements ..
      data xnu, cc0, cx, alf/15.75592D0, 0.004235D0, -0.001667212D0,
     &     0.09D0/
      data c1, c2, c3, c4/0.002568D0, 0.023266D0, 7.389D-6, 8.723D0/
      data c5, c6, a4/0.472D0, 7.389D-2, 100.0D0/
      data thrdm, thrd2/-0.333333333333D0, 0.666666666667D0/
C     ..
      bet = xnu*cc0
      delt = 2.0D0*alf/bet
      g3 = g**3
      g4 = g3*g
      pon = -delt*ec/(g3*bet)
      b = delt/(exp(pon)-1.0D0)
      b2 = b*b
      t2 = t*t
      t4 = t2*t2
      t6 = t4*t2
      rs2 = rs*rs
      rs3 = rs2*rs
      q4 = 1.0D0 + b*t2
      q5 = 1.0D0 + b*t2 + b2*t4
      q6 = c1 + c2*rs + c3*rs2
      q7 = 1.0D0 + c4*rs + c5*rs2 + c6*rs3
      cc = -cx + q6/q7
      r0 = (sk/fk)**2
      r1 = a4*r0*g4
      coeff = cc - cc0 - 3.0D0*cx/7.0D0
      r2 = xnu*coeff*g3
      r3 = exp(-r1*t2)
      h0 = g3*(bet/delt)*log(1.0D0+delt*q4*t2/q5)
      h1 = r3*r2*t2
      h = h0 + h1
C  LOCAL CORRELATION OPTION:
C     H = 0.0D0
C  ENERGY DONE. NOW THE POTENTIAL:
      ccrs = (c2+2.0*c3*rs)/q7 - q6*(c4+2.0*c5*rs+3.0*c6*rs2)/q7**2
      rsthrd = rs/3.0D0
      r4 = rsthrd*ccrs/coeff
      gz = ((1.0D0+zet)**thrdm-(1.0D0-zet)**thrdm)/3.0D0
      fac = delt/b + 1.0D0
      bg = -3.0D0*b2*ec*fac/(bet*g4)
      bec = b2*fac/(bet*g3)
      q8 = q5*q5 + delt*q4*q5*t2
      q9 = 1.0D0 + 2.0D0*b*t2
      q82 = q8*q8
      h0b = -bet*g3*b*t6*(2.0D0+b*t2)/q8
      h0rs = -rsthrd*h0b*bec*ecrs
      fact0 = 2.0D0*delt - 6.0D0*b
      fact1 = q5*q9 + q4*q9*q9
      h0bt = 2.0D0*bet*g3*t4*(q4*q5*fact0-delt*fact1)/q82
      h0rst = rsthrd*t2*h0bt*bec*ecrs
      h0z = 3.0D0*gz*h0/g + h0b*(bg*gz+bec*eczet)
      h0t = 2.0*bet*g3*q9/q8
      h0zt = 3.0D0*gz*h0t/g + h0bt*(bg*gz+bec*eczet)
      fact2 = q4*q5 + b*t2*(q4*q9+q5)
      fact3 = 2.0D0*b*q5*q9 + delt*fact2
      h0tt = 4.0D0*bet*g3*t*(2.0D0*b/q8-q9*fact3/q82)
      h1rs = r3*r2*t2*(-r4+r1*t2/3.0D0)
      fact4 = 2.0D0 - r1*t2
      h1rst = r3*r2*t2*(2.0D0*r4*(1.0D0-r1*t2)-thrd2*r1*t2*fact4)
      h1z = gz*r3*r2*t2*(3.0D0-4.0D0*r1*t2)/g
      h1t = 2.0D0*r3*r2*(1.0D0-r1*t2)
      h1zt = 2.0D0*gz*r3*r2*(3.0D0-11.0D0*r1*t2+4.0D0*r1*r1*t4)/g
      h1tt = 4.0D0*r3*r2*r1*t*(-2.0D0+r1*t2)
      hrs = h0rs + h1rs
      hrst = h0rst + h1rst
      ht = h0t + h1t
      htt = h0tt + h1tt
      hz = h0z + h1z
      hzt = h0zt + h1zt
      comm = h + hrs + hrst + t2*ht/6.0D0 + 7.0D0*t2*t*htt/6.0D0
      pref = hz - gz*t2*ht/g
      fact5 = gz*(2.0D0*ht+t*htt)/g
      comm = comm - pref*zet - uu*htt - vv*ht - ww*(hzt-fact5)
      dvcup = comm + pref
      dvcdn = comm - pref
C  LOCAL CORRELATION OPTION:
C     DVCUP = 0.0D0
C     DVCDN = 0.0D0
      return
c
      end
c
c     ==============================================================
c
      subroutine gcor(a,a1,b1,b2,b3,b4,p,rs,gg,ggrs)

C     CALLED BY SUBROUTINE CORLSD
C     .. Scalar Arguments ..
      double precision a, a1, b1, b2, b3, b4, gg, ggrs, p, rs
C     ..
C     .. Local Scalars ..
      double precision p1, q0, q1, q2, q3, rs12, rs32, rsp
C     ..
C     .. Intrinsic Functions ..
      intrinsic log, sqrt
C     ..
      p1 = p + 1.0D0
      q0 = -2.0D0*a*(1.0D0+a1*rs)
      rs12 = sqrt(rs)
      rs32 = rs12**3
      rsp = rs**p
      q1 = 2.0D0*a*(b1*rs12+b2*rs+b3*rs32+b4*rs*rsp)
      q2 = log(1.0D0+1.0D0/q1)
      gg = q0*q2
      q3 = a*(b1/rs12+2.0D0*b2+3.0D0*b3*rs12+2.0D0*b4*p1*rsp)
      ggrs = -2.0D0*a*a1*q2 - q0*q3/(q1**2+q1)
c
      return
c
      end
