c
c
      subroutine atftmt(ipr, a,ai,x,r,tnp,trans,ity,na,ib,ihg
     $                  ,ipm,li,nc,invers_no, mxdtyp, mxdatm)

      implicit none
c
c     INPUT:
c     -----
c
      integer 
     $     ipr,                 ! print flag
     $     ihg,                 ! holohedral group number
     $     ipm,                 ! print flag for multiplication table
     $     na,                  ! total number of atoms (of all kinds)
     $     mxdtyp,              ! max number of atom types
     $     mxdatm               ! max number of atoms of each type

      double precision 
     $     x(3,mxdtyp*mxdatm),  ! compact list of all coordinates (cartesian)
     $     r(49,3,3),           ! the rotation matrices as they come 
     $                          ! out of the pgl subroutine
     $     a(3,3),              ! realspace lattice vectors 
     $     ai(3,3)              ! inverse of lattice vectors (no 2pi)

c
c     INPUT/OUTPUT:
c
      integer 
     $     ity(mxdtyp*mxdatm),   ! compact list of all the types of all atoms
     $     ib(48),              ! index map for symmetry operations 
     $     nc                   ! number of symm-ops without/with basis
c
c
c     OUTPUT:
c     ------
c
      integer 
     $     li,                  ! something to do with inversion?
     $     invers_no            ! which operation is the inversion

      double precision 
     $     trans(3,3,48),       ! the cartesian realspace transf. matrices
     $     tnp(48,3)            ! the nonprimitive translations

c
c     DESCRIPTION:
c     -----------
c
c
c     subroutine atftmt determines the point group of the crystal, the
c     atom transformation table,f0, the fractional translations,tnp,
c     associated with each rotation and finally the multiplication table
c     mt, for the point group of the crystal. ib now contains
c     operations in the p.g. of the crystal and ntrans is the order of
c     this group.
c
c
c     1997 Bernd Pfrommer, based on a routine from Alberto Garcia's
c     code. I cleaned up some more, put in dynamic memory allocation, 
c     explicit typing, fortran 90 style, and more comments. Also,
c     some shortcuts were put in to speed up the case where the
c     symmetry is low. The rdiff array precomputes the rotated
c     vectors to speed up things

c
c     ------------------------ local variables -----------------------
c
C     ..
      double precision  v(3,48), vr(3), vt(3), xb(3)
      integer ia(48), ic(48), mt(48,48)

      double precision da, dif, eps, ts, vs, difmax
      parameter(eps=1.0d-8)    ! used to be 1.0d-8

      integer i, il, is, isy, iu, j, k, k1, k2, k3, k4, ks, l, m, n, n1,
     1        n2, n3, nca, ni

      double precision, allocatable :: rx(:,:), rdiff(:,:,:)
      integer, allocatable :: if0(:,:)


      character cst(7)*12
C     ..
C     .. External Subroutines ..
      external rlv
C     ..
C     .. Intrinsic Functions ..
      intrinsic abs, mod
C     ..
C     .. Data statements ..
c     
      data cst/'triclinic   ', 'monoclinic  ', 'orthorhombic',
     $     'tetragonal  ', 'cubic       ', 'trigonal    ',
     $     'hexagonal   '/
C     ..
      invers_no = 0

      allocate(rx(3,mxdtyp*mxdatm))
      allocate(if0(48,mxdtyp*mxdatm))
      allocate(rdiff(3,na,na))
c
c     eps should be slightly larger than computer precision
c
      nca = 0
      ni = 13
      if (ihg .lt. 6) ni = 25
      li = 0
      do 130 n = 1, nc          ! loop over all lattice symmetry operations 
         l = ib(n)              ! get index of symmetry operation
         ic(n) = ib(n)
c
c        operate on all atoms with symmetry operation l, and store in
c        list rx
c
         do k = 1, na
            do i = 1, 3
               rx(i,k) = 0.0d0
               do j = 1, 3
                  rx(i,k) = rx(i,k) + r(l,i,j)*x(j,k)
               end do
            end do
         end do
c
c     
c     This piece of code is pretty naive, and scales like order(n**3).
c     It basically checks if for each
c
c       R*x_1 - x_2   there is a matching R*x_3-x4
c
c     excluding the trivial case of x_1 .eq. x_3  and  x_2 .eq. x_4
c
c
         do k1 = 1, na          ! precompute the rdiffs first
            do k2 = 1, na
               xb(:) = rx(:,k1)-x(:,k2)
               call rlv(ai,xb,rdiff(1,k2,k1),il) ! rdiff = R*x_1 -x_2
c     subroutine rlv removes a direct lattice vector from xb
c     leaving the remainder in rdiff. if a nonzero lattice vector was
c     removed, il is made nonzero. 
            end do
         end do
c
c
c
         difmax=0
         do k1 = 1, na          ! double loop over compact atom list
            do k2 = 1, na
               if (ity(k1) .eq. ity(k2)) then ! same type atoms?
                  vr = rdiff(:,k2,k1) !vr stands for v-reference.
                  ks = 0
                  do k3 = 1, na
                     do k4 = 1, na
                        if (ity(k3) .eq. ity(k4)) then
                           vt = rdiff(:,k4,k3) ! vt stands for v-test
                           dif = 0.0d0
                           do i = 1, 3
                              da = abs(vr(i)-vt(i)) + eps
                              dif = dif + mod(da,1.d0)
                           end do
                           if (dif .le. 10.0D0*eps) then
                              if0(l,k3) = k4
c     if0 is the function defined in maradudin and vosko by
c     eq.(2.35). it defines the atom transformation table
                              ks = ks + k4
                              if (ks .eq. na*(na+1)/2) go to 110 ! found all
                              go to 80
                           end if
                        end if
                     end do
                     exit
 80                  continue
                  end do
c
c                 BP put in this shortcut (check carefully). If there
c                 is a single R*x_1- x_2 without match, then give up.
                  if(ks.eq.0) goto 130 ! this is not a symmetry operation
               end if
            end do
         end do
c
         go to 130              ! this was not a symmetry operation
c
 110     continue               ! we found a symmetry operation
         nca = nca + 1

c
c        v(i,l) is the i-th cartesian component of the fractional
c        translation associated with the rotation r(l).
c
         v(1:3,l) = vr(1:3)
c
         ib(nca) = l
         if (l .eq. ni) then
            li = l
            invers_no = nca
         endif
  130 continue

      deallocate(rdiff)
c
c     -------------- there are no gotos across this line -------------------
c
      if(ipr.eq.1) then
         if ((ihg .eq. 7 .and. nca .eq. 24) .or. 
     $        (ihg .eq. 5 .and. nca .eq. 48)) then
            write(9,9010) cst(ihg)
 9010       format(/' The point group of the crystal is the full '
     $           ,a12,'group')

         else
            write(9,9000) cst(ihg), (ic(i),i=1,nc)
 9000       format(/' The crystal system is ',a12
     $           ,' with operations: ',/5x,24i3,/5x,24i3,/)
         endif
      endif



      vs = 0.0d0
      nc = nca
      do n = 1, nc
         l = ib(n)
         vs = sum(abs(v(1:3,l)))
      end do
      if (vs .gt. eps) then
         if(ipr .eq. 1)  write(9,9030)

 9030    format(/' The space group is non-symmorphic',/,
     $        ' (Or a non standard  origin of coordinates is used)',/)
         isy = 0
         is = 0
      else
         if(ipr .eq. 1)  write(9,9020)
 9020    format(/' the space group of the crystal is symmorphic',/)
         isy = 1
         is = 1
      endif

c
c     construct the multiplication table
c
      do n1 = 1, nc
         do n2 = 1, nc
            l = ib(n1)
            m = ib(n2)
            do i = 1, 3
               do j = 1, 3
                  r(49,i,j) = 0.0d0
                  do k = 1, 3
                     r(49,i,j) = r(49,i,j) + r(l,i,k)*r(m,k,j)
                  end do
               end do
            end do
            do n3 = 1, nc
               n = ib(n3)
               ts = 0.0d0
               do i = 1, 3
                  do j = 1, 3
                     ts = ts + abs(r(49,i,j)-r(n,i,j))
                  end do
               end do
               if (ts .gt. 100.0D0*eps) cycle
               mt(l,m) = n
               exit
            end do
         end do
      end do
c
      il = 1
      iu = nc
      if (iu .gt. 24) iu = 24
  280 continue
      if(ipr .eq. 1) then
         write(9,9040) (ib(i),i=il,iu)
      endif
 9040 format(' Operation number  ',24i3)
      do 300 i = 1, na
         do 290 j = 1, nc
            l = ib(j)
            ia(j) = if0(l,i)
  290    continue
  300 continue
      if (nc-iu) 320, 320, 310
  310 continue
      if(ipr .eq. 1) then
         write(9,9050)
      endif
 9050 format(//)
      il = 25
      iu = nc
c
      go to 280
c
c     Print multiplication table and fractional translations.
c
  320 continue
      if (ipm .eq. 0) go to 410
      il = 1
      iu = nc
      if (nc .gt. 24) iu = 24
      if (is) 330, 330, 340
  330 continue
      if(ipr .eq. 1) then
         write(9,9060)
 9060    format('0',57x,'Multiplication table',30x,
     1      'Fractional translations')
         write(9,9070) (ib(i),i=il,iu)
 9070    format('0',4x,24i4)
         write(9,9080)
 9080    format('+',107x,'v(1)      v(2)      v(3)')
      endif
c
      go to 360
c
  340 continue
      if(ipr .eq. 1) then
         write(9,9090)
      endif
 9090 format('0',57x,'Multiplication table')
  350 continue
      if(ipr .eq. 1) then
         write(9,9100) (ib(i),i=il,iu)
      endif
 9100 format('0',4x,24i4)
  360 continue
      do 400 j = 1, nc
         l = ib(j)
         do 370 i = il, iu
            n = ib(i)
            ia(i) = mt(l,n)
  370    continue
         if (is) 380, 380, 390
  380    continue
         if(ipr .eq. 1) then
            write(9,9120) ib(j), (ia(i),i=il,iu)
            write(9,9110) (v(i,l),i=1,3)
 9110       format('+',102x,3f10.4)
         endif
c
         go to 400
c
  390    continue
         if(ipr .eq. 1) then
            write(9,9120) ib(j), (ia(i),i=il,iu)
         endif
  400 continue
 9120 format(i5,24i4)
      if (iu .eq. nc) go to 410
      il = 25
      iu = nc
      is = 1
c
      go to 350
c
  410 continue
c

      do 440 i = 1, nc
         l = ib(i)
         do 430 j = 1, 3
            tnp(i,j) = -v(j,l)
            do 420 k = 1, 3
               trans(j,k,i) = r(l,j,k)
  420       continue
  430    continue

  440 continue
c
      deallocate(rx)
      deallocate(if0)

      return
c
      end
c
c     ------------------------------------------------------------------------
c
c
      subroutine pgl(a,b,r,nc,ib,ihg)
c
C     .. Scalar Arguments ..
      integer ihg, nc
C     ..
C     .. Array Arguments ..
      double precision a(3,3), b(3,3), r(49,3,3)
      integer ib(48)
C     ..
C     .. Local Scalars ..
      double precision eps, tr
      parameter(eps=1.0d-8)    ! used to be 1.0d-8
      integer i, ihc, j, k, lx, n, nr
C     ..
C     .. Local Arrays ..
      double precision vr(3), xa(3)
C     ..
C     .. External Subroutines ..
      external rlv, rot
C     ..
C     .. Intrinsic Functions ..
      intrinsic abs
C     ..
c
c     eps should be slightly larger than computer precision
c
      
      ihc = 0
c
c     ihc is 0 for hexagonal groups and 1 for cubic groups.
c
      nr = 24
   10 continue
      nc = 0
      call rot(r,nr)
      do 60 n = 1, nr
         ib(n) = 0
         tr = 0.0d0
         do 50 k = 1, 3
            do 30 i = 1, 3
               xa(i) = 0.0d0
               do 20 j = 1, 3
                  xa(i) = xa(i) + r(n,i,j)*a(j,k)
   20          continue
   30       continue
            call rlv(b,xa,vr,lx)
            do 40 i = 1, 3
               tr = tr + abs(vr(i))
   40       continue
   50    continue
         if (tr .le. 10.0D0*eps) then
            nc = nc + 1
            ib(nc) = n
         end if
   60 continue
      if (ihc .eq. 0) then
         if (nc .eq. 12) then
            ihg = 6
c
            return
c
         end if
         if (nc .gt. 12) then
            ihg = 7
c
            return
c
         end if
         if (nc .lt. 12) then
            nr = 48
            ihc = 1
c
            go to 10
c
         end if
      else
         if (nc .eq. 16) then
            ihg = 4
c
            return
c
         end if
         if (nc .gt. 16) then
            ihg = 5
c
            return
c
         end if
         if (nc .lt. 16) then
            if (nc .eq. 4) then
               ihg = 2
c
               return
c
            end if
            if (nc .gt. 4) then
               ihg = 3
c
               return
c
            end if
            if (nc .lt. 4) then
               ihg = 1
c
               return
c
            end if
         end if
      end if
c
c     ihg stands for holohedral group number.
c
      end
c
      subroutine rot(r,nr)
c
C     .. Scalar Arguments ..
      integer nr
C     ..
C     .. Array Arguments ..
      double precision r(49,3,3)
C     ..
C     .. Local Scalars ..
      double precision f
      integer i, j, k, n, nv
C     ..
C     .. Intrinsic Functions ..
      intrinsic sqrt
C     ..
      do 30 n = 1, nr
         do 20 i = 1, 3
            do 10 j = 1, 3
               r(n,i,j) = 0.0d0
   10       continue
   20    continue
   30 continue
c
      if (nr .le. 24) then
c
c        define the generators for the rotation matrices
c                                 --hexagonal group
c
         f = sqrt(3.0d0)/2.0d0
         r(2,1,1) = 0.5d0
         r(2,1,2) = -f
         r(2,2,1) = f
         r(2,2,2) = 0.5d0
         r(7,1,1) = -0.5d0
         r(7,1,2) = -f
         r(7,2,1) = -f
         r(7,2,2) = 0.5d0
         do 40 n = 1, 6
            r(n,3,3) = 1.0d0
            r(n+18,3,3) = 1.0d0
            r(n+6,3,3) = -1.0d0
            r(n+12,3,3) = -1.0d0
   40    continue
c
c     generate the rest of the rotation matrices
c
         do 70 i = 1, 2
            r(1,i,i) = 1.0d0
            do 60 j = 1, 2
               r(6,i,j) = r(2,j,i)
               do 50 k = 1, 2
                  r(3,i,j) = r(3,i,j) + r(2,i,k)*r(2,k,j)
                  r(8,i,j) = r(8,i,j) + r(2,i,k)*r(7,k,j)
                  r(12,i,j) = r(12,i,j) + r(7,i,k)*r(2,k,j)
   50          continue
   60       continue
   70    continue
         do 100 i = 1, 2
            do 90 j = 1, 2
               r(5,i,j) = r(3,j,i)
               do 80 k = 1, 2
                  r(4,i,j) = r(4,i,j) + r(2,i,k)*r(3,k,j)
                  r(9,i,j) = r(9,i,j) + r(2,i,k)*r(8,k,j)
                  r(10,i,j) = r(10,i,j) + r(12,i,k)*r(3,k,j)
                  r(11,i,j) = r(11,i,j) + r(12,i,k)*r(2,k,j)
   80          continue
   90       continue
  100    continue
c
         do 130 n = 1, 12
            nv = n + 12
            do 120 i = 1, 2
               do 110 j = 1, 2
                  r(nv,i,j) = -r(n,i,j)
  110          continue
  120       continue
  130    continue
      else
c
c        define the generators for the rotation matrices
c                                          --cubic group
c
         r(9,1,3) = 1.0d0
         r(9,2,1) = 1.0d0
         r(9,3,2) = 1.0d0
         r(19,1,1) = 1.0d0
         r(19,2,3) = -1.0d0
         r(19,3,2) = 1.0d0
         do 160 i = 1, 3
            r(1,i,i) = 1.0d0
            do 150 j = 1, 3
               r(20,i,j) = r(19,j,i)
               r(5,i,j) = r(9,j,i)
               do 140 k = 1, 3
                  r(2,i,j) = r(2,i,j) + r(19,i,k)*r(19,k,j)
                  r(16,i,j) = r(16,i,j) + r(9,i,k)*r(19,k,j)
                  r(23,i,j) = r(23,i,j) + r(19,i,k)*r(9,k,j)
  140          continue
  150       continue
  160    continue
         do 190 i = 1, 3
            do 180 j = 1, 3
               do 170 k = 1, 3
                  r(6,i,j) = r(6,i,j) + r(2,i,k)*r(5,k,j)
                  r(7,i,j) = r(7,i,j) + r(16,i,k)*r(23,k,j)
                  r(8,i,j) = r(8,i,j) + r(5,i,k)*r(2,k,j)
                  r(10,i,j) = r(10,i,j) + r(2,i,k)*r(9,k,j)
                  r(11,i,j) = r(11,i,j) + r(9,i,k)*r(2,k,j)
                  r(12,i,j) = r(12,i,j) + r(23,i,k)*r(16,k,j)
                  r(14,i,j) = r(14,i,j) + r(16,i,k)*r(2,k,j)
                  r(15,i,j) = r(15,i,j) + r(2,i,k)*r(16,k,j)
                  r(22,i,j) = r(22,i,j) + r(23,i,k)*r(2,k,j)
                  r(24,i,j) = r(24,i,j) + r(2,i,k)*r(23,k,j)
  170          continue
  180       continue
  190    continue
         do 220 i = 1, 3
            do 210 j = 1, 3
               do 200 k = 1, 3
                  r(3,i,j) = r(3,i,j) + r(5,i,k)*r(12,k,j)
                  r(4,i,j) = r(4,i,j) + r(5,i,k)*r(10,k,j)
                  r(13,i,j) = r(13,i,j) + r(23,i,k)*r(11,k,j)
                  r(17,i,j) = r(17,i,j) + r(16,i,k)*r(12,k,j)
                  r(18,i,j) = r(18,i,j) + r(16,i,k)*r(10,k,j)
                  r(21,i,j) = r(21,i,j) + r(12,i,k)*r(15,k,j)
  200          continue
  210       continue
  220    continue
         do 250 n = 1, 24
            nv = n + 24
            do 240 i = 1, 3
               do 230 j = 1, 3
                  r(nv,i,j) = -r(n,i,j)
  230          continue
  240       continue
  250    continue
      end if
c
      end
c     ----------------------------------------------
      subroutine rlv(p,g,y,l)
      
      implicit none
c
c     INPUT:
c     -----
c
      double precision 
     $     g(3),                ! vector to be multiplied
     $     p(3,3)               ! multiplication matrix, e.g. lattice vectors
c      
c
c     OUTPUT:
c     ------
c
      double precision 
     $     y(3)                 ! mod(multiplied vector,lattice vector)
      integer l                 ! is nonzero if a lattice vector was removed

c     subroutine rlv removes a direct lattice vector from g by
c     operation p, leaving the remainder in y. If a nonzero lattice
c     vector was removed, l is made nonzero.

      intrinsic abs, nint

      y(1:3)=matmul(p(1:3,1:3),g(1:3))
      l=sum(nint(abs(y(1:3))))
      y(1:3)=y(1:3)-1.d0*nint(y(1:3))
      
      return
c
      end
c                
