
c
      subroutine gspace_gather(myproc, nproc, gsstruc, 
     $     nfft, gsvec, ng, istar, nstar, mstar, phase, inds)
c
c     packs a parallel gspace into gsstruc
c
      use vqmc_gspace_module

      implicit none             ! implicit? no!

m4_ifdef([CRAY],[m4_define([MPI_INTEGER4],[MPI_INTEGER])])
m4_ifdef([MPI],[
      include 'mpif.h'])

c
c     INPUT:
c
      integer
     $     myproc, nproc, 
     $     ng, info,
     $     gsvec(3,ng),         ! the local gvectors
     $     nfft(3),             ! the fft grid size
     $     nstar,               ! number of stars
     $     inds(ng),            ! maps gvectors to stars
     $     mstar(nstar)         ! size of each star

      complex(kind=8) phase(ng)  ! phase factors

      logical 
     $     istar                ! if star information is present

c
c     OUTPUT:
c     
      type (gspace) gsstruc    ! a structure containing the whole gspace
c
c     --------- local variables ---------------------------
c
      
      integer
     $     i,j, ngtot

c
c     set the fft grid dimensions
c

      gsstruc%fftsize(1)=nfft(1)
      gsstruc%fftsize(2)=nfft(2)
      gsstruc%fftsize(3)=nfft(3)
c
c     sum up across all processors to get total number of gvecs
c

      ngtot=ng

m4_ifdef([MPI],[
      if(nproc.gt.1) 
     $     call mpi_allreduce(ng,ngtot, 1, MPI_INTEGER4,
     $     MPI_SUM, MPI_COMM_WORLD, info)
])


      gsstruc%length = ngtot

c
c     allocate space, and collect all the data
c

      allocate(gsstruc%gvec(3,gsstruc%length))
      if(nproc.gt.1) then
m4_ifdef([MPI],[
      call mpi_allgather(gsvec,3*ng, MPI_INTEGER4,
     $     gsstruc%gvec,3*ng, MPI_INTEGER4,
     $     MPI_COMM_WORLD, info)
])
      else
         do i=1,ng
            gsstruc%gvec(1,i)= gsvec(1,i)
            gsstruc%gvec(2,i)= gsvec(2,i)
            gsstruc%gvec(3,i)= gsvec(3,i)
         end do
      endif
c
c     deal with the star information
c
      gsstruc%istar=istar

      if(istar) then
c
c        the mstar array is distributed already. No need to collect.
c         
         gsstruc%nstar=nstar
         allocate(gsstruc%mstar(nstar));
         gsstruc%mstar=mstar
c
c        the other ones have to be collected.
c

c
c        deal with the phase factors
c
         allocate(gsstruc%phase(gsstruc%length))

         if(nproc.gt.1) then
            m4_ifdef([MPI],[
            call mpi_allgather(phase(1),ng, MPI_DOUBLE_COMPLEX,
     $           gsstruc%phase(1),ng, MPI_DOUBLE_COMPLEX,
     $           MPI_COMM_WORLD, info)
            ])
         else
            gsstruc%phase=phase
         endif

c
c        handle the star index array
c
         allocate(gsstruc%inds(gsstruc%length))
         if(nproc.gt.1) then
            m4_ifdef([MPI],[
            call mpi_allgather(inds(1),ng, MPI_INTEGER4,
     $           gsstruc%inds(1),ng, MPI_INTEGER4,
     $           MPI_COMM_WORLD, info)
            ])
         else
            gsstruc%inds=inds
         endif

      else
         gsstruc%nstar=0
         nullify(gsstruc%inds); nullify(gsstruc%mstar); 
         nullify(gsstruc%phase);
      endif
         


      return

      end







