      subroutine gwout(fname, crys, syms,
     $     bands, gs, kgs, wfn,  kpoints,iflag)     

      use parallel_gspace_module
      use gspace_array_module
      use pseudo_potential_module
      use energy_module
      use force_module
      use band_module
      use hamiltonian_module
      use all_to_all_module
      use kpoint_module
      use symmetry_module
      use crystal_module


      implicit none             ! implicit? Just say no!

      include 'interface.h'
      include 'all_to_all.h'
c     include 'mpif.h'

c
c     writes a data array to disk such that it can be read and used in 
c     later GW calculations
c

 
c     INPUT:
c     -----

      type(crystal) crys
      type(symmetry) syms

      type(complex_gspace_array)
     $     wfn                  ! the wave functions
      type(band) bands
      type(parallel_gspace) 
     $     kgs(bands%nrk),      ! the gspaces at the different kpoints
     $     gs                   ! the big gspaces
      
      type (kpoint) kpoints     ! all the kpoints
      integer iflag             ! iflag =1 real version, iflag =2, complex

      character*(*) fname       ! file name
      
c     ---------------- local variables ----------------------------------
      integer 
     $     i, info, iproc, irk, is, j, k, n, ndim, ng, nkpt
      integer idum(1)
      integer, allocatable :: ifmin(:,:),ifmax(:,:)
      complex(kind=8), allocatable :: zc(:,:)

      if(gs%myproc.eq.0) then 
c
c     erase old file
c
        open(21,file=fname,status='replace',form='unformatted')

        write(21) ((crys%bdot(i,j),i=1,3),j=1,3)
        write(21) crys%vcell
c
        write(21) syms%ntrans
        do n=1,syms%ntrans
          write(21) ((syms%mtrx(i,j,n),i=1,3),j=1,3)
        enddo
        do n=1,syms%ntrans
          write(21) (syms%tnp(k,n),k=1,3)
        enddo
c
        write(21) crys%nspin
        write(21) kpoints%nrk
        write(21) (kpoints%grid(i),i=1,3)
        write(21) (kpoints%shift(i),i=1,3)
        write(21) (kpoints%w(i),i=1,kpoints%nrk)
        do j=1,kpoints%nrk
          write(21) (kpoints%rk(i,j),i=1,3)
        enddo
c
        write(21) bands%min
        allocate(ifmin(bands%nrk,crys%nspin))
        allocate(ifmax(bands%nrk,crys%nspin))
        ifmin=0
        ifmax=0
        do is=1,wfn%nspin
          do irk=1,bands%nrk
            do n=1,bands%nband(irk,is)
              if(bands%occup(n,irk,is).gt.1.0d-3) then
                if(ifmin(irk,is).eq.0) ifmin(irk,is)=n
                ifmax(irk,is)=n
              endif
            enddo
          enddo
        enddo
        write(21) ((ifmin(irk,is),irk=1,bands%nrk),is=1,crys%nspin)
        write(21) ((ifmax(irk,is),irk=1,bands%nrk),is=1,crys%nspin)
        do is=1,wfn%nspin
          do irk=1,bands%nrk
            write(21) (bands%energy(n,irk,is),n=1,bands%min)
          enddo
        enddo
c
        close(21)
      endif
c
      idum(1)=gs%length 
      call all_sum_all_integer(idum,1)
      ng=idum(1)
      if(gs%myproc.eq.0) then
        open(unit=21,file=fname,position='append',
     $       status='unknown',form='unformatted')
        write(21) (gs%fftsize(i),i=1,3)
        write(21) ng
        close(21)
      endif
      do iproc=0,gs%nproc-1
        if(iproc.eq.gs%myproc) then
          open(unit=21,file=fname,
     $         position = 'append',
     $         status='old',form='unformatted')
          do i=1,gs%length
            write(21) gs%gvec(1,i),gs%gvec(2,i),gs%gvec(3,i)
          enddo
          close(21)
        endif
        if(gs%nproc.gt.1) call parallel_barrier()
      enddo
c
      if(iflag.eq.1) call gwreal(bands,kgs,wfn,kpoints)
c
      do irk=1,kpoints%nrk
        ndim=kgs(irk)%length
        idum(1)=ndim
        call all_sum_all_integer(idum,1)
        nkpt=idum(1)
        if(gs%myproc.eq.0) then
          open(unit=21,file=fname,position='append',
     $         status='unknown',form='unformatted')
          write(21) nkpt
          close(21)
        endif 
        do iproc=0,gs%nproc-1
          if(iproc.eq.gs%myproc) then
            open(unit=21,file=fname,
     $           position = 'append',
     $           status='old',form='unformatted')
            do i=1,kgs(irk)%length
              write(21) kgs(irk)%gvec(1,i),
     $                  kgs(irk)%gvec(2,i),
     $                  kgs(irk)%gvec(3,i)
            enddo
            close(21)
          endif
          if(gs%nproc.gt.1) call parallel_barrier()
        enddo
        if(gs%myproc.eq.0)
     $    open(unit=21,file=fname,position='append',
     $         status='unknown',form='unformatted')
        allocate(zc(nkpt,wfn%nspin))
        do n=1,bands%min
          do is=1,wfn%nspin
            call data_gather(1,
     $              gs%myproc, gs%nproc,
     $              wfn%data(1+(n-1)*ndim,irk,is),
     $              zc(1,is),ndim)
          enddo
	  if(iflag.eq.1) then
            if(gs%myproc.eq.0)
     $        write(21) ((real(zc(j,k)),j=1,nkpt),k=1,wfn%nspin)
	  else
            if(gs%myproc.eq.0)
     $        write(21) ((zc(j,k),j=1,nkpt),k=1,wfn%nspin)
          endif
        enddo
        deallocate(zc)
        if(gs%myproc.eq.0) close(21)
      enddo

      return
      end 
