      subroutine gwreal(bands,kgs,wfn,kpoints)

      use parallel_gspace_module
      use gspace_array_module
      use pseudo_potential_module
      use energy_module
      use force_module
      use band_module
      use hamiltonian_module
      use all_to_all_module
      use kpoint_module
      use symmetry_module
      use crystal_module


      implicit none

      include 'interface.h'
      include 'all_to_all.h'
c     include 'mpif.h'
c
c     INPUT:
c     -----

      type(complex_gspace_array)
     $     wfn                  ! the wave functions
      type(band) bands
      type(parallel_gspace)
     $     kgs(bands%nrk)       ! the gspaces at the different kpoints

      type (kpoint) kpoints     ! all the kpoints

c     ---------------- local variables ----------------------------------
      integer dimension_span, reduced_span,
     $     i, idum, ig, inc, inum, irk, is, k, kprime, n, ng, wfninc
c     We assume that the maximum degeneracy is 10
      integer, parameter :: mdeg=10
      integer, allocatable :: imap(:),inull(:),null(:,:)
      double precision ii,rr,norm
      complex(kind=8), allocatable :: psi(:,:),phi(:,:),scr1(:),scr2(:)

      allocate(imap(bands%max))
      allocate(inull(bands%max))
      allocate(null(mdeg,bands%max))

      do is=1,wfn%nspin
        do irk=1,kpoints%nrk
          ng=kgs(irk)%length
c
c figure out degeneracies
c store degeneracy information in imap(inum)
c
          inum=1
          do n=1,bands%nband(irk,is)
            if(n.eq.bands%nband(irk,is)) then
              imap(inum)=n
              inum=inum+1
              goto 10
            endif 
            if(abs(bands%energy(n,irk,is)-
     $             bands%energy(n+1,irk,is))>1.0d-5) then
              imap(inum)=n
              inum=inum+1
            endif
 10         continue
          enddo
          inum=inum-1

c
c construct null vector map
c
          inc=1
          allocate(scr1(ng))
          allocate(scr2(ng))
          do i=1,inum
            inull(i)=1
            do n=inc,imap(i)
              do ig=1,ng
                scr1(ig)= 
     >          real(wfn%data(ig+(n-1)*ng,irk,is))
                scr2(ig)= 
     >          aimag(wfn%data(ig+(n-1)*ng,irk,is))
              enddo
              rr=parallel_zdotc(ng,scr1,1,scr1,1)
              if(sqrt(rr).lt.1e-2) null(inull(i),i)=0
              if(sqrt(rr).gt.1e-2) null(inull(i),i)=1 
              inull(i)=inull(i)+1
c
              ii=parallel_zdotc(ng,scr2,1,scr2,1)
              if(sqrt(ii).lt.1e-2) null(inull(i),i)=0
              if(sqrt(ii).gt.1e-2) null(inull(i),i)=1
              inull(i)=inull(i)+1 
            enddo
            inull(i)=inull(i)-1
            inc=imap(i)+1
          enddo
          deallocate(scr1)
          deallocate(scr2)
c
c i loops over each degenerate multiplet
c k loops over the index of the wavefunction in each
c    multiplet counting real and imaginary parts 
c
          inc=1
          wfninc=1
          allocate(psi(ng,mdeg))
          allocate(phi(ng,mdeg))
          do i=1,inum
            kprime=1
            do k=1,2*(imap(i)-inc)+1,2
              if(null(k,i).eq.1) then
                do ig=1,ng
                  phi(ig,kprime)=cmplx(real(
     >            wfn%data(ig+(wfninc-1)*ng,irk,is)),0.0)
                enddo
                kprime=kprime+1
              endif
              if(null(k+1,i).eq.1) then
                do ig=1,ng
                  phi(ig,kprime)=cmplx(aimag(
     >            wfn%data(ig+(wfninc-1)*ng,irk,is)),0.0)
                enddo
                kprime=kprime+1
              endif
              wfninc=wfninc+1
            enddo
            dimension_span=kprime-1
c normalize basis of spanning functions
            do n=1,dimension_span
              norm=parallel_zdotc(ng,phi(:,n),1,
     >                                       phi(:,n),1)
              do ig=1,ng
                phi(ig,n)=phi(ig,n)/sqrt(norm)
              enddo
            enddo 
c perform gramm-schimdt process
            call gramm_schmidt(psi,reduced_span,
     >       phi,dimension_span,ng)
            if(reduced_span.lt.(imap(i)-inc+1)) then
              write(9,*) 'you really messed up n1.ne.n2 ',
     >         reduced_span,(imap(i)-inc+1)
              stop 
            endif
c
c here change the wavefunctions to their real versions
c
            do n=1,(imap(i)-inc+1)
              do ig=1,ng
                wfn%data(ig+(inc+n-1-1)*ng,irk,is)=psi(ig,n)
              enddo
            enddo
c
            inc=imap(i)+1
          enddo
          deallocate(psi)
          deallocate(phi)
        enddo ! end of k-point loop
      enddo ! end of spin loop
      deallocate(imap)
      deallocate(inull)
      deallocate(null)
c end of gramm-schmidt process

      end subroutine
c-----------------------------------------------------------------------
      subroutine gramm_schmidt(psi,n_psi,phi,n_phi,npts)
c
      use all_to_all_module
      include 'all_to_all.h'
c
      complex(kind=8)  psi(npts,n_phi),phi(npts,n_phi)
      complex(kind=8), allocatable :: vec(:)
      double precision norm
      double precision scalar
      integer n_psi,n_phi,npts
      integer i,l
c
      allocate(vec(npts))
      norm = 0.0
      n_psi = 1
c intialize      
      norm = parallel_zdotc(npts,phi(:,1),1,phi(:,1),1)
      norm = sqrt(norm)
      do l=1,npts
        psi(l,1) = phi(l,1)/norm
      enddo

c begin gramm-scmidt algorithm
      do i=1,n_phi-1
c project out
        do l=1,npts
         vec(l) = phi(l,i+1)
        enddo
        do ii=1,n_psi
          scalar = parallel_zdotc(npts,phi(:,i+1),1,psi(:,ii),1)
          do l=1,npts
            vec(l) = vec(l) - scalar*psi(l,ii)
          enddo        
        enddo
        norm = parallel_zdotc(npts,vec,1,vec,1)
        norm = sqrt(norm)
c if we have a non-null projection, set new psi
        if (norm .gt. 1e-6) then
          n_psi = n_psi +1
          do l=1,npts
            psi(l,n_psi) = vec(l)/norm
          enddo
        endif
      enddo
      deallocate(vec)
      end subroutine
