C*
C*
c     @process extchk

      subroutine induced_current(curr, chi, bfield, gs, crys)

      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'

c
c     1996 Bernd Pfrommer


c
c     INPUT
c     -----

      type (parallel_gspace) gs ! gspace on which the data is defined
      type (fftstruc) ffts
      type (crystal) crys

      double precision bfield(3) ! the unnormalized direction of the B-field.

      complex(kind=8) chi(gs%length,3,3) ! susceptibility tensor field

c
c     OUTPUT:
c     ------

      complex(kind=8)
     $     curr(gs%length,3)    ! induced current as computed from chi, bfield

c
c     DESCRIPTION:
c     -----------
c
c     Given chi and the direction of the B field, this subroutine
c     computes the induced current in for a B-field of 1 Gauss.
c     The units are gauss/seconds = esu/(cm^2 s)
c
c      J = c/4pi * nabla x B_induced
c
c
c     Only the imaginary part of J should be used:
c
c     J_true(r) = Imag(FFT(J_returned))(r)
c
c     
c     ------------- local variables
c
      double precision 
     $     pi, bflen, gv(3)

      complex(kind=8)
     $     em(3), jm(3), bf(3)

      integer 
     $     ig, i


      pi = 4.d0 *atan(1.d0)

      bflen=dot_product(bfield,bfield)

      if(bflen.gt.1d-10)  bf=bfield/sqrt(bflen)

c
c     convert to gauss/seconds.
c
      bf = bf/(137.036**2.d0 * 4.d0*pi)*2.99792458d10


      do ig=1, gs%length
         do i=1,3
            em(i)= chi(ig,i,1)*bf(1)+chi(ig,i,2)*bf(2)+
     $           chi(ig,i,3)*bf(3)
         end do
         gv = matmul(crys%bvec,dble(gs%gvec(:,ig)))
         call mycmplxvecprod(jm(1),gv(1),em(1)) ! jm = gv x em

c
c        and stick it into final array
c
         curr(ig,:) = jm(:)

      end do

      return

      end

c
c     --------- computes complex vector cross-product ------------
c

      subroutine mycmplxvecprod(a,b,c)

      implicit none

c
c     computes a = b x c
c
c     where b is real, c is complex, and a is complex
c      
      complex(kind=8) a(3), c(3)
      double precision b(3)

      a(1)=b(2)*c(3)-b(3)*c(2)
      a(2)=b(3)*c(1)-b(1)*c(3)
      a(3)=b(1)*c(2)-b(2)*c(1)

      end
