/*
       Service routines for reading the input file 
       1995 Bernd Pfrommer
*/

#include "inputread.h"
#include "libcalls.h"


char *getline(FILE *infile, char *buf)
{
  return(fgets(buf, 999, infile));
}

int getnextline(FILE *infile, char *buf)
{
  char keybuf[1000];
  int ifound, ierr;

  while(getline(infile, buf)!=NULL) {
    ifound =sscanf(buf, "%s", keybuf);  /* get keyword */
    /* printf("gnl:%s, %d\n",keybuf,ifound); */
    if(ifound>0) {  /* it has found something, now check if comment char */
      if(strcmp(keybuf, COMMENT_CHAR)!=0) return(1);
    }
  }
  return(EOF);

}
int findkeyword(FILE *infile, keyword *name, char *buf)
{
  int ierr=0;
  char keybuf[1000];

  if( (name->number <= 0) || (name->number >2)) 
    {printf("findkeyword: number of keywords %d is not allowed!\n", 
	    name->number);      exit(-1);}

  rewind(infile);       /* go to beginning of file */
  
  while((ierr=getnextline(infile, buf))!=EOF)
    {
      if(keytest(buf,name)==KEYWORD_FOUND) return(KEYWORD_FOUND);
    }

  return(KEYWORD_NOT_FOUND);

}

void setkeyword(keyword *key, int n, char *s1, char *s2)
{
  if(n<1 || n >2) 
    {printf("setkeyword: number of keywords %d is not allowed!\n", n); 
     exit(-1);}
  
  key->number=n;
  strcpy(key->kw[0], s1);
  if(key->number>1) strcpy(key->kw[1], s2);
}
int keytest(char *buf, keyword *key)
{
  char mybuf[256];

  sscanf(buf,"%s", mybuf);
  if(strcmp(mybuf,key->kw[0])==0) 
    {
      if(key->number>1) {
	sscanf(buf,"%*s %s", mybuf);
	if(strcmp(mybuf,key->kw[1])==0) return(KEYWORD_FOUND);
	else return(KEYWORD_NOT_FOUND);
      }
      else return(KEYWORD_FOUND);
    }
  

  return(KEYWORD_NOT_FOUND);

}

void yell(char *buf, char *mess)
{
  printf("error in subroutine %s:\n",mess);
  printf("working on line : %s:\n",buf);
  exit(-1);
}

int getstr(char *buf, keyword *key, char *dstring, int n)
{
  char mybuf[25];
  int iscan;

  assert(key->number >0 && key->number <=2);
  if(key->number==1) {
    sprintf(mybuf,"%%*s %%%ds",n);
    /* printf("%s\n", mybuf); */
    iscan= sscanf(buf,mybuf,dstring,dstring);}
  else {
    sprintf(mybuf,"%%*s %%*s %%%ds",n);
    /* printf("%s\n", mybuf); */
    iscan=sscanf(buf,mybuf,dstring,dstring,dstring);}
  
  if(iscan<=0) return(-1);

  return(iscan);

}

int getdouble(char *buf, keyword *key, double *dpoi, int n)
{
  char mybuf[1000];
  int iscan,i;
  char *b = buf;

  strcpy(mybuf,buf);

  assert(key->number >0 && key->number <=2);

  if((b=strtok(mybuf," "))==NULL) return(-1);

  if(key->number>1)   if((b=strtok(NULL," "))==NULL) return(-1);

  for(i=0;i<n;i++){
    if((b=strtok(NULL," "))==NULL) return(-1);
    iscan=sscanf(b,"%lf",dpoi+i);
    if(iscan<=0) return(-1); }

  return(iscan);

}

int getint(char *buf, keyword *key, int *dpoi, int n)
{
  char mybuf[1000];
  int iscan,i;
  char *b = buf;

  strcpy(mybuf,buf);

  assert(key->number >0 && key->number <=2);

  if((b=strtok(mybuf," "))==NULL) return(-1);

  if(key->number>1)   if((b=strtok(NULL," "))==NULL) return(-1);

  for(i=0;i<n;i++){
    if((b=strtok(NULL," "))==NULL) return(-1);
#ifdef CVX    
    iscan=sscanf(b,"%d",dpoi+i);
#else
    iscan=sscanf(b,"%d",dpoi+i);
#endif
    if(iscan<=0) return(-1); }

  return(iscan);

}

int readdoublevar(FILE* infile, char *varname, double *var, int n)
{
  keyword key1;
  int ierr;
  char buf[500];
  
  setkeyword(&key1,1,varname,"");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {printf("cannot find '%s' in input file!\n",varname); exit(-1);}
  if(getdouble(buf,&key1,var,n)<0) yell(buf,"getdouble");

  return(1);
}

int askdoublevar(FILE* infile, char *varname, double *var, int n)
{
  keyword key1;
  int ierr;
  char buf[500];
  
  setkeyword(&key1,1,varname,"");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr==KEYWORD_FOUND) 
    if(getdouble(buf,&key1,var,n)<0) yell(buf,"getdouble");

  return(1);
}

int readintvar(FILE* infile, char *varname, int *var, int n)
{
  keyword key1;
  int ierr;
  char buf[500];
  
  setkeyword(&key1,1,varname,"");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {printf("cannot find '%s' in input file!\n",varname); exit(-1);}
  if(getint(buf,&key1,var,n)<0) yell(buf,"getint");

  return(1);
}

int askintvar(FILE* infile, char *varname, int *var, int n)
{
  keyword key1;
  int ierr;
  char buf[500];
  
  setkeyword(&key1,1,varname,"");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr==KEYWORD_FOUND) 
    if(getint(buf,&key1,var,n)<0) yell(buf,"getint");

  return(1);
}

int readstringvar(FILE* infile, char *varname, char *var, int n)
{
  keyword key1;
  int ierr;
  char buf[500];
  
  setkeyword(&key1,1,varname,"");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {printf("cannot find '%s' in input file!\n",varname); exit(-1);}
  if(getstr(buf,&key1,var,n)<0) yell(buf,"getstring");

  return(1);
}
int askstringvar(FILE* infile, char *varname, char *var, int n)
{
  keyword key1;
  int ierr;
  char buf[500];
  
  setkeyword(&key1,1,varname,"");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr==KEYWORD_FOUND) {
    if(getstr(buf,&key1,var,n)<0) yell(buf,"getstring");
    return(1);
  }
  else return(0);
}


void crystal_struc(char *fname, int *NTYP, int *MXATM, 
		   int **NATOMS, double **COORD,  double *LVEC, 
		   char **ATM, double **mass, double **SZET, 
		   double **MTS, double **DIPOLE)
{
  char buf[1000]; /* buffer to read in one line */
  int i,j,k;
  int ierr=0;
  int istop, istop2, natom, ntypes,mxdatm;
  int nvec;
  double vec[4],gvec[9], lv[3][3],  volume;

  int *natoms;
  double *coord, *lvec, *szet, *mts, *dipole;
  char *atm;
  char charbuf[3];

  keyword key1, key2, key3, key4;
  FILE *infile;

  infile=fopen(fname,"r");
  if(infile==NULL) { printf("couldn't open file %s\n",fname); exit(-1);}


  setkeyword(&key1,2,"begin","coordinates");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {printf("cannot find 'begin coordinates' in input file\n"); exit(-1);}

  setkeyword(&key1,1,"newtype","");
  setkeyword(&key2,1,"coord","");
  setkeyword(&key3,1,"end","");

  /* count the number of atomic types and coordinates per type */

  istop =1;  natom =0;  ntypes  =0;  mxdatm=0;

  while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {
    if(keytest(buf,&key1)==KEYWORD_FOUND){ntypes++;natom=0;}/* found new type */
    else {
      if(keytest(buf,&key2)==KEYWORD_FOUND) /* found coordinate */
	{natom++; if(natom>mxdatm) mxdatm=natom;}
      else{
	if(keytest(buf,&key3)==KEYWORD_FOUND) {istop=0;} /* found end */	
	else {
	  printf("found funny keyword in coordinate structure:\n%s\n", buf);}
      }
    }
  }

  /* printf("ntyp=%d, mxdatm=%d\n", ntypes, mxdatm); */
  
  if(ntypes<=0) {printf("cannot find atomic types in input file\n"); exit(-1);}
  if(mxdatm<=0){printf("cannot find coordinates in input file\n"); exit(-1);}


  /* Now read in the coordinates */

  /* allocate arrays */

  natoms= malloc(ntypes*sizeof(int));              
  coord = malloc(3*mxdatm*ntypes*sizeof(double));
  szet  = malloc(ntypes*sizeof(double));
  mts  = malloc(ntypes*sizeof(double));
  dipole  = malloc(ntypes*sizeof(double));
  atm   = malloc((2*ntypes+2)*sizeof(char));
  *mass  = (double *)malloc(ntypes*sizeof(double));

  /* reposition to begin coordinates*/
  setkeyword(&key1,2,"begin","coordinates");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {printf("cannot find 'begin coordinates' in input file\n"); exit(-1);}


  /* read the stuff in */


  setkeyword(&key1,1,"newtype","");
  setkeyword(&key2,1,"coord","");
  setkeyword(&key3,1,"end","");
  setkeyword(&key4,2,"fake","key");

  istop =1;
  ntypes  =-1;

  while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {

    if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new type */
      ntypes++;
      natoms[ntypes]=-1;
      if(getstr(buf,&key1,atm+ntypes*2,2)<0) yell(buf,"getstring"); 
      if(atm[ntypes*2+1]=='\0') {atm[ntypes*2+1]=atm[ntypes*2];
				 atm[ntypes*2]=' ';}
      charbuf[0]=atm[ntypes*2]; charbuf[1]=atm[ntypes*2+1]; charbuf[2]='\0';
      (*mass)[ntypes]=getmass(charbuf);
      szet[ntypes]=0.0; mts[ntypes]=1.0; dipole[ntypes]=0.0;
      if(getdouble(buf,&key4,vec,1)>=0) {
	szet[ntypes] = vec[0]; 
	if(getdouble(buf,&key4,vec,2)>=0) { 
	  mts[ntypes]=vec[1];
	  if(getdouble(buf,&key4,vec,3)>=0)
	    dipole[ntypes]=vec[2];
	}
      }
    }
    else {
      if(keytest(buf,&key2)==KEYWORD_FOUND) /* found coordinate */
	{ natoms[ntypes]++; 
	  if(getdouble(buf,&key2,vec,3)<0)  yell(buf,"getdouble");
	  coord[mxdatm*ntypes*3+3*natoms[ntypes]]   = vec[0];
	  coord[mxdatm*ntypes*3+3*natoms[ntypes]+1] = vec[1];
	  coord[mxdatm*ntypes*3+3*natoms[ntypes]+2] = vec[2];
	}
      else{
	if(keytest(buf,&key3)==KEYWORD_FOUND) {istop=0;} /* found end */
	else{
	  printf("found funny keyword in coordinate structure:\n%s\n", buf);}
      }
    }
  }
  ntypes++;

  
  for(i=0;i<ntypes;i++) {
    if(++natoms[i]<1) 
      {printf("found no coordinates for type %2s\n",atm+i*2); exit(-1);}
    /*
    printf("type %.2s natoms %d\n",atm+i*2,natoms[i]); 
    for(j=0;j<natoms[i];j++)
      {
	printf("%.2s %f %f %f\n",atm+i*2,coord[3*mxdatm*i+3*j],
	       coord[3*mxdatm*i+3*j+1],coord[3*mxdatm*i+3*j+2]);
      }
      */
    
  }
  
  /* --------------- read lattice vectors ---------------------- */

  
  lvec = LVEC;

  setkeyword(&key1,2,"begin","latticevecs");

  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {printf("cannot find 'begin latticevecs' in input file\n"); exit(-1);}

  setkeyword(&key1,1,"coord","");
  setkeyword(&key2,1,"volume","");
  setkeyword(&key3,1,"end","");

  istop =1;
  istop2 =1;
  nvec  =0;
  while((ierr=getnextline(infile, buf) !=EOF)&&
	((istop!=0)||(istop2!=0))) {
    if(keytest(buf,&key1)==KEYWORD_FOUND) /* found coordinate */
      { if(getdouble(buf,&key1,&(lv[nvec][0]),3)<0) 
	  yell(buf,"getdouble");
	nvec++;
	if(nvec>=3) istop=0;
      }
    if(keytest(buf,&key2)==KEYWORD_FOUND) /* found volume */
      { if(getdouble(buf,&key2,&volume,1)<0) 
	  yell(buf,"getdouble");
	istop2=0;
      }
    if(keytest(buf,&key3)==KEYWORD_FOUND) /* found end */
      { 
	if(istop!=0) {
	  printf("cannot find all coordinates for lattice vectors\n");
	  exit(-1);
	}
	if(istop2!=0) {
	  for(nvec=0;nvec<3;nvec++)
	    for(i=0;i<3;i++)
	      lvec[3*i+nvec]=lv[nvec][i];
	  volume=fabs(matinvert(lvec));
	  istop2=0;
	}
      }
  }

  for(nvec=0;nvec<3;nvec++)
    for(i=0;i<3;i++)
      lvec[3*i+nvec]=lv[nvec][i];
  

  

  /* rescale coordinates */

  for(i=0;i<9;i++) gvec[i]=lvec[i];

  volume=pow(volume/fabs(matinvert(gvec)),0.333333333333333);

  for(i=0;i<9;i++) lvec[i]=lvec[i]*volume;

  /*
  printf("lattice vectors:\n%f %f %f\n%f %f %f\n%f %f %f\n",
	 lvec[0],lvec[3],lvec[6],lvec[1],lvec[4],
	 lvec[7],lvec[2],lvec[5],lvec[8]);
	 */

/* -----calculate relative coordinates if given absolute ------- */

  setkeyword(&key1,2,"coordinates","absolute");
  if(findkeyword(infile,&key1,buf)==KEYWORD_FOUND){
    for(i=0;i<9;i++) gvec[i]=lvec[i];
    matinvert(gvec);
    for(i=0;i<ntypes;i++) { /* ----- loop over types ----- */
      for(j=0;j<natoms[i];j++) { /* ----- loop over atoms of this type--- */
	for(k=0;k<3;k++){ vec[k]=coord[i*mxdatm*3+j*3+k]; }
	coord[i*mxdatm*3+j*3]=vec[0]*gvec[0]+vec[1]*gvec[1]+vec[2]*gvec[2];
	coord[i*mxdatm*3+j*3+1]=vec[0]*gvec[3]+vec[1]*gvec[4]+vec[2]*gvec[5];
	coord[i*mxdatm*3+j*3+2]=vec[0]*gvec[6]+vec[1]*gvec[7]+vec[2]*gvec[8];
      }
    }
  }

  *MXATM=mxdatm;
  *NTYP = ntypes;
  *NATOMS = natoms;
  *COORD = coord;
  *SZET  = szet;
  *MTS   = mts;
  *DIPOLE = dipole;
  *ATM   = atm;

  fclose(infile);

}

double matinvert(double *mat)
{
  double rcond, det[2];
  int ipvt[3], dum1=3, dum2=3, dum4=3, dum3=11, info;
  double work[600], vol;

#if(defined(SGI_PC) || defined(CVX) || defined(SUN))
  DGECO(mat,&dum1, &dum2,ipvt,&rcond,work);
  if(fabs(rcond) <1e-7) {printf("matinvert: singular matrix!\n");exit(-1);}
  DGEDI(mat,&dum1, &dum2,ipvt,det,work,&dum3);
  return(det[0]*pow(10,det[1]));
#endif
#ifdef RS6K
  dgeicd(mat,3,3,2,&rcond,det,work,600);
  if(det[1] <-5) {printf("matinvert: singular matrix!\n");exit(-1);}
  return(det[0]*pow(10,det[1]));
#endif
#ifdef CRAY
  vol=(mat[0]*(mat[4]*mat[8]-mat[7]*mat[5])
	 -mat[1]*(mat[3]*mat[8]-mat[6]*mat[5])
	 +mat[2]*(mat[3]*mat[7]-mat[6]*mat[4]));
  SGETRF(&dum1,&dum2, mat, &dum4, ipvt, &info);
  SGETRI(&dum1, mat,&dum2, ipvt, work, &dum3, &info);
  return(vol);  
#endif
#ifdef ALPHA
  vol=(mat[0]*(mat[4]*mat[8]-mat[7]*mat[5])
	 -mat[1]*(mat[3]*mat[8]-mat[6]*mat[5])
	 +mat[2]*(mat[3]*mat[7]-mat[6]*mat[4]));
  dgetrf_(&dum1,&dum2, mat, &dum4, ipvt, &info);
  dgetri_(&dum1, mat,&dum2, ipvt, work, &dum3, &info);
  return(vol);  
#endif


}

/* a table with all the masses of the atoms in [u] */


struct mass_table {
  char name[3];
  double mass;
};

#define AT_TB_SIZE 105
struct mass_table 
table_of_masses[AT_TB_SIZE]= {
{"Ac",  227},
{"Al",  26.981539},
{"Am",  243},
{"Sb",  121.760},
{"Ar",  39.948},
{"As",  74.92159},
{"At",  210},
{"Ba",  137.327},
{"Bk",  247},
{"Be",  9.012182},
{"Bi",  208.98037},
{" B", 10.811},
{"Br",  79.904},
{"Cd",  112.411},
{"Cs",  132.90543},
{"Ca",  40.078},
{"Cf",  251},
{" C", 12.011},
{"Ce",  140.115},
{"Cl",  35.4527},
{"Cr",  51.9961},
{"Co",  58.93320},
{"Cu",  63.546},
{"Cm",  247},
{"Dy",  162.50},
{"Es",  252},
{"Er",  167.26},
{"Eu",  151.965},
{"Fm",  257},
{" F", 18.9984032},
{"Fr",  223},
{"Gd",  157.25},
{"Ga",  69.723},
{"Ge",  72.61},
{"Au",  196.96654},
{"Hf",  178.49},
{"Jl",  262},
{"He",  4.002602},
{"Ho",  164.93032},
{" H", 1.00794},
{"In",  114.818},
{" I", 126.90447},
{"Ir",  192.217},
{"Fe",  55.845},
{"Kr",  83.80},
{"La",  138.9055},
{"Lr",  262},
{"Pb",  207.2},
{"Li",  6.941},
{"Lu",  174.967},
{"Mg",  24.3050},
{"Mn",  54.93805},
{"Md",  258},
{"Hg",  200.59},
{"Mo",  95.94},
{"Nd",  144.24},
{"Ne",  20.1797},
{"Np",  237},
{"Ni",  58.6934},
{"Nb",  92.90638},
{" N", 14.00674},
{"No",  259},
{"Os",  190.23},
{" O", 15.9994},
{"Pd",  106.42},
{" P", 30.973762},
{"Pt",  195.08},
{"Pu",  244},
{"Po",  209},
{" K", 39.0983},
{"Pr",  140.90765},
{"Pm",  145},
{"Pa",  231.03588},
{"Ra",  226},
{"Rn",  222},
{"Re",  186.207},
{"Rh",  102.90550},
{"Rb",  85.4678},
{"Ru",  101.07},
{"Db",  261},
{"Sm",  150.36},
{"Sc",  44.955910},
{"Se",  78.96},
{"Si",  28.0855},
{"Ag",  107.8682},
{"Na",  22.989768},
{"Sr",  87.62},
{" S", 32.066},
{"Ta",  180.9479},
{"Tc",  98},
{"Te",  127.60},
{"Tb",  158.92534},
{"Tl",  204.3833},
{"Th",  232.0381},
{"Tm",  168.93421},
{"Sn",  118.710},
{"Ti",  47.867},
{" W", 183.84},
{" U", 238.0289},
{" V", 50.9415},
{"Xe",  131.29},
{"Yb",  173.04},
{" Y", 88.90585},
{"Zn",  65.39},
{"Zr",  91.224}
};

double getmass(char *s)
{
  int i;
  for(i=0;i<AT_TB_SIZE;i++)
    if(strcmp(s,table_of_masses[i].name)==0){
      return(table_of_masses[i].mass); }

  printf("Could not find the element %s in the table of masses!\n",s);
  exit(-1);
  return(0);
}

/* read all the parameters for the planewave code */

void read_pwparam(FILE *infile, struct pwparam *pwp)
{ 
  char sbuf[255],buf[255];
  int ierr,istop,nwv,nline, ifound,i;
  keyword key1,key2,key3;

  /* always update pwmain string array also. only add at the end ! */
#define NUM_PW_JOBS 5
  char *pw_jobstrings[NUM_PW_JOBS] ={
    "scf",
    "nmr_shift",
    "nmr_plot",
    "band_structure",
    "pot_plot"
  };
  
  pwp->nrk=0; askintvar(infile, "number_kpoints",&(pwp->nrk),1);
  if(pwp->nrk<=0) {
    pwp->ngrid[0]=1;pwp->ngrid[1]=1;pwp->ngrid[2]=1;
    askintvar(infile, "k_grid",pwp->ngrid,3);
    (*pwp).kshift[0]=0; (*pwp).kshift[1]=0;  (*pwp).kshift[2]=0; 
    askdoublevar(infile, "k_grid_shift",pwp->kshift,3);
  }
  pwp->qshift[0] = 0.00;
  pwp->qshift[1] = 0.00;
  pwp->qshift[2] = 0.00;
  askdoublevar(infile,"gw_shift",pwp->qshift,3);
  pwp->dsmear=0.05; askdoublevar(infile, "gaussian_smearing",&(pwp->dsmear),1);
  pwp->epsdiag=1e-12; askdoublevar(infile, "accuracy_diag",&(pwp->epsdiag),1);
  pwp->esafety=1.0; askdoublevar(infile, "diagsafety",&(pwp->esafety),1);
  pwp->randomstart=0.0; askdoublevar(infile, "randomize_diag_start_guess",&(pwp->randomstart),1);
  pwp->bandgap=0.0; askdoublevar(infile, "bandgap",&(pwp->bandgap),1);
  

  readdoublevar(infile, "energy_cutoff",&(pwp->ecutoff),1);
  readdoublevar(infile, "submatrix_energy_cutoff",&(pwp->ecutoffsub),1);
  readdoublevar(infile, "mixing_energy_cutoff",&(pwp->ecutoffmix),1);

  /* corrected kinetic energy functional */

  if(askstringvar(infile, "modify_kinetic_energy", sbuf, 255)==1){
    if(sscanf(sbuf,"%lf %lf %lf",&(pwp->modify_kinetic_energy[0]),
	      &(pwp->modify_kinetic_energy[1]),
	      &(pwp->modify_kinetic_energy[2]))<3) {
      if(strcmp(sbuf,"on")==0) {
	pwp->modify_kinetic_energy[0]=pwp->ecutoff*2.0; 
	pwp->modify_kinetic_energy[1]=pwp->ecutoff; 
	pwp->modify_kinetic_energy[2]=pwp->ecutoff/10.0;}
      else if(strcmp(sbuf,"off")==0) {
	pwp->modify_kinetic_energy[0]=0.0;  
	pwp->modify_kinetic_energy[1]=1000.0;
	pwp->modify_kinetic_energy[2]=1.0;}
      else{
	printf("unrecognized option to keyword modify_kinetic_energy: %s\n",
	       sbuf);} 
    }
  }
  else {  /* no correction of the kinetic energy functional */
    pwp->modify_kinetic_energy[0]=0.0;  pwp->modify_kinetic_energy[1]=1000.0;
    pwp->modify_kinetic_energy[2]=1.0;}

  pwp->nrg=10; askintvar(infile,"number_radial_gridpoints",&(pwp->nrg),1);

  pwp->correct_stress_cutoff=-1.0;
  askdoublevar(infile, "polished_energy_cutoff",
	       &(pwp->correct_stress_cutoff),1);

  pwp->ispn=1;            askintvar(infile, "number_of_spins",&(pwp->ispn),1);
  pwp->imaxitdiag=40; askintvar(infile,"max_iter_diag",&(pwp->imaxitdiag),1);
  pwp->imaxitscf=40; askintvar(infile,"max_iter_scfloop",&(pwp->imaxitscf),1);
  pwp->nprocpvm=1;          askintvar(infile, "nproc_pvm",&(pwp->nprocpvm),1);


  pwp->iscr=4;              askstringvar(infile, "screening_type",sbuf,255);
  if(strcmp(sbuf,"atomic")==0) pwp->iscr=5;
  if(strcmp(sbuf,"previous")==0) pwp->iscr=4;

  pwp->imonitor=0;             
  askstringvar(infile, "output_flags",sbuf,255);
  if(strstr(sbuf,"diagperf")!=NULL) pwp->imonitor+=1;
  if(strstr(sbuf,"vqmc")!=NULL) pwp->imonitor+=2;
  if(strstr(sbuf,"nmrshift")!=NULL) pwp->imonitor+=4;
  if(strstr(sbuf,"timing")!=NULL) pwp->imonitor+=8;
  if(strstr(sbuf,"cdplot")!=NULL) pwp->imonitor+=16;
  if(strstr(sbuf,"ballnstick")!=NULL) pwp->imonitor+=32;
  if(strstr(sbuf,"neighbor")!=NULL) pwp->imonitor+=64;
  if(strstr(sbuf,"dos")!=NULL) pwp->imonitor+=128;
  if(strstr(sbuf,"waveplot")!=NULL) pwp->imonitor+=256;
  if(strstr(sbuf,"wavefn")!=NULL) pwp->imonitor+=512;
  if(strstr(sbuf,"angdos")!=NULL) pwp->imonitor+=1024;
  if(strstr(sbuf,"eigval")!=NULL) pwp->imonitor+=2048;
  if(strstr(sbuf,"momden")!=NULL) pwp->imonitor+=4096;
  if(strstr(sbuf,"angle")!=NULL) pwp->imonitor+=8192;
  if(strstr(sbuf,"ballswrapped")!=NULL) pwp->imonitor+=16384;
  if(strstr(sbuf,"khoros")!=NULL) pwp->imonitor+=32768;
  if(strstr(sbuf,"memusage")!=NULL) pwp->imonitor+=65536;
  if(strstr(sbuf,"efield")!=NULL) pwp->imonitor+=131072;
  if(strstr(sbuf,"potplot")!=NULL) pwp->imonitor+=262144;
  if(strstr(sbuf,"gwr")!=NULL) pwp->imonitor+=524288;
  if(strstr(sbuf,"gwc")!=NULL) pwp->imonitor+=1048576;
  if(strstr(sbuf,"gwscreening")!=NULL) pwp->imonitor=2097152; 

  pwp->miscflag=0;             
  setkeyword(&key1,2,"no","restore_inversion");
  if(findkeyword(infile,&key1,buf)==KEYWORD_FOUND) 
    pwp->miscflag+=1;

  setkeyword(&key1,2,"checkpoint","nmr");
  if(findkeyword(infile,&key1,buf)==KEYWORD_FOUND) 
    pwp->miscflag+=2;

  pwp->checkpoint_wfn=1000000;
  askintvar(infile, "checkpoint_emin",&(pwp->checkpoint_wfn),1);


  pwp->icorr=0; /* default is ceperley-alder */             
  askstringvar(infile, "exchange_correlation",sbuf,255);
  if(strstr(sbuf,"perdew_wang_91")!=NULL) pwp->icorr=1;
  if(strstr(sbuf,"perdew_burke_ernzerhof")!=NULL) pwp->icorr=2;

  pwp->input=0;
  memset(sbuf,0,255);
  askstringvar(infile, "input_flags",sbuf,255);
  if(strstr(sbuf,"wavefn")!=NULL) pwp->input+=1;
  if(strstr(sbuf,"lukman")!=NULL) pwp->input+=2;
  if(strstr(sbuf,"chkptchi")!=NULL) pwp->input+=4;

  pwp->ioptimize=0;             
  askstringvar(infile, "optimize",sbuf,255);
  if(strstr(sbuf,"memory")!=NULL) pwp->ioptimize+=1;
  pwp->ioptimize+=2;
  if(strstr(sbuf,"nostartguess")!=NULL) pwp->ioptimize-=2;
  pwp->ioptimize+=4;
  if(strstr(sbuf,"nocgsolveguess")!=NULL) pwp->ioptimize-=4;
  if(strstr(sbuf,"insulator")!=NULL) pwp->ioptimize+=8;
  pwp->ioptimize+=16;
  if(strstr(sbuf,"nowfnreuse")!=NULL) pwp->ioptimize-=16;

  pwp->ioptimize+=32;
  if(strstr(sbuf,"noprecond")!=NULL) pwp->ioptimize-=32;

  if(strstr(sbuf,"metal")!=NULL) pwp->ioptimize+=64;
  if(strstr(sbuf,"descend")!=NULL) pwp->ioptimize+=128;



  pwp->ioccup=0;
  askstringvar(infile, "occupy_levels",sbuf,255);
  if(strstr(sbuf,"normal")!=NULL) pwp->ioccup=0;
  if(strstr(sbuf,"fixed")!=NULL) pwp->ioccup=1;
  if(strstr(sbuf,"antiferro")!=NULL) pwp->ioccup=2;

  pwp->fermi_level=0;
  askdoublevar(infile, "fermi_level",&(pwp->fermi_level),1);


  (*pwp).mixing[0] =0.77; askdoublevar(infile, "mixing_parameter",
				     pwp->mixing,1);
  (*pwp).mixing[1] =0.0; (*pwp).mixing[2] =0.6; (*pwp).mixing[3] =0.05; 
  askdoublevar(infile, "linear_mixing",&(pwp->mixing[1]),3);

  pwp->pot_conv = 1.0e-6;
  askdoublevar(infile, "potential_convergence_criterion",&(pwp->pot_conv),1);

  pwp->chi_q = 0.01; /* changed from 0.001 to 0.01 20.1.97 */
  askdoublevar(infile, "nmr_q",&(pwp->chi_q),1);


  (*pwp).g0mask[0]=0.6666666666666667;
  (*pwp).g0mask[1]=0;
  (*pwp).g0mask[2]=0;
  (*pwp).g0mask[3]=0;
  (*pwp).g0mask[4]=0.6666666666666667;
  (*pwp).g0mask[5]=0;
  (*pwp).g0mask[6]=0;
  (*pwp).g0mask[7]=0;
  (*pwp).g0mask[8]=0.6666666666666667;
  askdoublevar(infile, "nmr_g0mask",pwp->g0mask,9);


  (*pwp).visualize[0]=0;
  (*pwp).visualize[1]=0;
  (*pwp).visualize[2]=0; askintvar(infile, "visualize",pwp->visualize,3);
  pwp->ntrans=-1;    askintvar(infile, "number_symmetry_ops",&(pwp->ntrans),1);

  readintvar(infile, "number_bands",&(pwp->nband),1);
  /* pwp->ifmin=1; askintvar(infile, "band_ifmin",&(pwp->ifmin),1);
     pwp->ifmax=pwp->nband; askintvar(infile, "band_ifmax",&(pwp->ifmax),1); 
     */

  pwp->gmagnet[0]=0;  pwp->gmagnet[1]=0; pwp->gmagnet[2]=0; pwp->gmagnet[3]=0;
  pwp->h0=0.0;
  askintvar(infile, "magnetic_field_kvec",pwp->gmagnet,4);
  askdoublevar(infile, "magnetic_field_strength",&(pwp->h0),1);


  pwp->gepot[0]=0;  pwp->gepot[1]=0; pwp->gepot[2]=0; pwp->gepot[3]=0;
  pwp->eampli=0.0;
  askintvar(infile, "electric_potential_kvec",pwp->gepot,4);
  askdoublevar(infile, "electric_potential_strength",&(pwp->eampli),1);

  /* --------------- read the plot_wave_function information --------- */

  setkeyword(&key1,2,"begin","plot_wave_function");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {pwp->nplotwv=0; pwp->plotwv =NULL;}
  else {               

    setkeyword(&key1,1,"wavefunction","");
    setkeyword(&key2,1,"end","");

    /* count the number of wavefunctions to be plotted */

    istop = 1;
    nwv  =  0;

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {

      if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new line */
	nwv = nwv +1;}
      else{
	if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
	else{
	  printf("found funny keyword in plot_wavefunctions structure:\n%s\n",
		 buf);}
      }
    }
    pwp->nplotwv=nwv;

    pwp->plotwv=malloc(2*nwv*sizeof(int)); /* allocate memory */

    istop = 1;
    nwv  =  0;
    setkeyword(&key1,2,"begin","plot_wave_function");
    ierr=findkeyword(infile, &key1, buf);
    setkeyword(&key1,1,"wavefunction","");

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {
      setkeyword(&key1,1,"wavefunction","");
      if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new line */
	if(getint(buf,&key1,pwp->plotwv+nwv*2,2)<0)  yell(buf,"getint");
	nwv = nwv +1;}
      else{
	if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
	else{
	  printf("found funny keyword in plot_wavefunctions structure:\n%s\n",
		 buf);}
      }

    }
  }
  /* --------------- read the line_plot information --------- */

  (*pwp).bra[0]=1; (*pwp).bra[1]=0; (*pwp).bra[2]=0;
  (*pwp).ket[0]=1; (*pwp).ket[1]=0; (*pwp).ket[2]=0;
  askdoublevar(infile, "tensor_bra",pwp->bra,3);
  askdoublevar(infile, "tensor_ket",pwp->ket,3);

  setkeyword(&key1,2,"begin","line_plot");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {pwp->nlineplot=0; pwp->lineplot =NULL;}
  else {               

    setkeyword(&key1,1,"line","");
    setkeyword(&key2,1,"end","");

    /* count the number of lines to be plotted */

    istop = 1;
    nline  =  0;

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {

      if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new line */
	nline++;}
      else{
	if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
	else{
	  printf("found funny keyword in line_plot structure:\n%s\n",
		 buf);}
      }
    }
    pwp->nlineplot=nline;

    pwp->lineplot=malloc(7*nline*sizeof(double)); /* allocate memory */

    istop = 1;
    nline  =  0;
    setkeyword(&key1,2,"begin","line_plot");
    ierr=findkeyword(infile, &key1, buf);
    setkeyword(&key1,1,"line","");

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {
      if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new line */
	if(getdouble(buf,&key1,pwp->lineplot+nline*7,7)<0) 
	  yell(buf,"getdouble");
	nline++;}
      else{
	if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
	else{
	  printf("found funny keyword in line_plot structure:\n%s\n",
		 buf);}
      }

    }
  }
  /*         -------- read the joblist for the pw code -------- */
  
  setkeyword(&key1,2,"begin","pw_jobs");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) { /* default: do just the scf loop */
    pwp->njobs=1;
    pwp->joblist=malloc(pwp->njobs*sizeof(int));
    pwp->joblist[0]=0;
  }
  else{
    setkeyword(&key1,1,"pw_job","");
    setkeyword(&key2,1,"end","");

    /* count the number of pw_job */

    pwp->njobs=0;
    istop=1;

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {
      if(keytest(buf,&key1)==KEYWORD_FOUND) {pwp->njobs++;}/* found job */
      else {
	if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
	else {
	  printf("found funny keyword in pw_jobs:\n%s\n", buf);
	  exit(-1);
	}
      }
    }
    if(pwp->njobs<=0) {
      printf("pw_jobs list is empty!\n");
      exit(-1);
    }
    /* now read them in */

    pwp->joblist=malloc(pwp->njobs*sizeof(int));
    setkeyword(&key1,2,"begin","pw_jobs");
    ierr=findkeyword(infile, &key1, buf);

    setkeyword(&key1,1,"pw_job","");
    setkeyword(&key2,1,"end","");

    pwp->njobs=0;
    istop=1;

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {
      if(keytest(buf,&key1)==KEYWORD_FOUND) {/* found job */
	if(getstr(buf,&key1,sbuf,255)<0) yell(buf,"getstring"); 
	ifound=0;
	for(i=0;i<NUM_PW_JOBS;i++) {
	  if(strcmp(sbuf,pw_jobstrings[i])==0) {ifound=1; break;}
	}
	if(ifound==0) {
	  printf("illegal pw_job:\n %s\n",sbuf);
	  exit(-1);
	}
	pwp->joblist[pwp->njobs]=i;
	pwp->njobs++;}
      else {
	if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
	else {
	  printf("found funny keyword in pw_jobs:\n%s\n", buf);
	  exit(-1);
	}
      }
    }
    

  }

  /* ---------- read the energy window information ------------------ */



  setkeyword(&key1,2,"begin","energy_window");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {pwp->nenergywindow=0; pwp->energywindow =NULL;}
  else {               
    setkeyword(&key1,1,"window","");
    setkeyword(&key2,1,"end","");

    /* count the number of energy windows */

    istop = 1;
    nline  =  0;

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {

      if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new line */
	nline++;}
      else{
	if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
	else{
	  printf("found funny keyword in energy_window structure:\n%s\n",
		 buf);}
      }
    }
    if(nline<=0) {
      printf("found no energy windows in energy_window structure:\n");
      exit(-1);
    }
    pwp->nenergywindow=nline;

    pwp->energywindow=malloc(2*nline*sizeof(double)); /* allocate memory */

    istop = 1;
    nline  =  0;
    setkeyword(&key1,2,"begin","energy_window");
    ierr=findkeyword(infile, &key1, buf);
    setkeyword(&key1,1,"window","");

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {
      if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new line */
	if(getdouble(buf,&key1,pwp->energywindow+nline*2,2)<0) 
	  yell(buf,"getdouble");
	nline++;}
      else{
	if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
	else{ printf("found funny keyword in energy_window structure:\n%s\n",
		 buf);}
      }

    }
  }





  /* ---------- read the stay_put information ------------------ */



  setkeyword(&key1,2,"begin","stay_put");
  ierr=findkeyword(infile, &key1, buf);
  if(ierr!=KEYWORD_FOUND) 
    {pwp->nstay_put=0; pwp->stay_put =NULL;}
  else {               
    setkeyword(&key1,1,"stay_put","");
    setkeyword(&key2,1,"end","");
    setkeyword(&key3,1,"stay_put_lattice","");

    /* count the number of atoms to stay put */

    istop = 1;
    nline  =  0;

    while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {

      if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new line */
	nline++;}
      else if (keytest(buf,&key3)==KEYWORD_FOUND){ /* found new line */
        nline++;
      } else {
        if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
        else{
          printf("found funny keyword in energy_window structure:\n%s\n",
                 buf);}
      }
    }
    if(nline<=0) {
      printf("found no atoms to stay put in stay_put structure:\n");
      exit(-1);
    }
    pwp->nstay_put=nline;

    pwp->stay_put=malloc(nline*sizeof(int)); /* allocate memory */

    istop = 1;
    nline  =  0;
    setkeyword(&key1,2,"begin","stay_put");
    ierr=findkeyword(infile, &key1, buf);
    setkeyword(&key1,1,"stay_put","");

    {
      /* I'm going to try to be very user friendly.  I'll let peepul input
         their constraints as 'stay_put_lattice zx', etc.  Even xz or zx :) */
      keyword keyxx,keyyy,keyzz, keyxy, keyyz, keyzx, keyyx, keyzy, keyxz;
      setkeyword(&keyxx, 2, "stay_put_lattice", "xx");
      setkeyword(&keyyy, 2, "stay_put_lattice", "yy");
      setkeyword(&keyzz, 2, "stay_put_lattice", "zz");
      setkeyword(&keyxy, 2, "stay_put_lattice", "xy");
      setkeyword(&keyyx, 2, "stay_put_lattice", "yx");
      setkeyword(&keyyz, 2, "stay_put_lattice", "yz");
      setkeyword(&keyzy, 2, "stay_put_lattice", "zy");
      setkeyword(&keyzx, 2, "stay_put_lattice", "zx");
      setkeyword(&keyxz, 2, "stay_put_lattice", "xz");

      while((ierr=getnextline(infile, buf) !=EOF)&&(istop)) {
        if(keytest(buf,&key1)==KEYWORD_FOUND){ /* found new line */
          if(getint(buf,&key1,pwp->stay_put + nline,1)<0)
            yell(buf,"getint");
          nline++;
        } else if (keytest(buf,&keyxx)==KEYWORD_FOUND) { /* hold xx */
          pwp->stay_put[nline]=-1;
          nline++;
        } else if (keytest(buf,&keyyy)==KEYWORD_FOUND) { /* hold yy */
          pwp->stay_put[nline]=-2;
          nline++;
        } else if (keytest(buf,&keyzz)==KEYWORD_FOUND) { /* hold zz */
          pwp->stay_put[nline]=-3;
          nline++;
        } else if (keytest(buf,&keyxy)==KEYWORD_FOUND) { /* hold xy */
          pwp->stay_put[nline]=-4;
          nline++;
        } else if (keytest(buf,&keyyx)==KEYWORD_FOUND) { /* hold xy */
          pwp->stay_put[nline]=-4;
          nline++;
        } else if (keytest(buf,&keyzy)==KEYWORD_FOUND) { /* hold yz */
          pwp->stay_put[nline]=-5;
          nline++;
        } else if (keytest(buf,&keyyz)==KEYWORD_FOUND) { /* hold yz */
          pwp->stay_put[nline]=-5;
          nline++;
        } else if (keytest(buf,&keyzx)==KEYWORD_FOUND) { /* hold zx */
          pwp->stay_put[nline]=-6;
          nline++;
        } else if (keytest(buf,&keyxz)==KEYWORD_FOUND) { /* hold zx */
          pwp->stay_put[nline]=-6;
          nline++;
        } else if (keytest(buf,&key3)==KEYWORD_FOUND) { /* hold  # */
          if(getint(buf,&key3,pwp->stay_put + nline,1)<0)
            yell(buf,"getint");
          pwp->stay_put[nline]=-pwp->stay_put[nline];
          nline++;
        } else {
          if(keytest(buf,&key2)==KEYWORD_FOUND) {istop=0;} /* found end */
          else{ printf("found funny keyword in energy_window structure:\n%s\n",
                       buf);}
        }
      }
    }
  }





  /*         ---- generate the kpoints for a band structure plot -------- */

  istop=0; /* is any of the jobs a band structure calculation ? */
  for(i=0;i<pwp->njobs;i++) if(pwp->joblist[i]==3) istop=1;

  if(istop==1)
    generate_kpts(infile, pwp->nband, &(pwp->labels));

}



