/*
       header file for the subroutines to read the input file

*/

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <math.h>
#ifdef RS6K
#include <essl.h>
#endif
#ifdef CVX
#endif


#define KEYWORD_NOT_FOUND -2000
#define KEYWORD_FOUND 2000

#define COMMENT_CHAR "#"

typedef struct kwd {int number; char kw[2][256];} keyword;


struct pwparam {

  int nrk;               /* number of k-points. if <=0, then calculate */
  int nband;/* number of spins. 1 for no spin polarization, 2 for spin pol. */
  int ngrid[3];
  int ispn;
  int input;                        /* triggers additional inputs to be read */
  int imaxitdiag;           /* max iterations that diagonalization should do */
  int imaxitscf;         /* max number of iterations that scf loop should do */
  int ilog;              /* flag to trigger log output by pw code */
  int nprocpvm;                                   /* number of procs for PVM */
  int iscr;               /* type of screening to be used on first iteration */
  int ioccup;                                        /* how to occupy levels */
  int ntrans;                    /* number of symmetry operations of crystal */
  int visualize[3];                                    /* visualization data */
  int gmagnet[4];                     /* switch + gvector for magnetic field */
  int gepot[4];                   /* switch + gvector for electric potential */
  int nplotwv;                           /* number of wave functions to plot */
  int *plotwv;                    /* memory location where plotwv is located */
  int nlineplot;                         /* number of wave functions to plot */
  double *lineplot;               /* memory location where plotwv is located */
  double qshift[3];                    /* shift for gw */
  int nenergywindow;                             /* number of energy windows */
  double *energywindow;           /* energy windows for various plots, in eV */
  int nstay_put;                             /* number of atoms to stay puts */
  int *stay_put;                        /* list of atoms which must stay put */
  int miscflag;                                        /* carries misc flags */
  int imonitor;                                /* triggers monitoring output */
  int ioptimize;                                   /* triggers optimizations */
  int checkpoint_wfn;     /* every how many steps the wavefn should be saved */
  int njobs;           /* number of jobs to be performed */
  int *joblist;        /* the job codes */
  int nrg;            /* number of grid points for radial grid integrations */
  int icorr;                       /* the type of exchange-correlation used */
  char   *labels;          /* a list of character labels for the bstruc plot*/
  double dsmear;                                  /* gaussian smearing in eV */
  double kshift[3];
  double bra[3];              /* vector from the left for tensor visual */
  double ket[3];              /* vector from the right for tensor visual */
  double epsdiag;                      /* accuracy of matrix diagonalization */
  double esafety;        /* safety shift for diagonalization */
  double randomstart;   
  double bandgap;  /* estimated band gap for insulators for best performance */
  double ecutoff;                                  /* various energy cutoffs */
  double ecutoffsub;
  double ecutoffmix;
  double correct_stress_cutoff;
  double chi_q;
  double g0mask[9];                                      /* mask for g=0 */
  double h0;
  double eampli;                     /* amplitude for the electric potential */
  double pot_conv;                            /* accuracy of selfconsistency */
  double mixing[4];               /* mixing parameter for mixing v_in, v_out */
  double modify_kinetic_energy[3];       /* parameters for modification of T */
  double fermi_level;        /* the fermi level as input parameter */
};


/* Places the next line of the inputfile into the string buf. 
   The return value is EOF if the end of the file is reached */ 
char *getline(FILE *infile, char *buf);

/* same as "getline", but skips commented lines */
int getnextline(FILE *infile, char *buf);

/* Searches a file "infile" for a keyword "name", and stores the line where 
   the keyword is found into the buffer "buf". */

int findkeyword(FILE *infile, keyword *name, char *buf);

/* sets a keyword to a certain value */

void setkeyword(keyword *key, int n, char *s1, char *s2);

/* test line for a keyword */
int keytest(char *buf, keyword *key);

/* displays error message */
void yell(char *buf, char *mess);

/* gets a string of length n following a keyword */
int getstr(char *buf, keyword *key, char *dstring, int n);

/* gets a sequence of n doubles following a keyword */
int getdouble(char *buf, keyword *key, double *dpoi, int n);

/* gets a sequence of n ints */
int getint(char *buf, keyword *key, int *dpoi, int n);

/* reads the lattice vectors and the crystal structure */
void crystal_struc(char *fname, int *ntyp, int *mxdatm, int **natoms, 
		   double **coord,  double *lvec, char **names, double **mass,
		   double **szet, double **mts, double **dipole);

/* returns determinant and inverse of mat */
double matinvert(double *mat);

/* read mandatory variables */
int readdoublevar(FILE* infile, char *varname, double *var, int n);
int readintvar(FILE* infile, char *varname, int *var, int n);
int readstringvar(FILE* infile, char *varname, char *var, int n);

/* read optional variables */
int askdoublevar(FILE* infile, char *varname, double *var, int n);
int askintvar(FILE* infile, char *varname, int *var, int n);
int askstringvar(FILE* infile, char *varname, char *var, int n);


void read_pwparam(FILE *infile, struct pwparam *pwp);
double getmass(char *s);



