C*
C*
c     @process extchk
      subroutine lineplot(imode, crys, dscale, 
     $     data, gs,  nlineplot, line_plot, bra, ket, ndim,
     $     outfilename)

      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'

c
c     do line plots.
c
c     if the number of bins is >0, then a regular line plot is done
c     if <0, then the average on the plane perpendicular to the line 
c     is computed
c
c
c     1996 Bernd Pfrommer


c
c     INPUT
c     -----

      type (parallel_gspace) gs ! gspace on which the data is defined
      type (crystal) crys       ! for the metric in realspace


      integer 
     $     ndim,                ! dimensionality of the tensor data
     $     imode,               ! mode flag
     $                          ! imode = 1 print real part of data
     $                          !       = 2 print imaginary part of data
     $                          !       = 3 print absolute square of data
     $     nlineplot            ! number of line plots to do 

      character*7   outfilename ! name of output file. Suffix .dx will be added

      double precision
     $     dscale,              ! scaling factor
     $     bra(3),              ! for tensor data left vector
     $     ket(3),              ! for tensor data right vector
     $     line_plot(7,nlineplot) ! line plot ((start,end,bins), lines)

      complex(kind=8)
     $     data(gs%length,ndim)      ! data to be visualized



c     
c     ------------- local variables
c
      integer iline, nbins, n, i, j, ig1,ig2,ig3,i3start,i3end,
     $     igi, igtot, igmax, iproc
      
      double precision
     $     rp, rip,
     $     startp(3), endp(3),d(3),r(3),twopi,dr, ovl, nvec(3), nlen

      double precision, allocatable:: lgvec(:)
      complex(kind=8) , allocatable:: ldata(:)

      character*3 numstring

      complex(kind=8) ft(ndim), ftv

c
c     ----- variables for the gspace loop ---------
c
      integer
     $     igv(4),fftn(4), ffth(4),igv3, irod,iord,igs

      double precision gv(3)    ! if required


      fftn(1:3)=gs%fftsize(1:3)
      fftn(4)=gs%fftsize(3)
      ffth(:)=fftn(:)/2

      twopi = 8.d0*atan(1.d0)
      
      if((ndim.ne.1).and.(ndim.ne.3).and.(ndim.ne.9)) then
         write(9,*) '*** NDIM =',ndim,' NOT IMPLEMENTED YET'
         stop
      endif

      write(9,*)

      do iline=1, nlineplot
         write(numstring,'(i3.3)') iline
         if(gs%myproc.eq.0) then
            open(18,file=outfilename // '.line.' // numstring,
     $           form='formatted', status='unknown')
         end if
         write(9,10) iline, outfilename // '.line.' // numstring
         call myflush(9)
         startp(:)=line_plot(1:3,iline)
         endp(:)=line_plot(4:6,iline)
         nbins=int(line_plot(7,iline))
         if(nbins.gt.0) then
            d(:)=(endp(:)-startp(:))/nbins
            do n=0,nbins
               r(:)=startp(:)+dble(n)*d(:)
               dr=sqrt(dot_product(d,matmul(crys%adot,d)))
               ft=0
               igs=0
               do iord=1, gs%lorder ! loop through x/y gspace 
                  irod=gs%order(1,iord)
                  igv(1)= irod/gs%fftsize(2)
                  igv(2)= mod(irod,gs%fftsize(2))
                  igv(3)= gs%order(2,iord)
                  igv(4)= gs%order(3,iord)
                  igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
                  gv(1:2)=dble(igv(1:2))+gs%rk(1:2)
                  do igv3=igv(3),igv(4) ! loop over z axis
                     gv(3)=dble(igv3)+gs%rk(3)
                     igs = igs + 1
                     do i=1,ndim
                        ft(i)=ft(i)+data(igs,i)*
     $                       exp(cmplx(0.d0, twopi*
     $                       (r(1)*gv(1)+r(2)*gv(2)+r(3)*gv(3)),kind=8))
                     end do

                  end do
               end do
               call all_sum_all(ft,ndim)
               if(gs%myproc.eq.0) then
 1768             format(8g20.10)
                  ft = ft*dscale
                  if(ndim.eq.3) then
                     if(imode.eq.1)
     $                    write(18,1768) dble(n)*dr,(dble(ft(i)),
     $                    i=1,ndim)
                     if(imode.eq.2)
     $                    write(18,1768) dble(n)*dr,(aimag(ft(i)),
     $                    i=1,ndim)
                     if(imode.eq.3)
     $                    write(18,1768) dble(n)*dr,(abs(ft(i))*
     $                                       abs(ft(i)),i=1,ndim)
                     if(imode.eq.0)
     $                  write(18,1768) dble(n)*dr,(abs(ft(i)),i=1,ndim)
                  elseif(ndim.eq.9) then
                     ftv =0
                     do j=0,2
                        ftv = ftv + (bra(1)*ft(j*3+1)
     $                       +bra(2)*ft(j*3+2)
     $                       +bra(3)*ft(j*3+3))*ket(j+1)
                     end do
                     if(imode.eq.1)
     $                    write(18,1768) dble(n)*dr,dble(ftv)
                     if(imode.eq.2)
     $                    write(18,1768) dble(n)*dr,aimag(ftv)
                     if(imode.eq.3)
     $                    write(18,1768) dble(n)*dr,abs(ftv)*abs(ftv)
                     if(imode.eq.0)
     $                    write(18,1768) dble(n)*dr,abs(ftv)

                  else
                     ftv = ft(1)
                     if(imode.eq.1)
     $                    write(18,1768) dble(n)*dr,dble(ftv)
                     if(imode.eq.2)
     $                    write(18,1768) dble(n)*dr,aimag(ftv)
                     if(imode.eq.3)
     $                    write(18,1768) dble(n)*dr,abs(ftv)*abs(ftv)
                     if(imode.eq.0)
     $                    write(18,1768) dble(n)*dr,abs(ftv)
                  endif
                  
               endif


            end do
c
c     ------------------- compute plane average ----------------
c
         elseif(nbins.lt.0) then
            igmax=max(gs%fftsize(1),gs%fftsize(2),gs%fftsize(3))
            allocate(ldata(igmax))
            allocate(lgvec(igmax))
            nvec=endp-startp
            nlen=dot_product(nvec,matmul(crys%adot,nvec))
            rip =dot_product(startp,matmul(crys%adot,nvec))/sqrt(nlen)
            igs=0
            igi=0
            do iord=1, gs%lorder ! loop through x/y gspace 
               irod=gs%order(1,iord)
               igv(1)= irod/gs%fftsize(2)
               igv(2)= mod(irod,gs%fftsize(2))
               igv(3)= gs%order(2,iord)
               igv(4)= gs%order(3,iord)
               igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
               gv(1:2)=dble(igv(1:2))+gs%rk(1:2)
               do igv3=igv(3),igv(4) ! loop over z axis
                  gv(3)=dble(igv3)+gs%rk(3)
                  igs = igs + 1
                  ovl = twopi*dot_product(nvec,gv)
c                  write(9,*)  igs, igv(1), igv(2), igv3
                  if(abs(ovl*ovl-nlen*gs%ekin(igs)).lt.1.d-8) then
                     igi=igi+1
                     if(igi.gt.igmax) then
                        write(9,*) '*** lineplot: fatal error'
                        stop
                     endif
                     lgvec(igi) = ovl/sqrt(nlen)
                     if(ndim.eq.3) then
                        ftv = bra(1)*data(igs,1)+bra(2)*
     $                       data(igs,2)+bra(3)*data(igs,3)
                     elseif(ndim.eq.9) then
                        ftv =0
                        do j=0,2
                           ftv = ftv + (bra(1)*data(igs,j*3+1)
     $                          +bra(2)*data(igs,j*3+2)
     $                          +bra(3)*data(igs,j*3+3))*ket(j+1)
                        end do
                     else
                        ftv = data(igs,1)
                     endif
                     ldata(igi)=ftv
c                     write(9,*) igi,  lgvec(igi), ldata(igi)
c                     write(9,*) data(igs,1:ndim)
                  endif

               end do
            end do
c
c           now write the data to the file
c
            close(18)

            igtot=igi
            call all_sum_all(igtot)
            do iproc=0, gs%nproc-1
               if(gs%myproc.eq.iproc) then
                  if(gs%myproc.eq.0) then
                     open(18,file=outfilename // '.line.' //
     $                    numstring,form='formatted',
     $                    status='unknown')
                  else
                     open(18,file=outfilename // '.line.' //
     $                    numstring,form='formatted', position
     $                    ='append',status='unknown')
                  endif
                  if(gs%myproc.eq.0) then
                     write(18,'(a,g12.6)') 'r initial parallel:',rip
                     write(18,'(a,g12.6)') '|rf-ri| initial parallel:'
     $                    ,sqrt(nlen)
                     write(18,'(a,3g12.6)') 'direction (cartesian):', 
     $                    matmul(crys%adot,nvec)/sqrt(nlen)
                     write(18,*) igtot
                  endif
                  do i=1,igi
                     write(18,*) i,lgvec(i),dscale*ldata(i)
                  end do
                  call myflush(18)
                  close(18)
               end if
               call parallel_barrier()
            end do

            deallocate(lgvec)
            deallocate(ldata)
            if(gs%myproc.eq.0) then
               allocate(lgvec(igtot))
               allocate(ldata(igtot))
               open(18,file=outfilename // '.line.' // numstring,
     $              form='formatted', status='unknown')
               read(18,*) ; read(18,*) ; read(18,*) 
               read(18,*) igtot
               do i=1,igtot
                  read(18,*) j, lgvec(i), ldata(i)
               end do
               close(18)
               open(18,file=outfilename // '.fft.' // numstring,
     $              form='formatted', status='unknown')
               do i=1,iabs(nbins)
                  ftv=0
                  rp=rip+dble(i-1)* sqrt(nlen)/dble(iabs(nbins)-1)
                  do j=1, igtot
                     ftv=ftv+ldata(j)*exp(cmplx(0.d0,lgvec(j)*rp))
                  end do
                  write(18,*) rp,dble(ftv)
               end do
               deallocate(lgvec)
               deallocate(ldata)
            endif

            
         endif
         if(gs%myproc.eq.0) close(18)
         write(9,20) bra,ket,line_plot(1:7,iline)
         call myflush(9)
      end do

 10   format(' WRITING LINEPLOT #',i4,' TO FILE: ',a)
 20   format(' BRA:',3f6.2,' KET:',3f6.2,/' PATH: ', 7f8.3)

      end



