c     @process extchk
c
      subroutine magnetic_kkterm(ham, densham, 
     $     nloc_deriv_k, qmag, rk, pspot, 
     $     u_k, len, ksub_gspace, e_k, noccup,
     $     pw_params,crys, weight, uktil, outvv, outhh)
c     
c
c     1996 Bernd Pfrommer, UCB
c
c
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'
c
c     INPUT:
c     -----
      integer
     $     len,                 ! length of gspace. SGI compiler needs that
     $     noccup               ! number of occupied states

      double precision
     $     rk(3),               ! the kpoint
     $     qmag,                ! magnitude of q 
     $     weight,              ! kpoint weight
     $     e_k(noccup)           ! eigenvalues of the occupied states

      type(parallel_gspace) ksub_gspace ! the subspace for the starting guess
      type(hamiltonian) ham
      type(dense_hamiltonian) densham
      type(pw_parameter) pw_params
      type(pseudo_potential) pspot
      type(crystal) crys

      complex(kind=8)
     $     u_k(len,noccup) ! the wavefunctions for that kpoint

      complex(kind=8)
     $     nloc_deriv_k(ham%dim+ham%vnloc%nvecs, ! see note below
     $     ham%vnloc%nvecs,2,3) !  derivative of nonlocal potential at k

c
c     OUTPUT:
c     ------

      complex(kind=8) 
     $     uktil(len,noccup,*), ! if cgsolveguess, will contain starting guess
     $     outvv(3,3),          ! with   < dH/dk ><dH/dk>
     $     outhh(3,3)           ! with   < dH/dk >< p >

c
c     DESCRIPTION:
c     -----------
c
c     computes  Sum_(j=empty) <u_k,i|p1|u_k,j><u_k,j|p0|u_k,i>/(e_k,i-e_k,j)
c     for all occupied states i
c
c     To reduce memory usage, the nonlocal derivative array may be 
c     deallocated and recomputed afterwards. So if pw%optimize is set
c     for memory optimizing, nloc_deriv_k is ignored.
c     
c
c

c     ---------------- local variables ----------------------------------

      integer 
     $     i,                   ! band index
     $     pp,                  ! temp operator index
     $     p1,p0                ! operator index

      double precision 
     $     t0, gimmetime,
     $     oldshift             ! original shift of the hamiltonian
      external gimmetime

      complex(kind=8), pointer :: ! fortran 90 automatic array
     $     nloc_deri(:,:,:,:),
     $     phi(:,:)           ! the rhs for cgsolve


      if(iand(pw_params%output,8).eq.8)   t0=gimmetime()

      allocate(phi(ham%gspace%length,noccup))


      oldshift=ham%shift

      ham%shift=0.d0            ! no shift here


      outvv=0
      outhh=0

      if(iand(pw_params%optimize,1).eq.1)  then
         allocate(nloc_deri(ham%gspace%length+pspot%nanl, 
     $        pspot%nanl,2,3)) 
         call setup_nonlocal_derivative(pw_params%output,qmag,
     $        rk(1),pspot,ham%gspace, nloc_deri(1,1,1,1), crys)
      endif


      do p0=1,3                 ! loop over operator p0

         pp=1
         if(iand(pw_params%optimize,4).eq.4) pp =p0
c
c        compute gradient of uk for rhs of cgsolve
c
         if(iand(pw_params%optimize,1).eq.1) then
            do i=1, noccup
               call take_nonloc_deriv(p0,ham%gspace, rk(1),u_k(1,i), 
     $              ham%vnloc%nvecs,nloc_deri(1,1,1,1),phi(1,i)
     $              ,crys)
            end do         
            deallocate(nloc_deri)
         else
            do i=1, noccup
               call take_nonloc_deriv(p0,ham%gspace, rk(1),u_k(1,i), 
     $              ham%vnloc%nvecs,nloc_deriv_k(1,1,1,1),phi(1,i)
     $              ,crys)
            end do         
         endif
c
c        now solve for uktil
c
         
         call cg_blocksolve(pw_params%output, crys, p0,
     $        ham, pw_params%maxitcgsol, pw_params%epsmag, 
     $        ksub_gspace, ham%gspace%rk(1),
     $        noccup,  ksub_gspace%length,
     $        e_k(1), densham%energy(1),
     $        e_k(1), densham%energy(1), 
     $        densham%matrix(1),
     $        u_k(1,1),
     $        u_k(1,1), 
     $        uktil(1,1,pp), 1, phi(1,1))

         if(iand(pw_params%optimize,1).eq.1)  then
            allocate(nloc_deri(ham%gspace%length+pspot%nanl, 
     $           pspot%nanl,2,3)) 
            call setup_nonlocal_derivative(pw_params%output,qmag,
     $           rk(1),pspot,ham%gspace, nloc_deri(1,1,1,1), crys)
         endif

         do i=1, noccup          ! loop over all occupied bands
c     
c           apply operator p1 to u_k~
c                  
            do p1=1,3
c
c              -- vv term: take dH/dk left and right --
c
               if(iand(pw_params%optimize,1).eq.1) then
                  call take_nonloc_deriv(p1,ham%gspace,
     $                 ham%gspace%rk(1),uktil(1,i,pp),ham%vnloc%nvecs
     $                 ,nloc_deri(1,1,1,1),phi(1,1),crys)
               else
                  call take_nonloc_deriv(p1,ham%gspace,
     $                 ham%gspace%rk(1),uktil(1,i,pp),ham%vnloc%nvecs
     $                 ,nloc_deriv_k(1,1,1,1),phi(1,1),crys)
               endif

               outvv(p0,p1)=outvv(p0,p1)+
     $              parallel_zdotc2(ham%gspace%length,
     $              u_k(1,i),1,
     $              phi(1,1),1)*weight/dble(noccup)

c
c              -- hh term: take p  and dH/dk  ----
c

               call take_nonloc_deriv(p1,ham%gspace, ham%gspace%rk(1),
     $              uktil(1,i,pp),0,nloc_deriv_k(1,1,1,1),
     $              phi(1,1),crys)

               outhh(p0,p1)=outhh(p0,p1)+
     $              parallel_zdotc2(ham%gspace%length,
     $              u_k(1,i),1, phi(1,1),1)*weight/dble(noccup)

            end do

         end do

      end do

      if(iand(pw_params%optimize,1).eq.1)  then
         deallocate(nloc_deri)
      endif

      deallocate(phi)
      ham%shift=oldshift            ! no shift here

      if(iand(pw_params%output,8).eq.8) then
         write(9,100) gimmetime()-t0
         call myflush(9)
      endif





      return

 100  format(' TIME FOR KK-TERM:', f12.3)


      end




