c
c     @process extchk
      subroutine mixer(iter,
     $     pw_params, crys, gs, mixgs,
     $     vin, vout, energs, vion,
     $     ajac)
c
c     1996 by Bernd Pfrommer, while at UCB
c
c     Based on a code by JLM.
c
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
c     INPUT:
c     -----
      
      integer 
     $     iter                 ! iteration number

      type (crystal) crys       ! for vcell, nspin, ztot
      type (pw_parameter) pw_params ! for alphamix
      type (parallel_gspace) 
     $     gs,                  ! the gspace for the potentials
     $     mixgs                ! the (smaller mixing gspace)
      type (energy) energs      ! for vxc0

      complex(kind=8)
     $     vout(gs%length*crys%nspin)
C
c     INPUT/OUTPUT:
c     -------------

      complex(kind=8)
     $     vion(gs%length*crys%nspin), ! the screened ionic potential
     $     ajac(crys%nspin*mixgs%length,
     $     crys%nspin*mixgs%length), ! inverse of jacobian matrix
     $     vin(gs%length*crys%nspin) ! the screening potential with which the 
     $                          ! present eigenvectors where calculated
c
c     DESCRIPTION:
c     -----------
c
c     computes the new screened potential given 
c
c     * the input and the  output potential from the previous
c       iteration. 
c     * the approximate inverse of the hessian matrix
c     

c     A Broyden Quasi-Newton method is used to estimate the dielectric
c     matrix for the mixgs%length smallest G-vectors.
c
c     A simple linear mixing can also be used by setting 
c     pw_params%alphamix(2)>0.
c     G-vec components of the potential are intermixed with the previous 
c     step more strongly.
c
c     For the spin polarized case, the  assignment of the 
c     inverse Hessian is:
c
c
c           |1 | 2 3 .... nj | nj+1 .... 2*nj-1
c     ------+--+-------------+-------------------
c       1   |  |             |
c     ------+--+-------------+-------------------
c       2   |  |             |
c       .   |  |  up,up      |     up,down
c       .   |  |             |
c       .   |  |             |
c      nj   |  |             |
c     ------+--+-------------+-------------------
c     nj+1  |  |             |
c       .   |  |             |
c       .   |  |  down,up    |     down,down
c       .   |  |             |
c     2*nj-1|  |             |
c
c

c     ------------------ local variables ------------------------
      
      double precision 
     $     t0, gimmetime,
     $     qtf2,                ! square of thomas-fermi vector
     $     amix, pi, alpha, fac, qf, x, x2, gx, xmix, xmix1, vnorm,
     $     ekmmax

      integer 
     $     init,                ! flag, what initialization is required
     $     i, j, k, j0, findvec, isub, gsvi(3), isubi, is, it, jt


      complex(kind=8), allocatable ::
     $     av(:), va(:), vinm(:), voutm(:), wmix(:)



      complex(kind=8)
     $     vioni, vnew

      external findvec, gimmetime
c
c     ***************************************************************
c
      t0=gimmetime()

      pi = 4.d0*atan(1.d0)

      qtf2 = 4.d0*(3.d0*crys%ztot/(pi*crys%vcell))**(1.d0/3.d0)

      init = 0 ;     if(iter.eq.1) init=2

c
c     --------------------- use linear mixing ----------------------------
c
c
      if(pw_params%alphamix(2).gt.0) then

         do is=1,crys%nspin
            do i=1,gs%length
               amix = pw_params%alphamix(2) + 
     $              pw_params%alphamix(3)*
     $              dexp(-pw_params%alphamix(4)*gs%ekin(i))
               if(gs%ekin(i).gt.0) then 
                  vion(i+(is-1)*gs%length)=
     $                 vion(i+(is-1)*gs%length) - 
     $                 vin(i+(is-1)*gs%length) ! unscreen
                  vin(i+(is-1)*gs%length) =
     $                 amix*vin(i+(is-1)*gs%length)+
     $                 (1-amix)*vout(i+(is-1)*gs%length) ! mix new and old 
                  vion(i+(is-1)*gs%length) = 
     $                 vion(i+(is-1)*gs%length) + 
     $                 vin(i+(is-1)*gs%length) ! screen with new potential
               else
                  xmix   = pw_params%alphamix(2)
                  if(init.ne.2) then
                     energs%vxc0(is)   = xmix*energs%vxc0(is) 
     $                    + (1-xmix)*vin(i+(is-1)*gs%length)
                  endif
                  vin(i+(is-1)*gs%length) = energs%vxc0(is)
               endif
            end do
         end do

         
         write(9,'('' Mixing new and old potentials with a1=''' //
     $        ',f7.4, '', a2='',f7.4, '', a3='',f7.4)') 
     $        pw_params%alphamix(2),pw_params%alphamix(3),
     $        pw_params%alphamix(4)
         return
      endif


c
c     ----------------------- Broyden Mixing ----------------------------
c

c
c     allocate work arrays
c

      allocate(av(mixgs%length*crys%nspin))
      allocate(va(mixgs%length*crys%nspin))
      allocate(vinm(mixgs%length*crys%nspin))
      allocate(voutm(mixgs%length*crys%nspin))
      allocate(wmix(mixgs%nstar))


c     
c     figure out largest mixing energy and index of G=0 component
c

      ekmmax=0.d0               
      do i=1,mixgs%length
         if(mixgs%ekin(i).eq.0.d0) j0=i
         if(mixgs%ekin(i).gt.ekmmax) ekmmax=mixgs%ekin(i)
      end do

      alpha = pw_params%alphamix(1)
      fac = -6.0*alpha/(pi*pi*qtf2)
      qf = pi*qtf2/4.0

      if(mixgs%length.gt.1) then
c
c     initialize inverse jacobian matrix
c
      if (init .eq. 2) then
         do i=1,mixgs%length*crys%nspin
            it=mod(i-1,mixgs%length)+1
            do j=1,mixgs%length*crys%nspin
               jt=mod(j-1,mixgs%length)+1
               ajac(i,j) = (0.d0,0.d0)
               if ((i .eq. j).and.(it.ne.j0)) then
                  x = sqrt(mixgs%ekin(it))/qf
                  x2 = x/2.d0
                  gx = 1.d0/2.d0
                  if (x2 .ne. 1.d0) gx=(1.d0-(1.d0-x2*x2)*
     *                 log(abs((1.d0-x2)/(1.d0+x2)))/x)/2.d0
                  xmix = 1.d0/(1.d0+(fac+qtf2/mixgs%ekin(it))*gx)
                  ajac(i,j) = cmplx(- xmix,0.d0,kind=8)
               endif
            end do
         end do
c         write(9,*) 'initialized Broyden mixing matrix:'
c         do i=1, mixgs%length*crys%nspin
c            write(9,'(1000f8.4)') dble(ajac(i,:))
c         end do

      endif

c
c     restore the symmetry V(G)=cc(V(-G))      explicitly
c

      do is=1, crys%nspin
         call regspace(gs,vin(1+(is-1)*gs%length),
     $        mixgs, vinm(1+(is-1)*mixgs%length))
         call regspace(gs,vout(1+(is-1)*gs%length),
     $        mixgs, voutm(1+(is-1)*mixgs%length))

         call symmetrize_local(mixgs%length,
     $        vinm(1+(is-1)*mixgs%length), mixgs%nstar,
     $        mixgs%inds,mixgs%mstar, mixgs%phase,
     $        wmix(1), mixgs%inverse)
         call symmetrize_local(mixgs%length,
     $        voutm(1+(is-1)*mixgs%length), mixgs%nstar,
     $        mixgs%inds,mixgs%mstar, mixgs%phase,
     $        wmix(1), mixgs%inverse)
      end do

c
c     update inverse jacobian matrix a
c
      if (init .eq. 0) then

c         write(9,*) 'voutm     =',dble(voutm)
c         write(9,*) 'vinm      =',dble(vinm)
c         write(9,*) 'ajac(j0,j)=',dble(ajac(j0,:))
c         write(9,*) 'ajac(j,j0)=',dble(ajac(:,j0))
c         write(9,*) 'all       =',dble(voutm(:)-vinm(:)-
c     $        ajac(j0,:)+ajac(:,j0))


         do i=1, mixgs%length*crys%nspin
            it=mod(i-1,mixgs%length)+1
            if(it.ne.j0) then
               av(i)=(0.d0,0.d0)
               va(i)=(0.d0,0.d0)
               do j=1,mixgs%length*crys%nspin
                  jt=mod(j-1,mixgs%length)+1
                  if(jt.ne.j0) then
c                              -1
c                    av = f = J  (F_m - F_(m-1))
c
                     av(i) = av(i)  + ajac(i,j) * 
     $                    (voutm(j)-vinm(j)-ajac(j0,j)+ajac(j,j0))
c                          * -1
c                    va = g J
c
                     va(i) = va(i) + 
     $                    conjg(vinm(j)-ajac(j,j0)) * ajac(j,i)
                  endif
               end do
            endif
         end do

         vnorm = 0.d0
         do i=1,mixgs%length*crys%nspin
            it=mod(i-1,mixgs%length)+1
            if(it.ne.j0) then
               vnorm = vnorm + dble(conjg(vinm(i)-ajac(i,j0))*av(i))
            endif
         end do
         
c         write(9,*) 'va=',dble(va)
c         write(9,*) 'av=',dble(av)
c         write(9,*) 'vinm=',dble(vinm)
c         write(9,*) 'vnorm=',dble(vnorm)
c         write(9,*) 'ajac(:,j0)=',dble(ajac(:,j0))


         do i=1,mixgs%length*crys%nspin
            it=mod(i-1,mixgs%length)+1
            if(it.ne.j0) then
               do j=1,mixgs%length*crys%nspin
                  jt=mod(j-1,mixgs%length)+1
                  if(jt.ne.j0) then
                     ajac(i,j) = ajac(i,j)
     $                    + (vinm(i)-ajac(i,j0)-av(i)) * va(j) / vnorm
                  endif
               end do
            endif
         end do

c         write(9,*) 'updated Broyden mixing matrix to:'
c         do i=1, mixgs%length*crys%nspin
c            write(9,'(1000f8.4)') dble(ajac(i,:))
c         end do

      endif                     ! end of (if init.eq.0)
c
c     save vin and vout for next iteration
c

      
      do i=1,mixgs%length*crys%nspin
         it=mod(i-1,mixgs%length)+1
         if(it.ne.j0) then 
            ajac(i,j0) = vinm(i)
            ajac(j0,i) = voutm(i)
         endif
      end do
c
c     compute correction to vin from inverse jacobian
c
      do i=1,mixgs%length*crys%nspin
         it=mod(i-1,mixgs%length)+1
         if(it.ne.j0) then
            av(i) = (0.d0,0.d0)
            do j=1,mixgs%length*crys%nspin
               jt=mod(j-1,mixgs%length)+1
               if(jt.ne.j0) then
                  av(i) = av(i) + ajac(i,j) * (voutm(j)-vinm(j))
               endif
            end do
         endif
      end do

      endif                     ! of if(mixgs%length.gt.1)

c
c     loop over gspace
c
      do is=1,crys%nspin

      do i=1,gs%length
         if(gs%ekin(i).gt.0.d0) then
            if (gs%ekin(i) .gt. ekmmax) then ! do linear mixing for this part
               x = sqrt(gs%ekin(i))/qf
               x2 = x/2.d0
               gx = 1.d0/2.d0
               if (x2 .ne. 1.d0) gx=(1.d0-(1.d0-x2*x2)*
     *              log(abs((1.d0-x2)/(1.d0+x2)))/x)/2.d0
               xmix = 1.d0/(1.d0+(fac+qtf2/gs%ekin(i))*gx)
               xmix1 = 1.d0 - xmix
               vnew = xmix*vout(i+(is-1)*gs%length)+
     $              xmix1*vin(i+(is-1)*gs%length)
            else                ! and Broyden mixing here
               isub = findvec(gs%gvec(1,i),mixgs)
               if(isub.le.0) then
                  write(9,*) 'mixer: subspace mismatch!',isub
                  stop
               endif
               vnew = vin(i+(is-1)*gs%length) 
     $              - av(isub+(is-1)*mixgs%length)
               gsvi(1)=-gs%gvec(1,i)
               gsvi(2)=-gs%gvec(2,i)
               gsvi(3)=-gs%gvec(3,i)
               isubi = findvec(gsvi,mixgs)
               if(isubi.le.0) then
                  write(9,*) 'mixer: subspace mismatch!',isubi
                  stop
               endif

               if(abs(aimag(vinm(isub+(is-1)*mixgs%length)+
     $              vinm(isubi+(is-1)*mixgs%length))).gt.5.d-17)
     $              write(9,*) 'mixer-error:',
     $              isub+(is-1)*mixgs%length, 
     $              isubi+(is-1)*mixgs%length,gs%ekin(i), 
     $              aimag(vinm(isub+(is-1)*mixgs%length)+
     $              vinm(isubi+(is-1)*mixgs%length)),
     $              aimag(voutm(isub+(is-1)*mixgs%length)
     $              +voutm(isubi+(is-1)*mixgs%length))
            endif
c
c           replace old potential with new potential and compute
c           the dielectric function.
c
            vioni   = vion(i+(is-1)*gs%length)-vin(i+(is-1)*gs%length)
            vion(i+(is-1)*gs%length) = vioni +vnew
            vin(i+(is-1)*gs%length)  = vnew
         else                   ! linear mixing for the G=0 component
            xmix   = 0.2
            if(init.ne.2) then
               energs%vxc0(is)   = xmix*energs%vxc0(is) 
     $              + (1-xmix)*vin(i+(is-1)*gs%length)
            endif
            vin(i+(is-1)*gs%length) = energs%vxc0(is)
         endif
      end do

      end do

      deallocate(av) ; deallocate(va)
      deallocate(vinm) ; deallocate(voutm)
      deallocate(wmix)

      if(iand(pw_params%output,8).eq.8) 
     $     write(9,950) gimmetime()-t0


      return


 100  FORMAT('MIXING MATRIX',//,9(9(1X,2F6.2),/))
 950  format(' TIME FOR MIXING:',f12.3)

      end 

