c
c
      subroutine momentum_density(crys,kp,bands, k_gs, wfn)

      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'

c
c     INPUT:
c     -----
c
      type (crystal) crys
      type (kpoint) kp
      type (band) bands
      type (parallel_gspace) k_gs(kp%nrk)
      type (complex_gspace_array) wfn
c
c     1997 Bernd Pfrommer. Based on a routine by Balazs Kralik.
c
c
c
c     DESCRIPTION:
c     -----------
c
c     Computes the momentum density along special lines in the BZ. 
c     Those are given by the kpoint mesh for which the wave functions
c     are computed. Make sure the occupation numbers are correct, i.e.
c     between 0 and 1 with spin, or 0 and 2 without spin.
c
c     
c

c
c     ------------------- local variables ----------------------

      double precision 
     $     rk(3), rkc(3), rkm(3), klensq, dp, ecut, ekin,
     $     gvec(3), occp

      double precision, allocatable :: momdens(:)

      integer 
     $     ndim, n, irk, is

c
c     variables for the gspace loop
c
      integer
     $     igv(4),fftn(4), ffth(4),igv3, irod,iord,igs

c
c     -----------------------------------------------------------
c

      ecut=2d10                 ! does not make sense, dont use
c
c     do sanity checks
c

      if(k_gs(1)%myproc.gt.1) then
         write(9,*) 'momentum_density works only SERIAL!'
         stop
      endif

      allocate(momdens(bands%max))


      do is=1, crys%nspin
         write(9,*) '========= SPIN:',is,'=========='
         do irk=1, kp%nrk
            rk  = kp%rk(:,irk)
            rkc = matmul(crys%bvec,rk)
            rkm = matmul(crys%bdot,rk)
            klensq = rkc(1)*rkc(1)+rkc(2)*rkc(2)+rkc(3)*rkc(3) 
c
c           loop over gspace to get the desired components
c        
            fftn(1:3)=k_gs(irk)%fftsize(1:3)
            fftn(4)=k_gs(irk)%fftsize(3)
            ffth(:)=fftn(:)/2

            igs=0
            ndim = k_gs(irk)%length
            do iord=1, k_gs(irk)%lorder ! loop through x/y gspace 
               irod=k_gs(irk)%order(1,iord)
               igv(1)= irod/k_gs(irk)%fftsize(2)
               igv(2)= mod(irod,k_gs(irk)%fftsize(2))
               igv(3)= k_gs(irk)%order(2,iord)
               igv(4)= k_gs(irk)%order(3,iord)
               igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
               do igv3=igv(3),igv(4) ! loop over z axis
                  igs = igs + 1
                  ekin = k_gs(irk)%ekin(igs)
                  gvec(1) = dble(igv(1))+rk(1) 
                  gvec(2) = dble(igv(2))+rk(2)
                  gvec(3) = dble(igv3)+rk(3)
c
c                 check if this gvector is parallel to kpoint
c                  
                  dp=gvec(1)*rkm(1)+gvec(2)*rkm(2)+gvec(3)*rkm(3)

                  if(((klensq.ne.0).and.((dp*dp+1d-8.gt. ekin*klensq)
     $                 .and.(ekin.le.ecut))).or.((klensq.eq.0).and.
     $                 (ekin.eq.0))) then
c
c                    sum across all bands:
c
                     momdens=0.d0
                     do n=1, bands%nband(irk,is)
                        occp=bands%occup(n,irk,is)/
     $                       (kp%w(irk)*dble(crys%nspin))
                        momdens(n)=momdens(n)+occp*
     $                       abs(wfn%data(ndim*(n-1)+igs,irk,is))**2 
                     end do
                     write(9,'(3f12.6,200g18.6)') gvec,
     $                    sum(momdens(1:bands%nband(irk,is))),
     $                    momdens(1:bands%nband(irk,is))
                  endif
               end do
            end do


            
         end do
      end do


      deallocate(momdens)

      end
