c
c
c     @process extchk

      subroutine plot_bandstruc(kpoints, bands,crys,energs,bslabels)
c     
      include 'use.h'
      implicit none
      include 'interface.h'

c     INPUT:
c     -----

      type (kpoint) kpoints
      type (band) bands
      type (crystal) crys
      type (energy) energs
      character*100 bslabels(*) ! names of the k-points



c
c     DESCRIPTION:
c     -----------
c
c     plots band structure, with correct metric, into a file
c     The file can be read for instance with xmgr 
c
c     note that the label field is used to avoid jumps when
c     two different BZ lines follow each other. this works
c     because kpoints%label(k)=0.0 in that case.
c
c
c

c     ------------------ local variables ------------------------


      integer is,k,n,nregion,i,ii,l,itick

      character*100 dummy, labs(3)

      double precision 
     $     efermi(2),
     $     vxmin,vxmax,vymin,vymax,
     $     plabs(3),            ! positions of the three labels
     $     s, d, dk(3),slab, tmarg, bmarg,smid,emin,emax,xmax

      parameter(tmarg=1.d0, bmarg=1.d0) ! top and bottom margins
      parameter(vxmin=0.15,vymin=0.15, vxmax=0.85,vymax=0.85)


      open(11,file='BANDSTRUC',form='formatted',status='unknown')

c     efermi=energs%efermi

      efermi(1)=energs%inputfermi
      efermi(2)=energs%inputfermi
c
c     figure out max and minimum band energy
c
      emin=1d6
      emax=-1d6

      do is=1, bands%nspin
         do k=1, kpoints%nrk
            if(maxval(bands%energy(1:bands%nband(k,is),k,is))
     $           +energs%evalshift(is)-efermi(is) .gt.
     $           emax) emax= maxval(bands%energy(1:bands%nband(k,is),
     $           k,is))+energs%evalshift(is)-efermi(is) 
            if(minval(bands%energy(1:bands%nband(k,is),k,is))
     $           +energs%evalshift(is)-efermi(is) .lt.
     $           emin) emin= minval(bands%energy(1:bands%nband(k,is),
     $           k,is))+energs%evalshift(is)-efermi(is) 
         end do
      end do
      emin=emin*13.6058
      emax=emax*13.6058
c
c     find maximum abscissa value
c
      do is=1, bands%nspin
         s=0
         dk=0
         do k=1,kpoints%nrk
            if(k.lt.kpoints%nrk) 
     $           dk=kpoints%rk(:,k+1)-kpoints%rk(:,k)
            d=kpoints%label(k)*sqrt(dot_product(dk,
     $           matmul(crys%bdot,dk)))
            s=s+d;
         end do
      end do

      xmax=s
c
c     write first part of the header
c      
      write(11,5)
      write(11,6) energs%efermi(1:bands%nspin)*13.6058d0
      write(11,7) energs%evalshift(1:bands%nspin)*13.6058d0

      write(11,10) xmax,dble(int(emin-bmarg)),dble(int(emax+tmarg)),
     $     vxmin,vxmax,vymin,vymax
c
c     write the labels
c

      call myflush(9)

      dk=0
      nregion=1
      slab=0
      s=0
      itick=0
      do k=1,kpoints%nrk
         if(k.lt.kpoints%nrk) 
     $        dk=kpoints%rk(:,k+1)-kpoints%rk(:,k)
         d=kpoints%label(k)*sqrt(dot_product(dk,
     $        matmul(crys%bdot,dk)))
         if((d.eq.0.d0)) then   ! switched to new region
            nregion=nregion+1
c
c           decompose into left/middle/right
c
            dummy=trim(adjustl(bslabels(nregion-1)))
            l=len(dummy)
            if(l.le.0) exit     ! no label written, continue with do loop

            is=0
            do i=1,3
               ii=index(dummy(is+1:l),'_')+is
               if(ii.le.is) then
                  labs(i)=dummy(is+1:index(dummy,' '))
               else
                  labs(i)=dummy(is+1:ii-1)
               endif
               if(ii.le.is) exit
               is=ii            ! start new search after '_'
            end do
            
            if(i.eq.1) then
               plabs(1)=slab+(s-slab)/2.d0
            elseif(i.eq.2) then
               plabs(1)=slab ; plabs(2)=s
            else
               plabs(1)=slab; plabs(2)=slab+(s-slab)/2.d0; plabs(3)=s
            endif
            do l=1,i
               write(11,30) itick,plabs(l),itick,trim(labs(l))
               itick=itick+1
            end do

            slab=s
         endif
         s=s+d;
      end do
      write(11,40) itick

c
c     output the lines
c

c      write(11,19)
      dk=0
      nregion=1
      slab=0
      s=0
      do k=1,kpoints%nrk
         if(k.lt.kpoints%nrk) 
     $        dk=kpoints%rk(:,k+1)-kpoints%rk(:,k)
         d=kpoints%label(k)*sqrt(dot_product(dk,
     $        matmul(crys%bdot,dk)))
         if((d.eq.0.d0)) then   ! switched to new region
            nregion=nregion+1
c           draw the vertical line
c            write(11,20) s,dble(int(emin-bmarg)),s, 
c     $           dble(int(emax+tmarg))
            write(11,20) s/xmax*(vxmax-vxmin)+vxmin,
     $           vymin,s/xmax*(vxmax-vxmin)+vxmin,vymax
            slab=s
         endif
         s=s+d;
      end do



c
c     write the bands one by one
c
      do is=1, bands%nspin
         do n=1,bands%max
            write(11,'(''@TYPE xy'')')
            dk=0
            ii=0
            s=0
            do k=1,kpoints%nrk
               if(k.lt.kpoints%nrk) 
     $              dk=kpoints%rk(:,k+1)-kpoints%rk(:,k)
               d=kpoints%label(k)*sqrt(dot_product(dk,
     $              matmul(crys%bdot,dk)))
               if(bands%nband(k,is).ge.n) then
                  write(11,100) s, (bands%energy(n,k,is)+
     $                 energs%evalshift(is)-efermi(is)) *13.6058
                  ii=ii+1
               endif
               s=s+d;
            end do
            write(11,'(''&'')')
         end do
      end do

      close(11)


      write(9,200)


      return

 5    format(3('#',/),'#      BANDSTRUCTURE FOR USE WITH XMGR',
     $     3('#',/),'#    1996 Bernd Pfrommer')
 6    format(3('#',/),'# THE EIGENVALUES ARE SHIFTED BY VXC(0), AND',
     $     /'# ARE GIVEN WITH RESPECT TO THE INPUT FERMI LEVEL:',
     $     2f20.8,' eV','# MAKE SURE THE INPUT FERMI IS CORRECT!')
 7    format('# VXC0-SHIFT [eV]:',2f12.6)


 10   format('@with g0'/,
     $     '@    world xmin 0'/
     $     '@    world xmax ',f12.6/,
     $     '@    world ymin ',f12.6/,
     $     '@    world ymax ',f12.6/,
     $     '@    view xmin ',f12.6/,
     $     '@    view xmax ',f12.6/,
     $     '@    view ymin ',f12.6/,
     $     '@    view ymax ',f12.6/,
     $     '@    yaxis  tick on'/,
     $     '@    yaxis  tick major 5'/,
     $     '@    yaxis  tick minor 2.5'/,
     $     '@    xaxis  tick major off'/,
     $     '@    xaxis  tick minor off'/,
     $     '@    xaxis  ticklabel on'/,
     $     '@    xaxis  ticklabel type spec'/,
     $     '@    xaxis  tick type spec')

 19   format('@    line loctype world')
 20   format('@with line'/,
     $     '@    line',f12.6,',',f12.6,',',f12.6,',',f12.6/,
     $     '@line def')
 30   format('@    xaxis  tick ',i2,', ',f12.6/,
     $     '@    xaxis  ticklabel ',i2,', "',a,'"')
 40   format('@    xaxis  tick spec ',i3)

 100  format(f8.3, 500(f10.5))
 200  format(' WROTE BANDSTRUCTURE TO FILE BANDSTRUC!')


      end
