c-*-Fortran-*- 
c                                Parallel 
c                     Kleinman-Bylander Pseudopotential 
c                      Total Energy Plane Wave Program
c
c     1995/96 Bernd Pfrommer, Department of Physics,     UC Berkeley
c
c     based on a code by J.L. Martins (1990)
c
c     -----------------------------------------------------------------------

      subroutine pwmain(njobs, joblist,
     $     ntype, mxdatm, natom,
     $     nrkinput, nbandi, ngrid,
     $     ispn, icorr, iinput, maxnumit, 
     $     itmax, iwfnreuse, ichkptwfn,
     $     iscr, ioccup, intrans, nl, 
     $     nlineplot, line_plot, bra, ket,
     $     nplotwv, plotwv, imonitor, ioptimize, miscflag,
     $     idsmear, kshift, delta, bandgap, esafety, randomstart,
     $     emax, emaxsub, emaxmix, epscv,
     $     qx,qy,qz,
     $     ilog, alphamix, h0, gmagnet, eampli, gepot, 
     $     nenergywindow, energywindow,
     $     ekinmod, chi_q, g0mask, nrg,
     $     totmass,coord, szet, mtsphere, dipole, avec, fermil, 
     $     enew, ssum,fsum, nameattst, bslabels)

c     ---------------------------------------------------------------

      use all_to_all_module
      include 'use.h'
      implicit none             ! never uncomment this line. 
      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'


      include 'param.i'         ! the parameters for dimensioning the arrays
m4_ifdef([MPI],[
      include 'mpif.h' ])


c     INPUT:
c     ------

      integer 
     $     ichkptwfn,           ! how frequently wavefn should be checkpointed
     $     njobs,               ! the number of jobs to perform
     $     nenergywindow,       ! number of energy windows 
     $     nrg,                 ! number radial points for angdos integration
     $     nlineplot,           ! number of line plots to do 
     $     joblist(njobs),      ! the jobs to do
     $     ntype,               ! number of types
     $     mxdatm,              ! max number of atoms of one type
     $     nrkinput,            ! number of k-points to be read (<0 calc)
     $     nbandi,              ! number of bands to compute
     $     ngrid(3),            ! reducible k-point grid
     $     natom(ntype),        ! number of atoms for each type
     $     ispn,                ! number of spin components (1,2,or 3!)
     $     iinput               ! triggers various additional inputs
      integer 
     $     icorr,               ! which type of correlation
     $     maxnumit,            ! maximum number of iterations for cgdiag
     $     itmax,               ! max number of iterations to selfconsistency
     $     iwfnreuse,           ! noreuse=0, firstcall=1, reuse=2
     $     iscr,                ! iscr=5: use atomic cd, iscr=4: read scf cd
     $     ioccup,              ! occupy levels according to ioccup
     $     ilog,                ! input/flag for logging and job control
     $                          ! log = -1  compute bandstructure only
     $     intrans,             ! the number of symmetry operations
     $     nl(3),               ! # of unit cells to plot in direction 1,2,3
     $     nplotwv,             ! number of wavefunctions to be plotted
     $     gmagnet(4),          ! g-vector for magnetic wave
     $     gepot(4),            ! g-vector for electric potential
     $     miscflag,            ! carries assorted flags for various purposes
     $     imonitor,            ! triggers various monitoring output:
     $     ioptimize,           ! triggers different optimizations
     $                          !  bit0 = print performance output to stdout
     $     plotwv(2,nplotwv)    ! which ones to plot
      
      double precision
     $     dipole(ntype),       ! dipole strengths for each atomic type
     $     randomstart,         ! amount of random component in start guess
     $     totmass,             ! total mass of all atoms in the unit cell
     $     fermil,              ! input fermi level (needed for bandstructure etc)
     $     energywindow(2,nenergywindow), ! energy windows [ev] for var. purp.
     $     g0mask(9),           ! mask for the G=0 component of CHI tensor
     $     bra(3),              ! for tensor plot: vector from left
     $     ket(3),              ! for tensor plot: vector from right
     $     line_plot(7,*),      ! line plot ((start,end,bins), lines)
     $     alphamix(4),         ! mixing parameters. alphamix(1) for Broyden,
     $                          ! alphamix(2..4) for linear mixing
     $     kshift(3)            ! shift of k-point grid
      double precision 
     $     idsmear,             ! gaussian smearing parameter 
     $     delta,               ! required precision for eigenvectors
     $     bandgap,             ! estimated bandgap in eV
     $     esafety,             ! additional safety shift for matrix diag.
     $     emax,                ! the famous cutoff energy in Rydbergs
     $     emaxsub,             ! cutoff for small submatrix
     $     emaxmix,             ! cutoff energy for broyden mixing
     $     epscv,               ! precision for convergence of potential
     $     ekinmod(3),          ! parameters for the modification of ekin
     $     coord(3,mxdatm,ntype), ! atomic coordinates in lattice units
     $     avec(3,3),           ! realspace lattice vectors
     $     szet(ntype),         ! spin polarizations
     $     mtsphere(ntype),     ! size of muffin tin spheres
     $     chi_q,               ! |q|-vector (relative to reciprocal lattice)
     $     h0,                  ! strenght of the magnetic field applied
     $     eampli,              ! amplitude of applied electric potential
     $     qx,qy,qz             ! qshift for GW code

      character*(2*ntype) nameattst ! transfer string for name of atoms

      character*10000 bslabels  ! labels for the bandstructure plot


c     OUTPUT:
c     -------

      double precision
     $     ssum(6),             ! metric stress tensor [Ryd]
     $     fsum(3,mxdatm,ntype), ! sum of all forces [lattice coordinates]
     $     enew                 ! new energy



c     -------------------------- parameters --------------------------------

      
c
c     PRECISION PARAMETERS AND CONSTANTS
c     **********************************

      double precision 
     $     pi,                  ! 3.141.....
     $     zero

      complex (kind=8)     zone
      parameter (zero = 0.0d0, pi=3.141592653589793d0, zone=(1.d0,0.d0))

      character*6 outfilename   ! file names
      character*7 cdname        ! file name for cdfile
c
c     -------------------------- local variables -----------------------

      
      type (symmetry) syms       ! symmetry operations
      type (crystal) crys       ! crystal structure
      type (fftstruc) ffts      ! information for the fast fourier transform
      type (pw_parameter) pw_params ! plane wave parameters
      type (pseudo_potential) pspot
      type (energy) energs      ! energies and other information
      type (force)  ewaldf      ! ewald force and stress
      type (kpoint) altkpoints  ! alternative BZ integration kpoints
      type (kpoint),save :: kpoints ! BZ integration kpoints
      type (band), save :: bands ! eigenvalues, occupation numbers etc
      type (band) :: altbands   ! another set of eigenvalues etc..

      type (complex_gspace_array) 
     $     vion,                ! screened ionic potential
     $     denc,                ! core charge density
     $     denv,                ! valence charge density
     $     dena,                ! total charge density, alternate
     $     vin,                 ! input potential
     $     vout,                ! output potential
     $     ddc,                 ! derivative of core charge 
     $     dvql                 ! derivative of local ionic potential
      type (complex_gspace_array), save ::
     $     den                  ! total charge density

      type (complex_gspace_array) 
     $     wavefnbs             ! wave functions for band structure

      type (complex_gspace_array), save ::
     $     wavefn               ! wave functions for all kpoints, bands

      type (double_gspace_array) 
     $     vql,                 ! local pseudo pot in gspace for each type
     $     dnc                  ! the core charge in gspace for each type
      type (complex_rspace_array)
     $     chdr                 ! work array: charge density in realspace

      type (parallel_gspace) 
     $     pot_gspace
      
      type (parallel_gspace), allocatable :: k_gspace(:)

      complex(kind=8), allocatable::
     $     chi_trace(:), e_field(:,:),
     $     j_induced(:,:),      ! induced current in gspace
     $     chi_g(:,:,:)         ! rho_magnetic in gspace

c
      integer 
     $     i, j, k, n, is, irk, info,
     $     nproc,               ! number of processors started up
     $     myproc,              ! number of this processor: 0,1,...nproc-1
     $     nprow, npcol,        ! number of processor row or columns
     $     id9, idspin, ierr,
     $     mtrxdim,             ! total size of hamilton matrix
     $     maxlength,           ! maximum number of plane waves at any k-point
     $     ipr,                 ! print flag
     $     ijob,                ! job counter
     $     ii,jj,kk,il,im,nrgrid   ! parameters for angular momentum

c variables for angular momentum projection of wavefunctions
      double precision,allocatable :: bess(:,:)  ! more parameters
      double precision alpha(4,7)
      double precision gmax,step,rc,tau(3)
      integer,allocatable :: kx(:),ky(:),kz(:)
      integer ll
cc

      double precision 
     $     ddum,                ! double dummy
     $     t0,                  ! starting time
     $     factor
      double precision, allocatable :: 
     $     p_weights(:,:,:,:,:,:) ! weights for angular momentum DOS

      character*255 strbuf1     ! dummy string buffer 
      character*2 nameat(ntype) ! names of the atoms

c
c     maps job number to intelligible strings
c
      character*22 jobstring(5)
      data jobstring/ 'SELF-CONSISTENT FIELD', 'NMR SHIFT',
     $     'NMR PLOT', 'BAND STRUCTURE','PLOT POTENTIAL'/


      logical logidum, 
     $     lkwfn,               ! whether wave function should be kept
     $     lrwfn                ! whether wave function should be reused


c     ################## external functions and subroutines #############

      double precision 
     $     gimmetime            ! function which returns elapsed CPU time

      external gimmetime

c
c     --------- start up parallel stuff if not serial----------
c

m4_ifdef([MPI],[
      call mpi_comm_rank(MPI_COMM_WORLD, myproc, info)
      call mpi_comm_size(MPI_COMM_WORLD, nproc, info)
      m4_ifdef([SCALAPACK],[
      call layout_scalapack(100, info, nproc, nprow, npcol)
      call blacs_get(-1,0,blacs_context) ! get the default blacs context
      call blacs_gridinit(blacs_context, 'R', nprow, npcol) ! commit to  grid
      ],[blacs_context=-1])
      ],[
      blacs_context=-1; myproc=0; nproc=1; ])

      call allocate_shmem_work(nproc,1)

c     ----------------------------------------------------------------
c           TRANSFER DATA FROM THE PARAMETER LIST TO STRUCTURES 
c
      lrwfn =.false. ; lkwfn = .false.
      if(iwfnreuse.gt.1) lrwfn =.true. ! can reuse wave function
      if(iwfnreuse.gt.0) lkwfn =.true. ! should keep wave function

      fsum=0                    ! to avoid crashes if forces are not computed
      energs=0.d0
      energs%inputfermi=fermil
      do i=1,ntype
         nameat(i)=nameattst(2*i-1:2*i)
      end do
c
c     the pw parameters
c
      pw_params%mfield%gvec = gmagnet
      pw_params%mfield%h = h0
      pw_params%epot%gvec = gepot
      pw_params%epot%h = eampli
      pw_params%alphamix = alphamix
      pw_params%emaxsub = emaxsub
      pw_params%emaxmix = emaxmix
      pw_params%epscv = epscv
      pw_params%epsdiag = delta
      pw_params%bandgap = bandgap/13.6058
      pw_params%smearing = idsmear
      pw_params%emax = emax
      pw_params%miscflag = miscflag
      pw_params%output = imonitor
      pw_params%input =iinput
      pw_params%optimize = ioptimize
      pw_params%shiftsafety = esafety
      pw_params%maxitscf = itmax
      pw_params%maxitdiag = maxnumit
      pw_params%ekinmod = ekinmod
      pw_params%occupy = ioccup
      pw_params%icorr = 'ca'
      pw_params%q(1) = qx
      pw_params%q(2) = qy
      pw_params%q(3) = qz
      pw_params%random_startvec = randomstart
      if(icorr.eq.1)  pw_params%icorr = 'pw'
      if(icorr.eq.2)  pw_params%icorr = 'pb'
      pw_params%epsmag = pw_params%epsdiag ! set same accuracy as in diag.
      pw_params%maxitcgsol= pw_params%maxitdiag ! and same number of iterations
      pw_params%checkpoint_wfn = ichkptwfn ! how frequently wfn is chkpted

c-----------------------------------------------------------------------------
C
C                 OPEN OUTPUT FILES
C

c
c     --- open output files -------
c
      if(myproc.eq.0) then
         write(outfilename,'(a5,i1)') 'OUT._',myproc
      else
         write(outfilename,'(a6)') 'OUT.OT'
      endif

      open(9,file=outfilename,
     $     status='unknown',form='formatted')
      if(myproc.eq.0) then
         open(34,file='PW_LOG', 
     $        position='append',
     $        status='unknown',form='formatted')
      endif
c
c
c

      call tpage(nproc)              ! prints title page

c
c     look at crystal structure. This has to be done always.
c     

      call create_crystal(crys,mxdatm,ntype,coord,
     $     natom,nameat,avec,szet,  mtsphere, dipole, ispn)
      
      syms%ntrans = intrans
      bands%min = nbandi
      altbands%min = nbandi
      pspot%mxdlqp=mxdlqp

c     
c     gets crystal data and determines symmetry
c

      call crstl(pw_params%output, crys, syms, pw_params%miscflag)
c      call symmetrize_crystal(crys, syms)

      crys%totalmass = totmass

      altkpoints%nrk=iabs(nrkinput)
      altkpoints%grid=ngrid
      altkpoints%shift=kshift
      
      if(.not.lrwfn) then
         kpoints%nrk=iabs(nrkinput)
         kpoints%grid=ngrid
         kpoints%shift=kshift
      endif
      
      kpoints%generated=.false. 
      if(nrkinput.le.0) kpoints%generated=.true.
      kpoints%reduced=.false.
      if(nrkinput.ge.0) kpoints%reduced=.true.
      altkpoints%generated=kpoints%generated
      altkpoints%reduced=kpoints%reduced


      call print_pwparam(1,pw_params)

      call ball_and_stick(pw_params%output, crys, myproc)

      call neighbor(crys,pw_params)
      call myflush(9)
      
c
c     the main loop. Implement and document new subroutines from here
c
c

      do ijob=1, njobs
         write(9,30) jobstring(joblist(ijob)+1)
         t0=gimmetime()
         select case(joblist(ijob))

c        ------------ just the scf loop  ----------------------------
         case (0)  

c
c           generate large gspace for the potential
c
            call generate_potential_gspace(pot_gspace, 
     $           crys, syms,pw_params, myproc, nproc)
c
c           allocate work arrays for fft
c
            call create_fftstruc(ffts,pot_gspace)

c
c           read pseudopotential and set up valence/core charge
c           (the ddc,dvql, and dnc arrays are for force/stress calc)
c
            call read_pseudo(1,crys%nspin,pw_params, 
     $           pot_gspace,
     $           energs, pspot, crys,
     $           vion, denc, denv, ddc, dvql, vql,dnc)

c  
C           calculate ewald energy and forces 
C
            call ewald(pw_params%output,
     $           crys,pot_gspace, energs%ewald, ewaldf)
c
c           screen the ionic potential with the valence and core charge density
c           also creates arrays vin, den, chdr.

            call screen(iscr, lrwfn, pot_gspace, crys,
     $           ffts, energs, pw_params,
     $           vion, denv, denc, vin, den, chdr)

            call create_gspace_array(vout,pot_gspace%length,
     $           1,crys%nspin)

c
c           reads or calculates integration k-points, initializes bands.
c
            if(.not.lrwfn)
     $           call generate_kpoints(1,kpoints,crys,syms,bands,
     $                                  pw_params%output,pw_params%q)

c
c           now set up all the small gspaces at the various kpoints
c

            allocate(k_gspace(kpoints%nrk))

            call generate_k_gspaces(k_gspace, maxlength,
     $           pot_gspace, kpoints,crys,syms,emax, pw_params%output)

c
c           allocate space for wave functions, and do selfconsistent loop
c
            if(.not.lrwfn)
     $           call create_gspace_array(wavefn,maxlength*bands%max,
     $           kpoints%nrk,crys%nspin)

            if(iand(pw_params%optimize,8).eq.8) then ! direct minimization
               call direct_emin(pot_gspace, k_gspace(1),
     $              energs, pw_params, crys, syms, ffts,
     $              pspot, kpoints, bands, wavefn, lrwfn,
     $              vion%data(1,1,1),vin%data(1,1,1),vout%data(1,1,1),
     $              den%data(1,1,1), denc%data(1,1,1), 
     $              chdr%data(1,1,1))
            else                ! old scf iteration
               call scfloop(1, pot_gspace, k_gspace(1),
     $              energs, pw_params, crys, syms, ffts,
     $              pspot, kpoints, bands, wavefn,lrwfn,
     $              vion, vin, vout, den, denc, chdr)
            endif

c print out gw screening

            if(iand(pw_params%output,2097152) .eq. 2097152) then
               factor = 8.0d0 * pi /crys%vcell

C in GW calculations, the exchange-correlation potential
C should not include partial core correction. Assign denc%data = 0.

               denc%data(1:pot_gspace%length,1,1:crys%nspin)=0.0
      
               call velect(1,pot_gspace,crys,ffts,energs,pw_params,
     $              denc%data(1,1,1),vout%data(1,1,1),
     $              den%data(1,1,1),chdr%data(1,1,1))

c subtract out core charge
c subtract out hartree potential
               do i=1,pot_gspace%length
                  if (pot_gspace%ekin(i).gt.0) then
                     if (crys%nspin.eq.1) then
                        vout%data(i,1,1) = vout%data(i,1,1) -
     $                       factor/pot_gspace%ekin(i)*
     $                       den%data(i,1,1)
                     end if
c     when nspin=2, we should extract spin part separately and
c     den%data(i,1,1) is now the sum of charge density.
                     if (crys%nspin.eq.2) then
                        vout%data(i,1,1) = vout%data(i,1,1) -
     $                       factor/pot_gspace%ekin(i)*
     $                       (den%data(i,1,1)+den%data(i,1,2))
                        vout%data(i,1,2) = vout%data(i,1,2) -
     $                       factor/pot_gspace%ekin(i)*
     $                       (den%data(i,1,1)+den%data(i,1,2))
                     end if
                  endif
               enddo


               call writegsdat2(0,pot_gspace,vout%data(1,1,1),
     $              1,1,crys%nspin,'VXC',3)
               call writegsdat2(0,pot_gspace,den%data(1,1,1),
     $              1,1,crys%nspin,'CD95',4)
            endif
cc
            if(iand(pw_params%output,524288).eq.524288)
     $           call gwout('GWR',crys, syms,
     $           bands,pot_gspace,k_gspace, wavefn,
     $           kpoints,1)

c     angular momentum projection of wavefunctions. 
            if (myproc .eq. 0) then
               open(unit=239,file='project.dat')
            endif
            nrgrid = 3000
            gmax = sqrt(pw_params%emax)
            allocate(bess(nrgrid,4))
            rc = 4.0d0
            tau(1) = 0.0d0
            tau(2) = 0.0d0
            tau(3) = 0.0d0
            do ii = 1,bands%min
               do jj = 1,wavefn%nspin
                  do kk = 1,kpoints%nrk
                     allocate(kx(k_gspace(kk)%length))
                     allocate(ky(k_gspace(kk)%length))
                     allocate(kz(k_gspace(kk)%length))
                     do ll = 1,k_gspace(kk)%length
                        kx(ll) = k_gspace(kk)%gvec(1,ll)
                        ky(ll) = k_gspace(kk)%gvec(2,ll)
                        kz(ll) = k_gspace(kk)%gvec(3,ll)
                     enddo
                     call angular_wavefn(
     >                    wavefn%data((ii-1)*k_gspace(kk)%length+1,kk
     $                    ,jj),k_gspace(kk)%length,kx,ky,kz,bess
     $                    ,nrgrid,alpha,tau,ii*jj*kk,gmax,rc,step
     $                    ,crys%bdot,crys%bvec)
                     deallocate(kx)
                     deallocate(ky)
                     deallocate(kz)
                     if (myproc .eq.0) then
                        write(239,"(3i4)") ii,jj,kk
                        do il = 1,4
                           do im = 1,2*il-1
                              write(239,"(2i5,2x,f5.3)") il,im,
     >                             alpha(il,im)
                           enddo
                        enddo
                     endif
                  enddo
               enddo
            enddo
            deallocate(bess)
            
            if (myproc .eq. 0) close(239)
c     end of angular momentum projection of wavefunction
            
            if(iand(pw_params%output,1048576).eq.1048576)
     $           call gwout('GWC',crys, syms,
     $           bands,pot_gspace,k_gspace, wavefn,
     $           kpoints,2)
            
c     
c     output data to Balazs VQMC code
c

            if(iand(pw_params%output,2).eq.2)
     $           call vqmc('VQMC',crys, syms,
     $           bands,pot_gspace,k_gspace, wavefn,
     $           kpoints,energs%efermi(1), pw_params%smearing/13.6058)

c
c           compute forces and stress
c
            call force_stress(crys,syms, bands, kpoints,
     $           pspot, energs, pw_params, pot_gspace, k_gspace,
     $           vion%data(1,1,1), vin%data(1,1,1),
     $           vout%data(1,1,1), den%data(1,1,1), 
     $           denc%data(1,1,1), dvql%data(1,1,1), 
     $           ddc%data(1,1,1), vql%data(1,1,1), dnc%data(1,1,1),
     $           wavefn, ewaldf, ssum, fsum, ffts)

            call logfile(ipr, myproc, nproc, ilog,
     $           gimmetime()-t0, fsum, ssum, crys, pw_params, syms, 
     $           kpoints, energs)
c
c           plot charge density 
c
            if(iand(pw_params%output,16).eq.16) then
               if(crys%nspin.eq.1) then
                  call dxplot(1,pw_params%output, den%data(1,1,1),
     $                 pot_gspace,ffts,  crys, 'CHARGE_')
                  call lineplot(1,crys,1d0/crys%vcell,den%data(1,1,1), 
     $                 pot_gspace,  nlineplot, line_plot, bra, ket, 1,
     $                 'CHARGE_')
               else
                  call dxplot(1,pw_params%output, den%data(1,1,1),
     $                 pot_gspace,ffts,crys, 'CHAR_UP')
                  call dxplot(1,pw_params%output, den%data(1,1,2)
     $                 ,pot_gspace,ffts,crys, 'CHAR_DN')
                  call lineplot(1, crys, 1.d0, den%data(1,1,1), 
     $                 pot_gspace,  nlineplot, line_plot, bra,ket,1,
     $                 'CHAR_UP')
                  call lineplot(1, crys, 1.d0, den%data(1,1,2), 
     $                 pot_gspace,  nlineplot, line_plot, bra,ket,1,
     $                 'CHAR_DN')
               endif
               
               do i=1, nenergywindow
                  call resolved_charge(energywindow(1,i),
     $                 ffts, pw_params,pot_gspace, k_gspace(1),
     $                 bands, kpoints, wavefn, den%data(1,1,1),
     $                 energs)
                  do is=1,crys%nspin
                     write(cdname,20) is,i
                     call dxplot(1,pw_params%output, den%data(1,1,is),
     $                    pot_gspace,ffts,crys, cdname)
                     call lineplot(1,crys,1d0/crys%vcell,den%data(1,1
     $                    ,is),pot_gspace,  nlineplot, line_plot, bra,
     $                    ket, 1,cdname)
                  end do
               end do
              
            endif
c
c           compute and plot electric field
c
            if((iand(pw_params%output,131072).eq.131072)) then
               call electric_field(pw_params,crys,den%data(1,1,1),
     $              vion%data(1,1,1),vin%data(1,1,1),pot_gspace,
     $              nlineplot, line_plot,bra,ket)
            endif
               
c
c           plot wave functions
c
            if(iand(pw_params%output,256).eq.256) then
               do is=1,crys%nspin
                  do j=1, nplotwv
                     irk = plotwv(1,j) ! get kpoint number
                     if(irk.gt.kpoints%nrk) then
                        write(9,530) 'kpoint', irk,
     $                       'number of kpoints', kpoints%nrk
                        irk=kpoints%nrk
                     endif
                     n = plotwv(2,j) ! get band number
                     if(n.gt.bands%nband(irk,is)) then
                        write(9,530) 'band',n,'number of bands',
     $                       bands%nband(irk,is)
                        n=bands%nband(irk,is)
                     endif
c Write the square of modulus of the wavefunctions
                     if(n.lt.10)
     $                 write(strbuf1,'(a3,i1,a2,i1)') 'WF.',irk,'._',n
                     if(n.ge.10)
     $                 write(strbuf1,'(a3,i1,a1,i2)') 'WF.',irk,'.',n
                     call dxplot(3,pw_params%output,
     $                    wavefn%data(k_gspace(irk)%length
     $                    *(n-1)+1,irk,is),k_gspace(irk),
     $                    ffts,  crys, strbuf1)
c Write the real part of the wavefunctions
                     if(n.lt.10)
     $                 write(strbuf1,'(a3,i1,a2,i1)') 'WR.',irk,'._',n
                     if(n.ge.10)
     $                 write(strbuf1,'(a3,i1,a1,i2)') 'WR.',irk,'.',n
                     call dxplot(1,pw_params%output,
     $                    wavefn%data(k_gspace(irk)%length
     $                    *(n-1)+1,irk,is),k_gspace(irk),
     $                    ffts,  crys, strbuf1)
c Write the imaginary part of the wavefunctions
                     if(n.lt.10)
     $                 write(strbuf1,'(a3,i1,a2,i1)') 'WI.',irk,'._',n
                     if(n.ge.10)
     $                 write(strbuf1,'(a3,i1,a1,i2)') 'WI.',irk,'.',n
                     call dxplot(2,pw_params%output,
     $                    wavefn%data(k_gspace(irk)%length
     $                    *(n-1)+1,irk,is),k_gspace(irk),
     $                    ffts,  crys, strbuf1)

                     call lineplot(1, crys, 1.d0,
     $                    wavefn%data(k_gspace(irk)%length*(n-1)+1,
     $                    irk,is),k_gspace(irk),  nlineplot,
     $                    line_plot,bra,ket, 1, strbuf1)

                  end do
               end do
            endif

c
c                 compute the weights for the angular momentum DOS
c
            allocate(p_weights(7,crys%mxdatm,crys%ntype,bands%max,
     $           bands%nrk,crys%nspin)) 
            if(iand(pw_params%output,1024).eq.1024) then
               call angular(p_weights,k_gspace,ffts,crys,bands,
     $              kpoints, wavefn,syms, pw_params, nrg) 
            endif

            if((iand(pw_params%output,128).eq.128).or.
     $           (iand(pw_params%output,1024).eq.1024)) then
c
c              compute and print density of states
c
               call dos(myproc,nproc,p_weights,crys, bands, energs, 
     $              kpoints, pw_params)

            endif

            if((iand(pw_params%output,2048).eq.2048)) then
               call write_eigvals(myproc, pw_params, crys,
     $              energs,kpoints,bands, p_weights) 
            endif

            deallocate(p_weights)
            
c
c           deallocate all the arrays which have dynamical allocation
c
            call destroy(ffts)
            call destroy(ewaldf)
            if(.not.lkwfn) call destroy(bands)


            call destroy(pspot)
            call destroy(pot_gspace)
            do irk=1, kpoints%nrk
               call destroy(k_gspace(irk))
            end do
            deallocate(k_gspace)
            if(.not.lkwfn)  call destroy(kpoints)

c     
c           deallocate all the gspace arrays
c
            if(.not.lkwfn)           call destroy(wavefn)
            call destroy(vion); call destroy(denc);
            call destroy(denv); call destroy(ddc);
            call destroy(dvql); call destroy(vql);
            call destroy(dnc);  call destroy(vin);
            if(.not.lkwfn) call destroy(den);
            call destroy(chdr)
            call destroy(vout);

c
c
c            
c        ------------ compute NMR shift  ----------------------------
c
c
         case (1)  

c
c           generate large gspace for the potential
c
            call generate_potential_gspace(pot_gspace, 
     $           crys, syms,pw_params, myproc, nproc)
c
c           allocate work arrays for fft
c
            call create_fftstruc(ffts,pot_gspace)

c
c           read pseudopotential and set up valence/core charge
c           (the ddc,dvql, and dnc arrays are for force/stress calc)
c
            call read_pseudo(1,crys%nspin,pw_params, 
     $           pot_gspace, energs, pspot, crys,
     $           vion, denc, denv, ddc, dvql, vql,dnc)

c
c           screen the ionic potential with the valence and core charge density
c           also creates arrays vin, den, chdr.

            call screen(4, .false., pot_gspace, crys,
     $           ffts, energs, pw_params,
     $           vion, denv, denc, vin, dena, chdr)

c
c           reads or calculates integration k-points, initializes bands.
c
            call generate_kpoints(1,altkpoints,crys,syms,altbands,
     $             pw_params%output,pw_params%q)

c
c           compute NMR shift
c
            allocate(chi_g(pot_gspace%length,3,3))

            call magnetic_suscept(chi_q, g0mask(1),
     $           pw_params, ffts, altbands,
     $           pot_gspace, vion, crys, syms, pspot,altkpoints,
     $           chi_g(1,1,1))

c
c           and plot if desired
c
            if(iand(pw_params%output,4).eq.4) then
               allocate(chi_trace(pot_gspace%length))
               ddum =-1.d0/(3.d0*137.036**2.d0)*1.d6
               chi_trace(:)=ddum*
     $              (chi_g(:,1,1)+chi_g(:,2,2)+chi_g(:,3,3))
               call dxplot(1,pw_params%output,chi_trace,pot_gspace,ffts,
     $              crys, 'MCHI_TR')
               call lineplot(1, crys, 1.d0, 
     $              chi_trace(1), pot_gspace,  nlineplot, 
     $              line_plot, bra, ket, 1, 'MCHI_TR')
               deallocate(chi_trace)
            endif


c
c           deallocate all the arrays which have dynamical allocation
c

            deallocate(chi_g)

            call destroy(ffts)
            call destroy(altkpoints)
            call destroy(altbands)

            call destroy(pot_gspace)
            call destroy(pspot)
c     
c           deallocate all the gspace arrays
c
            call destroy(vion); call destroy(denc);
            call destroy(denv); call destroy(ddc);
            call destroy(dvql); call destroy(vql);
            call destroy(dnc);  call destroy(vin);
            call destroy(dena);  call destroy(chdr)


c            
c        ------------ read and plot NMR shift  ----------------------------
c
c
         case (2)  

c
c           generate large gspace for the potential
c
            call generate_potential_gspace(pot_gspace, 
     $           crys, syms,pw_params, myproc, nproc)

c
c           allocate work arrays for fft
c
            call create_fftstruc(ffts,pot_gspace)
            allocate(chi_g(pot_gspace%length,3,3))

            idspin=1 ; id9=9; ierr=0 ;
            call readgsdat(1,ierr, pot_gspace, chi_g, 
     $           9, id9,idspin,'CHI',3)
            if((id9.ne.9).or.(idspin.ne.1).or.(ierr.ne.0)) then
               write(9,40) 
            else
               call nmr_shift(pw_params,crys,pot_gspace,chi_g(1,1,1))
c
c              plot certain component of CHI along a line or plane
c
               call lineplot(1, crys,-1.d0/(137.036**2.d0)*1.d6, 
     $              chi_g(1,1,1),  pot_gspace,  nlineplot, 
     $              line_plot, bra, ket, 9, 'MCHI___')
c
c              plot the trace(CHI) as a function of space
c
               allocate(chi_trace(pot_gspace%length))
               ddum =-1.d0/(3.d0*137.036**2.d0)*1.d6
               chi_trace(:)=ddum*
     $              (chi_g(:,1,1)+chi_g(:,2,2)+chi_g(:,3,3))
               call dxplot(1,pw_params%output,chi_trace,pot_gspace,ffts,
     $              crys, 'MCHI_TR')
               call lineplot(1, crys, 1.d0, 
     $              chi_trace(1), pot_gspace,  nlineplot, 
     $              line_plot, bra, ket, 1, 'MCHI_TR')
               deallocate(chi_trace)

c
c              plot the current for an external field in direction of ket
c
               allocate(j_induced(pot_gspace%length,3))
               call induced_current(j_induced(1,1), chi_g(1,1,1),
     $              ket, pot_gspace, crys)
               call dxplot_field(2,pw_params%output,j_induced(1,1),
     $              pot_gspace,ffts, crys, 'J_INDUC')
               deallocate(j_induced)


c               chi_trace(:)=
c     $              (chi_g(:,1,1)+chi_g(:,2,2)+chi_g(:,3,3))/3.d0
c               call lineplot(1, crys, 1.d0/(137.036**2.d0)*1.d6, 
c     $              chi_trace(1), pot_gspace,  nlineplot, 
c     $              line_plot, bra,ket, 1, 'MCHI_TR')
c               call dxplot_field(1,pw_params%output,chi_g(1,1,1),
c     $              pot_gspace,ffts,   crys, 'B_INDUC')


            endif


c
c           deallocate all the arrays which have dynamical allocation
c

            deallocate(chi_g)

            call destroy(ffts)
            call destroy(pot_gspace)


c            
c        ------------ compute band structure  ----------------------------
c
c
         case (3)  

            call generate_potential_gspace(pot_gspace, 
     $           crys, syms,pw_params, myproc, nproc)
            call create_fftstruc(ffts,pot_gspace)

            call read_pseudo(1,crys%nspin,pw_params, 
     $           pot_gspace,
     $           energs, pspot, crys,
     $           vion, denc, denv, ddc, dvql, vql,dnc)

            call screen(iscr, .false., pot_gspace, crys,
     $           ffts, energs, pw_params,
     $           vion, denv, denc, vin, dena, chdr)

            call create_gspace_array(vout,pot_gspace%length,
     $           1,crys%nspin)

            logidum = altkpoints%generated
            altkpoints%generated=.false.
            call generate_kpoints(1,altkpoints,crys,syms,altbands,
     $             pw_params%output,pw_params%q)
            altkpoints%generated=logidum

            allocate(k_gspace(altkpoints%nrk))
            call generate_k_gspaces(k_gspace, maxlength,
     $           pot_gspace, altkpoints,  crys, syms,emax,
     $           pw_params%output)

            call create_gspace_array(wavefnbs,maxlength*altbands%max,
     $           altkpoints%nrk,crys%nspin)

            i=pw_params%maxitscf ; pw_params%maxitscf=1

            call scfloop(0, pot_gspace, k_gspace(1),
     $           energs, pw_params, crys, syms, ffts,
     $           pspot,altkpoints, altbands, wavefnbs, .false.,
     $           vion, vin, vout, dena, denc, chdr)
            pw_params%maxitscf=i

            if(myproc.eq.0) 
     $           call plot_bandstruc(altkpoints,altbands,crys,
     $           energs,bslabels)

            if(iand(pw_params%output,4096).eq.4096) then
               call momentum_density(crys,altkpoints,altbands,
     $              k_gspace, wavefnbs)
            endif
            
            if((iand(pw_params%output,2048).eq.2048)) then
               call write_eigvals(myproc, pw_params, crys,
     $              energs,altkpoints,altbands, p_weights) 
            endif
               
c    angular momentum projection for bs wavefunctions
          if (myproc .eq. 0) then
            open(unit=239,file='project.bs.dat')
          endif
          nrgrid = 3000
          gmax = sqrt(pw_params%emax)
          allocate(bess(nrgrid,4))
          rc = 4.0d0
          tau(1) = 0.0d0
          tau(2) = 0.0d0
          tau(3) = 0.0d0
            do ii=1,altbands%min
             do jj=1,wavefnbs%nspin
              do kk=1,altkpoints%nrk
               allocate(kx(k_gspace(kk)%length))
               allocate(ky(k_gspace(kk)%length))
               allocate(kz(k_gspace(kk)%length))
               do ll=1,k_gspace(kk)%length
                 kx(ll) =k_gspace(kk)%gvec(1,ll)
                 ky(ll) =k_gspace(kk)%gvec(2,ll)
                 kz(ll) = k_gspace(kk)%gvec(3,ll)
               enddo
               call angular_wavefn(
     >          wavefnbs%data((ii-1)*k_gspace(kk)%length+1,kk,jj)
     >           ,k_gspace(kk)%length,kx,ky,kz,
     >          bess,nrgrid,alpha,tau,ii*jj*kk,gmax,rc,step,
     >         crys%bdot,crys%bvec)
               deallocate(kx)
               deallocate(ky)
               deallocate(kz)
                if (myproc .eq.0) then
                  write(239,"(3i4)") ii,jj,kk
                  do il =1,4
                   do im =1,2*il-1
                    write(239,"(2i5,2x,f5.3)") il,im,
     >                  alpha(il,im)
                   enddo
                  enddo
                endif
             enddo
             enddo
             enddo
           deallocate(bess)
        if (myproc .eq. 0) close(239)


            do irk=1, altkpoints%nrk
               call destroy(k_gspace(irk))
            end do
            deallocate(k_gspace)

            call destroy(ffts)  ; call destroy(altkpoints)
            call destroy(altbands) ; call destroy(pot_gspace)
            call destroy(vion); call destroy(denc);
            call destroy(denv); call destroy(ddc);
            call destroy(dvql); call destroy(vql);
            call destroy(dnc);  call destroy(vin);
            call destroy(dena);  call destroy(chdr)
            call destroy(vout); call destroy(wavefnbs)
            call destroy(pspot)
c            
c           ------------ plot the potential and the charge ---------------
c
c
c
         case (4)  

c
c           generate large gspace for the potential
c
            call generate_potential_gspace(pot_gspace, 
     $           crys, syms,pw_params, myproc, nproc)
c
c           allocate work arrays for fft
c
            call create_fftstruc(ffts,pot_gspace)

c
c           read pseudopotential and set up valence/core charge
c           (the ddc,dvql, and dnc arrays are for force/stress calc)
c
            call read_pseudo(1,crys%nspin,pw_params, 
     $           pot_gspace, energs, pspot, crys,
     $           vion, denc, denv, ddc, dvql, vql,dnc)

c
c           screen the ionic potential with the valence and core charge density
c           also creates arrays vin, den, chdr.

            call screen(4, .false., pot_gspace, crys,
     $           ffts, energs, pw_params,
     $           vion, denv, denc, vin, dena, chdr)

c
c           do lineplot and printout for the data explorer
c
            if(iand(pw_params%output,262144).eq.262144) then
               call lineplot(1,crys,1d0,vion%data(1,1,1), 
     $              pot_gspace,  nlineplot, line_plot, bra, ket, 1,
     $              'VEFFAVG')
            
               call dxplot(1,pw_params%output,vion%data(1,1,1),
     $              pot_gspace,ffts, crys, 'VEFFAVG')
            endif
c
c           plot the charge density if the flag is set
c
            if(iand(pw_params%output,16).eq.16) then
               if(crys%nspin.eq.1) then
                  call dxplot(1,pw_params%output, dena%data(1,1,1),
     $                 pot_gspace,ffts,    crys, 'CHARGE_')
                  call lineplot(1,crys,1d0/crys%vcell,dena%data(1,1,1), 
     $                 pot_gspace,  nlineplot, line_plot, bra, ket, 1,
     $                 'CHARGE_')
               else
                  call dxplot(1,pw_params%output, dena%data(1,1,1),
     $                 pot_gspace,ffts,  crys, 'CHAR_UP')
                  call dxplot(1,pw_params%output, dena%data(1,1,2),
     $                 pot_gspace,ffts,  crys, 'CHAR_DN')
                  call lineplot(1, crys, 1.d0, dena%data(1,1,1), 
     $                 pot_gspace,  nlineplot, line_plot, bra,ket,1,
     $                 'CHAR_UP')
                  call lineplot(1, crys, 1.d0, dena%data(1,1,2), 
     $                 pot_gspace,  nlineplot, line_plot, bra,ket,1,
     $                 'CHAR_DN')
               endif
            endif

c
c           compute and plot electric field
c
            if((iand(pw_params%output,131072).eq.131072)) then
               call electric_field(pw_params,crys,dena%data(1,1,1),
     $              vion%data(1,1,1),vin%data(1,1,1),pot_gspace,
     $              nlineplot, line_plot,bra,ket)
            endif

            call destroy(ffts)
            call destroy(pot_gspace)
            call destroy(pspot)
c
c        ------------------------------------------------------------------
c



         case default
            write(0,*) 'ERROR: job ',joblist(ijob),
     $           ' not implemented yet!'
            stop
         end select
         write(9,300) gimmetime()-t0
         call myflush(9)

      end do                    ! end of gigantic loop 


      enew=energs%total

      call destroy(crys)

      call deallocate_shmem_work()
      m4_ifdef([SCALAPACK],[call blacs_gridexit(blacs_context)])
      close(9)
      close(34)

      return
C

 20   format('CDW',i1,i3.3)
 30   format(//' ================ JOB: ',a,' ====================='//)

 40   format(//' *** MAGNETIC SUSCEPTIBILITY FILE IS CORRUPT!')
 110  format(/' FIXING FERROMAGNETIC MOMENT TO BE ZERO')

 200  FORMAT(/,'  COMPUTING TIME FOR ITERATION ',I5,3X,F10.2)
 250  FORMAT(/,'  COMPUTING TIME FOR STARTING ',3X,F12.6,/)
 260  FORMAT(/,'  TOTAL TIME [s] ',3X,F12.6,/)

 300  FORMAT(/'  TOTAL COMPUTING TIME FOR JOB: ',F10.2)
 400  FORMAT('  E= ',F12.6)
 500  format(/,20x,'START OF SELFCONSISTENT CYCLE',/1x,78('-'))
 510  format(' SPIN(',i1,'):')
 520  format(' COMPUTED K-POINT: ', 3f12.6,' TIME[s]:',f12.6) 
 525  format(' diagonalization did not converge in ',i3,
     $     ' iterations:')

 530  format(' *** WARNING: ',a6,' NUMBER=',i3,' IN PLOTWV EXCEEDS ',
     $     a,' = ',i3)
 540  format(' *** REDUCED ',a6,' NUMBER TO ',a6,' = ',i5)

 600  format(' *** WARNING: ERROR WHEN READING OLD EIGENVECTORS',
     $      /' ***          GENERATING STARTING GUESS')   
 605  format(' *** WARNING: ERROR WHEN READING NCALC OR ESHIFT FILE',
     $      /' ***          GENERATING STARTING GUESS')   
 610  format(' NUMBER OF PROCESSORS: ',i4)

 620  format(' Spawning information:',
     $     /' -------------------- ',
     $     /' Program to spawn: ',a,
     $     /' Scratch directory: ',a,
     $     /' Group name:',a,
     $     /' Number of processors:',i4)

      END





