c
      subroutine qualitykp(crys, syms, kp)

      include 'use.h'
      implicit none             ! implicit? Just say no!

      include 'interface.h'
c
c     INPUT:
c     -----
c
      type (crystal)  crys   
      type (symmetry) syms
      type (kpoint)   kp
c
c     DESCRIPTION
c     -----------
c
c     Computes R_min, which is a measure of the quality of the kpoint
c     grid. 
c
c     1998 F. Mauri, B. Pfrommer, based on a code by A. de Vita.
c
c
c     ------------ local variables ------------------------------
c

      integer 
     $     mxr(3),              ! maximum grid limit
     $     nkpts                ! total number of symmetrized kpoints

      double precision, allocatable :: 
     $     symk(:,:),           ! symmetrized kpoints
     $     ws(:)                ! weights

      double precision 
     $     twopi,
     $     rmax,                ! maximum radius for which to check
     $     eps,                 ! criterion for which to decide on rmin
     $     sum,     
     $     phase,
     $     rr, rmin,
     $     rvec(3), vec(3)

      integer i, ix, iy, iz, k, ks

      parameter(eps = 1d-4)

c
c     unfold symmetrize kpoints into full grid by using the symmetry
c     operations.
c
      twopi = 8.d0*atan(1.d0)

      nkpts = syms%ntrans*kp%nrk

      allocate(symk(3,nkpts))
      allocate(ws(nkpts))
      
      do i=1, syms%ntrans
         do k=1, kp%nrk
            ks = k + (i-1)*kp%nrk
            ws(ks)=kp%w(k)
            symk(:,ks) = matmul(syms%mtrx(:,:,i), kp%rk(:,k))
         enddo 
      enddo 

      rmax = 10.d0* (3*crys%vcell/(2*twopi))**(1.d0/3.d0)
c     
c     find grid boundaries that match nicely 
c
      call findbound(rmax, crys%avec, mxr(1), mxr(2), mxr(3))
c
c     now find rmin
c
      rmin = rmax              

      do ix = -mxr(1), mxr(1)
         do iy = -mxr(2), mxr(2)
            do iz = -mxr(3), mxr(3)
               sum = 0
               do k=1, nkpts
                  phase = 
     $             twopi*(symk(1,k)*ix + symk(2,k)*iy + symk(3,k)*iz)
                  sum = sum +cos(phase)*ws(k)
               end do
               sum = abs(sum)
               if(sum.gt.eps) then
                  vec(1) = dble(ix)
                  vec(2) = dble(iy)
                  vec(3) = dble(iz)
                  rvec = matmul(crys%avec,vec)
                  rr   = sqrt(rvec(1)**2 + rvec(2)**2 + rvec(3)**2)
                  if(rr.gt.eps**2) rmin = min(rr,rmin)
               endif
            end do
         end do 
      end do

      if(rmin.lt.rmax) then
         write(9,1010) rmin
      else
         write(9,1020) rmin
      endif

      deallocate(symk)
      deallocate(ws)

 1010 format(/'    K POINT QUALITY ESTIMATE: R_MIN NONZERO =',f12.3)
 1020 format(/'    K POINT QUALITY ESTIMATE: R_MIN NONZERO >',f12.3)

      end
