C*
C*
      subroutine read_pseudo(ipr,
     $     ispn, pw_params, gs,  energs, pspot, crys,
     $     vion, denc, denv,ddc,dvql,  vql,dnc)

c     -----------------------------------------------------------------------
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'
c
c     INPUT:
c     -----
c     

      type (pw_parameter) pw_params
      integer 
     $     ipr,                 ! print flag
     $     ispn                 ! number of spins 


      type (parallel_gspace) gs

c
c     OUTPUT:
c     ------
c
      type (energy) energs      ! only alpha term is touched

      type (complex_gspace_array)
     $     vion,                ! local ionic potential
     $     denc,                ! core charge
     $     denv,                ! valence charge
     $     ddc,                 ! derivative of core charge
     $     dvql                 ! derivative of local ionic potential


      type (double_gspace_array)
     $     vql,                 ! local part of pseudopotl for atom,gvector
     $     dnc                  ! core charge for atom, gvector

      type (pseudo_potential) pspot 
    
c
c     INPUT/OUTPUT:
c     ------------
c
      type (crystal) crys       ! only valence charge zv() and ztot are altered


c
c     reads the fourier pseudo potentials and the
c     atomic core and valence charge densities from
c     files whose name depend on the chemical symbol.
c     it then computes the same quantities on the prototype
c     g-vectors.
c     adapted from sverre froyen plane wave program
c     written february 2 1990. jlm
c
c     Parallel and fortran 90 version 1996 Bernd Pfrommer
c
c     Multiple projector version 1997 Young-Gui Yoon and Bernd Pfrommer
c
c     Faster parallel i/o version 1997 Bernd Pfrommer. 
c     Now processor 0 only reads the file, and distributes the data
c     via broadcast. This is much faster than each processor
c     accessing separately. 
c     
c

c     ------------ local variables ---------------------------------

      character*1 symb(2)
      character*2 namel, icorrt
      character*3 irel
      character*4 icore
      character*8 tfile
      character*10 iray(6), ititle(7)
      character*2 icorr         ! type of correlation

      character*250 fnam        ! file name for open unit
      integer fnamlen, findvec
      external findvec


      double precision  zero, one, two, three, half
      
      parameter (zero = 0.0d0, one = 1.0d0, two=2.0d0, three=3.0d0)
      parameter (half = 0.5d0)

      double precision 
     $     vql0, fac, glmax, qj, xn, delql, xdum, 
     $     q2vn, q2vp, q2vm,
     $     vqj, dvqj, dcj, ddcj, dvj, fi
      
      double precision, allocatable :: vda(:)

      logical isp

      integer i,j,it,nt, izv, nql, norb, lp1, n, idum, 
     $     ind, ja, is, igvec(3)
c
c     -----------------------------------------------------------------------
c     
      icorr = pw_params%icorr   ! type of correlation
      fac = one / sqrt(crys%vcell)

      allocate(vda(pspot%mxdlqp)) ! just a temporary array
c
c     allocate memory for the pseudo potential structure
c
      pspot%ntype=crys%ntype
      allocate(pspot%vkb(pspot%mxdlqp,5,pspot%ntype))
      allocate(pspot%nqnl(pspot%ntype))
      allocate(pspot%delqnl(pspot%ntype))

      allocate(pspot%nkb(5,pspot%ntype))
      allocate(pspot%lo(5,pspot%ntype))
c
c     allocate memory for the gspace arrays
c

      call create_gspace_array(vion,gs%length,1,ispn)
      call create_gspace_array(denv,gs%length,1,ispn)

      call create_gspace_array(vql,gs%length,pspot%ntype,1)
      call create_gspace_array(dnc,gs%length,pspot%ntype,1)
      call create_gspace_array(denc,gs%length,1,1)
      call create_gspace_array(ddc,gs%length,1,1)
      call create_gspace_array(dvql,gs%length,1,1)


      isp = .false.
      if(ispn.eq.2) isp =.true.
 
      if (ipr .eq. 1) write(9,100) ! write heading

c
c      initialize arrays
c

      do i=1,gs%length
         denc%data(i,1,1) = 0
         dvql%data(i,1,1) = 0
         ddc%data(i,1,1)  = 0
      end do

      do is=1, ispn
         do i=1, gs%length
            vion%data(i,1,is) = 0
            denv%data(i,1,is) = 0
         end do
      end do

       energs%alpha = zero
       crys%ztot = zero
       tfile = '_POT.DAT'
C
C      START LOOP OVER ATOMIC TYPES
C

       do 20 nt=1,crys%ntype
         it = 10 + nt
C
C        ------ open file ------
C
         symb(1) = crys%nameat(nt)(1:1)
         symb(2) = crys%nameat(nt)(2:2)

         if(symb(1) .ne. ' ' .and. symb(2) .ne. ' ') then
            fnam = crys%nameat(nt)//tfile
         else if(symb(1) .eq. ' ') then
            fnam = symb(2)//tfile
         else
            fnam = symb(1)//tfile
         endif

         fnamlen = index(fnam,' ')-1

c
c        ------- read heading ----
c
         if(gs%myproc.eq.0) then
            open(unit=it,file=fnam(1:fnamlen),status='old',
     $           form='unformatted')
            read(it) namel,icorrt,irel,icore,(iray(j),j=1,6),
     $           (ititle(j),j=1,7)

            read(it) izv,nql,delql,vql0
         endif
         call my_broadcast(namel,2,0); call my_broadcast(icorrt,2,0)
         call my_broadcast(irel,3,0);  call my_broadcast(icore,4,0)
         do j=1,6 ; call my_broadcast(iray(j),10,0)
         end do
         do j=1,7 ; call my_broadcast(ititle(j),10,0)
         end do


         call my_broadcast(izv,0);    call my_broadcast(nql,0)
         call my_broadcast(delql,0);  call my_broadcast(vql0,0)

         crys%zv(nt) = dble(izv)
         pspot%delqnl(nt) =delql
         if (ipr .ge. 1) write(9,101) namel,icorrt,irel,icore,
     $        (iray(j),j=1,6),(ititle(j),j=1,7),nql,delql
         if (nql+2 .gt. pspot%mxdlqp) 
     $        call fatal(130,xdum,pspot%mxdlqp,'pseukb')
         if ((namel .ne. crys%nameat(nt))) then
            if(.not.((namel(1:1).eq.' ')
     $           .and.(crys%nameat(nt)(2:2).eq.' ')
     $           .and.(namel(2:2).eq.crys%nameat(nt)(1:1)).or.
     $          ((namel(2:2).eq.' ').and.(crys%nameat(nt)(1:1).eq.' ')
     $           .and.(namel(1:1).eq.crys%nameat(nt)(2:2)))))
     $           call warn(132,xdum,idum,'read_pseudo')

         endif
         if (icorrt .ne. icorr) then
              write(9,*) 'icorrt=',icorrt
              write(9,*) 'icorr=',icorr
              icorrt=icorr
              call warn(133,xdum,idum,'read_pseudo')
         endif
c
c        sum up vql0 for the alpha energy
c        and zv for the total valence charge
c
         energs%alpha = energs%alpha + vql0*dble(crys%natom(nt))
         crys%ztot = crys%ztot + crys%zv(nt)*dble(crys%natom(nt))

c
c        read pseudopotentials
c        note that q=0 is shifted to vda(2) and vkb(2,l,k)
c
         nql = nql + 2
         pspot%nqnl(nt) = nql 

c
c     --- read normalization and local potential in q-space -------------
c
         pspot%nkb(:,nt) = 0

c        maximum allowed norb is increased to 5
c        norb is actually nprojec in newkb

         if(gs%myproc.eq.0) 
     $        read(it) norb,(pspot%nkb(j,nt),j=1,min(norb,5))

         call my_broadcast(norb,0)
         call my_broadcast(pspot%nkb(1,nt),min(norb,5),0)

         if(norb .gt. 5) then
            write(9,*) 
     $           'TOO MANY NONLOCAL PROJECTORS FOUND IN read_pseudo'
            write(9,*) 'NUMBER OF PROJECTORS=', norb
            stop
         endif
c     
c     - local part
c
         if(gs%myproc.eq.0) 
     $        read(it) (vda(j),j=3,nql)
         call my_broadcast(vda(1),nql,0)
         
C
c     --- read nonlocal part of the Pseudopotential -----------

         do lp1 = 1,norb
            if(gs%myproc.eq.0) 
     $           read(it) (pspot%vkb(j,lp1,nt),j=2,pspot%nqnl(nt))
            call my_broadcast(pspot%vkb(1,lp1,nt),pspot%nqnl(nt),0)
         end do
c     
c        check argument limits and compare with gmax
c
         glmax = delql * dble(nql-3)

         if (glmax .lt. gs%gmax) then
            write(9,*) 'LOCAL POTENTIAL IS SET UP TO GLMAX=',glmax
            write(9,*) 'BUT GMAX IS LARGER:',gs%gmax
         endif
C
C        COMPUTE IONIC LOCAL POTENTIAL
C
         do j=1, gs%length
            vql%data(j,nt,1) = zero
C
C          QUADRATIC INTERPOLATION OF Q**2 * POTENTIAL
C
           qj = sqrt(gs%ekin(j))

           if(qj.gt.0.d0) then
              xn = qj / delql + two
              n  = xn + half
              if (n .lt. nql) then
                 if (n .le. 3) n = 4
                 xn = xn - dble(n)
                 q2vn = dble((n-2)*(n-2))*vda(n)
                 q2vp = dble((n-1)*(n-1))*vda(n+1)
                 q2vm = dble((n-3)*(n-3))*vda(n-1)
                 vqj  = q2vn * (one-xn) * (one+xn)
     1                + half * (q2vp*(one+xn) - q2vm*(one-xn)) * xn
                 vqj  = delql*delql * vqj / (crys%vcell*gs%ekin(j))
                 dvqj = -two * q2vn * xn
     1                + q2vp * (half+xn) - q2vm * (half-xn)
                 dvqj =(delql*dvqj/(two*crys%vcell*qj)-vqj)/gs%ekin(j)

                 do is=1, ispn
                    vion%data(j,1,is) = vion%data(j,1,is) 
     $                   + vqj  * gs%struc(nt,j)
                 end do

c                required for force/stress calculations:
                 dvql%data(j,1,1) = dvql%data(j,1,1) + 
     $                dvqj * gs%struc(nt,j) ! add all up
                 vql%data(j,nt,1) = vqj ! keep separately

              endif
           endif
        end do

c     ---------------- deal with core and valence charge -----------------

c
c       read core charge
c
        if(gs%myproc.eq.0) 
     $       read(it) (vda(j),j=3,nql)
        call my_broadcast(vda(1),nql,0)

        vda(1) = vda(3)
        vda(2) = vda(3) - (vda(4)-vda(3))/three
c
c       compute core charge
c
        do j=1,gs%length
           dnc%data(j,nt,1) = zero
c
c          interpolate vda
c
           qj = sqrt(gs%ekin(j))
           xn = qj/delql + two
           n = xn + half
           if (n .lt. nql) then
              xn = xn - dble(n)
              dcj = vda(n) * (one+xn) * (one-xn)
     $             + half * (vda(n+1)*(one+xn)-vda(n-1)*(one-xn)) * xn
              dnc%data(j,nt,1) = dcj
              
              denc%data(j,1,1) = denc%data(j,1,1) + dcj*gs%struc(nt,j)

              if(gs%ekin(j).lt.1d-16) then
                 ddc%data(j,1,1) = ddc%data(j,1,1) 
     $                + gs%struc(nt,j)*(vda(3)-vda(2))/(delql*delql)
              else
                 ddcj = -two * vda(n) * xn
     $                + vda(n+1)*(half+xn) - vda(n-1)*(half-xn)
                 ddcj = ddcj/(two*delql*qj)

                 ddc%data(j,1,1) = ddc%data(j,1,1) 
     $                + ddcj*gs%struc(nt,j)
              endif
              
           endif
        end do

c
c       read valence charge
c
        if(gs%myproc.eq.0) 
     $       read(it) (vda(j),j=3,nql)
        call my_broadcast(vda(1),nql,0)

        vda(1) = vda(3)
        vda(2) = crys%zv(nt)
c
c       compute valence charge
c     
        do j=1,gs%length
c
c          interpolate vda
c
           xn = sqrt(gs%ekin(j))/delql + two
           n = xn + half
           if (n .lt. nql) then

              xn = xn - dble(n)
              dvj = vda(n) * (one+xn) * (one-xn)
     $         + half * (vda(n+1)*(one+xn) - vda(n-1)*(one-xn)) * xn
c
c             sum up the charge density
c
              if(isp) then
                 denv%data(j,1,1) = denv%data(j,1,1) 
     $                + dvj * gs%struc(nt,j)
     $                *0.5*(1+crys%szet(nt)) ! spin up
                 denv%data(j,1,2) = denv%data(j,1,2) 
     $                + dvj * gs%struc(nt,j)
     $                *0.5*(1-crys%szet(nt)) ! spin down
              else
                 denv%data(j,1,1) = denv%data(j,1,1) 
     $                + dvj * gs%struc(nt,j)
              endif
           endif
        end do
c
c     pspot%lo is read for identification of l quantum number
c     To ensure compatiblity with old pspots, check if lo info
c     is available, and initialize correctly if not.
c
        pspot%lo(:,nt) = 0
        pspot%lo(1,nt)=0;  pspot%lo(2,nt)=1;  pspot%lo(3,nt)=2

c     read information about the angular momenta of the nonlocal
c     pspots
        if(gs%myproc.eq.0) then
           read(it, end=912)  (pspot%lo(j,nt),j=1,min(norb,5))
           goto 913

 912       continue
           write(9,*) ' ... consider upgrading to new kbconv ...'
 913       continue
        endif
        call my_broadcast(pspot%lo(1,nt),min(norb,5),0)
c
c       do some strange extrapolation to 0. Anybody knows what
c       this is good for?
c
        do lp1 = 1,norb
           pspot%vkb(1,lp1,nt) = (-1)**pspot%lo(lp1,nt)
     $          *pspot%vkb(3,lp1,nt)
        end do

        do lp1=1,norb
           do j=1,pspot%nqnl(nt)
              pspot%vkb(j,lp1,nt) = fac * pspot%vkb(j,lp1,nt)
           end do
        end do

        if(gs%myproc.eq.0) 
     $       close (unit=it)
 20   continue                  ! end of loop over atomic types


c
c     compute the alpha energy. this is the contribution
c     to the total energy coming from g=0.
c

      energs%alpha = crys%ztot*energs%alpha/crys%vcell

c
c     compute the number of components the nonlocal part has
c
c     For each atom, we have for a different potential component 
c
      pspot%nanl = 0

      do i=1,crys%ntype
         do j=1,5
         if(pspot%nkb(j,i) .ne. 0) 
     $        pspot%nanl = pspot%nanl 
     $       + (2 * pspot%lo(j,i) + 1) * crys%natom(i)
         end do
      end do

      if(ipr.ge.1)   write(9,200) pspot%nanl
      call myflush(9)

c
c     compute second derivatives for spline interpolation
c
      nql = maxval(pspot%nqnl(:)) - 2

      allocate(pspot%d2vkbdq2(nql,5,pspot%ntype))

      do i=1, crys%ntype
         do j= 1,5  
            if(pspot%nkb(j,i) .ne. 0) then
               call reg_grid_spline(pspot%vkb(3,j,i),
     $              pspot%nqnl(i)-2,pspot%d2vkbdq2(1,j,i))
            end if
         end do
      end do

      deallocate(vda)

c
c     put in an external electric potential if desired
c    
c     the shape is  V_e(r) = eampli * sin(Gr)
c
c     but then it is symmetrized with the symmetry operations
c     of the crystal.
c

      if(pw_params%epot%gvec(4).gt.0) then
         igvec = pw_params%epot%gvec(1:3)
         i   = findvec(igvec,gs)
         if(i.gt.0)   vion%data(i,1,:) = vion%data(i,1,:) 
     $           - 0.5d0 *cmplx(0.d0,pw_params%epot%h,kind=8)

         igvec = -pw_params%epot%gvec(1:3)
         i   = findvec(igvec,gs)
         if(i.gt.0)   vion%data(i,1,:) = vion%data(i,1,:) 
     $        + 0.5d0 *cmplx(0.d0,pw_params%epot%h,kind=8)
         do i=1,crys%nspin
            call symmetrize_scalar_global(vion%data(1,1,is),gs)
         end do
      endif

      return


 200  format(/' Nonlocal potential: ',/' ------------------',
     $     /' Number of projectors: ',i5)

 100  FORMAT(//,' POTENTIALS :',/,1X,12('-'))
 101  FORMAT(/,1X,A2,2X,A2,2X,A3,2X,A4,/,1X,6A10,/,1X,7A10,
     $     /,' NQL=',I4,' DELQL=',F5.3)

      END
