c
      subroutine regspace(gs, datas, gt, datat)

      use all_to_all_module
      include 'use.h'
      
      implicit none             ! just say no!
      include 'interface.h'
      include 'all_to_all.h'

c     1995 Bernd Pfrommer, UC Berkeley
c     
c

c     INPUT:
c     -----
      
      type(parallel_gspace)
     $   gs,                   ! source gspace
     $   gt                    ! target gspace


      complex(kind=8)
     $     datas(gs%length)             ! the source data 
c
c     OUTPUT:
c     ------

      complex(kind=8)
     $     datat(gt%length)     ! the target data

c
c     DESCRIPTION:
c     -----------
c
c     Transfers data from source gspace to target gspace.
c
c     The source gspace can be distributed across multiple processors,
c     whereas the target gspace must be replicated identically on
c     each processor. This is the case for instance with the mixing
c     gspace. If the source gspace is distributed, a lot of
c     communication has to be done, so use this routine frugally.
c
c     Meanwhile, with get_gset we have a better routine. It scales
c     like order(n*log(n)) with the size of the gspace and is clearly
c     superior. Please try to use that routine instead!
c     
c
c
c     ====================================================================
c
c     local variables:

      complex(kind=8)
     $     dcdum, rdat

      logical packit

      integer
     $     i,j,k, findvec, svec(3)

c
c     ----- variables for the gspace loop ---------
c
      integer
     $     igvt(4),fftnt(4), fftht(4),igv3t, irodt,iordt,igt,igs


      external findvec          ! finds index of gvec in gspace

      if(gt%nproc.gt.1) then
         write(9,*) 'regspace: target gspace must be replicated!'
         stop
      endif

      fftnt(1:3)=gt%fftsize(1:3)
      fftnt(4)=gt%fftsize(3)
      fftht(:)=fftnt(:)/2
c
      datat=0                   ! clear the target array

c     ----------------------------------------------------------------
c     Pick up the desired components while looping through 
c     the target gspace. when the target  gspace is smaller, this is
c     more efficient than looping through the source space, because
c     searching a gspace scales like n^(2/3).
c

      igt = 0 
      do iordt=1, gt%lorder     ! loop through x/y gspace 
         irodt=gt%order(1,iordt)
         igvt(1)= irodt/gt%fftsize(2)
         igvt(2)= mod(irodt,gt%fftsize(2))
         igvt(3)= gt%order(2,iordt)
         igvt(4)= gt%order(3,iordt)
         igvt(:)= mod(igvt(:)+fftht(:),fftnt(:))-fftht(:) 
         do igv3t=igvt(3),igvt(4) ! loop over z axis
            igt = igt + 1
c     
c              search in the source gspace
c
            svec(1:2)=igvt(1:2)
            svec(3)=igv3t
            igs = findvec(svec,gs)
            if(igs.gt.0) datat(igt)=datas(igs)
c            write(9,*) igs,datas(igs),igt,datat(igt)
         end do
      end do

c     In case the source gspace is distributed, the target array is
c     completed by a global sum. This is very brute force, but the 
c     simplest to program. Check if this causes an unacceptable
c     amount of communication.
c

      if(gs%nproc.gt.1) 
     $     call all_sum_all(datat,gt%length)
      
      end

