/*
 *   
 *  program to relax the atoms and the lattice vectors such that
 *  the total energy acquires a minimum
 *  
 *  1996 Bernd Pfrommer
 *
 */
#include "fortranize.h"
void C2FORTRANIZE(pwmain,PWMAIN)
(int *njobs, int *joblist,
	  int *ntyp, int *mxdatm, int *natom,
          int *nrk, int *nband, int *ngrid, 
	  int *ispn, int *icorr, int *input,int *imaxitdiag, 
	  int *imaxitscf, int *iwfnreuse, int *ichkpointwfn,
	  int *iscr, int *ioccup, int *ntrans, int *visualize,
	  int *nlineplot,double *lineplot, double *bra, double *ket,
          int *nplotwv, int *plotwv, int *imonitor, int *ioptimize, int *miscflag,
	  double *dsmear, double *kshift, double *epsdiag, double *bandgap, double *esafety, double *randomstart,
	  double *ecutoff, double *ecutoffsub, double *ecutoffmix, double *pot_conv,
          double *qx,double *qy,double *qz,
          int *ilog, double *mixing, double *h0, int *gmagnet, double *eampli, int *gepot, int *nenergywindow,
	  double *energywindow, 
	  double *modify_kinetic_energy, double *chi_q, double *g0mask, int *nrg,
	  double *totmass, double *coord, double *szet, double *mts, double *dipole,
	  double *a, double *fermil,
          double *etot, double *stress, double *force, char *atm,char *labels);


#include "libcalls.h"
#include "inputread.h"
#include "relax.h"

#include <stdlib.h>
#include <malloc.h>
#include <string.h>

struct pwparam pwp;


/* global variables */

FILE *logfile_c, *convergefile;

double lvec[3][3];
double vinit;
int    plen;
int    adjustpressure;
double relaxstress, relaxforce;
int    relax_nproc, relax_myproc;
double relax_total_mass;
int iwfnreuse_c;
int main_argc; char **main_argv;
/* David Roundy to fix xy plane for shearing */
int hold_xy_plane_fixed = 0, hold_xhat_fixed = 0;

main(int argc, char **argv)
{
  int i,j,k,iter,ierr,itmax,iprobe,pwcount=0,updateh, 
         decoup_rc;
  int iupdateh, ntyp, mxdatm, *natom, plen_squared, diaginfo;

  double qx, qy, qz, relaxepsfac, relaxcoordfac, accuracy, avmass,bulkmodulus, optfreq;
  double discrim, *coord, *mass, etot, etot2, e0,e_here;
  double lambda, lambdalin, L_p1, L_p2, stress[9], *force, *szet,*mts,*dipole;
  double *x, *H, *H0, *xold, *g, *gold, *metric;
  double *delta, *pv, *s, *u, *q;
  double eps[9],  lvecf[3][3], atam1[3][3],fsig[3][3], feps[3][3], fepsu[3][3];
  double p,v0,rho,adot,bdot,ldelta, lg, gnew, lgnew,lambdalimit;
  double pst[9],fpst[3][3],fupst[3][3];
  double *eigenvalues, *temp1,*temp2;
  double f0, g0, f1,g1, xsi, avar, bvar, cvar, lambda1, lambda2,disc;
  double lambda_tolerance, lambda2_tolerance;

  char vchar='V', lchar='L', buf[1000], jobstring[255], relaxstring[255];
  char *atm;

  FILE *infile;

  keyword key1;

  main_argc=argc;
  main_argv=argv;
#ifdef MPI
  MPI_Init(&argc, &argv); 
  MPI_Comm_rank(MPI_COMM_WORLD, &relax_myproc);
  MPI_Comm_size(MPI_COMM_WORLD, &relax_nproc);
#else
  relax_myproc =0;
  relax_nproc  =1;
#endif

  iwfnreuse_c =1; /* reuse wavefunctions by default */

  RSYSTEM("\\mv PW_LOG PW_LOG.old");
  RSYSTEM("\\mv STRUC_LOG STRUC_LOG.old");
  RSYSTEM("\\mv STRUC_CONVERGE STRUC_CONVERGE.old");

  /* --------- read crystal structure and initialize ------------------- */

  crystal_struc(INPUTFILENAME, &ntyp, &mxdatm,&natom,&coord,(double *)lvec,
		&atm,&mass, &szet, &mts, &dipole);


  
  infile=fopen(INPUTFILENAME,"r");
  if(infile==NULL) {printf("Cannot find input file!\n"); exit(-1);}
  if(relax_myproc==0) {
    logfile_c=fopen(LOGFILENAME,"w");
    if(logfile_c==NULL) {printf("Cannot open log file!\n"); exit(-1);}
    convergefile=fopen(CONVERGEFILENAME,"w");
    if(convergefile==NULL) {printf("Cannot open converge file!\n"); exit(-1);}
  }

  /* --------- get some other parameters needed ------------------ */

  for(i=0;i<9;i++) eps[i]=0.0;
  askdoublevar(infile, "starting_distortion",eps,9);

  for(i=0;i<3;i++) for(j=0;j<3;j++) feps[i][j]=lvec[i][j];
  vinit=matinvert((double *)feps);

  if(vinit<0) {FPRINTF(convergefile,
      "THE LATTICE VECTORS DO NOT FORM A DIRECT TRIAD!!\n"); exit(-1);}

  /* relaxation parameters for the lattice vectors */

  decoup_rc=0;
  if(askstringvar(infile, "decouple_coordinates", 
		  relaxstring, 255)==1)   {
    if(strstr(relaxstring,"yes")!=NULL) {
      decoup_rc=1;
      FPRINTF(convergefile,
      "DECOUPLING REALSPACE COORDINATES! COULD BREAK THE SYMMETRY!\n"); 
    }
  }

  bulkmodulus = 1.0; /* in MBar, good for silicon */
  askdoublevar(infile, "estimated_bulk_modulus", &bulkmodulus,1);
  if(bulkmodulus<=1e-6) {
    FPRINTF(logfile_c,"estimated_bulk_modulus too small: %lg MBar\n",
	    bulkmodulus);}
  relaxepsfac = 147.1076/(3.0*bulkmodulus*vinit);
  askdoublevar(infile, "relax_eps_factor",&relaxepsfac,1);



  /* mass for the plane wave code */
  relax_total_mass=0; 
  for(i=0;i<ntyp;i++)
    relax_total_mass+=mass[i]*natom[i];

  

  /* relaxation parameters for the coordinates */

  avmass=0.0;  /* compute average mass */
  for(i=0;i<ntyp;i++) avmass += mass[i];
  avmass =avmass/(double)ntyp;

  optfreq=15; /* in THz. Good for silicon */
  askdoublevar(infile, "estimated_optical_phonon_frequency", 
	       &optfreq,1);
  relaxcoordfac = 1.0/(avmass * optfreq*optfreq* 8.4157e-5);
  askdoublevar(infile, "relax_coord_factor",&relaxcoordfac,1);


  accuracy=0.01;
  askdoublevar(infile, "relax_accuracy",&accuracy,1);

  if(askstringvar(infile, "relax_pressure", relaxstring, 255)==1)
    {
      if(sscanf(relaxstring,"%lf",&p)==0) {p=0; adjustpressure=1;}
      else adjustpressure=0;
    }
  else {p=0;adjustpressure=0;}

  strcpy(jobstring,"relax");
  askstringvar(infile, "job", jobstring, 255);      /* what to do */
  lambdalimit=10;
  askdoublevar(infile, "lambda_limit",&lambdalimit,1);

  if(askstringvar(infile, "relax_what", relaxstring, 255)==1)
    {
      relaxstress=0.0; relaxforce=0.0;
      if(strstr(relaxstring,"stress")!=NULL) relaxstress=1.0;
      if(strstr(relaxstring,"force")!=NULL) relaxforce=1.0;
    }
  else {relaxstress=1.0; relaxforce=1.0;}


  /* pick different flavors of relaxations */

  iupdateh=1;
  if(askstringvar(infile, "relax_how", relaxstring, 255)==0)
    strcpy(relaxstring,"normal");

  lambda_tolerance=0.1; lambda2_tolerance=0.1;

  if(strstr(relaxstring,"gradient")!=NULL) iupdateh=0;
  if(strstr(relaxstring,"slow")!=NULL) 
    { lambda_tolerance=1e-6; lambda2_tolerance=1e-6;}
  if(strstr(relaxstring,"normal")!=NULL) 
    { lambda_tolerance=0.6; lambda2_tolerance=0.3;}
  if(strstr(relaxstring,"fast")!=NULL) 
    {lambda_tolerance=10.0; lambda2_tolerance=2.0;}



  FPRINTF(logfile_c,"--------RELAXATION STYLE: %s ---------\n", relaxstring);
  FPRINTF(logfile_c,"relax_eps_factor = %lf\n",relaxepsfac);
  FPRINTF(logfile_c,"relax_coord_factor = %lf\n",relaxcoordfac);
  FPRINTF(logfile_c,"Hessian update: %d\n", iupdateh);

  itmax=0;
  askintvar(infile, "relax_max_iter",&itmax,1);

  /* read all the parameters for the planewave code */
  read_pwparam(infile, &pwp); 
  qx = pwp.qshift[0];
  qy = pwp.qshift[1];
  qz = pwp.qshift[2];
  if(pwp.epsdiag>accuracy*accuracy) {
    FPRINTF(logfile_c,
	    "*** WARNING: Matrix diagonalization not accurate enough ***\n");
    FPRINTF(logfile_c,
	    "***          Decrease accuracy_diag to be less than %lg\n",
	    accuracy*accuracy);
  }

  if(pwp.pot_conv>accuracy) {
    FPRINTF(logfile_c,
	    "*** WARNING: Potential is not computed accurately enough ***\n");
    FPRINTF(logfile_c,
	    "***          Decrease potential_convergence_criterion to be less than %lg\n",
	    accuracy*accuracy);
  }

  /* Following line added by David Roundy to query file regarding
     fixing xy plane */
  askintvar(infile, "hold_xy_plane_fixed",&hold_xy_plane_fixed,1);
  askintvar(infile, "hold_xhat_fixed",&hold_xhat_fixed,1);

  fclose(infile);
  /* --------- allocate the memory needed ------- */

  force = malloc(3*mxdatm*ntyp*sizeof(double));

  plen = packlen(ntyp, natom);

  plen_squared = plen*plen;
  x = malloc(plen*sizeof(double));
  xold = malloc(plen*sizeof(double));
  metric = malloc(plen*plen*sizeof(double));

  g = malloc(plen*sizeof(double));
  gold = malloc(plen*sizeof(double));
  delta = malloc(plen*sizeof(double));

  pv = malloc(plen*sizeof(double));
  q = malloc(plen*sizeof(double));
  s = malloc(plen*sizeof(double));
  u = malloc(plen*sizeof(double));

  H = malloc(plen*plen*sizeof(double));
  H0 = malloc(plen*plen*sizeof(double));
  temp1 = malloc(plen*plen*sizeof(double));
  temp2 = malloc(plen*plen*sizeof(double));
  eigenvalues = malloc(plen*sizeof(double));
  
  FPRINTF(logfile_c,"--------JOB: %s ---------\n", jobstring);

  
  if(strcmp(jobstring,"relax_from_checkpoint")==0) {
    pack(ntyp,mxdatm, natom, x, coord, eps); 
    readcheckpoint(x, g, H, H0, &f1);  /* get stuff from checkpoint file */
    pwp.ilog =0;     
  }
  else if(strcmp(jobstring,"band_structure")==0) {
    printf(" *** job band_structure is not allowed any more\n");
    printf(" *** use the pw_job variable instead.\n");
    exit(-1);
  }
  else if(strcmp(jobstring,"relax")==0){
    /* pack initial values in x */
    
    pack(ntyp,mxdatm, natom, x, coord, eps); 
    
    initialize_hessian(x,H,relaxepsfac,relaxcoordfac,-1,decoup_rc);
    memcpy(H0, H, plen*plen*sizeof(double));
    
    pwp.ilog =0;     
    
    /* calculate stress and forces */
    f1 = funeval(ntyp,mxdatm, natom, x, coord,
		 atm,szet,mts,dipole, &p,&etot, force, g, qx, qy, qz);
    pwcount++;
  }
  else if(strcmp(jobstring,"relax_recycle_H")==0) {
    pack(ntyp,mxdatm, natom, x, coord, eps); 
    readcheckpoint(x, g, H, H0, &f1);  /* get stuff from checkpoint file */
    /* but ignore position and gradient */     
    pack(ntyp,mxdatm, natom, x, coord, eps); 
    pwp.ilog =0;     
    f1 = funeval(ntyp,mxdatm, natom, x, coord,
		 atm,szet,mts, dipole,&p,&etot, force, g, qx, qy, qz);
    pwcount++;
    
  }
  else {
    FPRINTF(logfile_c, "No valid job variable set: %s\n", jobstring);
    exit(-1);
  }

  /* memory_check("memory after pw call"); */

  matvec(plen, H, g,delta);

  MDCOPY(plen, g, 1, gold, 1);          /* set gold = g */
  MDCOPY(plen, x, 1, xold, 1);          /* set xold = x */
  lambda =1.0;

  computemetric(xold,xold,metric,3);
  rho=sqrt(dotprod(plen,gold,gold,metric));

  FPRINTF(logfile_c,"rho = %lf\n", rho);
  FPRINTF(convergefile,"iteration %d  rho = %lf  enthalpy = %lf\n", 
	  0, rho, f1);
  if(rho<=accuracy) {FPRINTF(logfile_c, "already done at iteration 0!\n"); 
#ifdef MPI
  MPI_Finalize();
#endif
		     exit(-1);}


  /*============================ begin main loop =================== */
  for(iter=1;iter<itmax;iter++) {
    pwp.ilog =iter;     
    FPRINTF(logfile_c, "\n\n--------- iteration %d -----------:\n", iter);

    MDCOPY(plen, xold,1,x,1); 

    /* FPRINTF(logfile_c,"Used following H matrix:\n");
       for(i=0;i<plen;i++) {
       for(j=0;j<plen;j++) FPRINTF(logfile_c,"%f ", H[i*plen+j]);
       FPRINTF(logfile_c,"\n"); } */
    
    update(x, delta, -1.0);              /* x = x - delta */

    f0 = f1;
    computemetric(xold,xold,metric,1);    
    g0 = -dotprod(plen, delta, g,metric);
    computemetric(xold,xold,metric,2);    
    ldelta=sqrt(dotprod(plen,delta,delta,metric));

    f1 = funeval(ntyp,mxdatm, natom, x, coord,
		 atm,szet, mts, dipole,&p,
		 &etot2, force, g,  qx, qy, qz);
    pwcount++;

    computemetric(xold,x,metric,1);
    g1= -dotprod(plen, delta, g,metric);

    computemetric(x,x,metric,3);
    rho=lg=sqrt(dotprod(plen,g,g,metric));

    if(rho<=accuracy) {
      goto cleanup;}

    lambdalin = -g0/(g1-g0);
    lambda = lambdalin;
    updateh = iupdateh;
    if(lambda<0) {
      FPRINTF(logfile_c, "!!!! set lambda to 1 and initialized H!!!!!\n"); 
      initialize_hessian(x,H,relaxepsfac,relaxcoordfac,-1, decoup_rc);
      memcpy(H0, H, plen*plen*sizeof(double));
      updateh=0;        /* dont update H */
      lambda =1.0;}
    if(lambda>lambdalimit){
      FPRINTF(logfile_c, "!!!! reduced lambda from %lf to %lf !!!!!\n", 
	      lambda, lambdalimit); 
      lambda=lambdalimit;}

    FPRINTF(logfile_c,"f0=%lf, g0=%lg, f1=%lf, g1=%lg\n",f0,g0,f1,g1);
    FPRINTF(logfile_c,"lambda from linear=%lf\n", lambda);
    
    FPRINTF(logfile_c,
	    "trialstep: normalized product between vectors (g1,delta): %lf\n",
	    -g1/ldelta/lg);

    
    

    if(fabs(lambda-1.0)>lambda_tolerance) {/* take shorter step if necessary */

      MDCOPY(plen, xold,1,x,1);
      update(x, delta, -lambda); 

      FPRINTF(logfile_c,"####### move to new x\n");
    
      f1 = funeval(ntyp,mxdatm, natom, x, coord,
		   atm,szet, mts, dipole, &p,
		   &etot2, force, g,  qx, qy, qz);

      computemetric(xold,x,metric,1);  gnew= -dotprod(plen, delta, g,metric);
      computemetric(x,x,metric,3);   rho=lgnew=sqrt(dotprod(plen,g,g,metric));
      if(rho<=accuracy) {
	FPRINTF(logfile_c,"converged at new x!\n");
	goto cleanup;}

      FPRINTF(logfile_c,
	      "linear: normalized product between vectors (gnew,delta): %lf\n",
	     -gnew/ldelta/lgnew);
      pwcount++;

      cvar = g0;
      avar = (g1-g0)*(g1-g0)*gnew/(g0*g1);
      bvar = g1-avar-g0;
      FPRINTF(logfile_c,"a=%lf, b=%lf, c=%lf\n", avar, bvar, cvar);

      discrim = bvar*bvar-4.0*avar*cvar;
      if(discrim<0.0) 
	{FPRINTF(logfile_c,"Error: complex roots. Taking linear lambda.\n"); 
	 lambda=lambdalin;}
      else  {
	if(fabs(avar)<fabs(cvar*1e-6)) {
	  FPRINTF(logfile_c,"quadratic term very small, doing linear fit.\n"); 
	  lambda = -cvar/bvar;
	} else {
	  L_p1 = (-bvar + sqrt(discrim))/(2*avar);
	  L_p2 = (-bvar - sqrt(discrim))/(2*avar);
	  lambda = (fabs(L_p1-lambdalin)<fabs(L_p2-lambdalin))?L_p1:L_p2;
	}
      }
      FPRINTF(logfile_c,"lambda from parabola=%lf\n",lambda);

      if(fabs(lambda-lambdalin)>lambda2_tolerance*lambda) {
	FPRINTF(logfile_c, "################# moving to parabolic x\n");
	MDCOPY(plen, xold,1,x,1);
	update(x, delta, -lambda); 

	  
	f1 = funeval(ntyp,mxdatm, natom, x, coord,
		     atm,szet, mts, dipole, &p,
		     &etot2, force, g,  qx, qy, qz);
	pwcount++;
	computemetric(xold,x,metric,1);  
	gnew= -dotprod(plen, delta, g,metric);
	computemetric(x,x,metric,3);   
	rho=lgnew=sqrt(dotprod(plen,g,g,metric));

	FPRINTF(logfile_c,
	   "parabolic: normalized product between vectors (gnew,delta): %lf\n",
		-gnew/ldelta/lgnew);
	if(rho<=accuracy) {
	  FPRINTF(logfile_c,"converged at new parabolic x!\n");
	  goto cleanup;}

      }
      else {lambda=lambdalin; 
	    FPRINTF(logfile_c,"Did not move to lambda quadratic!\n");
	  }
    }
    else {
      FPRINTF(logfile_c, "Trial step is close enough to estimated minimum!\n"); 
      FPRINTF(logfile_c,  "Normalized dot product between (g1,delta): %lf\n",
	      -g1/ldelta/lg); 
    }

cleanup:
    FPRINTF(convergefile,"iteration %d  rho = %lf  enthalpy = %lf\n", 
	    iter, rho, f1);

    FPRINTF(logfile_c,"rho of new minimum position = %lf\n", rho);
    FPRINTF(logfile_c,"So far %d call(s) to the pw program!\n",pwcount);

    
    /* ------------ update H ---------------------*/

    if(updateh==1) {
      for(i=0;i<plen;i++) pv[i]=0.0;
      MDAXPY(plen, -lambda,delta,1,pv,1); /* pv = -lambda*delta */

      MDCOPY(plen, g,1,q,1);             /* q= g-gold */
      MDAXPY(plen, -1.0,gold,1,q,1); 

      matvec(plen, H, q,s);           /* s = H*q     */
    
      MDDOT(adot,plen,pv,1,q,1);       /* adot = dot(pv,q) */
      MDDOT(bdot,plen,q,1,s,1);

      for(i=0;i<plen;i++) u[i]=0.0;

      MDAXPY(plen, -1.0/bdot,s,1,u,1); 
      MDAXPY(plen, 1.0/adot,pv,1,u,1); 


      for(i=0;i<plen;i++)
	for(j=0;j<plen;j++) {
	  H[i*plen+j]+= pv[i]*pv[j]/adot -s[i]*s[j]/bdot + bdot*u[i]*u[j];
	  temp1[i*plen+j]= H[i*plen+j];
	}
    }

    /* for(i=0;i<plen;i++) {
       for(j=0;j<plen;j++) FPRINTF(logfile_c,"%f ", H[i*plen+j]);
       FPRINTF(logfile_c,"\n"); } */
    
    checkpoint(x, g,H, H0, f1);
    if(rho<=accuracy) break;

    for(i=0;i<plen;i++)
      for(j=0;j<plen;j++) temp1[i*plen+j]= H0[i*plen+j];

    for(i=0;i<plen;i++)
      for(j=0;j<plen;j++) temp1[i*plen+j]= H[i*plen+j];

    /* ------------ apply H ---------------------*/

    matvec(plen, H, g,delta);

    FPRINTF(logfile_c,"negative of suggested step direction:\n");
    printpacked(plen,delta);


    /* ----------- save old variables ---------------- */
    MDCOPY(plen, g,1,gold,1);         
    MDCOPY(plen, x,1,xold,1);         

    if(relax_myproc==0) {
      fflush(stdout);
      fflush(logfile_c);
      fflush(convergefile);
    }

  }


  apply_eps_lat(x,lvec,lvecf);

  FPRINTF(logfile_c,"final lattice vectors:\n");
  for(i=0;i<3;i++)
    FPRINTF(logfile_c,"%lf %lf %lf\n", lvecf[i][0],lvecf[i][1],lvecf[i][2]);

  FPRINTF(logfile_c,"H matrix:\n");

  for(i=0;i<plen;i++) {
    for(j=0;j<plen;j++) FPRINTF(logfile_c,"%f ", H[i*plen+j]);
    FPRINTF(logfile_c,"\n"); }

  FPRINTF(logfile_c,"It took %d call(s) to the pw program!\n",pwcount);


  /* exploit the H matrix */
  

  free(x); free(force); free(xold); free(g); free(gold); free(H);
  free(delta); free(H0); free(mass);
  close(logfile_c);
  close(convergefile);

#ifdef MPI
  MPI_Finalize();
#endif

}






void pack(int ntyp, int mxdatm, int *natom, 
	  double *statevec, double *coord, double *eps)
{
  int i,j,k,l,m;
  double rc[3];

  /* write epsilon into right position */

  l=0;
  for(i=0;i<9;i++) statevec[l++]=eps[i];

  for(i=0;i<ntyp;i++) {
    for(j=0;j<natom[i];j++) {
      for(k=0;k<3;k++) {
	statevec[l++]=coord[3*i*mxdatm+3*j+k];
      }
    }
  }

}


void unpack(int ntyp, int mxdatm, int *natom, 
	    double *statevec, double *coord, double *eps)
{
  int i,j,k,l,m;
  double rc[3];
 

  l=0;
  for(i=0;i<9;i++) eps[i]=statevec[l++];

  for(i=0;i<ntyp;i++) {
    for(j=0;j<natom[i];j++) {
      rc[0]=statevec[l++];
      rc[1]=statevec[l++];
      rc[2]=statevec[l++];
      for(k=0;k<3;k++) {
	coord[3*i*mxdatm+3*j+k] = rc[k];
      }
    }
  }
}

int packlen(int ntyp, int *natom)
{
  int i,j,k;

  k=9;
  
  for(i=0;i<ntyp;i++)
    for(j=0;j<natom[i];j++) k +=3;

  return(k);
}

void printpacked(int xl, double *statevec)
{
  int i,j,k;
  double rc[3],a[3][3];

  FPRINTF(logfile_c,"strain part:\n");
  FPRINTF(logfile_c,
	"%10.4g %10.4g %10.4g\n %10.4g %10.4g %10.4g\n%10.4g %10.4g %10.4g\n",
	 statevec[0],statevec[1],statevec[2],
	  statevec[3], statevec[4],statevec[5],
	  statevec[6], statevec[7],statevec[8]);
  if(xl>9) {
    FPRINTF(logfile_c,"coordinate components:\n");
    for(i=9;i<xl;i+=3)
      FPRINTF(logfile_c,"%10.4g %10.4g %10.4g\n", statevec[i],statevec[i+1],
	      statevec[i+2]);
    FPRINTF(logfile_c,"in cartesian coordinates:\n");
    apply_eps_lat(statevec,lvec,a);
    for(i=9;i<xl;i+=3) {
      for(j=0;j<3;j++){
	rc[j]=0;
	for(k=0;k<3;k++) {
	  rc[j]+=a[j][k]*statevec[i+k];
	}
      }
      FPRINTF(logfile_c,"%10.4g %10.4g %10.4g\n", rc[0],rc[1],rc[2]);
    }
  }


}

void printunpacked(int ntyp, int mxdatm, int *natom,
		   double *coord, double *eps)
{

  int i,j;
  FPRINTF(logfile_c,"strain part: ");
  FPRINTF(logfile_c,
	"%10.4g %10.4g %10.4g\n %10.4g %10.4g %10.4g\n%10.4g %10.4g %10.4g\n",
	 eps[0],eps[1],eps[2],
	  eps[3], eps[4],eps[5],
	  eps[6], eps[7],eps[8]);

  for(i=0;i<ntyp;i++)
    for(j=0;j<natom[i];j++)
      FPRINTF(logfile_c,"%9.5f %9.5f %9.5f\n", 
	     coord[3*i*mxdatm + 3*j],coord[3*i*mxdatm + 3*j+1],
	     coord[3*i*mxdatm + 3*j+2]);
}

void apply_eps_lat(double *eps, double a[3][3], double c[3][3])
{
  /* 
   *       epsilon is assumed to have the FORTRAN convention:
   *
   *       e_11, e_12, e_13, e_21, e_22, e_23, e_31, e_32, e_33
   *
   *
   *       This was different before. Changed 11/16/96. BP
   */
  int i,j;
  double hilf[3][3];
  for(i=0;i<3;i++) {for(j=0;j<3;j++) {hilf[i][j]=eps[i*3+j]; 
				       if(i==j) hilf[i][j]+=1.0;}}

  matmul(c,hilf,a);
}

double dotprod(int xl,double *a, double *b, double *met)
{
  int i,j;
  double sum=0;

  for(i=0;i<xl;i++) 
    for(j=0;j<xl;j++) sum+=a[i]*met[i*xl+j]*b[j];
  return(sum);
}

void matvec(int xl, double *H, double *b, double *c)
{
  int i,j;

  for(i=0;i<xl;i++)
    { c[i]=0;
      for(j=0;j<xl;j++) {
	c[i]+=H[i*xl+j]*b[j];
      }
    }
}

void sym2full(double symmat[6],double fullmat[3][3])
{
  fullmat[0][0]=symmat[0];
  fullmat[1][1]=symmat[1];
  fullmat[2][2]=symmat[2];
  fullmat[0][1]=fullmat[1][0]=symmat[3];
  fullmat[1][2]=fullmat[2][1]=symmat[4];
  fullmat[0][2]=fullmat[2][0]=symmat[5];
}
void full2sym(double symmat[6],double fullmat[3][3])
{
  symmat[0]=fullmat[0][0];
  symmat[1]=fullmat[1][1];
  symmat[2]=fullmat[2][2];
  symmat[3]=0.5*(fullmat[0][1]+fullmat[1][0]);
  symmat[4]=0.5*(fullmat[1][2]+fullmat[2][1]);
  symmat[5]=0.5*(fullmat[0][2]+fullmat[2][0]);
}

void matmul(double c[3][3], double a[3][3], double b[3][3])
{
  int i,j,k;
  for(i=0;i<3;i++)
    for(j=0;j<3;j++) {
      c[i][j]=0.0;
      for(k=0;k<3;k++) c[i][j] +=a[i][k]*b[k][j];
    }
}
void mattrans(double a[3][3], double b[][3])
{
  int i,j,k;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      b[i][j]=a[j][i];
}

void pstress(double eps[3][3], double p, double pst[3][3])
{
  double v0; /* inverse of scaled volume */
  int i,j;

  for(i=0;i<3;i++) {for(j=0;j<3;j++) {pst[i][j]=eps[j][i]; 
				       if(i==j) pst[i][j]+=1.0;}}

  /* convert pressure from GPa to Ry/a.u.^3 */

  p = p/14710.75;

  /* calculate 1/V * p * dV/d(eps) */

  v0 =  matinvert((double *) pst);
  v0 *= vinit;

  for(i=0;i<3;i++) {for(j=0;j<3;j++) {pst[i][j]*=p*v0;}}

}

#ifdef RS6K
void memory_check(char *mess)
{
  struct mallinfo minfo;

  minfo = mallinfo();
  printf("%s\n", mess);
  
  printf("arena    = %d\n",minfo.arena);
  printf("#ordblks = %d\n",minfo.ordblks);
  printf("#smblks  = %d\n",minfo.smblks);
  printf("#hblks   = %d\n",minfo.hblks);
  printf("hblkhd   = %d\n",minfo.hblkhd);
  printf("usmblks  = %d\n",minfo.usmblks);
  printf("fsmblks  = %d\n",minfo.fsmblks);
  printf("uordblks = %d\n",minfo.uordblks);
  printf("fordblks = %d\n",minfo.fordblks);
}
#endif

void checkpoint(double *x, double *g, double *H, double *H0, double f1)
{
  int i;
  FILE *ckpfile;
  
  if(relax_myproc!=0) return;

  ckpfile = fopen(CHECKPOINTFILENAME,"w");
  if(ckpfile==NULL) {printf("Cannot open checkpoint file!\n"); exit(-1);}

  FPRINTF(ckpfile,"x=\n");
  for(i=0;i<plen;i++)
    FPRINTF(ckpfile,"%30.20lf \n", *(x++));
  FPRINTF(ckpfile,"g=\n");
  for(i=0;i<plen;i++)
    FPRINTF(ckpfile,"%30.20lf \n", *(g++));
  FPRINTF(ckpfile,"H=\n");
  for(i=0;i<plen*plen;i++)
    FPRINTF(ckpfile,"%30.20lf \n", *(H++));
  FPRINTF(ckpfile,"H0=\n");
  for(i=0;i<plen*plen;i++)
    FPRINTF(ckpfile,"%30.20lf \n", *(H0++));
  FPRINTF(ckpfile,"enthalpy =%30.20lf\n", f1);
  fclose(ckpfile);
}

void readcheckpoint(double *x, double *g, double *H,double *H0, double *f1)
{
  int i;
  FILE *ckpfile;
  double *xloc=x, *gloc=g, *Hloc=H, *H0loc=H0, dumdum;

  FPRINTF(logfile_c," READING FROM CHECKPOINT FILE:\n");
  if((ckpfile = fopen(CHECKPOINTFILENAME,"r"))==NULL) {
    printf("CANNOT OPEN CHECKPOINT FILE -- STOP!\n");
    exit(-1);
  }
  fscanf(ckpfile,"x=\n");
  for(i=0;i<plen;i++)
    fscanf(ckpfile,"%lf \n", xloc++);
  fscanf(ckpfile,"g=\n");
  for(i=0;i<plen;i++)
    fscanf(ckpfile,"%lf \n", gloc++);
  fscanf(ckpfile,"H=\n");
  for(i=0;i<plen*plen;i++)
    fscanf(ckpfile,"%lf \n", Hloc++);
  fscanf(ckpfile,"H0=\n");
  for(i=0;i<plen*plen;i++)
    fscanf(ckpfile,"%lf \n", H0loc++);
  fscanf(ckpfile,"enthalpy =%lf\n", f1);

  fclose(ckpfile);

  FPRINTF(logfile_c,"checkpoint state:\n");
  printpacked(plen, x);
  MDNRM2(dumdum,plen,g,1);
  FPRINTF(logfile_c,"checkpoint gradient has length %f:\n", dumdum);
  printpacked(plen, g);
  FPRINTF(logfile_c,"checkpoint enthalpy = %f\n",*f1 );

  if(relax_myproc==0)   fflush(logfile_c);
}

void update(double *x,double *delta, double lambda)
{
  MDAXPY(plen, lambda, delta,1, x, 1);   /* simple move */
}

void computemetric(double *x, double *xp, double *met, int imode)
{
  int i,j,k;
  double eps[3][3],epsp[3][3], a[3][3], ap[3][3], mmat[3][3], dum[3][3];

  for(i=0;i<3;i++)for(j=0;j<3;j++) {eps[i][j]=x[j+3*i]; epsp[i][j]=xp[j+3*i];}

  apply_eps_lat((double *)eps, lvec,a);
  apply_eps_lat((double *)epsp, lvec,ap);

  /* mode 1: compute metric between covariant*contravariant =   
             (at)*(a't)^-1                                      
     mode 2: compute metric between contravariant*contravariant =   
             (at)*(a')                                      
     mode 2: compute metric between covariant*covariant =   
             ((at)*(a'))^-1                                      

	     */

  switch(imode) {
  case 1:
    matinvert((double *)ap);
    matmul(dum,ap,a);
    mattrans(dum,mmat);
    break;
  case 2:
    mattrans(a,dum);
    matmul(mmat,dum,ap);
    break;
  case 3:
    mattrans(a,dum);
    matmul(mmat,dum,ap);
    matinvert((double *)mmat);
    break;
  default:      
    FPRINTF(logfile_c,"metric: mode %d is not allowed\n",imode);
    exit(-1);
    break;
  }
  for(i=0;i<plen;i++)
    for(j=0;j<plen;j++) met[i*plen+j]=0.0;

  for(i=0;i<9;i++)
    met[i*plen+i]= 1.0;

  for(i=9;i<plen;i+=3) {
    for(j=0;j<3;j++)
      for(k=0;k<3;k++)
	met[(i+j)*plen+i+k]=mmat[j][k];
  }
  /*
  FPRINTF(logfile_c,"Metric matrix for mode %d\n",imode);

    for(i=0;i<plen;i++) {
      for(j=0;j<plen;j++) FPRINTF(logfile_c,"%5.2f ", met[i*plen+j]);
      FPRINTF(logfile_c,"\n"); }
      */
}

void initialize_hessian(double *x,double *H, double relaxepsfac,
		     double relaxcoordfac, int mode, int decoup_rc)
{
  /* 
   *     NOTE: the Hessian is in C- convention 
   */
  double a[3][3],atam1[3][3],at[3][3],ainv[3][3],rc[3];
  int i,j,k, natoms, ic, iat, l;

  apply_eps_lat(x, lvec,a);
  memcpy(ainv,a,9*sizeof(double));
  matinvert((double *)ainv);

  mattrans(a,at);

  matmul(atam1,at,a);

  if(mode==-1)
    matinvert((double *)atam1);
  if(mode==0)
    memcpy(atam1,a,9*sizeof(double));


  for(i=0;i<plen;i++)
    for(j=0;j<plen;j++) H[i*plen+j]=0.0;
  for(i=0;i<9;i++) H[i*plen+i]=relaxepsfac;
  
  for(i=9;i<plen;i+=3) {
    for(j=0;j<3;j++)
      for(k=0;k<3;k++)
	H[(i+j)*plen+i+k]=relaxcoordfac*atam1[j][k];
  }

  /* initialize the off-diagonal elements if decoupling is desired 
   * 
   *    notice that  epsilon_ij =>    epsilon[3*j+i]
   *   (fortran convention)
   */

  

  if(decoup_rc==1) {
    natoms=(plen-9)/3;
    for(iat=0;iat<natoms;iat++) {  /* loop over atoms */
      for(l=0;l<3;l++) { /* compute cartesian coordinate */
	rc[l]=0;
	for(k=0;k<3;k++)
	  rc[l]+=a[l][k]*x[9+iat*3+k];}
      for(ic=0;ic<3;ic++) { /* loop over the vector components */
	for(k=0;k<3;k++)
	  for(l=0;l<3;l++) {
	    H[9+iat*3+ic+plen*(3*k+l)]=-relaxepsfac*ainv[ic][k]*rc[l];
	    H[(9+iat*3+ic)*plen + 3*k+l]=-relaxepsfac*ainv[ic][k]*rc[l];
	  }
      }
    }

  }
  /* FPRINTF(logfile_c,"Initialized H matrix:\n");
     for(i=0;i<plen;i++) {
     for(j=0;j<plen;j++) FPRINTF(logfile_c,"%f ", H[i*plen+j]);
     FPRINTF(logfile_c,"\n"); } */
  

}

/* 

void stress_test(double *e,double dede[3][3],double a[3][3])
{
  double alf[3],b[3][3],at[3][3],hilf[3][3];
  int i,j,k;

  alf[0]=1;
  alf[1]=2;
  alf[2]=3;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++) {b[i][j]=0;}
    
  b[0][0]=b[1][1]=b[2][2]=0.0;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++) {
      dede[i][j]=0;
      for(k=0;k<3;k++)
	dede[i][j]+=0.5*alf[k]*(lvec[j][k]*a[i][k]+lvec[j][k]*a[i][k])
	  -b[i][k]*lvec[j][k]; 
    }
 
  *e=0;
  mattrans(a,at);

  for(k=0;k<3;k++) {
    matmul(hilf,at,a);
    for(i=0;i<3;i++)
      *e=*e+0.5*alf[k]*hilf[i][i] - b[i][k]*a[i][k];
  }

}

void stress_test2(double *e,double eps[3][3],double dede[3][3],double a[3][3])
{
  double alf[3],b[3][3],at[3][3],hilf[3][3];
  double a11=1.0, a22=2.0, a33=3.0, a12=4.0, a21=5.0;
  double b11=0.0,b22=0.0,b33=0.2,b12=0.00, b21=0.00;
  int i,j,k;

  *e = a11*(eps[0][0]-b11)*(eps[0][0]-b11)+
       a22*(eps[1][1]-b22)*(eps[1][1]-b22)+
       a33*(eps[2][2]-b33)*(eps[2][2]-b22)+
       a12*(eps[0][1]-b12)*(eps[0][1]-b12)+
       a21*(eps[1][0]-b21)*(eps[1][0]-b21);
  
  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      dede[i][j]=0;
  
  dede[0][0]=2*a11*(eps[0][0]-b11); dede[1][1]=2*a22*(eps[1][1]-b22); 
  dede[2][2]=2*a33*(eps[2][2]-b33);
  dede[0][1]=2*a12*(eps[0][1]-b12);
  dede[1][0]=2*a21*(eps[1][0]-b21);
}

*/

double funeval(int ntyp, int mxdatm, int *natom, double *x, double *coord,
	       char *atm,double *szet, double *mts,  double *dipole, 
	       double *p,
	       double *etot, double *force, double *g, double qx, double qy,
               double qz)
{
  double a[3][3],at[3][3];
  double stress[6], fsig[3][3], feps[3][3], eps[3][3], fepsu[3][3], pst[3][3];
  double v0,dumdum;

  int i,j,k,joblist,njobs;

  /* ------------calculate stress and forces --------------------*/

  FPRINTF(logfile_c,"Current state:\n");
  printpacked(plen, x);

  unpack(ntyp,mxdatm, natom, x, coord,(double *)eps);
  /* the xy plane constant--David Roundy */
  if (hold_xy_plane_fixed) {
    eps[0][2] += eps[2][0];
    eps[1][2] += eps[2][1];
    eps[2][0] = 0.;
    eps[2][1] = 0.;
  }
  if (hold_xhat_fixed) {
    eps[0][1] += eps[1][0];
    eps[1][0] = 0.;
  }

  apply_eps_lat((double *)eps, lvec,at);
  mattrans(at,a); 

  if(relax_myproc==0) {fflush(logfile_c);  fflush(convergefile);}

  RSYSTEM("\\mv OUT._0 OUT.backup");

  if(pwp.correct_stress_cutoff>pwp.ecutoff + 1.0e-6) 
    (pwp.imonitor=pwp.imonitor|512); 

  if((pwp.ioptimize&16)==0) iwfnreuse_c=0; /* no wfn reuse is desired */
  if(relaxstress>0.0) iwfnreuse_c=0; /* no wfn reuse possible here */    

  joblist=0, njobs=1;

  C2FORTRANIZE(pwmain,PWMAIN)
      (&pwp.njobs,&(pwp.joblist[0]),
	  &ntyp, &mxdatm, natom,
          &pwp.nrk, &pwp.nband,&(pwp.ngrid[0]), 
	  &pwp.ispn, &pwp.icorr, &pwp.input, &pwp.imaxitdiag, 
	  &pwp.imaxitscf, &iwfnreuse_c, &pwp.checkpoint_wfn,
	  &pwp.iscr, &pwp.ioccup, &pwp.ntrans, pwp.visualize,
	  &pwp.nlineplot, pwp.lineplot, pwp.bra, pwp.ket,
          &pwp.nplotwv, pwp.plotwv,&pwp.imonitor, &pwp.ioptimize, &pwp.miscflag,
	  &pwp.dsmear, pwp.kshift, &pwp.epsdiag, &pwp.bandgap, &pwp.esafety, 
	  &pwp.randomstart,
	  &pwp.ecutoff, &pwp.ecutoffsub, &pwp.ecutoffmix, &pwp.pot_conv,
          &qx,&qy,&qz,
          &pwp.ilog, pwp.mixing,&pwp.h0, pwp.gmagnet, &pwp.eampli, pwp.gepot, &pwp.nenergywindow,
	  pwp.energywindow,
	  pwp.modify_kinetic_energy, &pwp.chi_q, pwp.g0mask,&pwp.nrg,
          &relax_total_mass, coord, szet, mts, dipole, (double *)&(a[0][0]), &pwp.fermi_level,
          etot, stress, force, atm, pwp.labels);
  iwfnreuse_c=2;
  if((pwp.ioptimize&16)==0) iwfnreuse_c=0; /* ... if the user wants it. */
  if(relaxstress>0.0) iwfnreuse_c=0; /* no wfn reuse possible here */    

  if(pwp.correct_stress_cutoff>pwp.ecutoff + 1.0e-6) {
    RSYSTEM("\\mv OUT._0 OUT.lowcut.backup");
    i=4; j=1; pwp.input=pwp.input|1;
    C2FORTRANIZE(pwmain,PWMAIN)
	(&pwp.njobs, pwp.joblist,
	    &ntyp, &mxdatm, natom,
	    &pwp.nrk, &pwp.nband, pwp.ngrid, 
	    &pwp.ispn, &pwp.icorr, &pwp.input, &pwp.imaxitdiag, 
	    &j,  &iwfnreuse_c, &pwp.checkpoint_wfn,
	    &i, &pwp.ioccup, &pwp.ntrans, pwp.visualize,
	    &pwp.nlineplot, pwp.lineplot,  pwp.bra, pwp.ket,
	    &pwp.nplotwv, pwp.plotwv,&pwp.imonitor,  &pwp.ioptimize, &pwp.miscflag,
	    &pwp.dsmear, pwp.kshift, &pwp.epsdiag,  &pwp.bandgap, &pwp.esafety,
	    &pwp.randomstart,
	    &pwp.correct_stress_cutoff, 
	    &pwp.ecutoffsub, &pwp.ecutoffmix, &pwp.pot_conv,
            &qx,&qy,&qz,
	    &pwp.ilog, pwp.mixing,&pwp.h0, pwp.gmagnet, &pwp.eampli, pwp.gepot,  &pwp.nenergywindow,
	    pwp.energywindow,
	    pwp.modify_kinetic_energy, &pwp.chi_q, pwp.g0mask, &pwp.nrg,
	    &relax_total_mass,coord, szet, mts, dipole, (double *)&(a[0][0]), &pwp.fermi_level,
	    etot, stress, force, atm, pwp.labels);
  }
  iwfnreuse_c=2;

  sym2full(stress,fsig);

  for(i=0;i<3;i++) {for(j=0;j<3;j++) {feps[i][j]=eps[j][i]; 
				       if(i==j) feps[i][j]+=1.0;}}
  v0 = vinit*matinvert((double *)feps);
  
  FPRINTF(logfile_c,"Enthalpy(%f)=%16.9f + %16.9f=%16.9f\n",
	  v0, *etot, *p/14710.75*v0, *etot+*p/14710.75*v0);

  matmul(fepsu,fsig,feps);
    
  if(adjustpressure==1) {
    *p=(stress[0]+stress[1]+stress[2])/3.0*14710.75/v0;
    adjustpressure=0;
  }

  MDSCAL(9, -1.0, (double *)fepsu, 1); 


  pstress(eps,*p,pst);
  FPRINTF(logfile_c,"pressure stress for p= %f GPa [Ry/a.u**3]:\n",*p);
  printpacked(9,(double *)pst);

  FPRINTF(logfile_c,"corrected LDA stress * Volume [Ry]:\n");
  printpacked(9,(double *)fepsu);

  MDAXPY(9, 1.0, (double *)pst, 1, (double *)fepsu, 1);   

  /* uncomment this to do tests for relaxation scheme 

  stress_test2(etot,eps, fepsu,a);   
  FPRINTF(logfile_c,"test energy:%lf\n", *etot); 
  for(i=0;i<plen-9;i++) force[i]=0;   */


  /* pack it into gradient vector */
  pack(ntyp,mxdatm, natom,g, force,(double *)fepsu); 


  for(i=0;i<9;i++) g[i]=relaxstress*g[i];            /* correct stress */
  for(i=9;i<plen;i++) g[i]=-relaxforce*g[i];         /* correct forces */

  /* now zero out forces on individual atoms */

  for(i=0;i<pwp.nstay_put;i++) {
    if((pwp.stay_put[i]>0)&&(pwp.stay_put[i]<=(plen-9)/3)) {
      g[9+(pwp.stay_put[i]-1)*3]=0.0;
      g[9+(pwp.stay_put[i]-1)*3+1]=0.0;
      g[9+(pwp.stay_put[i]-1)*3+2]=0.0;
    } else if ((pwp.stay_put[i]<0)&&(pwp.stay_put[i]>=-6)) {  /* Hold a strain fixed */
      switch (-pwp.stay_put[i]) {
      case 1:
        g[0]=0.0;
        break;
      case 2:
        g[4]=0.0;
        break;
      case 3:
        g[8]=0.0;
        break;
      case 4:
        g[1]=0.0;
        g[3]=0.0;
        break;
      case 5:
        g[5]=0.0;
        g[7]=0.0;
        break;
      case 6:
        g[2]=0.0;
        g[6]=0.0;
        break;
      }
    } else{
        FPRINTF(logfile_c,"Stay_put request ignored for atom %d\n",
                pwp.stay_put[i]);
    }
  }


  MDNRM2(dumdum,plen,g,1);
  FPRINTF(logfile_c,"gradient has length %f:\n",  dumdum);
  printpacked(plen, g);
  fflush(stdout);
  fflush(logfile_c);
  fflush(convergefile);

  return(*etot+(*p)/14710.75*v0);

}


