#include "fortranize.h"


#include <math.h>
#include <stdlib.h>


#define INPUTFILENAME "input"
#define CONVERGEFILENAME "STRUC_CONVERGE"
#define LOGFILENAME "STRUC_LOG"
#define CHECKPOINTFILENAME "CHECKPOINT"
#define FPRINTF if(relax_myproc==0) fprintf
#define RSYSTEM if(relax_myproc==0) system

#ifdef MPI
#include <mpi.h>
#endif


/* pack data into statevec */

void pack(int ntyp, int mxdatm, int *natom, 
	  double *statevec, double *coord, double *eps);


/* unpack data from statevec */

void unpack(int ntyp, int mxdatm, int *natom, 
	    double *statevec, double *coord, double *eps);


/* determine number of double variables contained in packed state vector */
int packlen(int ntyp, int *natom);


/* print out a packed state vector */
void printpacked(int xl, double *statevec);

/* print out unpacked state */
void printunpacked(int ntyp, int mxdatm, int *natom,
		   double *coord, double *eps);

/* apply packed (1+epsilon) to matrix */
void apply_eps_lat(double *eps, double a[3][3], double c[3][3]);

double dotprod(int xl, double *a, double *b, double *met);

void matvec(int xl, double *H, double *b, double *c);

void sym2full(double symmat[6],double fullmat[3][3]);
void full2sym(double symmat[6],double fullmat[3][3]);

/* simple matrix-matrix multiply */
void matmul(double c[3][3], double a[3][3], double b[3][3]);

/* take transpose of matrix a into b */
void mattrans(double a[3][3], double b[3][3]);

/* compute stress from p*V term */
void pstress(double eps[3][3], double p, double pst[3][3]);

/* prints message and current memory usage for the rs6k */
#ifdef RS6K
void memory_check(char *mess);
#endif

/* dumps checkpoint to disk */
void checkpoint(double *x, double *g, double *H,double *H0, double f1);
void readcheckpoint(double *x, double *g, double *H,double *H0, double *f1);

void update(double *x, double *delta, double lambda);
void computemetric(double *x, double *xp, double *met, int imode);

void initialize_hessian(double *x,double *H,double relaxepsfac,
		     double relaxcoordfac, int mode, int decoup_rc);


/* compute gradient of enthalpy surface */
double funeval(int ntyp, int mxdatm, int *natom, double *x, 
	       double *coord, 
	       char *atm,double *szet, double *mts, double *dipole, double *p,
	       double *etot, double *force, double *g, double qx, double qy,
	       double qz);


