C*
c     @process extchk
      subroutine resolved_charge(energy_window,ffts, pw_params,
     $     cd_gs, gs, bands, kpoints, wavefn,
     $     den, energs)
c     
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'

c
c     INPUT:
c     -----

c     
      type (fftstruc) ffts
      type (parallel_gspace) 
     $     cd_gs,               ! the gspace for the charge density and pot
     $     gs(*)                ! gspaces where wavefn were computed

      type (pw_parameter) pw_params ! for the smearing parameter
      type (band) bands         ! for nrk, occupation numbers ...
      type (kpoint) kpoints     ! for the weights kpoints%w
      type (complex_gspace_array) wavefn
      type (energy) energs      ! needed for the vxc0 shift
      double precision 
     $     energy_window(2)     ! energy window as start=(1), end=(2) in [eV]
c
c     OUTPUT:
c     ------

      complex(kind=8)
     $     den(cd_gs%length,bands%nspin) ! charge density 
c
c     DESCRIPTION:
c     -----------
c
c     computes the energy-resolved charge density which is generated 
c     by wavefunctions with eigenvalues inside the energy window
c
c     the weights are computed as:
c
c                              eps_upper  
c     f(eps) = 1/sqrt(pi) *   /             exp(-((eps-e)/sigma)^2) de
c                            eps_lower
c
c     with the definition t=eps/sigma, this becomes:
c
c     f(t) = 1 - 0.5*erfc(t-t_lower) - 0.5* erfc(t_upper-t)
c
c

c
c     1996 Bernd Pfrommer, UCB


c     ---------------- local variables ----------------------------------
       
      double precision
     $     t0, gimmetime, myderfc,
     $     erfc1c, erfc2c, occp, desm, range,
     $     dsmear,              ! gaussian smearing parameter [Ryd]
     $     ryd,                 ! the rydberg constant in eV
     $     small, dmin, dmax, cmin, cmax, zero, sqrpi, pi,spinf,delta

      external gimmetime, myderfc
      complex(kind=8) 
     $     zzero,
     $     cdsum, cdsumtot , cdsumtotr

      parameter (small = 1.0d-12, zzero=(0.d0, 0.d0), zero=0.d0,
     $     pi    =  3.141592653589793d0,
     $     range =  4.d0,
     $     ryd   =  13.60580d0)
      integer
     $     is,                  ! spin counter
     $     ns,                  ! number of stars
     $     ntot,                ! total number of points in realspace block
     $     irk,                 ! kpoint counter
     $     neig, ndim, iel, 
     $     i,j,k,
     $     ierr, idum, ntottot

      complex(kind=8), allocatable::
     $     dhd(:),              ! realspace array 
     $     rden(:)              ! accu for rspace dens



c     -----------------------------------------------------------------------
c
c     transfer data from structures to local variables
c
      t0=gimmetime()

      ns     = cd_gs%nstar
      ntot   = ffts%r_size     ! total number of points in realspace block
      dsmear = pw_params%smearing/ryd
      sqrpi  = sqrt(pi)
c
c     get dimensions of realspace block
c
      ntottot = ntot
      call all_sum_all(ntottot)
c
c     allocate various charge densities
c

      allocate(rden(ntot))    
      allocate(dhd(ntot))    
      cdsumtot =zzero
      cdsumtotr =zzero


      write(9,100) energy_window(1), energy_window(2)

      
      if(bands%nspin.ge.2) then ; spinf = 0.5d0 ; else ;  spinf =1.d0
      endif



      do is=1,bands%nspin        ! loop over spins
         rden=zzero

         do irk=1, bands%nrk    ! ------------   loop over k points ---------

            neig = bands%nband(irk,is)
            ndim = gs(irk)%length

       
            do j=1,neig
c
c              compute occupation number
c
c              first treat lower boundary
c
               desm = (bands%energy(j,irk,is)+energs%evalshift(is) 
     $              -energy_window(1)/ryd)/dsmear
               if(desm.ge.range) then ! well above lower margin
                  erfc1c=0.d0
               elseif(desm.lt.-range) then ! well below lower margin
                  erfc1c=1.d0
               else
                  erfc1c=0.5d0*myderfc(desm)
               endif
c
c              now treat upper bound
c
               desm = (energy_window(2)/ryd-bands%energy(j,irk,is)
     $              -energs%evalshift(is))/dsmear
               if(desm.ge.range) then ! well above lower margin
                  erfc2c=0.d0
               elseif(desm.lt.-range) then ! well below lower margin
                  erfc2c=1.d0
               else
                  erfc2c=0.5d0*myderfc(desm)
               endif
               occp =1.d0 - erfc1c - erfc2c
               occp = occp*kpoints%w(irk)         
c
c              fourier transform to real space
c     
               call fourier_transform(-1, ffts,
     $              gs(irk),wavefn%data((j-1)*ndim+1,irk,is),dhd)
c     
c              square and add
c     
               do i=1, ntot
                  rden(i) = rden(i) + dhd(i)*conjg(dhd(i))*occp
               end do
            end do

         end do                 ! ------------ end of loop over k points ----

c
c        fourier transform back to momentum space
c
         cdsum =zzero
         do i=1, ntot
            cdsum = cdsum + rden(i)
         end do
      
         call all_sum_all(cdsum)
         cdsum = cdsum/ntottot

         call fourier_transform(1, ffts,
     $        cd_gs, den(1,is), rden)
c     
c        symmetrize the charge density 
c        only if cd_gs%istar .eq. true
c        Francesco Mauri
c
         call symmetrize_scalar_global(den(1,is),cd_gs)

      end do                    ! end of loop over spins 

      deallocate(dhd)
      deallocate(rden)
      if(iand(pw_params%output,8).eq.8) 
     $     write(9,920) gimmetime()-t0

      return

 100  format(/' COMPUTING CHARGE DENSITY WITH ENERGY WINDOW:',
     $     f10.2, ' TO ', f10.2, 'eV')

 920  format(' TIME FOR ENERGY-RESOLVED CHARGE',
     $     ' DENSITY CALCULATION:',f12.3)


      end 



