c-*-Fortran-*- 
c     @process extchk
      subroutine scfloop(imode, pot_gspace, k_gspace,
     $           energs, pw_params, crys, syms, ffts,
     $           pspot, kpoints, bands, wavefn, lrwfn,
     $           vion, vin, vout, den, denc, chdr)
c
c     1996 Bernd Pfrommer
c
c
      include 'use.h'
      implicit none             ! never remove this line. 
      include 'interface.h'
c
c     INPUT:
c     -----
      
      integer imode             ! operation mode
      type (symmetry) syms      ! symmetry operations
      type (crystal) crys       ! crystal structure
      type (fftstruc) ffts      ! information for the fast fourier transform
      type (pw_parameter) pw_params ! plane wave parameters
      type (pseudo_potential) pspot ! all info about the pseudo potential
      type (kpoint) kpoints     ! BZ integration kpoints

      type (complex_gspace_array) 
     $     denc                 ! core charge density

      logical lrwfn             ! true if wavefn is to be reused, false if not
      type (parallel_gspace) 
     $     pot_gspace, k_gspace(*)


c
c     INPUT/OUTPUT:
c     -------------

      type (energy) energs      ! energies and other information
      type (band) bands         ! eigenvalues, occupation numbers etc

      type (complex_gspace_array) 
     $     vion,                ! screened ionic potential
     $     den,                 ! total charge density
     $     vin,                 ! input potential
     $     vout                 ! output potential

      type (complex_gspace_array) 
     $     wavefn               ! wave functions for all kpoints, bands

c
c     WORK:
c     ----
      type (complex_rspace_array)
     $     chdr                 ! work array: charge density in realspace
c
c     
c     DESCRIPTION:
c     -----------
c
c     Either computes self-consistent field (if imode =1) or
c     does just a single iteration (if imode =0, like for a
c     bandstructure   calculation). 
c
c     In case lrwfn (LogicalReuseWaveFuNction) is true, the
c     subroutine assumes that "bands" and "wavefn" contain a good
c     starting guess. 
c     
c
c     --------------- local variables -------------------------
c
      type (parallel_gspace)
     $     sub_gspace,          ! for the starting guess
     $     mix_gspace           ! gspace for mixing
      complex(kind=8), pointer ::
     $     vloc(:)              ! local potential in realspace
      type (hamiltonian)
     $     ham                  ! hamiltonian at a given k-point
      type (dense_hamiltonian)
     $     densham              ! subspace hamiltonian

      type (energy) oldenergs   ! the energies of the previous iteration step

      complex(kind=8), allocatable :: 
     $     pack(:),
     $     ajac(:)              ! inverse of hessian: mixing matrix

      double precision
     $     t0, tstart,          ! start time
     $     errscfconv           ! largest error in the potential

      integer 
     $     i,j,
     $     maxiter,             ! maximum number of scf iterations
     $     nsp, nvecs, 
     $     ierr,                ! error flag
     $     is,                  ! spin counter
     $     irk,                 ! kpoint counter
     $     iter                 ! iteration number
      
      logical 
     $     istartguess,         ! whether to compute the starting guess or not
     $     iscfconv,            ! indicates convergence of scf loop
     $     idiaconv             ! indicates convergence of diagonalization


      character*11 wfnname

      double precision gimmetime
      external gimmetime

c
c     --------------------------------------------------------------
c
      oldenergs=0.d0

      ham%ekinmod=pw_params%ekinmod

c
c     generate a gspace which is local to all processors
c     for the mixing. find inversion information, which
c     is used in the mixing routine.
c

      mix_gspace%gmax=sqrt(pw_params%emaxmix)
      mix_gspace%rk=(/0,0,0/);  ! no shift
      mix_gspace%nproc=1;       ! the same on all processors
      mix_gspace%name='mixing'
      mix_gspace%myproc=0; mix_gspace%fftsize=(/0,0,0,0,0/)
      mix_gspace%istar=.true. ; mix_gspace%igvec=.false.;
      mix_gspace%iekin=.true. ; mix_gspace%iinverse=.true.
      mix_gspace%istruc=.false.
      mix_gspace%ipackinfo=.false.

      call generate_gspace(pw_params%output,mix_gspace, crys, syms)
      allocate(ajac(crys%nspin*crys%nspin*
     $     mix_gspace%length*mix_gspace%length)) ! allocate mix matrix

      allocate(vloc(ffts%r_size))

      if(pw_params%maxitscf.gt.1) write(9,100)
      call myflush(9)
      
      if(iand(pw_params%input,1).eq.1) call read_band('BAND',bands)
      if(bands%nrk.le.0) write(9,155)

      if(iand(pw_params%input,2).eq.2) 
     $     call lukman(crys,pot_gspace, vion%data(1,1,1))

      if(imode.eq.1) then 
         maxiter =pw_params%maxitscf
      else
         maxiter = 1
      endif
      do iter=1, maxiter
c
c        compute the wave functions
c
         do is =1, crys%nspin
            call setup_local_potential(1,ffts,pot_gspace,
     $           vion%data(1,1,is),vloc(1))

            call myflush(9)
            do irk=1, kpoints%nrk
               call create_hamiltonian(ham,k_gspace(irk),pspot,
     $              vloc,ffts)
               call setup_nonlocal_potential(ham,pspot,crys)

               if((iter.eq.1).and.(.not.lrwfn)) then ! gen start guess or read
                  istartguess=.true.
                  if(iand(pw_params%input,1).eq.1) then ! read from file
                     istartguess=.false.
                     call read_band('BAND',bands)
                     if(bands%nrk.le.0) then ! error
                        write(9,155)
                        bands%nrk =kpoints%nrk; 
                        istartguess=.true.; goto 20
                     endif
                     write(wfnname,120) irk,is
                     wavefn%data(:,irk,is)=0
                     nsp = 1 ; nvecs = bands%nband(irk,is)
                     call readgsdat(1, ierr, k_gspace(irk), 
     $                    wavefn%data(1,irk,is), bands%nband(irk,is),
     $                    nvecs, nsp, wfnname,10)
                     if(ierr.ne.0) then
                        write(9,150) wfnname ; 
                        istartguess=.true. ; goto 20
                     endif
                     write(9,140) nvecs, wfnname
                     bands%nband(irk,is)=nvecs
                  endif
                     
 20               if(istartguess) then
                  sub_gspace%gmax=sqrt(pw_params%emaxsub)
                  sub_gspace%rk=ham%gspace%rk ! inherit shift from big brother
                  sub_gspace%nproc=1; ! scatter across all processors
                  sub_gspace%name='sub'
                  sub_gspace%myproc=0;sub_gspace%fftsize=(/0,0,0,0,0/)
                  sub_gspace%istar=.false. ;sub_gspace%igvec=.true.
                  sub_gspace%iekin=.true. ;sub_gspace%iinverse=.false.
                  sub_gspace%istruc=.false.
                  sub_gspace%ipackinfo=.false.

                  call generate_gspace(pw_params%output,sub_gspace, 
     $                 crys, syms)
                  call create_dense_hamiltonian(densham,sub_gspace)
                  call start_vectors(0,pw_params,ham, densham, 
     $                 bands%energy(1,irk,is), bands%max,
     $                 wavefn%data(1,irk,is),
     $                 pot_gspace, sub_gspace,vion%data(1,1,is),crys)

                  if(bands%nband(irk,is).gt.bands%min)
     $                 bands%nband(irk,is)=bands%nband(irk,is)-1
                  call destroy(densham)
                  call destroy(sub_gspace)
                  endif
               endif

               if(lrwfn.and.(iter.eq.1)) write(9,160)
               if(pw_params%bandgap.gt.0) then ! optimize for insulator
                  bands%nband(irk,is)=bands%min
                  bands%energy(bands%min+1,irk,is)=
     $                bands%energy(bands%min,irk,is)+pw_params%bandgap
               endif


               call diagonalize_hamiltonian(pw_params,
     $              pw_params%maxitdiag, idiaconv, bands%min,
     $              bands%nband(irk,is),pw_params%epsdiag,
     $              bands%energy(1,irk,is), ham, wavefn%data(1,irk,is))
               call kinet(ham,wavefn%data(1,irk,is), 
     $              bands%nband(irk,is),
     $              bands%ekn(1,irk,is))
               call destroy(ham)
               call myflush(9)
               
            end do              ! loop over k-points
         end do                 ! loop over spins


c
c        compute occupation numbers and fermi energy
c
         call flevel(imode, 2,pw_params,energs, crys,bands,kpoints)

         if(imode.eq.0) goto 10 ! leave the loop in this mode
c
c        compute new charge
c
         call charge(ffts, pw_params,pot_gspace, k_gspace, bands,
     $        kpoints,wavefn,den%data(1,1,1),energs)
c
c        calculate new screening potential
c     
         call velect(1,pot_gspace,crys,ffts,energs,pw_params, 
     $        denc%data(1,1,1),vout%data(1,1,1),
     $        den%data(1,1,1),chdr%data(1,1,1))
c
c        compute total energy
c
         call etotal(crys,
     $        iter, iscfconv, errscfconv, energs, oldenergs,
     $        pw_params, pot_gspace, vion%data(1,1,1), 
     $        vin%data(1,1,1), vout%data(1,1,1), den%data(1,1,1))
c
c        write charge density to disk
c
         call writegsdat(0, pot_gspace, den%data(1,1,1), 
     $        den%nvecs,den%nvecs,  den%nspin,'CD',2)

c
c        mix with previous iteration
c
         call mixer(iter, pw_params, crys,pot_gspace, mix_gspace,
     $        vin%data(1,1,1), vout%data(1,1,1), energs, 
     $        vion%data(1,1,1), ajac(1))

         if(iscfconv) goto 10

      end do                    ! iterative loop
      
      write(9,110) iter-1

 10   continue

c
c     free memory attached to local structures
c

      call destroy(mix_gspace)
      deallocate(vloc)
      deallocate(ajac)
c
c     write eigenvectors to disk 
c
      if(iand(pw_params%output,512).eq.512) then
         write(9,130)
         if(pot_gspace%myproc.eq.0) then
            call mysystem('\rm -f BAND')
            call write_band('BAND',bands)
         endif
         do is=1, crys%nspin
            do irk=1, kpoints%nrk
               write(wfnname,120) irk,is
               call writegsdat(0,k_gspace(irk), wavefn%data(1,irk,is)
     $              ,bands%nband(irk,is),bands%nband(irk,is),1
     $              ,wfnname,10)
            end do
         end do
      endif




      return
C
 100  format(/,20('-'),' BEGIN OF SELF-CONSISTENT LOOP ',20('-'))
 110  format(20('*'),/' WARNING: SCFLOOP NOT CONVERGED IN ', i3,
     $     ' ITERATIONS')
 120  format('WFN',i5.5,'.',i1)
 130  format(/' <<<<<<<< WRITING WAVEFUNCTIONS TO DISK >>>>>>>>>>')
 140  format(' <<<<<<<< READ ',i4,' BANDS FROM FILE ',a,'>>>>>>>>')
 150  format(' *** COULD NOT FIND WAVE FUNCTION ',a,'ON DISK!')
 155  format(' *** COULD NOT FIND BAND FILE ON DISK!')
 160  format(' <<<<<<<< REUSING WAVE FUNCTIONS >>>>>>>>')
      end
