c
      subroutine screen(iscr, lrcd,
     $     gs, crys, ffts, energs, pwp,
     $     vion, denv, denc, vin, den, chdr)

      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit ? ....

      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'

c     ---------------- arguments ----------------------------------------

c     INPUT:
c     -----

c      iscr           control parameter.
c                  1  use hartree screening.
c                  2  use fermi thomas screening.
c                  3  screen with dielectric function read from
c                     tape10.
c                  4  screen with electronic charge density read
c                     from tape10
c                  5  screen with a superposition of atomic
c                     charge densities read from tape1, tape2...
c
c      pwp%icorr   control parameter for correlation. it is
c                  used only if iscr = 3 or 4. for explanation
c                  see subroutine excorr

      integer 
     $     iscr

      logical lrcd              ! whether charge can be reused from previous step

      type (parallel_gspace) gs 
      type (crystal) crys       ! for nspin, ztot
      type (fftstruc) ffts
      type (pw_parameter) pwp

      type (complex_gspace_array)
     $     denv,                ! the initial valence charge density
     $     denc                 ! the core charge density
c
c     INPUT/OUTPUT:
c     ------------
      type (complex_gspace_array) 
     $     vion                 ! the ionic potential, screened on output
c
c     OUTPUT:
c     ------
      type (energy) energs
      type (complex_gspace_array) 
     $     vin,                 ! the input potential on output
     $     den                  ! the valence charge density on output

c
c     WORK:
c     ----

      type (complex_rspace_array)
     $     chdr                 ! a work array for the realspace charge
c
c      screens the ionic potential on lattice
c      it uses the subroutines velect and excorr to convert
c      charge density into potential.
c      adapted from sverre froyen plane wave program
c      written june 1 1987. jlm
c
c     parallel version 1995 Bernd Pfrommer

c     --------------- local variables -------------------------------------

      character*2  scr
      character*9  bdate
      character*8  btime
      character*10 ititle(7), iray(6)
      character*20 title

      double precision pi , zero, one, two, three, four, half, big
      parameter (pi=3.141592653589793d0)
      parameter (zero = 0.0d0, one = 1.0d0, four = 4.0d0)
      parameter (two = 2.0d0, three = 3.0d0, half = 0.5d0)
      parameter (big = 1.0d10)

      double precision qtf2, qf, x, x2, gx, fac,
     $     vinreal, vinimag, exc ! exc is discarded here

      integer
     $     i, ipr, ngr, ngmin, ierr, is, nspin, nv

      logical isp

      external velect
c
c     -------------- here begins the code itself --------------------------
C
      call create_gspace_array(vin,gs%length,1,crys%nspin)
      call create_rspace_array(chdr,ffts%r_size,1,crys%nspin)

      if(lrcd) then
         write(9,300)
      else
         call create_gspace_array(den,gs%length,1,crys%nspin)
      endif

      isp = .false.
      if(crys%nspin.eq.2) isp =.true.

      if(iscr .eq. 1) then      ! --------- Hartree screening (not debugged)
         write(9,*) ' Hartree screening is not debugged yet.'
         write(9,*) ' I would not use it!'
         qtf2 = four*(three*crys%ztot/(pi*crys%vcell))**(one/three)
         qf = pi*qtf2/four
         do i=1, gs%length
            x = sqrt(gs%ekin(i))/qf
            if(x.gt.zero) then  ! skip g=0 vector
               x2 = x/two
               gx = half
               if (x2 .eq. one) gx = (one-(one-x2*x2)*
     *              log(abs((one-x2)/(one+x2)))/x)/two
               fac = one/(one+gx*qtf2/gs%ekin(i)) - one
               do is=1,crys%nspin
                  vin%data(i,1,is) = fac*vion%data(i,1,is)
               end do
            endif
         end do
         

      else if(iscr .eq. 2) then ! ----- Thomas Fermi screening (not debugged)

         write(9,*) ' Thomas-Fermi screening is not debugged yet.'
         write(9,*) ' I would not use it!'

         qtf2 = four*(three*crys%ztot/(pi*crys%vcell))**(one/three)

         do i=1, gs%length
            if(gs%ekin(i).gt.zero) then
               fac = one/(one+qtf2/gs%ekin(i)) - one
               do is=1, crys%nspin
                  vin%data(i,1,is) = fac*vion%data(i,1,is)
               end do
            endif
         end do

      else            ! -------- screen with charge or dielectric function
     
         write(9,100)

         if((iscr .eq. 5).and.(.not.lrcd)) then   ! screen with atomic charge 
            call mzcopy(gs%length*crys%nspin, denv%data(1,1,1), 
     $           1, den%data(1,1,1), 1)
         endif

         if((iscr .eq. 4).and.(.not.lrcd)) then
c
c          read the charge density from tape.
c          if file empty or doesnt exist switch to atomic screening.
c

            do is=1,crys%nspin
               do i=1, gs%length       ! clear valence charge density
                  den%data(i,1,is)=(0.d0, 0.d0)
               end do
            end do

            nv = den%nvecs ; nspin = den%nspin
            call readgsdat(1,ierr,
     $           gs, den%data, den%nvecs, nv,nspin,'CD',2)
            if(nspin.ne.den%nspin) then
               write(9,298)
               ierr=-1
            endif
            if(nv.ne.den%nvecs) then
               write(9,299)
               ierr=-1
            endif
            if(ierr.ne.0) then
               write(9,200)
               call mzcopy(gs%length*crys%nspin,denv%data(1,1,1),
     $              1,den%data(1,1,1),1)
            else
c              symmetrize the read-in charge density
               do is=1, crys%nspin
                  call fourier_transform(-1,ffts,gs,den%data(1,1,is),
     $                 chdr%data(1,1,1))
                  chdr%data(:,1,1)=abs(chdr%data(:,1,1))
                  call fourier_transform(1,ffts,gs,den%data(1,1,is),
     $                 chdr%data(1,1,1))
                  call symmetrize_scalar_global(den%data(1,1,is),gs)
               end do
            endif
         endif

         if((iscr.eq.4).or.(iscr.eq.5)) then
c
c          compute the potential from the charge density
c
            call velect(2,gs,crys,ffts,energs,pwp,
     $           denc%data(1,1,1),vin%data(1,1,1),
     $           den%data(1,1,1),chdr%data(1,1,1))
         endif


         IF (ISCR .EQ. 3) THEN
            write(9,*) 'velect: the dielectric function screening '//
     $           'is not implemented yet'
            stop
         endif
      endif
c
c        add screening potential to the ionic potential.
c
c

      do is=1,crys%nspin
         do i=1,gs%length
            if(gs%ekin(i).gt.0) then
               vion%data(i,1,is) = vion%data(i,1,is) 
     $              + vin%data(i,1,is)
            endif
         end do
      end do
      return

 100  FORMAT(//,' SCREENING :',/,1X,11('-'))
      
 200  format(' *** COULD NOT FIND INTACT OLD CHARGE ON DISK!',
     $     /' *** ATOMIC CHARGE USED TO SET UP INITIAL POTENTIAL!')
 298  format(' *** SPIN NUMBER MISMATCH FOR CHARGE DENSITY ON DISK!')
 299  format(' *** CHARGE DENSITY ON DISK LOOKS FUNNY!')
 300  format(' <<<<<<<< REUSING CHARGE IN  MEMORY >>>>>>>>')

       end
