/*
   The following routines are called from the fortran code to do
   various system jobs.
*/


#include<stdlib.h>
#include<string.h>
#include<malloc.h>
#include<stdio.h>
#include "libcalls.h"
#include "fortranize.h"

void FORTRANIZE(frepack,FREPACK)
(const int *packsize,  zomplex *out, const int *pack2, 
             const zomplex *in, const int *pack1)
{
  register int i,j,k,k1,j1,p2;

#ifdef CRAY
#ifndef T3E
#pragma _CRI ivdep
#endif
#endif

 
#if defined(SUN)  || defined(CVX)
  p2 = *packsize- (*packsize)%2;
  for(i=0;i<p2;i+=2) {
    j  = pack2[i]-1;
    j1 = pack2[i+1]-1;
      
    k  = pack1[i]   -1;
    k1 = pack1[i+1] -1;

      
    out[j].real=in[k].real;
    out[j].imag=in[k].imag;

    out[j1].real=in[k1].real;
    out[j1].imag=in[k1].imag;
  }
  for(;i<*packsize;i++) {
    j=pack2[i]-1;
    k=pack1[i]-1;
    out[j].real=in[k].real;
    out[j].imag=in[k].imag;
  }

#else
  for(i=0;i<(*packsize);i++) {
    j=pack2[i]-1;
    k=pack1[i]-1;
    out[j]=in[k];
  }

#endif
}

void FORTRANIZE(fftlt2,FFTLT2)
   (const int *dir, const int *chunk, Zomplex *rspace, Zomplex *t1buf, 
	    const int *nnx, const int *nny, const int *ldx, const int *ldy) 
{
  int  bailout = 1;
  int  const one = 1;
  int  ichunk, srod, irod, sy, slen, swidth, drod, dx, ix;

  if(*dir < 0) { /* repacking for the inverse transform */
#ifdef CVX
#pragma _CNX no_loop_dependence
#endif
    for(ichunk=0; bailout>0; ichunk+=8) {
      srod   = chunk[ichunk];
      sy     = chunk[ichunk+1];
      
      slen   = chunk[ichunk+2];
      bailout = chunk[ichunk+3];
      swidth = abs(bailout);
      
      drod   = chunk[ichunk+4];
      dx     = chunk[ichunk+5];
      
      for(irod=srod; irod<srod+swidth; irod++) {
	ix = (dx+(irod-srod))%(*nnx);
	/* printf("slen=%d, rs_start=%d, t1_start=%d\n",
	   slen, drod*(*nnx)+ix, irod*(*nny)+sy); */
	MZCOPY( slen,  &t1buf[irod* (*ldy)+sy], one, 
	        &rspace[drod* (*ldx)+ix], *ldx );
      }
    }
  } else {
#ifdef CVX
#pragma _CNX no_loop_dependence
#endif
    for(ichunk=0; bailout>0; ichunk+=8) {
      srod   = chunk[ichunk];
      sy     = chunk[ichunk+1];
      
      slen   = chunk[ichunk+2];
      bailout = chunk[ichunk+3];
      swidth = abs(bailout);
      
      drod   = chunk[ichunk+4];
      dx     = chunk[ichunk+5];
      
      for(irod=srod; irod<srod+swidth; irod++) {
	ix = (dx+(irod-srod))%(*nnx);
	MZCOPY(slen, (ZOMPLEX *)&rspace[drod*(*ldx)+ix], *ldx,
	       (ZOMPLEX *)&t1buf[irod*(*ldy)+sy], one);
      }
    }
    

  }

}


void fftunfoldc_(zomplex *folded, zomplex *unfolded, int *NNZ,
	       int *gsorder, int *GSLORDER) {

  /*

     c
     c     DESCRIPTION:
     c     -----------
     c
     c     Transfers the rods from the packed gspace arrangement onto
     c     a linear grid of length gs%fftsize(3), such that a subsequent
     c     1d FFT can be performed
     c
     c
     */

  int gslorder, nz, ii, iord, istart, iend, idiff, iid, nzdiff,iuffz,ns ;



  gslorder = *GSLORDER;
  nz = *NNZ;
  ns = sizeof(zomplex);
	
  memset((void *)unfolded,0,gslorder*nz*ns);

  ii=0;
  iuffz=0;
  for(iord=0;iord<gslorder; iord++) {
    istart = gsorder[3*iord+1];
    iend = gsorder[3*iord+2];
    idiff =iend - istart;
    if(idiff>=0) { /* there is only one contiguous block to copy */
      iid = ii + (idiff+nz)%nz;
      memcpy(&unfolded[istart+iuffz],&folded[ii],(idiff+1)*ns);
    }
    else { /* there are two blocks to copy */
      nzdiff=nz-istart;
      iid = ii + nzdiff -1;
      memcpy(&unfolded[istart+iuffz],&folded[ii], nzdiff*ns);
      ii     = iid+1;
      iid    = ii + iend;
      memcpy(&unfolded[iuffz],&folded[ii],(iend+1)*ns);
    }
    iuffz += nz;
    ii = iid+1;
  }

}









void mysetenv_(char* variab, char* valu, char* buf)
{
  char cst[2];
  char* bufend;
  int stend;

  cst[0]='='; cst[1]='\0';
  buf[0]='\0';

  strcat(buf,variab);   strcat(buf,cst);   strcat(buf,valu); 
  bufend=memchr(buf,' ',254);
  
  *bufend='\0';
  putenv(buf);
}

/* 
               provide a fortran substitute for memory copy

*/

void FORTRANIZE(fmemcpy,FMEMCPY)
(void *dest, const void *source, int *NUM)
{
  int num;

  num = *NUM;
  memcpy(dest, source, num);
}

/*      clear double complex array */
void fcleardcarray_(void *dest, int *NUM)
{
  memset(dest,0,16*(*NUM));
}


void mymalloc(int *size, void **poi)
{
  /* printf("mallocing %d bytes\n",*size); */
  *poi=malloc(*size);
  /* printf("malloced %d bytes\n",*size); */
}


void myfree(void **poi)
{
  free(*poi);
}
#ifdef RS6K

#include <sys/time.h>

void mygettimeofday(double *time) {
struct timeval Tp;
struct timezone Tzp;
int idum;

idum = gettimeofday(&Tp,&Tzp);
*time = (double )Tp.tv_sec + 1e-6*(double) Tp.tv_usec;
}

#endif




/*      this one is for debugging on the IBMs. it prints out statistics 
	about current memory usage of the process. if you want to use 
	this function, you have to use mymalloc and myfree to allocate 
	and free memory.
*/

#ifdef RS6K
void mymallinfo()
{
  FILE *memy;

  struct mallinfo minfo;
  minfo = mallinfo();

  memy=fopen("mem_state","a");
  fprintf(memy, "memory state:\n");
  
  fprintf(memy,"arena    = %d\n",minfo.arena);
  fprintf(memy,"#ordblks = %d\n",minfo.ordblks);
  fprintf(memy,"#smblks  = %d\n",minfo.smblks);
  fprintf(memy,"#hblks   = %d\n",minfo.hblks);
  fprintf(memy,"hblkhd   = %d\n",minfo.hblkhd);
  fprintf(memy,"usmblks  = %d\n",minfo.usmblks);
  fprintf(memy,"fsmblks  = %d\n",minfo.fsmblks);
  fprintf(memy,"uordblks = %d\n",minfo.uordblks);
  fprintf(memy,"fordblks = %d\n",minfo.fordblks);

  fclose(memy);
}

#endif

#ifdef CVX
#include<math.h>
double myderf(double *x)
{
return erf(*x);
}
double myderfc(double *x)
{
return erfc(*x);
}
#endif

