c-*-Fortran-*- 
c
c     @process extchk optimize(2)

      subroutine setup_nonlocal_derivative(imonitor,
     $     qmag, rk, pspot, k_gspace, nloc_deriv_proj, crys)

      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
c 
c     INPUT:
c     -----

      type (pseudo_potential) pspot
      type (crystal) crys
      type (parallel_gspace) k_gspace

      integer imonitor          ! flag for printouts
      

      double precision
     $     rk(3),               ! the origin kpoint
     $     qmag                 ! magnitude of delta q to compute derivative


c
c     OUTPUT:
c     ------
c
c     derivative operator
c
      complex(kind=8)
     $     nloc_deriv_proj(k_gspace%length+pspot%nanl, pspot%nanl, 2,3)
c
c
c     DESCRIPTION:
c     -----------
c
c
c     Sets up the nonlocal potential part of the hamiltonian at 
c
c         k+qmag*e^  for all three cartesian directions directions e^
c                    and for the negative -e^.
c
c     This is required to compute the derivative dV_nloc/dk.
c
c     dV_nloc/dk_x |phi> =  
c       (nloc_deriv_proj(:,2,x)-nloc_deriv_proj(:,1,x)) |phi>
c
c     notice that there is no xnorm application necessary: it is already
c     incorporated into the projectors

c
c
c     --------------------- local variables ------------------------------
c
c     ----- variables for the gspace loop ---------
c
      integer
     $     igv(4),fftn(4), ffth(4),igv3, irod,iord,igs

      double precision gv(3)


      integer 
     $     ind, i, k, kk, 
     $     l, ni, ni1, lmmin, lmmax, lm, iq, iqsign

      double precision
     $     pi2,                 ! 2*pi
     $     dq(3,3),             ! delta q in lattice coordinates for
     $                          ! cartesian displacement in x,y,z
     $     qi, qk(3), qcar(3), flm(9), qinv, fi, xni,
     $     oosq2q, t0, 
     $     zero, one, two, three, half, eps, 
     $     vq, xi, xdum, xa, xb, gimmetime


      parameter ( zero = 0.0d0, one = 1.0d0, two = 2.0d0)
      parameter ( three = 3.0d0, half = 0.5d0)
      parameter ( eps = 1.0d-8)

      external gimmetime

      complex(kind=8), allocatable ::
     $     st(:)                ! work array for structure factors

C     
c
c     -----------------------------------------------------------------------


      if(iand(imonitor,8).eq.8) t0=gimmetime()

      allocate(st(crys%mxdatm))
      pi2 = 8.d0*datan(1.d0)
      fftn(1:3) = k_gspace%fftsize(1:3)
      fftn(4)   = k_gspace%fftsize(3)
      ffth(:)   = fftn(:)/2

c
c     
c
      dq=qmag*transpose(crys%avec)/pi2
      oosq2q=1.d0/sqrt(4.d0*qmag) ! 2.0 for Hart<-Ryd, 2.0 for +-q derivative
c
c      starts loop over g-vectors in small gspace 
c
      igs = 0 
      do 10 iord=1, k_gspace%lorder    ! loop through x/y gspace 
         irod=k_gspace%order(1,iord)
         igv(1)= irod/k_gspace%fftsize(2)
         igv(2)= mod(irod, k_gspace%fftsize(2))
         igv(3)= k_gspace%order(2,iord)
         igv(4)= k_gspace%order(3,iord)
         igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
         gv(1:2)=dble(igv(1:2))
         do 15 igv3=igv(3),igv(4)     ! loop over z axis
            gv(3)=dble(igv3)
            igs = igs + 1
            do iq=1, 3          ! loop over all q-points
               do iqsign=1,-1,-2 ! sign of q   
                  qk=gv(:)+rk(:)+ dble(iqsign)*dq(:,iq)

                  qcar(1) = crys%bvec(1,1)*qk(1)+crys%bvec(1,2)*
     $                 qk(2)+crys%bvec(1,3)*qk(3)
                  qcar(2) = crys%bvec(2,1)*qk(1)+crys%bvec(2,2)*
     $                 qk(2)+crys%bvec(2,3)*qk(3)
                  qcar(3) = crys%bvec(3,1)*qk(1)+crys%bvec(3,2)*
     $                 qk(2)+crys%bvec(3,3)*qk(3)
                  qi    = sqrt(qcar(1)*qcar(1) + qcar(2)*qcar(2) +
     $                 qcar(3)*qcar(3)) 

c
c                 ----- compute angular functions
c
                  flm(1) = one
                  if(qi .gt. eps) then
                     qinv = one/qi
                     flm(2) = qcar(1)*qinv
                     flm(3) = qcar(2)*qinv
                     flm(4) = qcar(3)*qinv
                     qinv = qinv*qinv
                     flm(5) = sqrt(three)*qcar(1)*qcar(2)*qinv
                     flm(6) = sqrt(three)*qcar(2)*qcar(3)*qinv
                     flm(7) = sqrt(three)*qcar(3)*qcar(1)*qinv
                     flm(8) = three*qcar(3)*qcar(3)*qinv/two - one/two
                     flm(9) = sqrt(three)*(qcar(1)*qcar(1)-qcar(2)
     $                    *qcar(2))*qinv/two
                  else
                     do lm=2,9
                        flm(lm) = zero
                     end do
                  endif
c
c                 starts loop over second index
c

                  ind = 0
                  do 20 k=1,crys%ntype
                     do kk=1,crys%natom(k) !  compute complex phase factor
                        fi = gv(1)*crys%rat(1,kk,k) +
     $                       gv(2)*crys%rat(2,kk,k) +
     $                       gv(3)*crys%rat(3,kk,k)
                        st(kk) = exp(cmplx(0.d0, fi,kind=8))
                     end do
C     

                     do l=1,5   ! loop over angular momenta l
                        if(pspot%nkb(l,k) .ne. 0) then 
                           xni = qi/pspot%delqnl(k) + two 
c     
c                          cubic spline interpolation
c
                           vq=zero
                           ni = xni 
                           if(ni .le. 2) ni = 3
                           ni1 = ni+1
                           
                           if(ni .lt. pspot%nqnl(k)) then
                              xa = dble(ni1)-xni
                              xb = xni-dble(ni)
                              vq = xa*pspot%vkb(ni,l,k) + 
     $                           xb*pspot%vkb(ni1,l,k) + 
     $                           ((xa**3-xa)*pspot%d2vkbdq2(ni-2,l,k)
     $                           +(xb**3-xb)*pspot%d2vkbdq2(ni1-2,l,k
     $                           ))/6.d0
                           endif


                           lmmin = pspot%lo(l,k)*pspot%lo(l,k)+1
                           lmmax = (pspot%lo(l,k)+1)*(pspot%lo(l,k)+1)
c     
                           do lm=lmmin,lmmax ! loop over m quantum number
                              do kk=1,crys%natom(k)
                                 ind = ind + 1
                         nloc_deriv_proj(igs,ind,(iqsign+3)/2,iq)
     $                               =conjg(st(kk)*flm(lm)*vq)*oosq2q
                              end do
                           end do
                        endif
                     end do 
 20               continue      ! end of loop over atomic types
               end do           ! loop over iqsign
            end do              ! loop over iq

 15      continue               ! end of loop over 3rd dimension

 10   continue                  ! end of loop over small gspace

      deallocate(st)

c
c     attach the normalization factors to the end of the nonloc_deriv
c     array

      ind = 0
      do k=1,crys%ntype
         do l=1,5
            if(pspot%nkb(l,k) .ne. 0) then

               lmmin = pspot%lo(l,k)*pspot%lo(l,k) + 1
               lmmax = (pspot%lo(l,k)+1)*(pspot%lo(l,k)+1)

               do lm=lmmin,lmmax
                  do kk=1, crys%natom(k)
                     ind = ind + 1
                     nloc_deriv_proj(k_gspace%length+ind,1,1,1)
     $                    = cmplx(dble(pspot%nkb(l,k)),0.d0,kind=8)
                  end do
               end do
            endif
         end do
      end do

      if(iand(imonitor,8).eq.8) then
         write(9,920) gimmetime()-t0
         call myflush(9)
      endif

      return
 920  format(' TIME FOR NONLOCAL DERIVATIVE:', f12.3)


      end

