c-*-Fortran-*- 
      subroutine setup_nonlocal_potential(ham,pspot,crys)

      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'

      type (hamiltonian) ham
      type (pseudo_potential) pspot
      type (crystal) crys

c
c     Set up the nonlocal part of the Hamiltonian for a given k-point.
c     Kleinman-Bylander Pseudopotential.
c
c     all it does is setting up the arrays ham%vnloc and ham%xnorm
c     at the kpoint specified by ham%gspace%rk
c     
c
c
c     1996 Bernd Pfrommer (parallel version)
c          I changed the JLM representation "anlga" to be the conjugate
c          complex "vnloc", because it allows a more efficient use of
c          BLAS calls later in the diagonalization routine.
c
c     1990 JLM
c     adapted from Sverre Froyen plane wave program


c     --------------------- local variables ------------------------------

c
c     ----- variables for the gspace loop ---------
c
      integer
     $     igv(4),fftn(4), ffth(4),igv3, irod,iord,igs

      double precision gv(3)

      integer 
     $     ind, i, k, kk,
     $     l, ni, ni1, lmmin, lmmax, lm 

      double precision
     $     qi, qk(3), qcar(3), flm(9), qinv, fi, xni,
     $     zero, one, two, three, half, eps, 
     $     vq, xi, xdum, xa, xb


      parameter ( zero = 0.0d0, one = 1.0d0, two = 2.0d0)
      parameter ( three = 3.0d0, half = 0.5d0)
      parameter ( eps = 1.0d-8)

      complex(kind=8) sum
      complex(kind=8), allocatable ::
     $     st(:)           ! work array for structure factors

C     
c
c     -----------------------------------------------------------------------

      allocate(st(crys%mxdatm))

      fftn(1:3) = ham%gspace%fftsize(1:3)
      fftn(4)   = ham%gspace%fftsize(3)
      ffth(:)   = fftn(:)/2
c
c      starts loop over g-vectors in small gspace 
c
         
      igs = 0 
      do 10 iord=1, ham%gspace%lorder    ! loop through x/y gspace 
         irod=ham%gspace%order(1,iord)
         igv(1)= irod/ ham%gspace%fftsize(2)
         igv(2)= mod(irod, ham%gspace%fftsize(2))
         igv(3)= ham%gspace%order(2,iord)
         igv(4)= ham%gspace%order(3,iord)
         igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
         gv(1:2)=dble(igv(1:2))
         do 15 igv3=igv(3),igv(4)     ! loop over z axis
            gv(3)=dble(igv3)
            igs = igs + 1
            qi    = sqrt(ham%gspace%ekin(igs)) ! get kinetic energy from array
            qk(:) = ham%gspace%rk(:) +gv(:)
c
c           cartesian of k+G
c
            qcar(1) = crys%bvec(1,1)*qk(1)+crys%bvec(1,2)*
     $           qk(2)+crys%bvec(1,3)*qk(3)
            qcar(2) = crys%bvec(2,1)*qk(1)+crys%bvec(2,2)*
     $           qk(2)+crys%bvec(2,3)*qk(3)
            qcar(3) = crys%bvec(3,1)*qk(1)+crys%bvec(3,2)*
     $           qk(2)+crys%bvec(3,3)*qk(3)
c
c           ----- compute angular functions
c
            flm(1) = one
            if(qi .gt. eps) then
               qinv = one/qi
               flm(2) = qcar(1)*qinv
               flm(3) = qcar(2)*qinv
               flm(4) = qcar(3)*qinv
               qinv = qinv*qinv
               flm(5) = sqrt(three)*qcar(1)*qcar(2)*qinv
               flm(6) = sqrt(three)*qcar(2)*qcar(3)*qinv
               flm(7) = sqrt(three)*qcar(3)*qcar(1)*qinv
               flm(8) = three*qcar(3)*qcar(3)*qinv/two - one/two
               flm(9) = sqrt(three)*(qcar(1)*qcar(1)-qcar(2)*qcar(2))
     $              *qinv/two
            else
               do lm=2,9
                  flm(lm) = zero
               end do
            endif
c
c        starts loop over second index
c

            ind = 0
            do 20 k=1,crys%ntype

               do kk=1,crys%natom(k) !           compute complex phase factor
                  fi = gv(1)*crys%rat(1,kk,k) +
     $                 gv(2)*crys%rat(2,kk,k) +
     $                 gv(3)*crys%rat(3,kk,k)
                  st(kk) = exp(cmplx(0.d0, fi))
               end do
C

c          loop over lo is replaced by loop over l
c          and l quantum number is given  explicitely by referencing pspot%lo

               do l=1,5        ! loop over angular momenta l
                  if(pspot%nkb(l,k) .ne. 0) then ! found pot of that ang. mom
                     xni = qi/pspot%delqnl(k) + two ! interpolate potential
c
c               this was the old extrapolation. changed 1996 pfrommer/mauri
c
c                     ni  = xni + half
c                     if(ni .le. 3) ni = 4
c                     vq = zero
c                     if(ni .lt. pspot%nqnl(k)) then
c                        xi  = xni - dble(ni)
c                        vq = pspot%vkb(ni,l,k) * (one+xi) * (one-xi)
c     $                       + half * (pspot%vkb(ni+1,l,k)*(one+xi)
c     $                       -pspot%vkb(ni-1,l,k)*(one-xi)) * xi
c                     endif


c
c                    cubic spline interpolation
c
                     vq=zero
                     ni = xni 
                     if(ni .le. 2) ni = 3
                     ni1 = ni+1

                     if(ni .lt. pspot%nqnl(k)) then
                        xa = dble(ni1)-xni
                        xb = xni-dble(ni)
                        vq = xa*pspot%vkb(ni,l,k) + 
     $                       xb*pspot%vkb(ni1,l,k) + 
     $                       ((xa**3-xa)*pspot%d2vkbdq2(ni-2,l,k) +
     $                       (xb**3-xb)*pspot%d2vkbdq2(ni1-2,l,k))/6.d0
                     endif


                     lmmin = pspot%lo(l,k)*pspot%lo(l,k) + 1
                     lmmax = (pspot%lo(l,k)+1)*(pspot%lo(l,k)+1)
c     
                     do lm=lmmin,lmmax ! loop over m quantum number
                        do kk=1,crys%natom(k)
                           ind = ind + 1
                ham%vnloc%data(igs,ind,1) = conjg(st(kk)*flm(lm)*vq)
                        end do
                     end do
                  endif
               end do 


 20         continue            ! end of loop over atomic types

 15      continue               ! end of loop over 3rd dimension

 10   continue                  ! end of loop over small gspace
      
c      do k=1,crys%ntype
c         do l=1,5
c            if(pspot%nkb(l,k) .ne. 0) then ! found pot of that ang. mom
c               sum =0
c               do i=1, size(pspot%vkb(:,l,k))
c                  sum = sum + pspot%vkb(i,l,k)*pspot%vkb(i,l,k)
c     $            *sqrt(crys%vcell)
c     $            *pspot%delqnl(k)*(2*4*atan(one))**(-3)*4*4*atan(one)
c               enddo
c            endif
c         end do
c      end do


c      check for norm
c     
c     do ind=1, pspot%nanl
c        sum =0 
c        do i=1, ham%gspace%length
c           sum = sum +
c    $           ham%vnloc%data(i,ind,1)*
c    $           conjg(ham%vnloc%data(i,ind,1))
c        end do
c        write(9,*) 'second a2norm/anorm for projector',ind,'is:', sum 
c     end do
         
c
c      normalization matrix
c
       ind = 0
       do k=1,crys%ntype
          do l=1,5
             if(pspot%nkb(l,k) .ne. 0) then
                lmmin = pspot%lo(l,k)*pspot%lo(l,k) + 1
                lmmax = (pspot%lo(l,k)+1)*(pspot%lo(l,k)+1)
                do lm=lmmin,lmmax
                   do kk=1,crys%natom(k)
                      ind = ind + 1
                      ham%xnorm(ind) = dble(pspot%nkb(l,k))
                   end do
                end do
             endif
          end do
       end do
c

c       write(9,200)
c       do ind=1,pspot%nanl
c          write(9,300) (ham%vnloc%data(k,ind,1), k=1,ham%gspace%length)
c       end do



       deallocate(st)

       return


 200   format(/' Vectors: ')

 300   format(2000('(',f10.4,',',f10.4,')'))

       end

