c-*-Fortran-*-
c     
c     @process extchk
      subroutine setup_packinfo(gs)
 
m4_ifdef([T3E],[m4_define([SGI_PC])])
m4_ifdef([ALPHA],[m4_define([SGI_PC])])
m4_ifdef([CRAY],[m4_define([MPI_INTEGER4],[MPI_INTEGER])])
      use all_to_all_module
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'

m4_ifdef([MPI],[      
      include 'mpif.h' ])

c     1996 by Bernd Pfrommer while at UC Berkeley
c
c     INPUT/OUTPUT:
c     ------------

      type (parallel_gspace) gs ! the gspace for which the data is set up

c     
c     DESCRIPTION:
c     -----------
c
c
c     Sets up the necessary arrays to perform the 3d fft in parallel.
c     This involves attaching the following memory blocks to the
c     gspace structure:
c
c                                      ,---- pack/unpack
c     gs%packinfo(maxpacksize_t1,nproc,2)
c                     \              
c                      `-- which element of the packet
c
c                        ,---- pack/unpack
c     gs%packsize(nproc,2,2)                 = number of dc variables sent
c                          \              
c                           `-- which phase of the transpose (first or second)
c
c                     ,---- inverse=1, forward =2 fft
c     gs%pickup(nproc,2)  = where on the other proc the packed data
c                           for the t2 phase resides (only used for shmem)
c     
c
c     packing index:
c
c     the index=1 refers to "packing    for proc p" for inverse FFT
c                       and "unpacking from proc p"     forward
c     the index=2 refers to "unpacking from proc p" for inverse FFT
c                       and "packing    for proc p"     forward
c
c     transposition index:
c
c     the index=1 refers to  1st transpose of inverse FFT
c                        and 2nd              forward
c               2 refers to  2nd transpose of inverse FFT
c                        and 1st              forward
c
c     Also, gs%maxpacksize_t1/t2 is set to the maximum package size 
c     for the first/second transpose
c
c
c     packinfo and packsize contain the necessary information to 
c     pack and unpack the messages exchanged between the processors
c     during the transposition phase of the inverse and forward FFT.
c
c     For the second transpose of the FFT grid, the following array
c     is used:
c                    1    2                 
c     gs%chunk(1..4,pack/unpack,whichchunk,proc)
c
c     Each chunk is described by 4 numbers in the first index:
c
c     1 = starting rod, as indexed on the local processor
c     2 = starting y offset for packing, x offset for unpacking
c     3 = length of chunk along y 
c     4 =  width of chunk along x 
c
c     For the last chunk, the width is negative. This serves as a 
c     flag to stop packing/unpacking.
c
c
c
c
c     WARNING: do not call and ask to set up packinfo with gs%nproc=1
c     unless there is really only one processor up and running.
c
c     This will crash, since the gather routine
c     will try to collect data into the arrays, which are dimensioned
c     too small then.
c
c
c     ------------------ local variables ------------------------------
c
      integer
     $     nnx, nny, nnz, maxpacksize, maxpacksize_t2, ptarget, 
     $     ipack, iord, irod, totpacksize_t2,
     $     rodrange(2), nx, ny, nz, nxp, nrmxz, nrmyz,
     $     nxs, nys, irods, k, ipack1, ldx, ldy, ldz,
     $     j, i, ierror, nrodst1, nrodst2, ilen, nchnk,
     $     nrodst2t, b2t, na2t, dnx,b,b2,na,na2, ip

      logical inewchunk

      integer, allocatable :: 
     $     mypack(:), helper(:), destinfo(:,:), myunpack(:)
      integer, allocatable :: aofb(:), packdum(:),
     $ recvbuf(:,:,:)


      if(.not.gs%ipackinfo) then
         gs%maxpacksize_t1=-1
         gs%maxpacksize_t2=-1
         nullify(gs%packinfo)
         nullify(gs%packsize)
         nullify(gs%pickup)
         return
      endif

      nnx=gs%fftsize(1) ; ldx = nnx;
      nny=gs%fftsize(2) ; ldy = nny;
      nnz=gs%fftsize(3) ; ldz = nnz;
      m4_ifdef([CRAY],[
      m4_ifdef([T3E],[],[
      if(mod(ldx,2).eq.0) ldx=ldx+1
      if(mod(ldy,2).eq.0) ldy=ldy+1
      if(mod(ldz,2).eq.0) ldz=ldz+1])
      ])
c
c     x-dimension size of the slab obtained after the first FFT
c     and the first transpose
c
      dnx=mod(gs%fftsize(4)-gs%fftsize(5)+nnx,nnx)+1
c
c     minimum number of rows each processor holds after the first FFT
c     and the first transpose
c
      b = (dnx*nnz)/gs%nproc 
c
c     number of processors with an additional rod. The lower-number
c     processors hold the additional rods.
c
      na = mod(dnx*nnz,gs%nproc)

c
c     number of rods on this processor after first fft
c
      nrodst1 = b
      if(na.gt.gs%myproc) nrodst1=nrodst1+1
c
c     minimum number of rows each processor holds after the 2nd
c     transpose
c
      b2 = (nny*nnz)/gs%nproc 
c
c     number of processors with an additional rod after the 2nd FFT. 
c     The lower-number processors hold the additional rods.
c
      na2 = mod(nny*nnz,gs%nproc)
c
c     number of rods this processor holds
c
      nrodst2=b2
      if(na2.gt.gs%myproc)  nrodst2=nrodst2+1

c
c     figure out the maximum packet size,
c     including the packet to itself
c
      nrmxz = (dnx*nnz)/gs%nproc+1
      nrmyz = (nny*nnz)/gs%nproc+1

      maxpacksize = 0
      do ptarget=0,gs%nproc-1   ! where the packet goes
         rodrange(1)=b*ptarget+min(ptarget,na)
         rodrange(2)=rodrange(1)+b
         if(na.le.ptarget) rodrange(2)=rodrange(2)-1
         ipack=0
         do iord = 1,gs%lorder
            irod = gs%order(1,iord)
            nx = irod/nny
            ny = mod(irod,nny)
            nxp = mod(nx-gs%fftsize(5)+nnx,nnx)
            do nz=0,nnz-1
               i=nz*dnx+nxp     ! rod index
               if((i.ge.rodrange(1)).and.(i.le.rodrange(2))) then
                  ipack=ipack+1
               endif
            end do
         end do
         m4_ifdef([SHMEM],[
         if(ptarget.ne.gs%myproc) ipack=2*ipack
         ])

         if(ipack.gt.maxpacksize) maxpacksize=ipack
      end do

      call all_max_all(maxpacksize)

      gs%maxpacksize_t1=maxpacksize

      allocate(gs%packinfo(maxpacksize,gs%nproc,2))
      gs%packinfo=0
      allocate(gs%packsize(gs%nproc,2,2));      gs%packsize=0
      allocate(gs%pickup(gs%nproc,2)) ;    gs%pickup=0

      
      allocate(mypack(maxpacksize))
      allocate(helper(max(nny*nrmxz,nnx*nrmyz)))

      m4_ifdef([SHMEM],[
      allocate(myunpack(maxpacksize))
      ])

c
c     ---- set up for the 1st transposition phase of FFT ---------
c


c      write(9,*) '------ first transposition packing info----- '
c      call myflush(9)

c
c     
c
      do ptarget=0,gs%nproc-1   ! where the packet goes
c         write(9,*) ' === target proc:',ptarget
c
c        those are the rod indices held by the target processor
c
         rodrange(1)=b*ptarget+min(ptarget,na)
         rodrange(2)=rodrange(1)+b
         if(na.le.ptarget) rodrange(2)=rodrange(2)-1
         
m4_ifdef([SGI_PC],[
c
c        for better cache behavior, loop in stride one through the
c        target array.
c
c
c        first set up a helper index array to cut down operation count
c
         helper(:)=-1
         do iord = 1,gs%lorder
            irod = gs%order(1,iord)
            nx = irod/nny
            ny = mod(irod,nny)
            nxp = mod(nx-gs%fftsize(5)+nnx,nnx)
            do nz=0,nnz-1
               i=nz*dnx+nxp     ! rod index
               if((i.ge.rodrange(1)).and.(i.le.rodrange(2))) then
                  helper((i-rodrange(1))*nny+ny+1)=(iord-1)*nnz+nz+1
               endif
            end do
         end do

c
c        now pick up the data points
c
         ipack=0
         do irod=rodrange(1),rodrange(2)
            nz = irod/dnx
            nx = mod(irod,dnx)
            do ny=0, nny-1
               if(helper((irod-rodrange(1))*nny+ny+1).ge.0) then
                  ipack = ipack+1
c
c                 set up unpacking(inverse) and packing(forward) info. 
c
                  mypack(ipack)=(irod-rodrange(1))*nny+ny+1

                  k=ipack
                  m4_ifdef([SHMEM],[
                  if(ptarget.ne.gs%myproc) then
                     k=2*ipack
                     myunpack((ipack-1)*2+1) = 
     $                    (helper((irod-rodrange(1))*nny+ny+1)-1)*2
                     myunpack((ipack-1)*2+2) = 
     $                    (helper((irod-rodrange(1))*nny+ny+1)-1)*2+1
                  else
                     myunpack(ipack)=
     $                    helper((irod-rodrange(1))*nny+ny+1)
                  endif
                  ],[
                  gs%packinfo(ipack,ptarget+1,1)=
     $                 helper((irod-rodrange(1))*nny+ny+1)
                  ])
                  if(k.gt.gs%maxpacksize_t1) then
                     write(9,*)'bombout in setup_packinfo, 1st transp:',
     $                    k,gs%maxpacksize_t1
                     stop
                  endif

               endif
            end do
         end do
],[
c
c        this one works too, but is slower on the SGI PowerChallenge
c
         ipack=0
         do iord = 1,gs%lorder
            irod = gs%order(1,iord)
            nx = irod/nny
            ny = mod(irod,nny)
            nxp = mod(nx-gs%fftsize(5)+nnx,nnx)
            do nz=0,nnz-1
               i=nz*dnx+nxp     ! rod index
               if((i.ge.rodrange(1)).and.(i.le.rodrange(2))) then
                  ipack=ipack+1
c
c                 set up the unpacking info. There is no difference
c                 here between shmem and mpi.

                  mypack(ipack)=(i-rodrange(1))*ldy+ny+1
c
c                 set up the packing info. The shmem implementation
c                 requires to double the index array, and to communicate.
c                  
                  k=ipack
                  m4_ifdef([SHMEM],[
                  if(ptarget.ne.gs%myproc) then
                     k=2*ipack
                     myunpack((ipack-1)*2+1) = ((iord-1)*ldz+nz)*2
                     myunpack((ipack-1)*2+2) = ((iord-1)*ldz+nz)*2+1
                  else
                     myunpack(ipack)= (iord-1)*ldz+nz+1
                  endif 
                  ],[
                  gs%packinfo(ipack,ptarget+1,1)=(iord-1)*ldz+nz+1
                  ])
                  
                  if(k.gt.gs%maxpacksize_t1) then
                     write(9,*)'bombout in setup_packinfo, 1st transp:',
     $                    k,gs%maxpacksize_t1
                     stop
                  endif

c                  write(9,*) 'coordinate:',nx,ny,nz
c                  write(9,*) 'pack as:',(iord-1)*ldz+nz+1
c                  write(9,*) 'unpack as:',mypack(ipack)
               endif
            end do
         end do
])
c
c        communicate the unpacking info and packet size 
c        to the target processor. This is done by specifying
c        ptarget as root of the gather call.
c
m4_ifdef([MPI],[
         call mpi_gather(mypack, maxpacksize, MPI_INTEGER4, 
     $        gs%packinfo(1,1,2),maxpacksize, MPI_INTEGER4, 
     $        ptarget,  MPI_COMM_WORLD,ierror) 

         call mpi_gather(ipack, 1, MPI_INTEGER4, 
     $        gs%packsize(1,2,1),1, MPI_INTEGER4, ptarget,
     $        MPI_COMM_WORLD,ierror) 
m4_ifdef([SHMEM],[
c
c        For shmem, we need to also communicate the packing info
c     
         if(ptarget.ne.gs%myproc)       ipack=ipack*2 

         call mpi_gather(myunpack, maxpacksize, MPI_INTEGER4, 
     $        gs%packinfo(1,1,1),maxpacksize, MPI_INTEGER4, 
     $        ptarget,  MPI_COMM_WORLD,ierror) 

         call mpi_gather(ipack, 1, MPI_INTEGER4, 
     $        gs%packsize(1,1,1),1, MPI_INTEGER4, ptarget,
     $        MPI_COMM_WORLD,ierror) 

],[
         gs%packsize(ptarget+1,1,1)=ipack
])
],[
c
c        serial version is simply this
c
         gs%packinfo(1:ipack,1,2)=mypack(1:ipack)
         gs%packsize(1,2,1)=ipack
])
      end do

m4_ifdef([MPI_SCATTER],[
      allocate(aofb(gs%maxpacksize_t1))
      allocate(packdum(gs%maxpacksize_t1))
      allocate(gs%packtype(gs%nproc))
      aofb=1;                   ! all blocks are of length one

      do ip=0, gs%nproc-1
         if(ip.ne.gs%myproc) then
            packdum(:)=gs%packinfo(:,ip+1,1)-1
            call mpi_type_indexed(gs%packsize(ip+1,1,1),
     $           aofb(1),packdum(1),
     $           MPI_DOUBLE_COMPLEX, gs%packtype(ip+1), ierror)
            call mpi_type_commit(gs%packtype(ip+1), ierror)
            write(9,*) 'setup packtype:', ip,
     $           gs%packtype(ip+1),ierror
         endif
      end do
      deallocate(aofb)
      deallocate(packdum)
      ],[
      nullify(gs%packtype)
])

      m4_ifdef([SHMEM],[
      deallocate(myunpack)
      ])







c
c     ---- set up for the 2nd transposition phase of  FFT ---------
c

c     
c     startrod (not index)
c     start x/y
c     len
c     width       1       2
c     chunk(1..4,pack/unpack,whichchunk,proc)
c     

c      write(9,*) '------ second transposition packing info----- '
c      call myflush(9)

      gs%maxnchunk = gs%nproc*(gs%fftsize(3)/gs%nproc+3)
      allocate(gs%chunk(4,2,gs%maxnchunk,gs%nproc))
      allocate(destinfo(4,gs%maxnchunk))
      m4_ifdef([MPI],[allocate(recvbuf(4,gs%maxnchunk,gs%nproc))])

      destinfo=0
      gs%chunk = 0
      maxpacksize_t2 =0
      do ptarget=0,gs%nproc-1   ! where the packet goes
         rodrange(1)=b2*ptarget+min(ptarget,na2)
         rodrange(2)=rodrange(1)+b2
         if(na2.le.ptarget) rodrange(2)=rodrange(2)-1

         ipack = 0              ! initial packet length is 0
         nchnk = 0              ! assume no chunks from this proc to ptarget

         inewchunk=.true.
         do irod=0,nrodst1-1    ! loop through all rods on this proc
            i = b*gs%myproc + min(gs%myproc,na) + irod ! source rod index
            nz  = i/dnx
            nxp = mod(i,dnx)
            nx  = mod(nxp+gs%fftsize(5),nnx)
c     
c           see if a new chunk starts here
c     
            if(nx.eq.gs%fftsize(5)) inewchunk=.true.

c     
c           if yes, then find out about its length, and whether it goes to
c           ptarget at all.
c            
            if(inewchunk) then  ! this could be a new chunk. lets see...
               ilen = 0
               do ny=0,nny-1    ! run along y to see if it really is a new chunk
                  j = ny+nz*nny ! target rod index
                  if((j.ge.rodrange(1)).and.(j.le.rodrange(2))) then
c     
c     found start of new chunk
c
                     if(ilen.eq.0) then
                        if((nchnk+1).gt.gs%maxnchunk) then
                           write(9,*)
     $                          'nchnk too big in setup_packinfo'
                           stop
                        endif
                        gs%chunk(2,1,nchnk+1,ptarget+1)=ny
                        destinfo(1,nchnk+1)=j-rodrange(1) ! target rod index
                     endif
                     ilen = ilen + 1
                  endif
               end do
               if(ilen.gt.0) then ! there really is a new chunk for this processor
c                  write(9,*) ' found new chunk!'
                  nchnk = nchnk+1
                  gs%chunk(3,1,nchnk,ptarget+1) = ilen ! length of chunk
                  destinfo(3,nchnk)=ilen ! length of chunk is same for destination
c     
c     determine width of chunk
c     
                  gs%chunk(4,1,nchnk,ptarget+1) = 
     $                 min(dnx-nxp,nrodst1-irod)
                  destinfo(4,nchnk) = gs%chunk(4,1,nchnk,ptarget+1)
c     
c     this is the rod on the source processor where the chunk starts
c
                  gs%chunk(1,1,nchnk,ptarget+1) = irod
c     
c     this is the x offset for the destination processor
c
                  destinfo(2,nchnk) = nx
c
c                 increase the total packet size
c
                  ipack = ipack + ilen*gs%chunk(4,1,nchnk,ptarget+1)
               endif
               inewchunk=.false. ! go on with search for chunk start
            endif

         end do

         if(nchnk.gt.0) then
c     
c           mark last chunk by setting the width field negative!
c
            gs%chunk(4,1,nchnk,ptarget+1) = 
     $           -gs%chunk(4,1,nchnk,ptarget+1) 
            destinfo(4,nchnk) = - destinfo(4,nchnk)

         endif
c
c        communicate the unpacking info in destinfo to the target
c        processor. 
c        This is done by specifying   ptarget as root of the gather call.
c

         gs%packsize(ptarget+1,1,2)=ipack 
         m4_ifdef([MPI],[
         call mpi_gather(destinfo(1,1), gs%maxnchunk*4, MPI_INTEGER4, 
     $        recvbuf(1,1,1),gs%maxnchunk*4, MPI_INTEGER4, 
     $        ptarget,  MPI_COMM_WORLD,ierror) 
         gs%chunk(:,2,:,:)=recvbuf(:,:,:) ! massage into shape...
         call mpi_gather(ipack, 1, MPI_INTEGER4, 
     $        gs%packsize(1,2,2),1, MPI_INTEGER4, ptarget,
     $        MPI_COMM_WORLD,ierror) 
         ],[
         gs%chunk(:,2,:,1)=destinfo(:,:)
         gs%packsize(1,2,2)=ipack
         ])
         if(ipack.gt.maxpacksize_t2) maxpacksize_t2 = ipack
      end do                    ! end of loop over target processor

      call all_max_all(maxpacksize_t2)
      gs%maxpacksize_t2 = maxpacksize_t2
      m4_ifdef([MPI],[deallocate(recvbuf)])

      deallocate(destinfo)
      deallocate(helper)
      deallocate(mypack)
      gs%totpacksize_t2 = gs%maxpacksize_t2



      m4_ifdef([MPI],[
c
c     Now set up the pickup info for shmem. This small array tells
c     shmem_get where to pick up the packed data from the remote
c     node during the t2 stage.
c
c     First, we fill the helper array with the location of the
c     packets to the target processors. If there is no packet, the  
c     entry will be zero.
c
      allocate(helper(gs%nproc))
      helper=0
      ipack=1 
      do ip=1, gs%nproc-1
         ptarget=mod(ip+gs%myproc,gs%nproc) ! the target processor 
         if(gs%packsize(ptarget+1,1,2).gt.0) then
            helper(ptarget+1) =ipack
            ipack = ipack+gs%packsize(ptarget+1,1,2)
         endif
      end do
      totpacksize_t2 = ipack
c
c     now communicate the pickup information to all the other procs
c
      do ip=1, gs%nproc
         call mpi_gather(helper(ip), 1, MPI_INTEGER4, 
     $        gs%pickup(1,1),1, MPI_INTEGER4, ip-1,
     $        MPI_COMM_WORLD,ierror) 
      end do
c
c     Now the same thing for the reverse direction, i.e. the t2 stage
c     during the forward FFT. This is also a good occasion to 

      helper=0
      ipack=1 
      do ip=1, gs%nproc-1
         ptarget=mod(ip+gs%myproc,gs%nproc) ! the target processor
         if(gs%packsize(ptarget+1,2,2).gt.0) then
            helper(ptarget+1) =ipack
            ipack = ipack+gs%packsize(ptarget+1,2,2)
         endif
      end do

      gs%totpacksize_t2 = max(totpacksize_t2,ipack)

c
c     now communicate the pickup information to all the other procs
c
      do ip=1, gs%nproc
         call mpi_gather(helper(ip), 1, MPI_INTEGER4, 
     $        gs%pickup(1,2),1, MPI_INTEGER4, ip-1,
     $        MPI_COMM_WORLD,ierror) 
      end do


      deallocate(helper)
      ])
c

      end

c

