c     ============================================================
c
      subroutine spec_tet(vol,e,f,ngrid,ebound, edos, fgrid,neqtet)

      implicit none             ! nope.

c     Modified by Alberto Garcia (March 1991) to implement
c     the method in Gilat and Bharatiya, PRB 12, 3479 (1975).
c     
c     1996 additional documentation. Paul Delaney
c
c     1996 removed compton-profile specific comments and features,
c     and assimilated it into paratec. Resistance is futile...
c     Bernd Pfrommer
c     
c     ---------------------------------------------------------------
c
c     INPUT:
c     -----
c
      integer 
     $     ngrid                ! number of grid points on which energy
     $                          ! is stored
      double precision 
     $     ebound(2),           ! start(1) and end(2) energies
     $                          ! will be computed.
     $     f(4),                ! the spectral function at the corners
     $                          ! of the tetrahedron
     $     e(4),                ! energy levels at the corner of the 
     $                          ! tetrahedron
     $     vol                  ! the volume of the tetrahedron in k-space

c
c     INPUT/OUTPUT: (NOTE: THE CONTRIBUTION FROM THIS TETRAHEDRON IS ADDED)
c     ------------
      integer neqtet            ! counter for  tetrahedra with equal corners

      double precision 
     $     edos(ngrid),         ! the contribution to DOS
     $     fgrid(ngrid)         ! the contribution to DOS*F(E)
c
c     DESCRIPTION:
c     -----------
c
c     spec_tet calculates the contribution to the spectral function
c
c
c     and the DOS from one tetrahedron. This is done for energies
c     defined by ebound.
c

c
c     ------------    local variables -----------------------
c
      double precision tolerance
      parameter (tolerance=1.d-12)

      double precision delta
      parameter (delta=1.d-11)

      double precision 
     $     d21, d31, d32, d41, d42, d43,
     $     f21, f31, f32, f41, f42, f43,
     $     dn1, dn2, dn4, ei, x, s,
     $     fm, fms, denerg, estart
      integer i, j, ind(4), istart,iend
      double precision es(4), fs(4)

      external sort

      double precision xx, yy, eq_tol
      logical equal
      equal(xx,yy,eq_tol) = (abs(xx-yy) .le. eq_tol)

c
c     --------------------------------------------------------
c


c     sort energy levels at corners in increasing order

      call sort(4,e,ind)
c
      es(1) = e(ind(1))
      es(2) = e(ind(2))
      es(3) = e(ind(3))
      es(4) = e(ind(4))

c
      fs(1) = f(ind(1))
      fs(2) = f(ind(2))
      fs(3) = f(ind(3))
      fs(4) = f(ind(4))

c     if energies slightly different set them equal.
c     this is important for numerical stability

      if (equal(es(2),es(1),tolerance)) then
         es(2) = es(1)
      end if

      if (equal(es(3),es(2),tolerance)) then
         es(3) = es(2)
      end if

      if (equal(es(4),es(3),tolerance)) then
         es(4) = es(3)
      end if


      if ( es(1) .eq. es(2) .and. es(2) .eq. es(3) .and. 
     $     es(3) .eq. es(4)) then
         neqtet = neqtet +1     ! found tetrahedron with four corners the same
         return
      endif

c     find energy and matrix element differences 

      d21 = es(2) - es(1)
      d31 = es(3) - es(1)
      d32 = es(3) - es(2)
      d41 = es(4) - es(1)
      d42 = es(4) - es(2)
      d43 = es(4) - es(3)

      f21 = fs(2) - fs(1)
      f31 = fs(3) - fs(1)
      f32 = fs(3) - fs(2)
      f41 = fs(4) - fs(1)
      f42 = fs(4) - fs(2)
      f43 = fs(4) - fs(3)

c     compute denominators

c     Hold on computing Gilat's eps's, since we might divide by
c     zero...

      dn1 = (d41*d31*d21)/vol
      dn2 = (d42*d32*d21)/vol
      dn4 = (d43*d42*d41)/vol

c     Note that these D's have the volume built into them.

      denerg=dabs(ebound(2)-ebound(1))/dble(ngrid)
      estart = ebound(1)

c      write(9,*) 'corner values:',es
c
c     find start and end value for better speed
c
      istart = max(1,int((es(1)-ebound(1))/denerg)-2)
      iend   = min(ngrid-int((ebound(2)-es(4))/denerg)+2,ngrid)

      do i = istart, iend       !  loop over energy grid

         ei = estart+denerg*dble(i)
         do j=1,4
            if (equal(es(j),ei,delta)) then
               if (ei .lt. es(j)) then
                  ei=ei+2.02d0*delta
               else
                  ei=ei+1.01d0*delta
               end if
            end if
         end do

         x = ei - es(1)

c
c     distinguish the different energy regions
c

         if (ei .le. es(1)) then !   -------------     E <= E1 : do nothing
            s = 0.d0
            fms = 0.d0
c            write(9,*) 'case1:', i, ei, s
         else if (ei .le. es(2)) then ! ----------    E1 < E <= E2
            
            s = 3.d0 * x**2 /dn1
            fm = ( f21/d21 + f31/d31 + f41/d41 ) * x / 3.d0
            fms = fm * s
c            write(9,*) 'case2:', i, ei, s
         else if (ei .le. es(3)) then ! ----------    E2 < E <= E3
            
            if (es(1) .ne. es(2)) then
               s = 3.d0/dn1 * x*x  - 3.d0/dn2 * (x-d21)**2
               fms = ( f21/d21 + f31/d31 + f41/d41 ) * x**3 /dn1 
     &              - ( f21/d21 + f32/d32 + f42/d42 ) * (x-d21)**3
     $              /dn2- 3.d0 * f21 * (x-d21)**2 / dn2
            else
               s = 3.d0 * vol * x /(d41*d31) *
     &              ( 2.d0 - x*(1.d0/d31 + 1.d0/d41))
               
               fms = vol * x / (d41*d31) *  (
     &              3.d0 * f21 +
     &              3.d0 * x * ( f32/d32 + f42/d42) -
     &              x**2 / (d41*d31) * (
     &              d41 * (f31/d31+f32/d32+f41/d41) 
     &              + d31 * (f31/d31+f41/d41+f42/d42)
     &              - f21 )          
     &              )
            endif
c            write(9,*) 'case3:', i, ei, s
         else if (ei .le. es(4)) then ! -----------  E3 < E <= E4

            s = 3.d0 * (x-d41)**2 / dn4
            fm = (f41/d41+f42/d42+f43/d43) * (x-d41) / 3.d0 + f41
            fms = fm * s
c            write(9,*) 'case4:', i, ei, s
         else                   !----   E > E4 : increment integrated DOS only
            s = 0.d0
            fms = 0.d0
c            write(9,*) 'case5:', i, ei, s
         endif

         edos(i)  = edos(i) + s
         fgrid(i) = fgrid(i) + (fs(1)*s + fms)

      end do
      
      return
      
      end
      
