
      subroutine start_vectors(ipr, pw_params,
     $     ham, densham, eval, neig, xvec, 
     $     pot_gspace, sub_gspace, vion, crys)

      include 'use.h'
      use flibcalls_module
      use all_to_all_module
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'flibcalls.ph'
      include 'all_to_all.h'

c     --------------------  arguments ------------------------------------

c     INPUT:
c     -----
      
      integer 
     $     ipr,                 ! print flag
     $     neig                 ! number of eigenvectors to search for.
     $                          ! this is the number of approximate eigenvecs
     $                          ! returned in the array xvecs
      

      type(pw_parameter) pw_params ! several plane wave parameters 

      type (parallel_gspace)
     $     pot_gspace,          ! the gspace structure for the potential
     $     sub_gspace           ! the gspace structure for the submatrix

      complex(kind=8)
     $     vion(*)              ! screened ionic potential in fourier space

      type (hamiltonian)
     $     ham                  ! all information necessary for the hamiltonian

      type (crystal)
     $     crys                 ! crystal structure info,e.g. inv symmetry


c
c     OUTPUT:
c     ------

      type (dense_hamiltonian)  ! a structure containing the spectrum etc.
     $     densham

      double precision eval(neig) ! the approximate eigenvalues on return

      complex(kind=8)
     $     xvec(*)              ! approx. eigenvectors, on the full gspace 
     $                          ! xvec(neig*ham%gspace%length)


c
c     DESCRIPTION:
c     -----------
c
c     Generates approximate eigenvectors in the full G-space by
c     diagonalizing a Hamiltonian set up only on a smaller gspace.
c     The approximate eigenvectors are then expanded onto the full G
c     -space by either filling in zeroes, or throwing in random
c     numbers. 
c
c     The full spectrum of the approximate eigenvectors is returned 
c     in densham%matrix. If scalapack is used, only those components
c     and eigenvectors that belong to this processor are stored in
c     densham%matrix. The component densham%energy will have the full
c     spectrum in the subspace.
c      
c     1997 Bernd Pfrommer, Andrew Canning 

c
c     --------------------- local variables ------------------------------
c
      integer
     $     gr(3), gc(3), 
     $     ngc, ngr,            ! number of gvecs in cols/rows on this proc
     $     b,  na, ngv, 
     $     n,                   ! submatrix size
     $     ibc, ibr,            ! block counters
     $     ic, ir,              ! col/row counters
     $     igc, igr,            ! col/row gvec counters
     $     nbc,                 ! number of blocks in one column
     $     nbr,                 ! number of blocks in one row
     $     nb, nprow, npcol
      integer
     $     nprocblacs,          ! number of procs as blacs sees it
     $     myprow, mypcol,      ! myprocs place in blacs proc grid
     $     myproc, nproc,
     $     idiff,               ! index into G-G' array
     $     ngdiff,              ! number of G-G' vectors needed on this proc
     $     ngdiffmax,           ! max num of G-G' vectors needed on this proc
     $     ndsqh, ndsq, worksubtot,
     $     idum, i1, i2, i,j,k,l, subcomm, info,
     $     dg(3), nfft(3), kk, ngmax, 
     $     ioutput              ! output flags


      double precision 
     $     t0, gimmetime,   tdiag, x,y,alf,bet, zbuff_real, zbuff_imag
      complex(kind=8)
     $     hsum, vnonloc, dcdum, vloc,phfac


      double precision, allocatable ::    
     $     rworksub(:)          ! size 3*mxdsdim, for matrix diagonalization


      complex(kind=8), allocatable ::
     $     vnlsubr(:,:),        ! the nonlocal projectors for the subspace
     $     vnlsubc(:,:),        ! the nonlocal projectors for the subspace
     $     worksub(:)

      integer, allocatable :: 
     $     wfnlist(:),          ! which wfns are on this proc
     $     glist(:),            ! ordering list for G-G' vectors in subspace
     $     glistr(:),           ! ordering list for G vectors in subspace
     $     glistc(:)            ! ordering list for G' vectors in subspace
      external findbound, regspace, gimmetime


c     ---------  sanity checks etc  --------------------------------
c
      ioutput=pw_params%output
      myproc = pot_gspace%myproc; nproc = pot_gspace%nproc
      n = sub_gspace%length
      

      t0=gimmetime()

      if(neig.gt.sub_gspace%length) then
         write(9,920) sub_gspace%length,neig
         stop
      endif
      if(sub_gspace%length.gt.densham%gspace%length) then
         write(9,*)  'dense hamiltonian is too small for start vectors!'
      endif

c     ---------  do the parallel data layout  -----------------------

c
c     generate list of G-G' vectors required to set up those
c     components of H_sub(G,G') residing on this processor
c
      m4_ifdef([SCALAPACK],[
      call layout_scalapack(n, nb, nproc, nprow, npcol)
 
      call blacs_gridinfo(blacs_context, nprow, npcol, myprow, mypcol) ! figure where myproc is in proc grid
      write(9,400) n,nprow, npcol, nb

      if(myprow.eq.-1) then
         write(9,*) '*** start_vectors: BLACS ERROR on proc', myproc
         stop
      endif
      ],[
      worksubtot= 3*sub_gspace%length
      allocate(worksub(worksubtot))
      allocate(rworksub(3*sub_gspace%length))
      nb=1; nprow=1; npcol=1; mypcol=0; myprow=0;
      ])
c
c
c     figure out number of blocks per processor in the column/row
c
      nbc = n/(nb *npcol) ; if(mod(n,(nb*npcol)).gt.mypcol*nb) nbc=nbc+1
      nbr = n/(nb *nprow) ; if(mod(n,(nb*nprow)).gt.myprow*nb) nbr=nbr+1
c
c     ------- set up the local potential from large to small gspace --------
c
c
      ngmax     = nb*max(nbc,nbr) ! upper bound for g list
      nfft=pot_gspace%fftsize(1:3)
c
c     set up the list of column gvectors
c
      allocate(glistc(ngmax))
      ngc =0
      do ibc=0, nbc-1           ! loop over column blocks
         do ic=(ibc*npcol+mypcol)*nb+1, 
     $        min((ibc*npcol+mypcol)*nb+nb,n) 
            ngc = ngc + 1
            gc(1:3) = sub_gspace%gvec(1:3,ic)
            glistc(ngc) =  mod(gc(3)+nfft(3), nfft(3))
     $           + mod(gc(2)+nfft(2), nfft(2))*nfft(3)
     $           + mod(gc(1)+nfft(1), nfft(1))*nfft(3)*nfft(2)
         end do
      end do
c
c     set up the list of row gvectors as well
c
      allocate(glistr(ngmax))
      ngr =0
      do ibr =0, nbr-1          ! loop over row blocks
         do ir=(ibr*nprow+myprow)*nb+1, 
     $        min((ibr*nprow+myprow)*nb+nb,n) ! loop over row
            ngr = ngr + 1
            gr(1:3) = sub_gspace%gvec(1:3,ir)
            glistr(ngr) =  mod(gr(3)+nfft(3), nfft(3))
     $           + mod(gr(2)+nfft(2), nfft(2))*nfft(3)
     $           + mod(gr(1)+nfft(1), nfft(1))*nfft(3)*nfft(2)
         end do
      end do

c
c     now set up list of g-g' vectors
c
      ngdiffmax=ngc*ngr         ! should be exact
      allocate(glist(ngdiffmax))

      ngdiff=0 ;
      do ibc=0, nbc-1           ! loop over column blocks
         do ic=(ibc*npcol+mypcol)*nb+1, min((ibc*npcol+mypcol)*nb+nb,n) ! loop over column
            do ibr =0, nbr-1    ! loop over row blocks
               do ir=(ibr*nprow+myprow)*nb+1, 
     $              min((ibr*nprow+myprow)*nb+nb,n) ! loop over row
                  dg = sub_gspace%gvec(:,ir)- sub_gspace%gvec(:,ic) 
                  ngdiff = ngdiff + 1
                  if(ngdiff.gt.ngdiffmax) then
                     write(9,*) 'start_vectors: ngdiff too large:'
     $                    ,ngdiff
                     stop
                  endif
                  glist(ngdiff) =  mod(dg(3)+nfft(3), nfft(3))
     $                 + mod(dg(2)+nfft(2), nfft(2))*nfft(3)
     $                 + mod(dg(1)+nfft(1), nfft(1))*nfft(3)*nfft(2)
               end do
            end do
         end do
      end do

      if(associated(densham%matrix))
     $     deallocate(densham%matrix) ! make sure we have no leak...
      allocate(densham%matrix(ngr*ngc))
c
c     reset local potential from pot_gspace to 2*sub_gspace
c
      call get_gset(ngdiff, glist(1), 1, densham%matrix(1),
     $     2*sub_gspace%gmax,  pot_gspace, vion(1)) 
c
c     reset the nonlocal potential projectors from the normal gspace
c     onto the sub gspace
c

      if(ham%vnloc%nvecs.gt.0) then
         allocate(vnlsubr(ngr,ham%vnloc%nvecs))
         call get_gset(ngr, glistr(1), ham%vnloc%nvecs, vnlsubr(1,1),
     $        sub_gspace%gmax, ham%gspace, ham%vnloc%data(1,1,1)) 
         allocate(vnlsubc(ngc,ham%vnloc%nvecs))
         call get_gset(ngc, glistc(1), ham%vnloc%nvecs, vnlsubc(1,1),
     $        sub_gspace%gmax, ham%gspace, ham%vnloc%data(1,1,1)) 
         
      endif



c
c     --------- construct Hamiltonian matrix in subspace -----------------
c
      allocate(wfnlist(neig))



      igc=0 ;   idiff =0
      do ibc=0, nbc-1           ! loop over column blocks
         do ic=(ibc*npcol+mypcol)*nb+1, min((ibc*npcol+mypcol)*nb+nb,n) 
            igc = igc + 1
            if(igc.le.neig) wfnlist(igc) = ic
            igr =0
            do ibr =0, nbr-1    ! loop over row blocks
               do ir=(ibr*nprow+myprow)*nb+1, 
     $              min((ibr*nprow+myprow)*nb+nb,n) ! loop over row
                  igr = igr + 1

                  idiff = idiff +1
                  vloc = densham%matrix(idiff) ! local part: V(G_col-G_row)
                  vnonloc = (0.d0,0.d0)
                  do kk=1, ham%vnloc%nvecs
                     vnonloc = vnonloc +
     $                    vnlsubr(igr,kk)*conjg(vnlsubc(igc,kk))
     $                    * ham%xnorm(kk)
                  end do
                  if(ir.eq.ic) then ! add kinetic energy for the diagonal
                     hsum = sub_gspace%ekin(ir) + vnonloc
                  else
                     hsum = vloc + vnonloc ! loc pot if off-diagonal
                  endif
                  densham%matrix(igr+(igc-1)*ngr) = hsum
c                  write(9,410) ir,ic, hsum, vloc, vnonloc
               end do
            end do
         end do
      end do
      call myflush(9)

c     ------------ now diagonalize the submatrix -------------------------
 
      tdiag = gimmetime() 

      if(.not.crys%icomplex) then ! if system has inversion symm, pack hsub -
         ndsq=ngc*ngr
         ndsqh=ndsq/2

         if(ipr.ge.2) then
            write(9,*)  'hsub:'
            do i=1,ngr
               write(9,'(2000f20.14)') 
     $              (dble(densham%matrix(i+(j-1)*ngr)), 
     $              j=1,ngc)
            end do
         endif


c        pack hsub: Remove all the imaginary parts to prepare call to
c                   real symmetric eigensolver

         do i=0,ndsqh-1
            densham%matrix(i+1)=cmplx(dble(densham%matrix(2*i+1)),
     $           dble(densham%matrix(2*i+2)),kind=8)
         end do
         
         if(mod(ndsq,2).eq.1) then
            densham%matrix(ndsqh+1)=
     $           cmplx(dble(densham%matrix(ndsq)),0.d0,kind=8)
         endif

c        diagonalize it
         m4_ifdef([SCALAPACK],[
         call rdiag_scalapack(blacs_context,n,nb,nprow,npcol,
     $        densham%matrix(1), ngc,ngr,densham%energy(1))
         ],[
         call mdsyev('V','L',sub_gspace%length, 
     $        densham%matrix,sub_gspace%length,densham%energy, 
     $        rworksub,  3*sub_gspace%length,idum)
         ])
c        now we have to unpack it from real to complex again

c        ! the if statement must go first!


         if(mod(ndsq,2).eq.1) then
            densham%matrix(ndsq)=
     $           cmplx(dble(densham%matrix(ndsqh+1)),0.d0,kind=8)
         endif

         do i=ndsqh-1,0,-1
            zbuff_real = dble(densham%matrix(i+1))
            zbuff_imag = aimag(densham%matrix(i+1))
            
            densham%matrix(2*i+1)=zbuff_real
            densham%matrix(2*i+2)=zbuff_imag
         end do


         if(ipr.ge.2) then
            write(9,*)  'packed eigenvectors hsub (column vectors):'
            do i=1,ngr
               write(9,'(3f20.14)') 
     $              (dble(densham%matrix(i+(j-1)*ngr)), 
     $              j=1,ngc)
            end do
         endif


      else                      ! the complex case is easier. 
         m4_ifdef([SCALAPACK],[
         call zdiag_scalapack(blacs_context,n,nb,nprow,npcol,
     $        densham%matrix(1), ngc,ngr,densham%energy(1))
c        
c        enforce the phase
c
c         call all_split_all(mypcol,iam,subcomm)

c         do igc =1,ngc
c            if(myprow.eq.0) then
c               x = dble(densham%matrix((igc-1)*ngr+1))
c               y = aimag(densham%matrix((igc-1)*ngr+1))
c               if(abs(y).gt.1d-10) then
c                  bet = 1/sqrt(1+(x/y)**2)
c                  alf = -x*bet/y
c                  phfac = cmplx(alf,bet,kind=8)
c               else
c                  phfac = cmplx(1.d0,0.d0,kind=8)
c               endif
c            else
c               phfac =0
c            endif
c            call all_sum_all_sub(subcomm,phfac)
c            densham%matrix((igc-1)*ngr+1:igc*ngr)=
c     $           densham%matrix((igc-1)*ngr+1:igc*ngr)*phfac

c         end do
c         call mpi_comm_free(subcomm, info)


         ],[          
          m4_ifdef([CRAY],[
          call zdiagh(sub_gspace%length, densham%matrix,
     $         sub_gspace%length, densham%energy, 
     $         worksub(1), worksubtot, rworksub(1))
          ],[
          m4_ifdef([SUN],[
          call zdiagh(sub_gspace%length, densham%matrix,
     $         sub_gspace%length, densham%energy, 
     $         worksub(1), worksubtot, rworksub(1))
          ],[
          m4_ifdef([RS6K],[
          call zdiagh(sub_gspace%length, densham%matrix,
     $         sub_gspace%length, densham%energy, 
     $         worksub(1), worksubtot, rworksub(1))
          ],[
          call zdiagh(sub_gspace%length, densham%matrix(1),
     $          sub_gspace%length, densham%energy(1), 
     $          worksub(1), worksubtot, rworksub(1))
          ])
          ])
          ])

         ])

      endif

      tdiag = gimmetime() - tdiag

      if(ipr.ge.1) then
         write(9,100)
         write(9,110) (densham%energy(l),l=1, neig)
         call myflush(9)
      endif
      densham%neig=sub_gspace%length ! have all of them

c     ---------- expand the packed eigenvectors into arrays ------------

c      do j=0, neig-1 ! check norm
c         hsum =0
c         do i=1,sub_gspace%length         
c            hsum=hsum+densham%matrix(j*sub_gspace%length+i)*
c     $           conjg(densham%matrix(j*sub_gspace%length+i))
c         end do
c         call all_sum_all(hsum)
c         write(9,*) 'unexpanded norm',j+1,hsum
c      end do

      xvec(1:neig*ham%gspace%length) =0

      call put_gset(ngr, glistr,min(neig,ngc), wfnlist,
     $     densham%matrix(1), sub_gspace%gmax, ham%gspace,xvec(1),neig)
      
c      do j=0, neig-1 ! check norm
c         hsum =0
c         do i=1,ham%gspace%length         
c            hsum=hsum+xvec(j*ham%gspace%length+i)*
c     $           conjg(xvec(j*ham%gspace%length+i))
c         end do
c         call all_sum_all(hsum)
c         write(9,*) 'expanded norm',j+1,hsum
c      end do

      deallocate(glistr)
      deallocate(glistc)
      deallocate(glist)
      deallocate(wfnlist)

      eval(1:neig) = densham%energy(1:neig)



      if(pw_params%random_startvec.gt.0) 
     $     call randomize_startguess(crys, ham%gspace,
     $     sub_gspace%gmax, pw_params%random_startvec, neig, xvec(1))

         


      if(ipr.ge.3) then
         write(9,300)
         do j=1, ham%gspace%length
            if(.not.crys%icomplex) then
               write(9,310) (dble(xvec((l-1)*ham%gspace%length+
     $              j)),l=1, neig)
            else
               write(9,310) 
     $              (xvec((l-1)*ham%gspace%length+j),l=1, neig)
            endif
         end do
      endif

      m4_ifdef([SCALAPACK],[],[
      deallocate(rworksub)
      deallocate(worksub)
      ])

      if(ham%vnloc%nvecs.gt.0) then
         deallocate(vnlsubr)
         deallocate(vnlsubc)
      endif

      if(iand(ioutput,65536).eq.65536) then
         write(9,930) 
     $        2*(ngmax+ngdiffmax)*8+16*ngdiff
     $        +(ngc+ngr)*ham%vnloc%nvecs*16
      endif
      if(iand(ioutput,8).eq.8) then
         write(9,910) sub_gspace%length,tdiag,gimmetime()-t0
         call myflush(9)
      endif
      
      return


 100  format (/' Eigenvalues from submatrix diagonalization:',
     $     /' -------------------------------------------')
 110  format (4f19.14)
 200  format ('Eigenvectors from submatrix diagonalization:',
     $     /'-------------------------------------------')
 210  format (1008f9.3)
      
 300  format ('Startvectors after unpacking:',
     $     /'-------------------------------------------')
 310  format (400f20.14)

 400  format (' START GUESS SIZE ',i5,
     $     /' PROCESSOR GRID ',i3,' x ',i3,' BLOCKSIZE ',i3)
 410  format (2i5,8f20.12)

 910  format(' START GUESS SIZE ',i4,' TOOK ',
     $     f8.2,' SECONDS FOR DIAG, AND ',f8.2,' TOTAL')
         
 920  format(' *** ERROR: the subspace for the starting guess',
     $     /' ***        has size ',i5,', but you have asked for',i5,
     $     /' ***        bands to be computed. Increase the energy',
     $     /' ***        cutoff of the subspace.')

 930  format(' START GUESS MEMORY USAGE IS ',i10,' BYTES')

      end


