c-*-Fortran-*-
c
c     Fortran 90 structures.
c     
c     (1996) Bernd Pfrommer
c     
c     Just include the modules you need
c

c
c     ========== parallel gspace structure ==========================
c
      module parallel_gspace_module

      implicit none
c
c     first some public data which is global to ALL GSPACES
c
      integer blacs_context     ! the blacs context (processor grid layout)

      type parallel_gspace
      sequence
c     ------ true information --------------

      double precision  gmax    ! cutoff length
      double precision rk(3)    ! the k-vector by which it is shifted
      integer length            ! number of g-space vectors on this proc
      integer totlength         ! total number of g-space vectors (all procs)

      integer nstar             ! total number of stars in gspace
      integer ig0               ! index of the g=0. If not on this proc,=-1

      integer ntype             ! number of types of atoms
      logical igvec             ! if gvectors are available

      logical iekin             ! if kinetic energy is available
      logical iinverse          ! if inverse index array is available

      logical istruc            ! if structure factors are there
      logical istar             ! if star arrays are available

      integer, pointer :: gvec(:,:) ! the gspace vectors themselves
      integer, pointer :: inverse(:) ! index to inverse of G. only for local.
      double precision, pointer :: ekin(:) ! kinetic energy |k+G|^2
      complex(kind=8), pointer :: struc(:,:) ! struc factors struc(ntype,length)
      integer, pointer :: inds(:) ! maps gvecs to stars inds(length)
      integer, pointer :: mstar(:) ! size of stars mstar(nstar)
      complex(kind=8), pointer :: phase(:) ! phases(length)

c     ------ parallel information ----------
      integer, pointer :: order(:,:) ! orders the gspace layout order(3,lorder)
      integer, pointer :: fastfind(:,:) ! help array for fastfind(3,fftsize(1))
      integer lorder            ! length of order array
      integer nproc             ! number of procs for gspace
      integer myproc            ! myproc for this gspace

c     ------ FFT information ---------------

      integer fftsize(5)        ! FFT grid boundaries for gspace
      integer naux(4)           ! the size of the four workspaces
      integer r_size            ! the size of the complementary realspace array

      integer maxpacksize_t1    ! sizeof 1st dim of packinfo (1st transp)
      integer maxpacksize_t2    ! maximum packet size for second transpose

      integer totpacksize_t2    ! total size of all packets sent during t2
      integer maxnchunk         ! dimension of 2nd index of chunk array
      integer ipad(1)

      integer, pointer :: packinfo(:,:,:) ! packing/unpacking information
      integer, pointer :: chunk(:,:,:,:) ! chunk(4,pack/unpack,maxnchunk,nprocs)
      integer, pointer :: packsize(:,:,:) ! see setup_packinfo routine
      integer, pointer :: pickup(:,:) ! pickup(nproc,inverse/forward)
      integer, pointer :: packtype(:) ! mpi data type array

      character*16 name         ! a string identifying the gspace
      logical ipackinfo         ! if packinfo is there or not
      integer ipad2(1)

      end type

      contains
      subroutine destroy_parallel_gspace(gs)
      type (parallel_gspace) gs
      if(associated(gs%order)) deallocate(gs%order)
      if(associated(gs%gvec)) deallocate(gs%gvec)
      if(associated(gs%ekin)) deallocate(gs%ekin)
      if(associated(gs%inds))deallocate(gs%inds)
      if(associated(gs%mstar)) deallocate(gs%mstar)
      if(associated(gs%phase))deallocate(gs%phase)
      if(associated(gs%inverse))deallocate(gs%inverse)
      if(associated(gs%struc))deallocate(gs%struc)
      if(associated(gs%packinfo)) deallocate(gs%packinfo)
      if(associated(gs%packsize)) deallocate(gs%packsize)
      if(associated(gs%fastfind)) deallocate(gs%fastfind)
      if(gs%ipackinfo .and. associated(gs%packsize)) 
     $     deallocate(gs%chunk)
      if(gs%ipackinfo .and. associated(gs%pickup)) 
     $     deallocate(gs%pickup)
      if(gs%ipackinfo .and. associated(gs%packtype)) 
     $     deallocate(gs%packtype)
      end subroutine

c     assign two gspaces to each other

      subroutine parallel_gspace_assignment(gt,gs) 
      type(parallel_gspace), intent(out) :: gt
      type(parallel_gspace), intent(in) :: gs
      gt%gmax=gs%gmax
      gt%rk=gs%rk
      gt%length=gs%length
      gt%totlength=gs%totlength
      gt%nstar=gs%nstar

      gt%igvec=gs%igvec
      if(gs%igvec.and.associated(gs%gvec)) then
         allocate(gt%gvec(3,gs%length))
         gt%gvec=gs%gvec
      endif
      gt%iekin=gs%iekin
      if(gs%iekin.and.associated(gs%ekin)) then
         allocate(gt%ekin(gs%length))
         gt%ekin(:)=gs%ekin(:)
      endif

      gt%iinverse=gs%iinverse
      if(gs%iinverse.and.associated(gs%inverse)) then
         allocate(gt%inverse(gs%length))
         gt%inverse(:)=gs%inverse(:)
      endif

      gt%istruc=gs%istruc
      gt%ntype=gs%ntype
      if(gs%istruc.and.associated(gs%struc)) then
         allocate(gt%struc(gs%ntype,gs%length))
         gt%struc(:,:)=gs%struc(:,:)
      endif
      gt%istar=gs%istar
      if(gs%istar) then
         allocate(gt%inds(gs%length))
         gt%inds(:)=gs%inds(:)
         allocate(gt%mstar(gs%nstar))
         gt%mstar(1:gs%nstar)=gs%mstar(1:gs%nstar)
         allocate(gt%phase(gs%length))
         gt%phase(:)=gs%phase(:)
      endif
      gt%lorder=gs%lorder
      allocate(gt%order(3,gs%lorder))
      gt%order(:,:)=gs%order(:,:)
      gt%fastfind(:,:)=gt%fastfind(:,:)
      gt%nproc=gs%nproc
      gt%myproc=gs%myproc
      gt%fftsize=gs%fftsize
      gt%naux=gs%naux
      gt%r_size=gs%r_size
      gt%name=gs%name
      gt%maxpacksize_t1=gs%maxpacksize_t1
      gt%maxpacksize_t2=gs%maxpacksize_t2

      gt%ipackinfo=gs%ipackinfo
      if(gs%ipackinfo.and.associated(gs%packinfo)) then
         allocate(gt%packinfo(gs%maxpacksize_t1,gs%nproc,2))
         gt%packinfo=gs%packinfo
      endif
      if(gs%ipackinfo.and.associated(gs%packsize)) then
         allocate(gt%packsize(gs%nproc,2,2))
         gt%packsize=gs%packsize
      endif
      if(gs%ipackinfo.and.associated(gs%chunk)) then
         allocate(gt%chunk(4,2,gs%maxnchunk,gs%nproc))
         gt%chunk=gs%chunk
      endif
      if(gs%ipackinfo.and.associated(gs%pickup)) then
         allocate(gt%pickup(gs%nproc,2))
         gt%pickup=gs%pickup
      endif
      if(gs%ipackinfo.and.associated(gs%packtype)) then
         allocate(gt%packtype(gs%nproc))
         gt%packtype=gs%packtype
      endif

      end subroutine

      end module  parallel_gspace_module





c
c     ==================== FFT structure =============================
c
      module fftstruc_module
      use parallel_gspace_module
      type fftstruc
      sequence
      integer fftsize(3)        ! the three fftgrid sizes
      integer r_size            ! the size of the realspace array
      integer naux(4)           ! the size of the four workspaces
      complex(kind=8), pointer :: unfoldbuf(:) ! unfolding buffer 
      complex(kind=8), pointer :: rspacebuf(:) ! for convolutions etc
      complex(kind=8), pointer :: t1buf(:) ! buffer for intermediate fft
      complex(kind=8), pointer :: sendbuf(:) ! send buffer for mpi
      complex(kind=8), pointer :: recbuf(:) ! receive buffer for mpi
      complex(kind=8), pointer :: workspace(:) ! space for call to local fft
      integer, pointer :: mpireq(:) ! space for mpi requests
      integer, pointer :: mpistat(:) ! space for mpi status
      integer unfoldbufsize      ! sizes of the various buffers
      integer t1bufsize
      integer sendbufsize
      integer recbufsize
      end type 

      contains

      subroutine destroy_fftstruc(ffts)
      type(fftstruc) ffts
      if(associated(ffts%unfoldbuf)) deallocate(ffts%unfoldbuf)
      if(associated(ffts%rspacebuf)) deallocate(ffts%rspacebuf)
      if(associated(ffts%t1buf)) deallocate(ffts%t1buf)
      if(associated(ffts%sendbuf)) deallocate(ffts%sendbuf)
      if(associated(ffts%recbuf)) deallocate(ffts%recbuf)
      if(associated(ffts%workspace)) deallocate(ffts%workspace)
      if(associated(ffts%mpireq)) deallocate(ffts%mpireq)
      if(associated(ffts%mpistat)) deallocate(ffts%mpistat)
      end subroutine

      subroutine create_fftstruc(ffts,gs) ! allocate memory for fft
      integer mpistatsize, alcstat
      type(fftstruc) ffts
      type(parallel_gspace) gs

      ffts%fftsize=gs%fftsize(1:3)

      ffts%r_size = (gs%fftsize(2)*gs%fftsize(3))/gs%nproc

      if( mod(gs%fftsize(2)*gs%fftsize(3),gs%nproc).gt.gs%myproc)
     $     ffts%r_size=ffts%r_size+1
      ffts%r_size=ffts%r_size*gs%fftsize(1)


      allocate(ffts%rspacebuf(ffts%r_size),stat=alcstat) ! realspace buffer
      call alccheck('rspacebuf',ffts%r_size,alcstat)

      ffts%sendbufsize=max(gs%maxpacksize_t1,gs%maxpacksize_t2,
     $     gs%totpacksize_t2)
      nullify(ffts%sendbuf)
      if(ffts%sendbufsize.gt.0) 
     $     allocate(ffts%sendbuf(ffts%sendbufsize),stat=alcstat)         
      call alccheck('sendbuf',ffts%sendbufsize,alcstat)

      ffts%recbufsize = max(gs%maxpacksize_t1,gs%maxpacksize_t2)
      nullify(ffts%recbuf)
      if(ffts%recbufsize.gt.0) 
     $     allocate(ffts%recbuf(ffts%recbufsize),stat=alcstat)
      call alccheck('recbuf',ffts%recbufsize,alcstat)

      call fft_workspace(gs%nproc,ffts%fftsize,ffts%naux,
     $     ffts%unfoldbufsize, ffts%t1bufsize,mpistatsize) !size of fft arrays

      allocate(ffts%mpireq(gs%nproc))
      allocate(ffts%mpistat(gs%nproc*mpistatsize))
      allocate(ffts%t1buf(ffts%t1bufsize),stat=alcstat)
      call alccheck('t1buf',ffts%t1bufsize,alcstat)

      allocate(ffts%workspace(ffts%naux(1)+
     $     ffts%naux(2)+ffts%naux(3)+ffts%naux(4)))
      allocate(ffts%unfoldbuf(ffts%unfoldbufsize),stat=alcstat)
      call alccheck('unfoldbuf',ffts%unfoldbufsize,alcstat)

      call fft_iworkspace(ffts%workspace(1), ffts%naux(1), 
     $     ffts%fftsize(1))


      end subroutine


      end module fftstruc_module

c
c     ==================== crystal structure =========================
c
      module crystal_module
      use iolib_module
      type crystal
      sequence
      double precision avec(3,3) ! lattice vectors in cartesian coordinates
      double precision adot(3,3) ! adot(i,j) = a_i dot a_j
      double precision bvec(3,3) ! reciprocal lattice vectors
      double precision bdot(3,3) ! b_i dot b_k for reciprocal lattice vectors
      double precision, pointer :: szet(:) ! spin polarization szet(ntype)
      double precision, pointer :: dipole(:) ! dipole strength dipole(ntype)
c
c     positions of the atoms rat(3,mxdatm, ntype), with respect to the
c     lattice vectors, multiplied by a factor of 2 pi

      double precision, pointer :: rat(:,:,:) 
      double precision ztot     ! total charge of all atoms
      double precision, pointer ::  zv(:) ! valence charge zv(ntype)
      double precision, pointer ::  mtsphere(:) ! size of mt sphere in a.u.
      double precision vcell             ! unit cell volume
      double precision totalmass ! the total mass of all the atoms in [u]


      integer, pointer :: natom(:) ! number of atoms of each type natom(ntype)
      integer nspin             ! number of spins
      integer ntype             ! number of types
      integer mxdatm            ! maximum number of atoms of one type
      logical icomplex          ! false if crystal has inversion symmetry
      character(len=2), pointer :: nameat(:) ! names of atoms nameat(ntype)

      end type 

      contains
      subroutine destroy_crystal(crys)
      type(crystal) crys
      if(associated(crys%rat)) deallocate(crys%rat)
      if(associated(crys%natom))deallocate(crys%natom)
      if(associated(crys%nameat))deallocate(crys%nameat)
      if(associated(crys%szet)) deallocate(crys%szet)
      if(associated(crys%dipole)) deallocate(crys%dipole)
      if(associated(crys%zv)) deallocate(crys%zv)
      if(associated(crys%mtsphere)) deallocate(crys%mtsphere)
      end subroutine

      subroutine create_crystal(crys,mxdatm,ntype,rat,natom,nameat,
     $     avec,szet, mtsphere, dipole, nspin)
      type(crystal) crys
      integer mxdatm, ntype, natom(ntype),nspin,i
      double precision rat(3,mxdatm,ntype), avec(3,3), 
     $     szet(ntype), mtsphere(ntype), dipole(ntype)
      character*2 nameat(ntype)

      allocate(crys%rat(3,mxdatm,ntype))
      allocate(crys%natom(ntype))
      allocate(crys%nameat(ntype))
      allocate(crys%szet(ntype))
      allocate(crys%zv(ntype))
      allocate(crys%mtsphere(ntype))
      allocate(crys%dipole(ntype))

      crys%ntype=ntype; crys%mxdatm=mxdatm ; crys%nspin=nspin;
      crys%avec=avec
      crys%natom=natom
      crys%szet=szet
      crys%mtsphere=mtsphere
      crys%dipole =dipole
      crys%nameat=nameat
      crys%rat=rat

      end subroutine

      subroutine write_crystal(fname, crys) ! ------- iolib write crystal ----
      implicit none

      character*(*) fname       ! the filename
      type(crystal) crys        ! the crystal structure to write

      integer   mymarker, info, i,j,k,l
      parameter(mymarker=7)

      open(unit=21,file=fname,
     $     position = 'append',
     $     status='unknown',form='formatted')

      write(21,*) mymarker, ' crystal'
      write(21,*) crys%avec
      write(21,*) crys%adot
      write(21,*) crys%bvec
      write(21,*) crys%bdot
      write(21,*) crys%ztot, crys%vcell, crys%nspin, 
     $     crys%ntype, crys%mxdatm, crys%icomplex

      write(21,*) crys%nameat
      write(21,*) crys%natom
      write(21,*) crys%szet
      write(21,*) crys%zv
      write(21,*) crys%mtsphere
      write(21,*) crys%rat
      close(21)
      end subroutine
c
c
c
      subroutine read_crystal(fname,crys)

      implicit none             ! implicit? no!

      character*(*) fname       ! the filename
      type (crystal) crys


      integer mymarker, info, i,j,k,l
      parameter(mymarker=7) 

      open(unit=21,file=fname,
     $     status='unknown',form='formatted')
      call iolib_find_marker(mymarker,21,info)
      if(info.eq.0) then
         write(0,*) 'iolib: could not find marker:', mymarker
         stop
      endif

      read(21,*,end=102) crys%avec
      read(21,*,end=102) crys%adot
      read(21,*,end=102) crys%bvec
      read(21,*,end=102) crys%bdot
      read(21,*,end=102) crys%ztot, crys%vcell, crys%nspin,
     $     crys%ntype, crys%mxdatm, crys%icomplex

      allocate(crys%rat(3,crys%mxdatm,crys%ntype))
      allocate(crys%natom(crys%ntype))
      allocate(crys%nameat(crys%ntype))
      allocate(crys%szet(crys%ntype))
      allocate(crys%zv(crys%ntype))
      allocate(crys%mtsphere(crys%ntype))
      
      read(21,*,end=102) crys%nameat
      read(21,*,end=102) crys%natom
      read(21,*,end=102) crys%szet
      read(21,*,end=102) crys%zv
      read(21,*,end=102) crys%mtsphere
      read(21,*,end=102) crys%rat

      close(21)
      return


 102  continue
      write(0,*) 'iolib: reached eof before reading object:', 
     $     mymarker
      stop


      end subroutine
      


      end module crystal_module


c
c     ==================== symmetry structure =========================
c
      module symmetry_module
      use iolib_module
      type symmetry
      sequence
      double precision tnp(3,48) ! the nonprimitive translations
      double precision rsymmat(3,3,48) ! the cartesian symmetry transformations
      integer mtrx(3,3,48)      ! the transformation matrices
      integer rmtrx(3,3,48)     ! the transformation matrices in realspace
      integer ntrans
      end type
c
c     Assume the matrix h has the lattice vectors in its COLUMNS.
c     Suppose a symmetry operation S takes a vector from r to r':
c
c         r' = Sr = alpha r + b          
c
c     where alpha = mirror-rotation,    b = nonprimitive translation.
c
c     then    mtrx = (h^(-1) alpha^-1 h)^T
c
c     such that mtrx gives the transformation for the G vectors
c     in reciprocal lattice vector coordinates
c
c        G' = alpha G       (in cartesian coordinates)
c        G' =  mtrx^T * G   (in lattice coordinates)
c
c     It turns out that:
c
c       rsymmat = alpha
c
c
c     the translations tnp are actually 2pi* b, and given relative
c     to the realspace lattice coordinates


      contains

      subroutine write_symmetry(fname, syms) ! ----- iolib write symmetry ----
      implicit none

      character*(*) fname       ! the filename
      type(symmetry) syms       ! the symmetry info to write

      integer   mymarker, info
      parameter(mymarker=8)

      open(unit=21,file=fname,
     $     position = 'append',
     $     status='unknown',form='formatted')

      write(21,*) mymarker, ' symmetry'
      write(21,*) syms%ntrans
      write(21,*) syms%mtrx   
      write(21,*) syms%tnp    
      write(21,*) syms%rmtrx  
      write(21,*) syms%rsymmat
      close(21)
      end subroutine
c
c
c
      subroutine read_symmetry(fname,syms)

      implicit none             ! implicit? no!

      character*(*) fname       ! the filename
      type (symmetry) syms

      integer mymarker, info
      parameter(mymarker=8) 

      open(unit=21,file=fname,
     $     status='unknown',form='formatted')
      call iolib_find_marker(mymarker,21,info)
      if(info.eq.0) then
         write(0,*) 'iolib: could not find marker:', mymarker
         stop
      endif

      read(21,*,end=102) syms%ntrans
      read(21,*,end=102) syms%mtrx   
      read(21,*,end=102) syms%tnp    
      read(21,*,end=102) syms%rmtrx  
      read(21,*,end=102) syms%rsymmat
      close(21)
      return

 102  continue
      write(0,*) 'iolib: reached eof before reading object:', 
     $     mymarker
      stop
      end subroutine
      
      end module symmetry_module
c
c     ==================== magnetic field structure =========================
c
      module magnetic_field_module
      type magnetic_field
      sequence
      double precision h        ! strength of magnetic field
      integer gvec(4)  ! direction of magnetic field, switch
      end type

      end module
c
c     ==================== pw parameter structure =========================
c

      module pw_parameter_module
      use magnetic_field_module
      type pw_parameter
      sequence
      double precision 
     $     q(3),                ! gw use
     $     alphamix(4),         ! linear mixing parameters
     $     smearing,            ! gaussian smearing parameter [eV]
     $     emaxsub,             ! cutoff energy for submatrix
     $     emaxmix,             ! mixing energy cutoff
     $     emax,                ! energy cutoff
     $     epsdiag,             ! accuracy of diagonalization
     $     shiftsafety,         ! extra shift to avoid bombout in matrix diag.
     $     random_startvec,     ! how much random startvector to mix in [0, 1]
     $     bandgap,             ! estimated bandgap in Ryd
     $     epsmag,              ! accuracy of cg inversion for magnetic suscep
     $     epscv,               ! convergence precision for potential
     $     ekinmod(3)           ! modification parameters for kinetic energy
      type (magnetic_field) mfield ! magnetic field
      type (magnetic_field) epot ! electric potential
      integer 
     $     checkpoint_wfn,      ! how frequently wfn is chkpted 
     $     occupy,              ! how to occupy the bands with electrons
     $     maxitdiag,           ! maximum number of iterations for diagonal.
     $     maxitcgsol,          ! maximum number of iterations for cg solver
     $     maxitscf,            ! maximum number of iterations for scf loop
     $     optimize,            ! flag to trigger optimizations
     $     miscflag,            ! miscellaneous flags
     $     input,               ! flag to trigger additional input
     $     output               ! flag to trigger additional output

      logical isymmetrize       ! true if k-point symmetrization is necessary
      character*2 icorr         ! the correlation type
      end type

      end module

c
c     ==================== kpoint structure =========================
c
      module kpoint_module

      use iolib_module          ! because here already the iolib functions
c                               ! are available as regular member functions

      type kpoint
      sequence
      integer nrk               ! number of k-points
      integer grid(3)           ! the generation grid
      double precision shift(3) ! shift vector
      logical generated         ! true if generated, false if read
      logical reduced           ! whether it is a reduced k-point grid
      integer, pointer ::
     $     kmap(:),             ! map: the irred kpoint of each reducib. kp
     $     ind(:,:)             ! index of six nearest neighbors: ind(6,nrk)
      double precision, pointer :: 
     $     w(:),                ! symmetry weights w(nrk)
     $     label(:)             ! label for bandstructure plot
      double precision, pointer :: rk(:,:) ! k-points rk(3,nrk)
      end type

      contains
      subroutine destroy_kpoint(kp)
      type(kpoint) kp
      if(associated(kp%w)) deallocate(kp%w)
      if(associated(kp%rk)) deallocate(kp%rk)
      if(associated(kp%ind)) deallocate(kp%ind)
      if(associated(kp%label)) deallocate(kp%label)
      end subroutine

      subroutine create_kpoint(kp,nrk)
      type(kpoint) kp
      integer nrk

      kp%nrk =nrk
      allocate(kp%w(nrk)) ; allocate(kp%label(nrk))
      allocate(kp%rk(3,nrk)) ; allocate(kp%ind(6,nrk))
      kp%label=0;
      kp%ind=0;
      kp%w=0;
      kp%rk=0;
      end subroutine

      subroutine write_kpoint(fname,struc)

      implicit none             ! implicit? no!
      character*(*) fname       ! the filename
      type (kpoint) struc 
c     ----------- local variables
      integer   mymarker, info, i,j,k,l
      parameter(mymarker=4)

      open(unit=21,file=fname,
     $     position = 'append',
     $     status='unknown',form='formatted')

      write(21,*) mymarker, ' kpoint'
      write(21,*) struc%nrk
      write(21,*) struc%grid
      write(21,*) struc%shift
      write(21,*) struc%generated
      write(21,*) struc%reduced
      write(21,*) struc%kmap
      write(21,*) struc%ind
      write(21,*) struc%w
      write(21,*) struc%label
      write(21,*) struc%rk
      close(21)
      return
      end subroutine


      subroutine read_kpoint(fname,struc)

      implicit none             ! implicit? no!
      character*(*) fname       ! the filename

      type (kpoint) struc
c     ----------- local variables
      integer mymarker, info, i,j,k,l
      parameter(mymarker=4) 

      open(unit=21,file=fname,
     $     status='unknown',form='formatted')
      call iolib_find_marker(mymarker,21,info)
      if(info.eq.0) then
         write(0,*) 'iolib: could not find marker:', mymarker
         stop
      endif

      read(21,*,end=102) struc%nrk
      allocate(struc%w(struc%nrk))
      allocate(struc%rk(3,struc%nrk))
      allocate(struc%ind(6,struc%nrk))
      allocate(struc%label(struc%nrk))
      read(21,*,end=102) struc%grid
      read(21,*,end=102) struc%shift
      read(21,*,end=102) struc%generated
      read(21,*,end=102) struc%reduced
      allocate(struc%kmap(struc%grid(1)*struc%grid(2)*struc%grid(3)))
      read(21,*,end=102) struc%kmap
      read(21,*,end=102) struc%ind
      read(21,*,end=102) struc%w
      read(21,*,end=102) struc%label
      read(21,*,end=102) struc%rk

      close(21)
      return

 102  continue
      write(0,*) 'iolib: reached eof before reading object:', 
     $     mymarker
      stop


      end subroutine

      end module
c
c     ==================== pseudopotential  structure ==================
c
      module pseudo_potential_module
      type pseudo_potential
      sequence
      integer, pointer :: nqnl(:) ! mesh points for nonlocal pot nqnl(ntype)
      integer, pointer :: nkb(:,:) ! norm for nonlocal pot nkb(3,ntype)
      integer, pointer :: lo(:,:)  ! l quantum number for nonlocal projector
      
      double precision, pointer :: delqnl(:) !nloc pot mesh dist delqnl(ntype)
      double precision, pointer :: vkb(:,:,:) ! nloc pspot vkb(mxdlqp,3,ntype)
      double precision, pointer :: d2vkbdq2(:,:,:) ! d^2 vkb/ dq^2

      integer nanl              ! number of nonlocal projectors
      integer ntype             ! number of atomic types
      integer mxdlqp            ! max number of mesh points for nonloc pot


      end type 

      contains
      subroutine destroy_pseudo_potential(pspots)
      type(pseudo_potential) pspots
      if(associated(pspots%nqnl)) deallocate(pspots%nqnl)
      if(associated(pspots%nkb)) deallocate(pspots%nkb)
      if(associated(pspots%lo)) deallocate(pspots%lo)
      if(associated(pspots%delqnl)) deallocate(pspots%delqnl)
      if(associated(pspots%vkb)) deallocate(pspots%vkb)
      if(associated(pspots%d2vkbdq2)) deallocate(pspots%d2vkbdq2)
      end subroutine

      end module

c
c     ==================== gspace_array structure ==================
c
      module gspace_array_module
      use parallel_gspace_module

      type complex_gspace_array
      sequence
      complex(kind=8), pointer :: data(:,:,:) ! data(length,nvecs,nspin)
      integer nspin             ! number of spin components
      integer nvecs             ! number of one-dimensional arrays
      integer length            ! length of a single vector
      end type 

      type double_gspace_array
      sequence
      double precision, pointer :: data(:,:,:) ! data(length,nvecs,nspin)
      integer nspin             ! number of spin components
      integer nvecs             ! number of one-dimensional arrays
      integer length            ! length of a single vector
      end type 


      contains
      subroutine destroy_complex_gspace_array(cgsarrays)
      type(complex_gspace_array) cgsarrays
      if(associated(cgsarrays%data)) deallocate(cgsarrays%data)
      end subroutine

      subroutine 
     $     create_complex_gspace_array(cgsarrays,len,nvecs,nspin)
      type(complex_gspace_array) cgsarrays
      integer len, nvecs,nspin, alcstat
      if((len.gt.0).and.(nvecs.gt.0).and.(nspin.gt.0)) then
         allocate(cgsarrays%data(len,nvecs,nspin),stat=alcstat)
         call alccheck('gsarray',len*nvecs*nspin,alcstat)
      else
         nullify(cgsarrays%data)
      endif
      cgsarrays%nspin=nspin
      cgsarrays%length=len
      cgsarrays%nvecs=nvecs
      end subroutine

      subroutine print_complex_gspace_array(gs,cgsa)
      type(complex_gspace_array) cgsa
      type(parallel_gspace) gs
      integer i,is,n
      do is=1,cgsa%nspin
         write(9,*) 'spin: ',is
         do n=1, cgsa%nvecs
            write(9,*) 'vector: ',n
            do i=1, gs%length
               write(9,*) i, cgsa%data(i,n,is)
            end do
         end do
      end do
      end subroutine

      subroutine destroy_double_gspace_array(dgsarrays)
      type(double_gspace_array) dgsarrays
      if(associated(dgsarrays%data)) deallocate(dgsarrays%data)
      end subroutine

      subroutine 
     $     create_double_gspace_array(dgsarrays,len,nvecs,nspin)
      type(double_gspace_array) dgsarrays
      integer len, nvecs,nspin,alcstat
      if((len.gt.0).and.(nvecs.gt.0).and.(nspin.gt.0)) then
         allocate(dgsarrays%data(len,nvecs,nspin),stat=alcstat)
         call alccheck('gsarray',len*nvecs*nspin,alcstat)
      else
         nullify(dgsarrays%data)
      endif
      dgsarrays%nspin=nspin
      dgsarrays%length=len
      dgsarrays%nvecs=nvecs
      end subroutine

      end module

c
c     ==================== rspace_array structure ==================
c
      module rspace_array_module

      type complex_rspace_array
      sequence
      complex(kind=8), pointer :: data(:,:,:)

      integer nspin             ! number of spin components
      integer nvecs             ! number of one-dimensional arrays
      integer length            ! length of a single vector
      end type 


      contains
      subroutine destroy_complex_rspace_array(crsarrays)
      type(complex_rspace_array) crsarrays
      if(associated(crsarrays%data)) deallocate(crsarrays%data)
      end subroutine

      subroutine create_complex_rspace_array(crsarrays,len,nvecs,nspin)
      type(complex_rspace_array) crsarrays
      integer len, nvecs,nspin,alcstat
      allocate(crsarrays%data(len,nvecs,nspin),stat=alcstat)
      call alccheck('rs_array',nvecs*nspin*len,alcstat)
      crsarrays%nspin=nspin
      crsarrays%length=len
      crsarrays%nvecs=nvecs
      end subroutine


      end module


c
c     ==================== energy structure =========================
c
      module energy_module
      type energy
      sequence
      double precision          ! all energies in Rydbergs
     $     inputfermi,          ! fermi energy as given by input
     $     afmagmom,            ! antiferromagnetic sublattice moment in Bohr
     $     vxc0(2),             ! spin up and down of the xc potential at G=0
     $     magnetic,            ! magnetic energy due to external field
     $     hxc,                 ! hartree exchange-correlation correction
     $     xc,                  ! the exchange-correlation energy
     $     ewald,               ! ewald energy, with compensating background
     $     eband,               ! band energy
     $     ion,                 ! ion-electron energy
     $     nloc,                ! nonlocal energy contribution
     $     hartree,             ! hartree electron-electron energy
     $     ektot,               ! total kinetic energy
     $     esmear,              ! smearing energy contribution
     $     efermi(2),           ! fermi energy for both spins
     $     evalshift(2),        ! shift of the energy eigenvalues
     $     total,               ! the total energy
     $     totnonv,             ! etot - 1/2 TS term, is not variational!
     $     alpha                ! the alpha term which contains divergent terms
      end type

      contains

      subroutine energy_assignment(et,es) 
      type(energy), intent(out) :: et
      type(energy), intent(in) :: es

      et%inputfermi=es%inputfermi
      et%afmagmom=es%afmagmom
      et%vxc0 = es%vxc0
      et%magnetic = es%magnetic
      et%hxc=es%hxc
      et%xc=es%xc
      et%ewald=es%ewald
      et%eband=es%eband
      et%ion=es%ion
      et%nloc=es%nloc
      et%hartree=es%hartree
      et%ektot=es%ektot
      et%esmear=es%esmear
      et%efermi=es%efermi
      et%total=es%total
      et%alpha=es%alpha
      et%totnonv=es%totnonv
      et%evalshift=es%evalshift
      end subroutine

      subroutine energy_assignment0(et,e0) 
      type(energy), intent(out) :: et
      double precision, intent(in) :: e0

      et%inputfermi=e0
      et%afmagmom=e0
      et%vxc0 = e0
      et%magnetic = e0
      et%hxc=e0
      et%xc=e0
      et%ewald=e0
      et%eband=e0
      et%ion=e0
      et%nloc=e0
      et%hartree=e0
      et%ektot=e0
      et%esmear=e0
      et%efermi=e0
      et%total=e0
      et%totnonv=e0
      et%alpha=e0
      et%evalshift=e0
      end subroutine

      function energy_subtract(et,es) result(en_s)
      type(energy) en_s
      type(energy), intent(in) :: et
      type(energy), intent(in) :: es

      en_s%inputfermi=et%inputfermi
      en_s%afmagmom=et%afmagmom-es%afmagmom
      en_s%vxc0 =et%vxc0 - es%vxc0
      en_s%magnetic =et%magnetic - es%magnetic
      en_s%hxc = et%hxc-es%hxc
      en_s%xc = et%xc-es%xc
      en_s%ewald = et%ewald-es%ewald
      en_s%eband = et%eband-es%eband
      en_s%ion = et%ion-es%ion
      en_s%nloc = et%nloc-es%nloc
      en_s%hartree = et%hartree-es%hartree
      en_s%ektot = et%ektot-es%ektot
      en_s%esmear = et%esmear-es%esmear
      en_s%efermi = et%efermi-es%efermi
      en_s%total = et%total-es%total
      en_s%totnonv = et%totnonv-es%totnonv
      en_s%alpha = et%alpha-es%alpha
      en_s%evalshift = et%evalshift-es%evalshift
      end function

      end module 

c
c     ==================== force structure =========================
c
      module force_module
      type force
      sequence
      integer mxdatm
      integer ntype
      double precision
     $     stress(6)            ! the symmetric stress xx,yy,zz,xy,yz,xz
      double precision, pointer :: 
     $     force (:,:,:)        ! forces on the atoms force(3,mxdatm,ntype)
      end type

      contains
      subroutine destroy_force(f)
      type(force) f
      if(associated(f%force)) deallocate(f%force)
      end subroutine


      end module 

c
c     ==================== band structure =========================
c
      module band_module
      
      use iolib_module

      type band
      sequence
      integer nrk               ! number of kpoints
      integer min               ! minimum number of bands computed
      integer max               ! maximum number of bands computed
      integer nspin             ! number of spins
      integer, pointer :: 
     $     nband(:,:),          ! number of bands computed nband(nrk,nspin)
     $     ifmax(:,:)         ! highest occupied band ifmax(nrk,nspin)
      double precision, pointer :: 
     $     energy(:,:,:),       ! eigenvalues energy(max,nrk,nspin)
     $     ekn(:,:,:),          ! kinetic energy(max,nrk,nspin)
     $     occup(:,:,:)         ! occupation numbers occup(max,nrk,nspin)
      
      end type

      contains
      subroutine destroy_band(b)
      type(band) b
      if(associated(b%ifmax)) deallocate(b%ifmax)
      if(associated(b%energy)) deallocate(b%energy)
      if(associated(b%ekn)) deallocate(b%ekn)
      if(associated(b%occup)) deallocate(b%occup)
      if(associated(b%nband)) deallocate(b%nband)
      end subroutine

      subroutine create_band(b,nrk, max,nspin)
      type(band) b
      integer nrk,max,nspin
      b%nrk   = nrk
      b%max   = max
      b%nspin = nspin
      allocate(b%nband(nrk,nspin)) ; b%nband=0
      allocate(b%occup(max,nrk,nspin)) ; b%occup=0
      allocate(b%energy(max,nrk,nspin)) ; b%energy=0
      allocate(b%ekn(max,nrk,nspin)) ; b%ekn=0
      allocate(b%ifmax(nrk,nspin)) ; b%ifmax=0

      end subroutine


      subroutine write_band(fname,struc)
      implicit none             ! implicit? no!
      character*(*) fname       ! the filename
      type (band) struc 
c     ----------- local variables
      integer   mymarker, info, i,j,k,l
      parameter(mymarker=6)

      open(unit=21,file=fname,
     $     position = 'append',
     $     status='unknown',form='formatted')

      write(21,*) mymarker, ' band info'
      write(21,*) struc%nrk, struc%min, struc%max, struc%nspin
      write(21,*) ((struc%nband(k,j),k=1,struc%nrk),j=1,struc%nspin)
      write(21,*) ((struc%ifmax(k,j),k=1,struc%nrk),j=1,struc%nspin)

      write(21,*) (((struc%energy(l,k,j),l=1,struc%max),
     $     k=1,struc%nrk),j=1,struc%nspin)
      write(21,*) (((struc%ekn(l,k,j),l=1,struc%max),
     $     k=1,struc%nrk),j=1,struc%nspin)
      write(21,*) (((struc%occup(l,k,j),l=1,struc%max),
     $     k=1,struc%nrk),j=1,struc%nspin)

      close(21)
      return
      end subroutine


      subroutine read_band(fname,bands)

      implicit none             ! implicit? no!
      character*(*) fname       ! the filename

      type (band) bands
c     ----------- local variables
      integer 
     $     mymarker, info, i,j,k,l, nrk,mmin,mmax,nspin
      
      parameter(mymarker=6) 

      open(unit=21,file=fname,
     $     status='unknown',form='formatted')
      call iolib_find_marker(mymarker,21,info)
      if(info.eq.0) then
         bands%nrk=-1
         close(21)
         return
      endif

      read(21,*,end=102) nrk,mmin,mmax,nspin
      if(nrk.ne.bands%nrk) then
         write(9,*) 'read_band: kpoint mismatch: kpoints in file:',nrk
         stop
      endif
      if(nspin.ne.bands%nspin) then
         write(9,*) 'read_band: spin mismatch, spins in file:',nspin
         stop
      endif
      if(mmax.gt.bands%max) then
         write(9,*) 'read_band: too many bands in file:',mmax
         stop
      endif
      if((.not.associated(bands%nband)).or.
     $     (.not.associated(bands%ifmax)).or.
     $     (.not.associated(bands%energy)).or.
     $     (.not.associated(bands%ekn)).or.
     $     (.not.associated(bands%occup))) then
         write(9,*) 'read_band: structure not allocated!' ; stop
      endif

      read(21,*,end=102) ((bands%nband(k,j),k=1,nrk),j=1,nspin)
      read(21,*,end=102) ((bands%ifmax(k,j),k=1,nrk),j=1,nspin)

      read(21,*,end=102) (((bands%energy(l,k,j),l=1,mmax),
     $     k=1,nrk),j=1,nspin)
      read(21,*,end=102) (((bands%ekn(l,k,j),l=1,mmax),
     $     k=1,nrk),j=1,nspin)
      read(21,*,end=102) (((bands%occup(l,k,j),l=1,mmax),
     $     k=1,nrk),j=1,nspin)

      close(21)
      return

 102  continue
      write(0,*) 'iolib: reached eof when reading bands' 
      stop


      end subroutine

      end module 

c
c     ==================== hamiltonian structure =========================
c
c     by convention, only the vnonloc part of the Hamiltonian gets
c     allocated and deallocated upon creation and destruction.
c     For the dense Hamiltonian, different rules apply

      module hamiltonian_module
      use parallel_gspace_module
      use rspace_array_module
      use gspace_array_module
      use pseudo_potential_module
      use fftstruc_module

      type hamiltonian
      sequence
      double precision shift    ! global energy shift for the hamiltonian [Ry]
      double precision ekinmod(3) ! modification parameters for kinetic energy
      type (parallel_gspace), pointer :: gspace
      complex(kind=8), pointer :: vloc(:) ! local potential
      double precision, pointer :: xnorm(:) ! normalization xnorm(nanl)
      type (fftstruc), pointer :: fftstruc
      type (complex_gspace_array) vnloc ! nonlocal potential projectors
      integer dim               ! local number plane waves for this hamiltonian
      end type

      type dense_hamiltonian
      sequence
      type (parallel_gspace), pointer :: gspace
      complex(kind=8), pointer :: matrix(:) ! the dense matrix itself
      double precision, pointer :: energy(:) ! the spectrum
      integer neig              ! number of eigenvalues
      end type

      contains
      subroutine destroy_hamiltonian(h)
      type(hamiltonian) h
      call destroy_complex_gspace_array(h%vnloc)
      if(associated(h%xnorm)) deallocate(h%xnorm)
      end subroutine

      subroutine create_hamiltonian(h,gs,pspot,vloc,ffts)
      type(hamiltonian) h
      type(parallel_gspace),target :: gs
      type(pseudo_potential) pspot
      type(fftstruc), target :: ffts
      complex(kind=8), pointer :: vloc(:) ! local potential
      nullify(h%gspace)
      h%gspace=>gs 
      h%dim=gs%length
      nullify(h%vloc)
      h%vloc=>vloc
      nullify(h%fftstruc)
      h%fftstruc=>ffts
      if(pspot%nanl.gt.0)  then
         call create_complex_gspace_array(h%vnloc,
     $        gs%length,pspot%nanl,1)
         allocate(h%xnorm(pspot%nanl))
      else
         h%vnloc%nspin=0
         h%vnloc%nvecs=0
         h%vnloc%length=0
         nullify(h%vnloc%data)
         nullify(h%xnorm)
      endif
      end subroutine



      subroutine destroy_dense_hamiltonian(h)
      type(dense_hamiltonian) h
      nullify(h%gspace)
      if(associated(h%matrix)) deallocate(h%matrix)
      if(associated(h%energy)) deallocate(h%energy)
      h%neig=0
      end subroutine

      subroutine create_dense_hamiltonian(h,gs)
      type(dense_hamiltonian) h
      type(parallel_gspace),target :: gs
      h%gspace=>gs 
      nullify(h%matrix)
      allocate(h%energy(gs%length))
      h%neig=0
      end subroutine

      end module

c
c     =========== vqmc_gspace_module ==========================
c

      module vqmc_gspace_module

      use iolib_module
      type gspace
      sequence                  ! want sequential memory layout.
      complex(kind=8), pointer :: phase(:) ! phases(length)
      integer, pointer :: gvec(:,:) ! the gspace vectors themselves
      integer, pointer :: inds(:) ! maps gvecs to stars inds(length)
      integer, pointer :: mstar(:) ! size of stars mstar(nstar)

      integer fftsize(3)        ! FFT grid boundaries for gspace
      integer length            ! the total number of g-space vectors included
      integer nstar             ! total number of stars in gspace
      logical istar             ! if star arrays are available
      
      end type gspace

      contains

      subroutine destroy_gspace(gs)
      type(gspace) gs
      if(associated(gs%gvec)) deallocate(gs%gvec)
      if(associated(gs%inds)) deallocate(gs%inds)
      if(associated(gs%mstar)) deallocate(gs%mstar)
      if(associated(gs%phase)) deallocate(gs%phase)
      end subroutine


      subroutine write_gspace(fname,gsstruc)

      implicit none             ! implicit? no!

      character*(*) fname       ! the filename
      type (gspace) gsstruc     ! the gspace structure

      integer 
     $     mymarker, info, i,j

      parameter(mymarker=1)     ! the marker for the gspace object
      open(unit=21,file=fname,
     $     position = 'append',
     $     status='unknown',form='formatted')

      write(21,*) mymarker, ' gspace'
      write(21,*) (gsstruc%fftsize(i),i=1,3)
      write(21,*) gsstruc%length
      write(21,*) ((gsstruc%gvec(j,i),j=1,3),
     $     i=1,gsstruc%length)
      write(21,*) gsstruc%istar 
      if(gsstruc%istar) then
         write(21,*) (gsstruc%inds(i),   i=1,gsstruc%length)
         write(21,*) (gsstruc%phase(i),   i=1,gsstruc%length)
         write(21,*) gsstruc%nstar
         write(21,*) (gsstruc%mstar(i),  i=1,gsstruc%nstar)
      else
         write(21,*) ; write(21,*) ; write(21,*) ; write(21,*) 
      endif

      close(21)
      return
      end subroutine


      subroutine read_gspace(fname,gsstruc)

      implicit none             ! implicit? no!
      character*(*) fname       ! the filename
      type (gspace) gsstruc     ! the gspace structure

      integer 
     $     mymarker, info, i,j

      parameter(mymarker=1)     ! the marker for the gspace object
      open(unit=21,file=fname,
     $     status='unknown',form='formatted')

      call iolib_find_marker(mymarker,21,info)

      if(info.eq.0) then
         write(0,*) 'iolib: could not find marker:', mymarker
         stop
      endif

      read(21,*,end=102) (gsstruc%fftsize(i),i=1,3)
      read(21,*,end=102) gsstruc%length
      allocate(gsstruc%gvec(3,gsstruc%length))
      read(21,*,end=102) ((gsstruc%gvec(j,i),j=1,3),
     $     i=1,gsstruc%length)
      read(21,*) gsstruc%istar 
      if(gsstruc%istar) then
         allocate(gsstruc%inds(gsstruc%length))
         read(21,*) (gsstruc%inds(i),   i=1,gsstruc%length)
         allocate(gsstruc%phase(gsstruc%length))
         read(21,*) (gsstruc%phase(i),   i=1,gsstruc%length)
         read(21,*) gsstruc%nstar
         allocate(gsstruc%mstar(gsstruc%nstar))
         read(21,*) (gsstruc%mstar(i),  i=1,gsstruc%nstar)
      else
         gsstruc%nstar=0
         nullify(gsstruc%phase);nullify(gsstruc%mstar);
         nullify(gsstruc%inds);
         read(21,*); read(21,*); read(21,*); read(21,*); 
      endif

      close(21)
      return
 102  continue
      write(0,*) 'iolib: reached eof before reading object:', 
     $     mymarker
      stop
      end subroutine

      end module

c
c     ==================== isrt structure ===============
c

      module vqmc_isrt_module

      use iolib_module
      type isrt
      sequence
      integer, pointer :: kndim(:) ! number of plane-waves at each k-point
      integer, pointer :: isrt(:,:) ! the actual isrt array: isrt(lda,nrk)
      integer nrk               ! number of k-points
      integer lda               ! leading dimension of isrt array

      end type

      contains 
      subroutine destroy_isrt(is)
      type(isrt) is
      if(associated(is%kndim)) deallocate(is%kndim)
      if(associated(is%isrt)) deallocate(is%isrt)
      end subroutine


      subroutine write_isrt(fname,struc)

      implicit none             ! implicit? no!

      character*(*) fname       ! the filename
      type (isrt) struc 

      integer 
     $     mymarker, info, i,j

      parameter(mymarker=2)

      open(unit=21,file=fname,
     $     position = 'append',
     $     status='unknown',form='formatted')

      write(21,*) mymarker, ' isrt'
      write(21,*) struc%nrk
      write(21,*) (struc%kndim(i),i=1,struc%nrk)
      write(21,*) struc%lda
      write(21,*) ((struc%isrt(i,j),
     $     i=1,struc%lda),j=1,struc%nrk)

      close(21)
      return
      end subroutine


      subroutine read_isrt(fname,struc)

      implicit none             ! implicit? no!
      character*(*) fname       ! the filename
      type (isrt) struc

      integer 
     $     mymarker, info, i,j

      parameter(mymarker=2) 
      open(unit=21,file=fname,
     $     status='unknown',form='formatted')

      call iolib_find_marker(mymarker,21,info)

      if(info.eq.0) then
         write(0,*) 'iolib: could not find marker:', mymarker
         stop
      endif

      read(21,*,end=102) struc%nrk
      allocate(struc%kndim(struc%nrk))
      read(21,*,end=102) (struc%kndim(i),i=1,struc%nrk)
      read(21,*,end=102) struc%lda
      allocate(struc%isrt(struc%lda,struc%nrk))
      read(21,*,end=102) ((struc%isrt(i,j),
     $     i=1,struc%lda),j=1,struc%nrk)

      close(21)
      return


 102  continue
      write(0,*) 'iolib: reached eof before reading object:', 
     $     mymarker
      stop

      end subroutine

      end module
c
c     ==================== wavefn arrays structure ===============
c
      module vqmc_wavefn_module
      use iolib_module

      type wavefn
      sequence
      integer nrk               ! number of k-points
      integer nspin             ! number of spins, 1 or 2
      integer mnband            ! maximum number of bands at all k-points
      integer lda               ! leading dimension of wfns array
      integer, pointer :: kndim(:) ! number of plane-waves at each k-point
      integer, pointer :: nband(:,:) ! number of bands computed (kpoint,spin)

c     the actual wfn array: wfn(lda,mnband,nrk,nspin)

      complex(kind=8), pointer :: wfn(:,:,:,:) 

      end type

      contains

      subroutine destroy_wavefn(wf)
      type(wavefn) wf
      if(associated(wf%kndim)) deallocate(wf%kndim)
      if(associated(wf%nband)) deallocate(wf%nband)
      if(associated(wf%wfn)) deallocate(wf%wfn)
      end subroutine

      subroutine write_wavefn(fname,struc)

      implicit none             ! implicit? no!
      character*(*) fname       ! the filename
      type (wavefn) struc 

c     ----------- local variables

      integer   mymarker, info, i,j,k,l
      parameter(mymarker=3)

      open(unit=21,file=fname,
     $     position = 'append',
     $     status='unknown',form='formatted')

      write(21,*) mymarker, ' wavefn'
      write(21,*) struc%nrk
      write(21,*) (struc%kndim(i),i=1,struc%nrk)
      write(21,*) struc%nspin
      write(21,*) struc%mnband
      write(21,*) struc%lda
      write(21,*) ((struc%nband(i,j),i=1,struc%nrk),
     $     j=1,struc%nspin)
      write(21,*) ((((struc%wfn(i,j,k,l),
     $     i=1,struc%lda),j=1,struc%mnband),k=1,struc%nrk),
     $     l=1,struc%nspin)


      close(21)
      return
      end subroutine

      subroutine read_wavefn(fname,struc)

      implicit none             ! implicit? no!
      character*(*) fname       ! the filename

      type (wavefn) struc
c     ----------- local variables
      integer mymarker, info, i,j,k,l
      parameter(mymarker=3) 

      open(unit=21,file=fname,
     $     status='unknown',form='formatted')
      call iolib_find_marker(mymarker,21,info)
      if(info.eq.0) then
         write(0,*) 'iolib: could not find marker:', mymarker
         stop
      endif

      read(21,*,end=102) struc%nrk
      allocate(struc%kndim(struc%nrk))
      read(21,*,end=102) (struc%kndim(i),i=1,struc%nrk)
      read(21,*,end=102) struc%nspin
      read(21,*,end=102) struc%mnband
      read(21,*,end=102) struc%lda
      allocate(struc%nband(struc%nrk,struc%nspin))
      read(21,*,end=102) ((struc%nband(i,j),i=1,struc%nrk),
     $     j=1,struc%nspin)
      allocate(struc%wfn(struc%lda,
     $     struc%mnband,struc%nrk,struc%nspin))
      read(21,*,end=102) ((((struc%wfn(i,j,k,l),
     $     i=1,struc%lda),j=1,struc%mnband),k=1,struc%nrk),
     $     l=1,struc%nspin)

      close(21)
      return


 102  continue
      write(0,*) 'iolib: reached eof before reading object:', 
     $     mymarker
      stop
      end subroutine
      end module

c
c     ==================== level array structure ===============
c

      module vqmc_level_module
      use iolib_module

      type level
      sequence
      integer, pointer :: nband(:,:) ! number of bands computed (kpoint,spin)
      integer, pointer :: ifmin(:,:) ! lowest occupied band (kpoint,spin)
      integer, pointer :: ifmax(:,:) ! highest occupied band
      double precision, pointer :: occup(:,:,:) !occup number(band,kpoint,spin)
      double precision, pointer :: el(:,:,:) ! energy [Ryd] (band,kpoint,spin)
      double precision smearing ! the smearing parameter used [Ryd]
      double precision fermilevel ! the fermi level [Ryd]
      integer nrk               ! number of k-points
      integer mnband            ! maximum number of bands
      integer nspin             ! number of spins
      end type

      contains

      subroutine destroy_level(lv)
      type(level) lv
      if(associated(lv%nband)) deallocate(lv%nband)
      if(associated(lv%ifmin)) deallocate(lv%ifmin)
      if(associated(lv%ifmax)) deallocate(lv%ifmax)
      if(associated(lv%occup)) deallocate(lv%occup)
      if(associated(lv%el)) deallocate(lv%el)
      end subroutine

      subroutine write_level(fname,struc)

      implicit none             ! implicit? no!
      character*(*) fname       ! the filename
      type (level) struc 

c     ----------- local variables

      integer   mymarker, info, i,j,k,l
      parameter(mymarker=5)

      open(unit=21,file=fname,
     $     position = 'append',
     $     status='unknown',form='formatted')

      write(21,*) mymarker, ' level'
      write(21,*) struc%nrk
      write(21,*) struc%mnband
      write(21,*) struc%nspin
      write(21,*) ((struc%nband(i,j),i=1,struc%nrk),
     $     j=1,struc%nspin)
      write(21,*) ((struc%ifmin(i,j),i=1,struc%nrk),
     $     j=1,struc%nspin)
      write(21,*) ((struc%ifmax(i,j),i=1,struc%nrk),
     $     j=1,struc%nspin)
      write(21,*) (((struc%occup(k,i,j),k=1,struc%mnband),
     $     i=1,struc%nrk), j=1,struc%nspin)
      write(21,*) (((struc%el(k,i,j),k=1,struc%mnband),
     $     i=1,struc%nrk), j=1,struc%nspin)
      write(21,*) struc%smearing
      write(21,*) struc%fermilevel
      close(21)
      return
      end subroutine

      subroutine read_level(fname,struc)

      implicit none             ! implicit? no!

      character*(*) fname       ! the filename

      type (level) struc
c     ----------- local variables
      integer mymarker, info, i,j,k,l
      parameter(mymarker=5) 

      open(unit=21,file=fname,
     $     status='unknown',form='formatted')
      call iolib_find_marker(mymarker,21,info)
      if(info.eq.0) then
         write(0,*) 'iolib: could not find marker:', mymarker
         stop
      endif

      read(21,*,end=102) struc%nrk
      read(21,*,end=102) struc%mnband
      read(21,*,end=102) struc%nspin
      allocate(struc%nband(struc%nrk,struc%nspin))
      read(21,*,end=102) ((struc%nband(i,j),i=1,struc%nrk),
     $     j=1,struc%nspin)
      allocate(struc%ifmin(struc%nrk,struc%nspin))
      allocate(struc%ifmax(struc%nrk,struc%nspin))
      allocate(struc%occup(struc%mnband,struc%nrk,struc%nspin))
      allocate(struc%el(struc%mnband,struc%nrk,struc%nspin))
      read(21,*,end=102) ((struc%ifmin(i,j),i=1,struc%nrk),
     $     j=1,struc%nspin)
      read(21,*,end=102) ((struc%ifmax(i,j),i=1,struc%nrk),
     $     j=1,struc%nspin)
      read(21,*,end=102) (((struc%occup(k,i,j),k=1,struc%mnband),
     $     i=1,struc%nrk), j=1,struc%nspin)
      read(21,*,end=102) (((struc%el(k,i,j),k=1,struc%mnband),
     $     i=1,struc%nrk), j=1,struc%nspin)
      read(21,*,end=102) struc%smearing
      read(21,*,end=102) struc%fermilevel

      close(21)
      return


 102  continue
      write(0,*) 'iolib: reached eof before reading object:', 
     $     mymarker
      stop


      end subroutine


      end module

