c
      subroutine symmetrize_tensor_global(ioutput, tens, syms,gs)
c
c
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'

m4_ifdef([CRAY],[m4_define([MPI_INTEGER4],[MPI_INTEGER])])
m4_ifdef([MPI],[
      include 'mpif.h'])


c     INPUT:
c     -----
c
      type (symmetry) syms      ! symmetry operations
      type (parallel_gspace) gs ! the gspace corresponding to the data

      integer ioutput           ! flag for timing printout

c     INPUT/OUTPUT:
c     ------------
c
      complex(kind=8)
     $     tens(gs%length,3,3)  ! the tensor quantity to be symmetrized

c
c     DESCRIPTION:
c     -----------
c
c     Symmetrizes the tensor field tens in gspace across multiple 
c     processors.
c     
c     1996 Bernd Pfrommer
c


c
c     ---------------- local variables -----------------------------
c

      integer
     $     i,j,ierr,            ! dummies
     $     ig,                  ! gvector index
     $     ivec,                ! index of found gvector
     $     iproc,               ! sending processor index
     $     il,                  ! list of gvec counter
     $     llen,                ! length of list of gvecs
     $     n,                   ! counter for symmetry operations
     $     findvec,
     $     mylen,               ! number of gvecs here
     $     mylist(2,48),        ! first is index of gvec, second index in glist
     $     oplist(48),          ! list of operations
     $     gtrans(3),           ! transformed gvector
     $     glist(3,48)          ! list of gvectors

      double precision  pi2, gimmetime, t0,
     $     alf(3,3), alfm1(3,3)

      complex(kind=8) 
     $     phase,               ! the phase factor exp(-iG*tau)
     $     symtens(3,3),
     $     t(3,3,48),
     $     tr(3,3,48) 

      logical 
     $     inew,                ! flag for new gvector found
     $     issym(gs%length)     ! array to indicate which gvecs have been symm.



      external findvec, gimmetime



      pi2= 8.d0*datan(1.d0)

      t0=gimmetime()

      if(syms%ntrans.eq.1) return ! need not symmetrize

      issym=.false.             ! nothing is symmetrized so far

      do iproc=0, gs%nproc-1    ! loop over all procs

         if(iproc.eq.gs%myproc) then ! i'm symmetrizing
            do ig=1, gs%length
               if(.not.issym(ig)) then ! has not been treated yet
                  t=0
                  t(:,:,1)=tens(ig,:,:)
                  issym(ig)=.true.

c                 generate a list of symmetry-related gvectors by 
c                 acting on the g-vector with the symmetry operations
c     
                  llen = 0 
                  do n=1, syms%ntrans
                     gtrans=matmul(syms%mtrx(:,:,n), gs%gvec(:,ig))
                     inew=.true.
                     do i=1,llen ! check if already in the list
                        if((glist(1,i).eq.gtrans(1)).and.
     $                       (glist(2,i).eq.gtrans(2)).and.
     $                       (glist(3,i).eq.gtrans(3))) then
                           inew=.false. ! is already in list
                           exit
                        endif
                     end do
                     if(inew) then ! is new, add to list
                        llen=llen+1
                        glist(:,llen)=gtrans
                        oplist(llen)=n
                     endif
                  end do
c                  write(9,*)  'glist for g=', ig
c                  do il=1,llen
c                     write(9,*) il,oplist(il), glist(:,il)
c                  end do

c
c                 communicate with other processors if there are
c                 other vectors in the list.
c
                  glist(1,1)=llen ! abuse temporarily for lenght info
                  m4_ifdef([MPI],[
                  if(llen.gt.1) 
     $                 call mpi_bcast(glist(1,1),3*48,
     $                 MPI_INTEGER4,iproc, MPI_COMM_WORLD,ierr) ])

                  glist(:,1)=gs%gvec(:,ig) ! put the original back in place
c
c                 gather the tensor components for symmetrization
c
c
c                 first look in my own backyard if I have one of the 
c                 gvectors 
c
                  mylen = 1
                  mylist(1,mylen)=ig
                  mylist(2,mylen)=1

                  do i=2,llen
                     ivec = findvec(glist(1,i),gs)
                     if(ivec.gt.0) then
                        t(:,:,i)=tens(ivec,:,:)
                        issym(ivec)=.true.
                        mylen = mylen+1
                        mylist(1,mylen)=ivec
                        mylist(2,mylen)=i 
                     endif
                  end do
c                  write(9,*)  'mylist:'
c                  do il=1,mylen
c                     write(9,*) il,mylist(1,il), mylist(2,il)
c                  end do

c     
c                 gather the data from the rest of the world
c
                  tr=t
                  m4_ifdef([MPI],[
                  if(llen.gt.1) 
     $                 call mpi_reduce(t(1,1,1),tr(1,1,1),
     $                 3*3*llen,MPI_DOUBLE_COMPLEX, MPI_SUM,
     $                 iproc,MPI_COMM_WORLD,ierr) 
                  ])
c 
c                now symmetrize 
c
c                  write(9,*)  'unsymmetrized tensors:'
c                  do il=1,llen
c                     write(9,*)  il, ':'
c                     write(9,'(6g12.5)') tr(:,:,il)
c                  end do

                  symtens=0
                  do n=1, syms%ntrans
                     gtrans=matmul(syms%mtrx(:,:,n),
     $                    gs%gvec(:,ig))

                     phase=exp(cmplx(0.d0,
     $                    1.d0*dot_product(dble(gtrans), 
     $                    syms%tnp(:,n)),kind=8))

c                       find place in the list
                        
                     do il=1,llen
                        if((glist(1,il).eq.gtrans(1)).and.
     $                       (glist(2,il).eq.gtrans(2)).and.
     $                       (glist(3,il).eq.gtrans(3))) 
     $                       exit
                     end do
                     if(il.eq.llen+1) then
                        write(9,*) 'error!'
                        stop
                     endif
                     do i=1,3
                        alf(i,:)=syms%rsymmat(i,:,n)
                        alfm1(:,i)=syms%rsymmat(i,:,n)
                     end do
c                     write(9,*) n, phase, syms%tnp(:,n), gs%gvec(:,ig)
c                     write(9,'(6g12.5)') matmul(alfm1,matmul(tr(:,:
c     $                    ,il),alf))*phase

                     symtens = symtens + matmul(alfm1,matmul(tr(:,:
     $                    ,il),alf))*phase
                     
                  end do
                  symtens=symtens/dble(syms%ntrans)

c                  write(9,*)  'prototype symmetrized tensor:'
c                  write(9,'(6g12.5)') symtens

c
c                 from the symmetrized tensor at G,
c                 compute rotated tensors
c
                  do il=1,llen
                     n=oplist(il)
                     do i=1,3
                        alf(i,:)=syms%rsymmat(i,:,n)
                        alfm1(i,:)=syms%rsymmat(:,i,n)
                     end do
                     phase=exp(cmplx(0.d0,
     $                    -dot_product(dble(glist(:,il)), 
     $                    syms%tnp(:,n)),kind=8)) 
                     t(:,:,il) =
     $                    matmul(alf,matmul(symtens,alfm1))*phase
                  end do

c                  write(9,*)  'symmetrized tensors:'
c                  do il=1,llen
c                     write(9,*)  il, ':'
c                     write(9,'(6g12.5)') t(:,:,il)
c                  end do

c
c                 ship back to the rest of the world
c
                  m4_ifdef([MPI],[
                  if(llen.gt.1) 
     $                 call mpi_bcast(t,9*llen, MPI_DOUBLE_COMPLEX,
     $                 iproc,MPI_COMM_WORLD,ierr)])
c
c                 put it into right place here
c

                  do i=1,mylen
                     tens(mylist(1,i),:,:)=t(:,:,mylist(2,i))
c                     write(9,*) 'gvec=', mylist(1,i)
                  end do
                  
               endif            ! endif not symmetrized
            end do              ! end of loop over all gvectors
c
c           Signal the other processors that i'm done
c
            glist(1,1)=-1234567
            m4_ifdef([MPI],[
            call mpi_bcast(glist(1,1),3*48,
     $           MPI_INTEGER4,iproc, MPI_COMM_WORLD,ierr)])
c
c           signal other processors that i'm done with symmetrizing
c
         else                   ! i'm responding

c
c           wait for the search requests to come in
c           
            do while (1.eq.1)
               m4_ifdef([MPI],[
               call mpi_bcast(glist(1,1),3*48,
     $              MPI_INTEGER4, iproc, MPI_COMM_WORLD, ierr)])

               llen=glist(1,1)
c               write(9,*) 'received list of length', llen
c               call myflush(9)
               if(llen.eq.-1234567) exit ! last search request received
c
c              look and see if I have the g-vector
c
               t=0
               mylen=0
               do i=2,llen
                  ivec = findvec(glist(1,i),gs)
                  if(ivec.gt.0) then  ! i have it
                     t(:,:,i)=tens(ivec,:,:)
                     issym(ivec)=.true.
                     mylen = mylen+1
                     mylist(1,mylen)=ivec
                     mylist(2,mylen)=i 
                  endif
               end do
c                     
c              send the local tensor components back to the root
c
               m4_ifdef([MPI],[
               call mpi_reduce(t(1,1,1),tr(1,1,1),
     $              3*3*llen,MPI_DOUBLE_COMPLEX, MPI_SUM,iproc,
     $              MPI_COMM_WORLD,ierr) ])
c
c              get back the symmetrized stuff from root
c
               m4_ifdef([MPI],[
               call mpi_bcast(t,9*llen, MPI_DOUBLE_COMPLEX,iproc, 
     $              MPI_COMM_WORLD,ierr)])
c
c              put it into right place here
c
               do i=1,mylen
                  tens(mylist(1,i),:,:)=t(:,:,mylist(2,i))
               end do

            end do
         endif

      end do                    ! loop over all processors

c      write(9,*)  'symmetrized tensors:'

c      do ig=1, gs%length
c         write(9,'(i6,/3(3g12.5/))') ig, dble(tens(ig,:,:))
c      end do

      if(iand(ioutput,8).eq.8) then
         write(9,100) gimmetime()-t0
         call myflush(9)
      endif



      return

 100  format(' TIME FOR TENSOR GLOBAL SYMMETRIZATION:', f12.3)

      end



