c     @process extchk
c
      subroutine take_nonloc_deriv(dir,gs, rk, psi, nanl,
     $     nloc_deriv,dpsi, crys) 

      use all_to_all_module

      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'


c
c     INPUT:
c     -----

      type(parallel_gspace) gs 

      type(crystal) crys

      integer
     $     nanl,                ! number of nonlocal projectors
     $     dir                  ! direction into which to take the derivative

      double precision
     $     rk(3)

      complex(kind=8)
     $     psi(gs%length),      ! the wavefn to which dVnl/dk is applied
     $     nloc_deriv(gs%length+nanl,nanl,2,3) !  derivative of nonlocal 
     $                          ! potential at k
c
c     OUTPUT:
c     ------
c
      complex(kind=8)
     $     dpsi(gs%length)            ! the dVnl/dk * |psi>

c
c     DESCRIPTION:
c     -----------
c
c     * applies the nonlocal potential derivative operator to the wave
c       function 
c
c     * computes derivative wrt spatial direction: dir
c       we compute data(g)*(k+G), so set the k-point of the gspace
c       to zero if you just want the gradient!
c
c     (1996) Bernd Pfrommer
c

c
c     ----------- local variables -----------------------------------
c
      complex(kind=8) dcone, dczero, dcmone
      parameter(dcone=(1.d0,0.d0), dczero=(0.d0,0.d0),
     $     dcmone=(-1.d0,0.d0))
      complex(kind=8), allocatable ::
     $     work(:)
      integer k

c
c     ----- variables for the gspace loop ---------
c
      integer
     $     igv(4),fftn(4), ffth(4),igv3, irod,iord,igs

      double precision gv(3)
c
c
      if((dir.lt.1).or.(dir.gt.3)) then
         write(9,*) ' derivative_gspace: illegal direction: ',dir
         stop
      endif
c
c     --------- apply the nonlocal part of the hamiltonian ----------
c

      dpsi=0

c      goto 200                  ! use this to disable nonlocal part...

      if(nanl.gt.0) then
         allocate(work(nanl))
c
c        the +q part
c
         call mzgemm('C', 'N', nanl, 1, gs%length, dcone, 
     $        nloc_deriv(1,1,2,dir), gs%length+nanl,psi(1),gs%length,
     $        dczero, work(1),nanl)

         do k=1,nanl
            work(k)=work(k)*dble(nloc_deriv(gs%length+k,1,1,1))
         end do

         call all_sum_all(work,nanl)
c
c        linearly combine the projectors to get +q part
c     
         call mzgemm('N','N',gs%length,1,nanl,dcone,
     $        nloc_deriv(1,1,2,dir),gs%length+nanl, work(1), nanl,
     $        dczero, dpsi(1),gs%length)
c
c        the -q part
c
         call mzgemm('C', 'N', nanl, 1, gs%length, dcone, 
     $        nloc_deriv(1,1,1,dir), gs%length+nanl, psi(1),gs%length,
     $        dczero,work(1),nanl);

         do k=1,nanl
            work(k)=work(k)*dble(nloc_deriv(gs%length+k,1,1,1))
         end do


         call all_sum_all(work,nanl)
c
c        linearly combine the projectors to get -q part
c     
         call mzgemm('N','N',gs%length,1,nanl,dcmone,
     $        nloc_deriv(1,1,1,dir),gs%length+nanl, work(1), nanl,
     $        dcone, dpsi(1),gs%length); 

         deallocate(work)
      endif
      
 200  continue
c
c     --------- take the gradient ----------
c
      fftn(1:3)=gs%fftsize(1:3)
      fftn(4)=gs%fftsize(3)
      ffth(:)=fftn(:)/2
      igs = 0 
      do iord=1, gs%lorder    ! loop through x/y gspace 
         irod=gs%order(1,iord)
         igv(1)= irod/gs%fftsize(2)
         igv(2)= mod(irod,gs%fftsize(2))
         igv(3)= gs%order(2,iord)
         igv(4)= gs%order(3,iord)
         igv(:)= mod(igv(:)+ffth(:),fftn(:))-ffth(:) 
         gv(1:2)=dble(igv(1:2))+rk(1:2)
         do igv3=igv(3),igv(4)     ! loop over z axis
            gv(3)=dble(igv3)+rk(3)
            igs = igs + 1
            dpsi(igs)=dpsi(igs)+psi(igs)*(crys%bvec(dir,1)*gv(1)+
     $           crys%bvec(dir,2)*gv(2)+crys%bvec(dir,3)*gv(3))
         end do
      end do
      return

      end
