c-*-Fortran-*- 
      subroutine update_hamiltonian(kp,ham,ldawfn, len,neig,
     $     wfn,pot_gspace, bands, crys, energs, pw_params, denc, 
     $     rden, den,  vion, vbareion)
c
c     1996 Bernd Pfrommer
c
      use all_to_all_module
      include 'use.h'
      implicit none             ! never uncomment this line. 
      include 'interface.h'
      include 'all_to_all.h'
c
c     INPUT:
c     -----
      type(kpoint) kp

      integer 
     $     len(kp%nrk),         ! length of the different gspaces
     $     neig,                ! number of occupied wave functions
     $     ldawfn               ! leading dimension of wavefn array

      type (parallel_gspace)    ! gspace for charge density and potential
     $     pot_gspace
      type (crystal) crys
      type (energy) energs
      type (pw_parameter) pw_params
      type (band) bands         

      complex(kind=8) 
     $     vbareion(pot_gspace%length), ! bare ionic potential
     $     denc(pot_gspace%length), ! the core charge density
     $     wfn(ldawfn,kp%nrk) ! the wave functions in gspace
      
c
c     INPUT/OUTPUT:
c     -------------

      type (hamiltonian)
     $     ham(kp%nrk)          ! the Hamiltonian to be updated

c
c     OUTPUT:
c     ------
      complex(kind=8) den(pot_gspace%length) ! charge density in fourier space

c
c     WORK ARRAYS:
c     -----------

      complex(kind=8) 
     $     rden(pot_gspace%r_size), ! realspace work array rden(r_size)
     $     vion(pot_gspace%length) ! work array for vion

c
c     DESCRIPTION:
c     -----------
c
c     Updates the Hamiltonian to the new wave functions
c
c
c     It is assumed that all Hamiltonians have the same realspace
c     grid  size for their representation, and it matches the
c     realspace grid for the potential.
c
c     Further, the Hamiltonians are expected to have only a single
c     local potential, since it doesn't vary with the kpoint. This
c     means that the vloc pointer of all Hamiltonians points to the
c     same location.
      
c
c     --------------- local variables -------------------------
c
      complex(kind=8) cdtot
      integer 
     $     n, i,ntot,ids,irk

c
c     figure out occupation numbers
c

      call flevel(1, 0,pw_params,energs, crys,bands,kp)
c
c     ------------- compute new charge -------------------------
c
      ntot = ham(1)%gspace%fftsize(1)*ham(1)%gspace%fftsize(2)*
     $     ham(1)%gspace%fftsize(3) 

      rden(1:ham(1)%fftstruc%r_size)=0

      cdtot = 0
      do irk=1, kp%nrk
         do n=1,neig            ! loop over occupied bands
c     
c           fourier transform to realspace
c     
            call fourier_transform(-1, ham(1)%fftstruc,
     $           ham(irk)%gspace,wfn(1+(n-1)*len(irk),irk),
     $           ham(1)%fftstruc%rspacebuf(1))
c     
c           square and add to the charge density
c
            do i=1, ham(1)%fftstruc%r_size

c               cdtot = cdtot+ ham(1)%fftstruc%rspacebuf(i)
c     $              *conjg(ham(1)%fftstruc%rspacebuf(i))
c     $              *bands%occup(n,irk,1)

               rden(i) = rden(i) + ham(1)%fftstruc%rspacebuf(i)
     $              *conjg(ham(1)%fftstruc%rspacebuf(i))
     $              *bands%occup(n,irk,1)
            end do
c            call all_sum_all(cdtot)
c            write(9,*) irk,n,cdtot/dble(ntot)
         end do
      end do  
c
c     fourier transform back to momentum space
c
      call fourier_transform(1, ham(1)%fftstruc,
     $     pot_gspace, den(1), rden(1))

c
c     symmetrize charge
c
c     symmetrize only if pot_gspace%istar true 
c     because if  pot_gspace%istar false the arrays are not defined
c     Francesco Mauri
c
      if(pw_params%isymmetrize) then
         call symmetrize_scalar_global(den(1),pot_gspace)
      endif

c
c     ------------- compute new electronic potential vion --------
c
      call velect(0, pot_gspace,crys,ham(1)%fftstruc,
     $     energs,pw_params, 
     $     denc,vion,den,rden(1))

c
c     ------------- add new vhxc to vion -------------------------
c
      vion = vion+ vbareion

      if(pot_gspace%ig0.gt.0) vion(pot_gspace%ig0)=0

c
c     ------------ setup new local potential for Hamiltonian -------
c
      call setup_local_potential(0,ham(1)%fftstruc, pot_gspace,
     $     vion(1),ham(1)%vloc(1))


      return
C
      end


