C*
C*
      subroutine velect(ipr, gs, crys, ffts, energs, pwp,
     $     denc, vout, den, chdr)
c
c     1995/96 Bernd Pfrommer
c
      use all_to_all_module
      include 'use.h'

      implicit none             ! implicit? Just say no!
      include 'interface.h'
      include 'all_to_all.h'
      include 'flibcalls.ph'

c     ---------------- arguments ----------------------------------------

c     INPUT:
c     -----

c      pwp%icorr       control parameter for correlation. it is
c                  used only if iscr = 3 or 4. for explanation
c                  see subroutine excorr

      integer 
     $     ipr                  ! print flag

      type (parallel_gspace) gs 
      type (crystal) crys
      type (fftstruc) ffts
      type (pw_parameter) pwp

      complex(kind=8) 
     $     denc(gs%length)      !  core charge density
c
c     OUTPUT:
c     ------
c
      type (energy) energs
      complex(kind=8) 
     $     vout(gs%length,crys%nspin), ! the new V_hartree+V_xc on output
     $     den(gs%length,crys%nspin) ! the valence charge density on output

c
c     WORK:
c     ----

      complex(kind=8) chdr(gs%r_size,crys%nspin)

c     DESCRIPTION:
c     -----------
c
c     * computes the xc energy  energs%xc
c     * computes the V_xc(G=0) component for both spins
c     * computes the spin-energy due to a magnetic field
c     * computes the antiferromagnetic moment energs%afmagmom

c     
c     ------------- local variables
c

      complex(kind=8)  
     $     zone, zhalf,
     $     td

      double precision fac, pi, tden(2), t0,t1, gimmetime, dum,
     $     afmagm               ! total antiferromagnetic moment

      parameter(zone=(1.d0,0.d0),zhalf=(0.5d0,0.d0),
     $     pi=3.141592653589793d0)

      integer
     $     i,j,k,
     $     imagvec,             ! index of  G of the magnetic field
     $     imagveci,            ! index of -G
     $     is, n123, ntottot, 
     $     findvec,
     $     my_ipr, isymmet,
     $     invgmagnet(4)           ! g-vector for magnetic wave

      complex(kind=8)
     $     densmag(2,2)         ! densmag(i,j) = spin i component of n(G),n(-G)

      external gimmetime, findvec
      logical isp
c
c     --------------------------------------------------------------------
c
      t0=gimmetime()

      isp = .false.
      if(crys%nspin.eq.2) isp =.true.

      ntottot = gs%r_size

      call all_sum_all(ntottot)

c     ---- add core charge density ------------

      if(isp) then
         vout(1:gs%length,1:crys%nspin)=0

         call mzaxpy(gs%length,zhalf,denc(1), 1, 
     $        vout(1,1), 1) ! take HALF of core cd
         call mzaxpy(gs%length,zhalf,denc(1), 1, 
     $        vout(1,2), 1)

         call mzaxpy(2*gs%length, zone, den(1,1), 1, 
     $        vout(1,1), 1)     ! add valence charge
      else
         call mzcopy(gs%length,denc(1), 1, 
     $        vout(1,1), 1)
         call mzaxpy(gs%length, zone, den(1,1), 1,
     $        vout(1,1), 1)
      endif

c
c     find charge components corresponding to staggered magnetic field
c
      if(isp.and.(pwp%mfield%gvec(4).eq.1)) then
         imagvec=findvec(pwp%mfield%gvec,gs)
         if(imagvec.ge.1) then
            densmag(1,1) = vout(imagvec,1)
            densmag(2,1) = vout(imagvec,2)
         endif
         invgmagnet(1:3) = -pwp%mfield%gvec(1:3)
c        cos(Gr) = 0.5*(exp(iGr) + exp(-iGr)), so have to take inverse as well

         imagveci=findvec(invgmagnet,gs)
         if(imagveci.ge.1) then
            densmag(1,2) = vout(imagveci,1)
            densmag(2,2) = vout(imagveci,2)
         endif

      endif


c
c      compute exchange and correlation potential/energy
c
c
      my_ipr=ipr

      if(pwp%icorr.ne.'pw'.and.pwp%icorr.ne.'pb') then
c
c        inverse fourier transform the charge density to real space
c
         do is=1,crys%nspin
            call fourier_transform(-1, ffts,gs,vout(1,is),chdr(1,is))
         end do

c         do i=0, gs%r_size
c            write(9,*) i, chdr(i,1)
c         end do

         call excorr(my_ipr,pwp%icorr, crys%nspin,
     $        gs%r_size, gs%fftsize(1)*gs%fftsize(2)*gs%fftsize(3),
     $        chdr(1,1), crys%vcell, energs%xc, afmagm)


         do is=1,crys%nspin
            call fourier_transform(1,ffts,gs,vout(1,is),chdr(1,is))
         end do

      else
         if(pwp%icorr.eq.'pw') then
            call pw91_excorr(my_ipr, ffts, crys, gs, vout(1,1), 
     $           energs%xc,   afmagm) 
         else
            call pbe_excorr(my_ipr, ffts, crys, gs, vout(1,1), 
     $           energs%xc,   afmagm) 
         endif
      endif

c
c      compute and add v hartree to the exchange correlation potential 
c

      fac = 8.d0*pi/crys%vcell

      energs%vxc0=0


      if(isp) then
         do i=1, gs%length
            if(gs%ekin(i).gt.0) then
               td = fac*(den(i,1)+den(i,2))/gs%ekin(i)
               vout(i,1) = vout(i,1) + td
               vout(i,2) = vout(i,2) + td
            else
               energs%vxc0(1) = dble(vout(i,1)) ! save G=0 of xc pot
               energs%vxc0(2) = dble(vout(i,2))
            endif
         end do
      else
         do i=1, gs%length
            if(gs%ekin(i).gt.0) then
               vout(i,1) = vout(i,1) 
     $              + fac*den(i,1)/gs%ekin(i)
            else
               energs%vxc0(1) = dble(vout(i,1)) ! save G=0 of xc pot
            endif
         end do
      endif
c
c     add staggered magnetic field if necessary
c
      if(isp.and.(pwp%mfield%gvec(4).eq.1)) then
         if(imagvec.ge.1) then
            vout(imagvec,1) = vout(imagvec,1)
     $           - .5d0*cmplx(pwp%mfield%h,0.d0,kind=8) ! v=-mu * H
            vout(imagvec,2) = vout(imagvec,2) 
     $           + .5d0*cmplx(pwp%mfield%h,0.d0,kind=8) ! v=+mu * H
         endif

c        cos(Gr) = exp(iGr) + exp(-iGr), so have to take inverse as well

         if(imagveci.ge.1) then
            vout(imagveci,1)=vout(imagveci,1)- 
     $           .5d0*cmplx(pwp%mfield%h,0.d0,kind=8) ! v=-mu * H
            vout(imagveci,2)=vout(imagveci,2)+ 
     $           .5d0*cmplx(pwp%mfield%h,0.d0,kind=8) ! v=+mu * H
         endif
         energs%magnetic = -0.5*pwp%mfield%h*
     $        abs((densmag(1,1)-densmag(2,1)+
     $        densmag(1,2)-densmag(2,2)))
c
c        must symmetrize afterwards
c
         call symmetrize_scalar_global(vout(1,1),gs)
         call symmetrize_scalar_global(vout(1,2),gs)

      else
         energs%magnetic =0.d0
      endif
      
      call all_sum_all(energs%vxc0,crys%nspin)
      
      energs%afmagmom = afmagm
      if((ipr.ge.2).and.isp)  write(9,300) afmagm

      if((ipr.gt.0) .and.(iand(pwp%output,8).eq.8))
     $     write(9,930) gimmetime()-t0


      return

 930  format(' TIME FOR VELECT:',f12.3)
 300  format(' TOTAL ANTIFERROMAGNETIC MOMENT:',f12.6)

      

       end




