c     @process extchk
c
      subroutine wrapup_chi(crys, ffts,
     $     gs, rho, qmag, g0mask, chi_g, chi)

c     
      include 'use.h'
      implicit none             ! implicit? Just say no!
      include 'interface.h'
c
c     INPUT:
c     -----

c     

      type (crystal) crys       ! for the lattice vectors
      type (fftstruc) ffts
      type (parallel_gspace) 
     $     gs                   ! potential gspace

      complex(kind=8)
     $     chi(3,3),            ! G=0 component
     $     rho(ffts%r_size,3,3) ! raw f(r,p0,p1)

      double precision 
     $     g0mask(3,3),         ! mask for g=0 component
     $     qmag                 ! magnitude of q

c
c     OUTPUT:
c     ------

      complex(kind=8)
     $     chi_g(gs%length,3,3) ! chi in gspace
c
c
c     1996 Bernd Pfrommer, UCB
c
c

c     ---------------- local variables ----------------------------------
       
      integer i,j,ig,ia,ib,itwoq,iq, iubxq,p0,p1, info

      double precision
     $     fac, uaxg(3), ua(3), ubxqsign, pi4vcell,
     $     met(3,3,3)           ! metric : met(i,j,alpha) = (u_alpha x b_j)_i

      complex(kind=8), allocatable ::
     $     r_g(:,:,:) ! rho_magnetic in gspace

c
c
c
      allocate(r_g(gs%length,3,3), stat=info)
      if(info.ne.0) then
         write(9,*) '*** ERROR: ALLOCATION FAILED IN WRAPUP_CHI!'
         write(9,*) '*** CURE:  REDUCE MEMORY CONSUMPTION:'
         stop
      endif



      pi4vcell=16.d0*datan(1.d0)/crys%vcell

c     
c     transform to fourier space
c

      do p0=1,3
         do p1=1,3
            call fourier_transform(1,ffts,gs,
     $           r_g(1,p0,p1), rho(1,p0,p1))
         end do
      end do

      
c
c     compute metric tensor met(i,j,alpha) = (u_alpha x b_j)_i
c
      do ia=1,3
         ua=0
         ua(ia)=1.d0
         do j=1,3
            call myvecprod(met(1,j,ia),ua,crys%bvec(1,j))
         end do
      end do
c
c     pick up contributions to chi in gspace.
c

      do ig=1, gs%length        ! loop over gspace
         chi_g(ig,:,:)=0     
         if(gs%ekin(ig).gt.0.d-8) then 
c
c        conversion factor fac:
c
c        2   for spin up/down
c        2   for Hartree-> Rydberg
c        1/2 for averaging over the two q directions perpendicular to b0
c        1/2q  because derivative is ( f(q)-f(-q) )/2q
c        4pi because B = 4pi M
c        1/vcell for the definition of the Fourier transform

            fac= pi4vcell/(qmag*gs%ekin(ig))
            do ia=1,3           ! loop over alpha index
               uaxg=matmul(met(:,:,ia),dble(gs%gvec(:,ig))) ! uaxg = u_alpha x G
               do ib=1,3        ! loop over beta index
                  do j=1,3      ! handle the u_alpha x G
                     chi_g(ig,ia,ib) = chi_g(ig,ia,ib) 
     $                    + uaxg(j)*r_g(ig,ib,j)
                  end do
c
c                 normalize
c
                  chi_g(ig,ia,ib) = chi_g(ig,ia,ib)*fac
               end do
            end do
         else
c
c           put in the G=0 component. See Jackson for details.
c           by default, assume spherical sample, i.e 2/3 of full screening
            
            do ia=1,3
               do ib=1,3
                  chi_g(ig,ia,ib) = chi(ia,ib)*pi4vcell*g0mask(ia,ib)
               end do
            end do
c            write(9,*) 'G=0 component:',chi_g(ig,:,:)
         endif
      end do
c     
c     
c
      deallocate(r_g)
      return
      end
c
c     ------- local subroutines ------------------------
c     

      subroutine myvecprod(a,b,c)
      implicit none
c
c     computes a = b x c
c      
      double precision a(3),b(3),c(3)

      a(1)=b(2)*c(3)-b(3)*c(2)
      a(2)=b(3)*c(1)-b(1)*c(3)
      a(3)=b(1)*c(2)-b(2)*c(1)

      end 




